var io = require('socket.io');
var redis = require("redis");
var https = require('https');
var express = require('express');
var fs = require('fs');

var PORT_SSL = 8080; //working https port

var sslOptions = {
    key: fs.readFileSync('/etc/apache2/ssl/local.key'), //path to key
    cert: fs.readFileSync('/etc/apache2/ssl/local.crt'), //path to cert
    ca: fs.readFileSync('/etc/apache2/ssl/local.crt') // path to ca
};

var app = express();

var serverHttps = https.createServer(sslOptions, app);
var ioHttps = io.listen(serverHttps);
serverHttps.listen(PORT_SSL);

function start() {
    console.log("Server has started.");

    var subscriber = redis.createClient();
    var recorderChannel = 'recording:status';
    var scheduleChannel = 'schedule:update';
    var scheduleDeleteChannel = 'schedule:delete';
    var recordCutChannel = 'record:cut';
    var mosaicUpdateChannel = 'mosaic:update';

    var sockets = [];
    var serverHost = '';

    ioHttps.configure(function () {
        ioHttps.enable('browser client minification');  // send minified client
        ioHttps.enable('browser client etag');          // apply etag caching logic based on version number
        ioHttps.enable('browser client gzip');          // gzip the file
        ioHttps.set('log level', 1);                    // reduce logging
        ioHttps.set('heartbeat timeout', 40);            // time from first no answer to disconnect
        ioHttps.set('heartbeat interval', 10);           // request time interval
        ioHttps.set('flash policy port', 8080);
        ioHttps.set('transports', [
            'websocket'
            , 'flashsocket'
            , 'htmlfile'
            , 'xhr-polling'
            , 'jsonp-polling'
        ]);
    });

    subscriber.on('message', function (channel, json) {
        console.log("Client channel recieve from channel : %s, the message : %s", channel, json);

        if (channel == recorderChannel) {
            ioHttps.sockets.emit(recorderChannel, json);
        }
        if (channel == scheduleChannel) {
            ioHttps.sockets.emit(scheduleChannel, json);
        }
        if (channel == scheduleDeleteChannel) {
            ioHttps.sockets.emit(scheduleDeleteChannel, json);
        }
        if (channel == recordCutChannel) {
            ioHttps.sockets.emit(recordCutChannel, json);
        }
        if (channel == mosaicUpdateChannel) {
            ioHttps.sockets.emit(mosaicUpdateChannel, json);
        }
    });

    subscriber.subscribe(recorderChannel);
    subscriber.subscribe(scheduleChannel);
    subscriber.subscribe(scheduleDeleteChannel);
    subscriber.subscribe(recordCutChannel);
    subscriber.subscribe(mosaicUpdateChannel);
}

exports.start = start;