<?php

namespace FS\ApiBundle\Controller\v1;

use FS\CamBundle\Entity\Record;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class RecordController
 * @package FS\ApiBundle\Controller\v1
 * @Route("/recordings")
 */
class RecordingController extends Controller
{
    /**
     * @Route(path="",methods={"GET"})
     * @param Request $request
     * @return JsonResponse
     */
    public function indexAction(Request $request)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $recordings = $em->getRepository('FSCamBundle:Recording')->findAll();

        $result = [];

        foreach ($recordings as $id => $recording) {
            $result[$id]['id'] = $recording->getRecord()->getId();

            $user = $this->getUser();
            $accessToRecord = 0;
            if ($user != null && is_object($user)) {
                $accessToRecord = ($user->hasAccessToRecord($recording->getRecord()) == true) ? 1 : 0;
            }

            $result[$id]['record_access'] = $accessToRecord;
            $result[$id]['name'] = $recording->getRecord()->getName();

            $result[$id]['cameras'] = $recording->getRecord()->getCamerasInRecord();
            $result[$id]['duration'] = $this->get('fs_cam.recorder')->getTime(
                $recording->getRecord()
            );
            $result[$id]['roomId'] = $recording->getRoom()->getId();
            $result[$id]['author'] = $recording->getRecord()->getCreatedBy()->getUsername();
        }

        return new JsonResponse(
            [
                'data' => [
                    'recordings' => $result
                ]
            ]
        );
    }

    /**
     * @Route(path="/{id}", methods={"GET"})
     * @param Request $request
     * @param Record $record
     * @return JsonResponse
     */
    public function recordingInfoAction(Request $request, Record $record)
    {
        if ($record->isCompleted()) {
            $result['status'] = 'completed';
        } else {
            $result['status'] = 'recording';
            $result['time'] = $this->get('fs_cam.recorder')->getTime(
                $record
            );
        }
        $result['cameras'] = $record->getCamerasInRecord();

        $result['id'] = $record->getId();
        $result['roomId'] = $record->getRoom()->getId();
        $result['name'] = $record->getName();

        $user = $this->getUser();
        $accessToRecord = 0;
        if ($user != null && is_object($user)) {
            $accessToRecord = ($user->hasAccessToRecord($record) == true) ? 1 : 0;
        }

        $result['author'] = $record->getCreatedBy()->getUsername();
        $result['record_access'] = $accessToRecord;

        return new JsonResponse(
            [
                'data' => $result
            ]
        );
    }
}
