<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 25.03.2016
 * Time: 13:26
 * Autor: <vladislav@fora-soft.com>
 */

namespace FS\ApiBundle\Controller\v1;

use FS\ApiBundle\Util\FSParameters;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Recording;
use FS\CamBundle\Entity\Room;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class RoomController
 * @package FS\ApiBundle\Controller\v1
 * @author <vladislav@fora-soft.com>
 * @Route(path="/rooms")
 */
class RoomController extends Controller
{
    /**
     * @Route(path="", methods={"GET"})
     * @param Request $request
     * @return JsonResponse
     */
    public function indexAction(Request $request)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $rooms = $em->getRepository('FSCamBundle:Room')
            ->findAll();

        $result = [];

        foreach ($rooms as $id => $room) {
            $result[$id]['id'] = $room->getId();
            $result[$id]['name'] = $room->getName();
            $result[$id]['hasRecording'] = $room->hasRecording();
            if ($room->hasRecording()) {
                $result[$id]['recording']['name'] = $room->getRecording()->getRecord()->getName();
                $result[$id]['recording']['id'] = $room->getRecording()->getRecord()->getId();
            }
            foreach ($room->getCameras() as $i => $camera) {
                $result[$id]['cameras'][$i]['name'] = $camera->getName();
                $result[$id]['cameras'][$i]['id'] = $camera->getId();
            }
        }

        return new JsonResponse(
            [
                'data' => [
                    'rooms' => $result
                ]
            ]
        );
    }

    /**
     * @Route(path="/{id}", methods={"GET"})
     * @param Request $request
     * @return JsonResponse
     */
    public function roomAction(Request $request, Room $room)
    {
        $result['id'] = $room->getId();
        $result['name'] = $room->getName();
        $result['hasRecording'] = $room->hasRecording();
        if ($room->hasRecording()) {
            $result['recording']['name'] = $room->getRecording()->getRecord()->getName();
            $result['recording']['id'] = $room->getRecording()->getRecord()->getId();
        }
        foreach ($room->getCameras() as $i => $camera) {
            $result['cameras'][$i]['name'] = $camera->getName();
            $result['cameras'][$i]['id'] = $camera->getId();
        }

        return new JsonResponse(
            [
                'data' => $result,
            ]
        );
    }

    /**
     * @Route(path="/{id}/record/start", methods={"POST"})
     * @param Request $request
     * @param Room $room
     * @return JsonResponse
     */
    public function startRecordAction(Request $request, Room $room)
    {
        $parameters = FSParameters::serialize($request->request->all());

        if ($room->hasRecording()) {
            return new JsonResponse(
                [
                    'data' => [
                        'message' => 'This room is recording',
                        'start' => false
                    ]
                ],
                400
            );
        }

        if (FSParameters::keyExist('name', $parameters)) {

            $em = $this->getDoctrine()
                ->getManager();

            $user = $this->getUser();


            $record = new Record();
            $record->setRoom($room);
            $record->setName($parameters['name']);
            $record->setCreatedBy($this->getUser());
            $record->setTemplate($user->getUserGroup()->getInfoTemplate());

            if (FSParameters::keyExist('share', $parameters)) {
                if (FSParameters::keyExist('users', $parameters['share'])) {
                    foreach (explode(",", $parameters['share']['users']) as $userId) {
                        $userInShare = $em->getRepository('FSUserBundle:User')->find($userId);
                        if ($userInShare) {
                            $record->setAccessedByUsers($userInShare);
                        }
                    }
                }
                if (FSParameters::keyExist('user_groups', $parameters['share'])) {
                    foreach (explode(",", $parameters['share']['user_groups']) as $userGroupId) {
                        $userGroupInShare = $em->getRepository('FSUserBundle:UserGroup')->find($userGroupId);
                        if ($userGroupInShare) {
                            $record->setAccessedByGroups($userGroupInShare);
                        }
                    }
                }

            }
            if (FSParameters::keyExist('template', $parameters)) {
                $record->setTemplateData($parameters['template']);
            }
            $record->setContainer(
                $this->get('fs_user.container_manager')
                    ->getUserContainer()
            );

            $em->persist($record);
            $em->flush();

            $recording = new Recording();
            $recording->setRecord($record);
            $recording->setRoom($record->getRoom());
            $recording->setStartedAt(new \DateTime);
            $em->persist($recording);
            $em->flush();

            $startResult = $this->get('fs_cam.recorder')->startRecording($record);

            if (!$startResult) {
                $em->remove($recording);
                $em->remove($record);
                $em->flush();

                return new JsonResponse(
                    [
                        'data' => [
                            'message' => 'Record start fail',
                            'start' => false
                        ]
                    ],
                    500
                );
            }
            $em->persist($record);
            $em->flush();

            return new JsonResponse(
                [
                    'data' => [
                        'id' => $record->getId(),
                        'start' => true
                    ]
                ]
            );

        } else {
            return new JsonResponse(
                [
                    'data' => [
                        'message' => 'Validation Error',
                        'fields' => [
                            'name' => 'name',
                            'message' => "This value should not be blank."
                        ]
                    ]
                ],
                400
            );
        }
    }

    /**
     * @Route(path="/{id}/record/stop", methods={"POST"})
     * @param Request $request
     * @param Room $room
     * @return JsonResponse
     */
    public function stopRecordAction(Request $request, Room $room)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $recording = $room->getRecording();

        if ($recording) {
            $schedule = $recording->getSchedule();
            if ($schedule) {
                $schedule->setStatus(\FS\CamBundle\Entity\Schedule::STATUS_PENDING);
            }
            $this->get('fs_cam.recorder')->stopRecording($recording->getRecord());
            $recording->getRecord()->setIsCompleted();

            $em->remove($recording);
            $em->flush();

            return new JsonResponse(
                [
                    'data' => [
                        'id' => $recording->getRecord()->getId(),
                        'status' => true
                    ]
                ]
            );
        }

        return new JsonResponse(
            [
                'data' => [
                    'message' => "recording in room not found",
                    'status' => false
                ]
            ],
            404
        );
    }
}