<?php

namespace FS\BasicBundle\Controller;

use FS\UserBundle\Entity\UserGroupRight;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\ExpressionLanguage\Expression;
use Symfony\Component\HttpFoundation\JsonResponse;

class DefaultController extends Controller
{
    /**
     * @Route(path="/", name="home")
     * @Template()
     */
    public function indexAction()
    {
        $context = $this->get('security.context');
        $links = [];
        if ($context->isGranted(new Expression('user.hasRight("monitoring") || ("ROLE_ADMIN" in user.getRoles())'))) {
            $links[] = array(
                'title' => 'Observe',
                'icon' => 'video-camera fa-2x',
                'path' => $this->get('router')->generate('live'),
            );
        }

        if ($context->isGranted(new Expression('user.hasRight("review") || ("ROLE_ADMIN" in user.getRoles())'))) {
            $links[] = array(
                'title' => 'Review',
                'icon' => 'film fa-2x',
                'path' => $this->get('router')->generate('record'),
            );
        }

        if ($context->isGranted(new Expression('user.hasRight("schedule") || ("ROLE_ADMIN" in user.getRoles())'))) {
            $links[] = array(
                'title' => 'Schedule',
                'icon' => 'calendar fa-2x',
                'path' => $this->get('router')->generate('schedule'),
            );
        }

        if ($context->isGranted(new Expression("user.hasRight('upload') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $links[] = array(
                'title' => 'Upload',
                'icon' => 'upload fa-2x',
                'class' => 'ajax-modal',
                'dataOp' => 'reload',
                'path' => $this->get('router')->generate('record_upload_new'),
            );
        }

        if ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--general') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $links[] = array(
                'title' => 'Admin',
                'icon' => 'cog fa-2x',
                'path' => $this->get('router')->generate('admin_general'),
            );
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--users-and-groups') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $links[] = array(
                'title' => 'Admin',
                'icon' => 'cog fa-2x',
                'path' => $this->get('router')->generate('admin_user'),
            );
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--templates') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $links[] = array(
                'title' => 'Admin',
                'icon' => 'cog fa-2x',
                'path' => $this->get('router')->generate('admin_template'),
            );
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--rooms') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $links[] = array(
                'title' => 'Admin',
                'icon' => 'cog fa-2x',
                'path' => $this->get('router')->generate('admin_camera'),
            );
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--help') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $links[] = array(
                'title' => 'Admin',
                'icon' => 'cog fa-2x',
                'path' => $this->get('router')->generate('admin_help'),
            );
        }


        return array('links' => $links);
    }

    /**
     * @Route(path="/help", name="help")
     * @Template()
     */
    public function helpAction()
    {
        $em = $this->getDoctrine()->getManager();

        $topics = $em->getRepository('FSCamBundle:HelpTopic')->findAll();

        return ['topics' => $topics];
    }

}
