<?php

/**
 *
 * @author:  Gabriel BONDAZ <gabriel.bondaz@idci-consulting.fr>
 * @licence: GPL
 *
 */

namespace FS\BasicBundle\Form\Type;

use FS\BasicBundle\Form\Transformer\DurationToIntTransformer;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class DurationType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $hours = [];
        for ($i = 0; $i <= 3; $i++) {
            $hours[$i] = $i . ' Hours';
            switch ($i) {
                case 1:
                    $hours[$i] = $i . ' Hour';
                    break;
                default:
                    $hours[$i] = $i . ' Hours';
            }

        }

        $minutes = [];
        for ($i = 0; $i <= 59; $i = $i + 5) {
            $minutes[$i] = $i . ' Minutes';
        }

        $builder
          ->add(
            'hours',
            'choice',
            [
              'choices' => $hours,
              'label' => false,
              'horizontal_input_wrapper_class' => 'col-w-2 schedule-time-mg'
            ]
          )
          ->add(
            'minutes',
            'choice',
            [
              'choices' => $minutes,
              'label' => false,
              'horizontal_input_wrapper_class' => 'col-w-2 schedule-time-mg'
            ]
          );

        $transformer = new DurationToIntTransformer();
        $builder->addModelTransformer($transformer);
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
          [
            'error_bubbling' => false,
            'attr' => ['class' => 'custom-row']
          ]
        );
    }

    public function getParent()
    {
        return 'form';
    }

    public function getName()
    {
        return 'fs_duration';
    }
}
