<?php

namespace FS\BasicBundle\Menu;

use Knp\Menu\FactoryInterface;
use Symfony\Component\ExpressionLanguage\Expression;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\SecurityContext;

class Builder
{

    private $factory;

    private $securityContext;

    /**
     * @param FactoryInterface $factory
     */
    public function __construct(FactoryInterface $factory, SecurityContext $securityContext)
    {
        $this->factory = $factory;
        $this->securityContext = $securityContext;
    }


    public function createMainMenu(Request $request)
    {
        $menu = $this->factory->createItem(
          'root',
          ['navbar' => true]
        );

        $context = $this->securityContext;

        $menu->addChild('Home', array('route' => 'home'));
        if ($context->isGranted(new Expression("user.hasRight('monitoring') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Observe', array('route' => 'live'));
        }
        if ($context->isGranted(new Expression("user.hasRight('review') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Review', array('route' => 'record'));
        }
        if ($context->isGranted(new Expression("user.hasRight('schedule') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Schedule', array('route' => 'schedule'));
        }


// todo separated make admin page with redirect to granted page
        if ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--general') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Admin', array('route' => 'admin_general'));
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--users-and-groups') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Admin', array('route' => 'admin_user'));
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--templates') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Admin', array('route' => 'admin_template'));
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--rooms') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Admin', array('route' => 'admin_camera'));
        } elseif ($context->isGranted(new Expression("user.hasRight('admin') && user.hasRight('admin--help') || 'ROLE_ADMIN' in user.getRoles()"))) {
            $menu->addChild('Admin', array('route' => 'admin_help'));
        }

        $routeName = $request->get('_route');
        switch (true) {
            case preg_match('/^admin?_/', $routeName):
                if ($admin = $menu->getChild('Admin')) {
                    $admin->setCurrent(true);
                }
                break;
            case preg_match('/^record?_/', $routeName):
                if ($record = $menu->getChild('Review')) {
                    $record->setCurrent(true);
                }
                break;
            default:
                $menu->setCurrent(true);
        }


        // ... add more children
        return $menu;
    }
}
