<?php

namespace FS\CamBundle\Command;

use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Recur;
use FS\CamBundle\Entity\RetentionRule;
use FS\CamBundle\Entity\Schedule;
use FS\CamBundle\Entity\Wowza;
use FS\UserBundle\Entity\Log;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use FS\CamBundle\Entity\Recording;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Validator\Constraints\True;

class DeleteOutdatedCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName('recorder:outdated:delete')
            ->setDescription('Run outdated records deletion');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        //date_default_timezone_set ("UTC");
        /** @var \FS\CamBundle\Utils\Recorder $recorder */
        $recordFiles = $this->getContainer()->get('fs_cam.record_files');

        $now = new \DateTime('now');

        /** @var \Doctrine\ORM\EntityManager $em */
        $em = $this->getContainer()->get('doctrine')->getManager();

        //TODO Clean old retentions

        $records = $em->getRepository("FSCamBundle:Record")->findAll();
        $toDelete = [];
        foreach ($records as $record) {
            /** @var RetentionRule $rule */
            $rule = $record->getRetentionRule();
            if (is_null($rule))
                $rule = $record->getParentRetentionRule();
            if (is_null($rule))
                $rule = $record->getCreatedBy()->getUserGroup()->getDefaultRetentionRule();

            $outDate = clone $now;
            switch ($rule->getType()) {
                case RetentionRule::KEEP_FOR_DAYS:
                    $outDate = $record->getStartedAt()
                        ->add(new \DateInterval("P" . $rule->getKeepTime() . "D"));
                    break;
                case RetentionRule::KEEP_FOR_HOURS:
                    $outDate = $record->getStartedAt()
                        ->add(new \DateInterval("PT" . $rule->getKeepTime() . "H"));
                    break;
                case RetentionRule::KEEP_FOR_MINUTES:
                    $outDate = $record->getStartedAt()
                        ->add(new \DateInterval("PT" . $rule->getKeepTime() . "M"));
                    break;
                case RetentionRule::KEEP_UNTIL:
                    $outDate = $rule->getKeepUntil();
                    break;
                default:
                    continue;
            }
            if ($now->diff($outDate)->invert)
                $toDelete[] = $record;
        }
        /** @var Record $record */
        foreach ($toDelete as $record) {
            $output->writeln("Removing Record #" . $record->getId());
            $buzz = $this->getContainer()->get('buzz');
            if ($record->getRoom() != null) {
                $wowzas = $record->getRoom()->getWowzas();

            } else {
                $wowzas = $this
                    ->getContainer()
                    ->get('fs_cam.wowza_manager')
                    ->getAllWowzas();
            }
            if ($wowzas != null) {
                /** @var Wowza $wowza */
                foreach ($wowzas as $wowza) {
                    $buzz->post(
                        'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                        ['Content-Type' => 'application/json'],
                        json_encode([
                                'command' => 'deleteRecord',
                                'recordId' => (string)$record->getId(),
                                'type' => 'live',
                                'userId' => "1",
                            ]
                        )
                    );
                }
            }
            $extraFields = function(Log $log) {
                $log->setExtraInformation('Retention delete record');
            };

            $this->getContainer()->get('fs_user.util.logger')->logAction(
                Log::OB_TYPE_VIDEO,
                $record,
                null,
                Log::$actions[Log::OB_TYPE_VIDEO]['delete'],
                $extraFields);
        }

        $em->flush();
    }
}
