<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 29.10.2015
 * Time: 16:33
 */

namespace FS\CamBundle\Command;


use FS\CamBundle\Entity\MosaicQueue;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class MosaicCreateCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName('recorder:mosaic:create')
            ->setDescription('Run mosaic create');
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int|null|void
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output->writeln('<info>Start</info>');
        $now = new \DateTime();
        $output->writeln('<info>' . $now->format('m/d/Y H:i:s') . '</info>');
        try {
            /** @var \Doctrine\ORM\EntityManager $em */
            $em = $this->getContainer()->get('doctrine')->getManager();
            $wowzaManager = $this->getContainer()->get('fs_cam.wowza_manager');
            $wowzaClient = $this->getContainer()->get('fs_cam.wowza_sender_client');
        } catch (\Exception $e) {
            $output->writeln('<info>Exception get services </info>');
            $output->writeln('<info>' . $e->getMessage() . '</info>');
            return;
        }
        $output->writeln('<info>get All services done</info>');

        //run every 10 minutes, and check for new mosaic requests
        $mosaics = $em->getRepository('FSCamBundle:MosaicQueue')->getAllWaiting();

        $output->writeln('<info>Mosaic get done</info>');
        $output->writeln('<info>mosaics waiting count = ' . count($mosaics) . '</info>');

        try {
            $localWowza = $wowzaManager->getLocalWowza();
        } catch (\Exception $e) {
            $output->writeln('<info>Exception get Local Wowza </info>');
            $output->writeln('<info>' . $e->getMessage() . '</info>');
            return;
        }

        if($localWowza != null){
            $output->writeln('<info>Local wowza get Done</info>');
        } else {
            $output->writeln('<info>Local wowza get False</info>');
            return;
        }

        /** @var MosaicQueue $mosaic */
        foreach ($mosaics as $mosaic) {
            try {
                //prepare
                $output->writeln('<info>Start prepare mosaic id= ' . $mosaic->getId() . '</info>');
                $mosaic->setStatus(MosaicQueue::STATUS_DOWNLOADING);
                $res = $wowzaManager->prepareToMosaic($mosaic->getRecord());
                if (!$res) {
                    $output->writeln('<info>Downloading fail</info>');
                    $mosaic->setStatus(MosaicQueue::STATUS_FAIL);
                    $em->persist($mosaic);
                    continue;
                }

                $res = $wowzaClient->post(
                    'http://' . $localWowza->getAddress() . ':8086/dustinSystemData',
                    ['Content-Type' => 'application/json'],
                    json_encode([
                            'command' => 'makeMosaic',
                            'id' => (string)$mosaic->getRecord()->getId(),
                        ]
                    )
                );

                if ($res['msg'] != 'good') {
                    $output->writeln('<info>Send fail.</info>');
                    $mosaic->setStatus(MosaicQueue::STATUS_FAIL);
                    $em->persist($mosaic);
                    continue;

                }
                $mosaic->setStatus(MosaicQueue::STATUS_CREATING);
                $em->persist($mosaic);
                $output->writeln('<info>All good</info>');
                // after preparation done, send to script
                //script done creating mosaic and set it to the done
            } catch (\Exception $e) {
                $output->writeln('<info>Exception mosaic id= ' . $mosaic->getId() . '</info>');
                $output->writeln('<info>' . $e->getMessage() . '</info>');
            }
        }
        $em->flush();

        //step 2 check
        // mosaic is waiting for 2 weeks then deleted
        // and all failed mosaic delete
        $old = $em->getRepository('FSCamBundle:MosaicQueue')->getOldForDelete();
        foreach ($old as $mosaic) {
            try {
                $output->writeln('<info>Removing mosaic id= ' . $mosaic->getId() . '</info>');
                $res = $wowzaClient->post(
                    'http://' . $localWowza->getAddress() . ':8086/dustinSystemData',
                    ['Content-Type' => 'application/json'],
                    json_encode([
                            'command' => 'deleteMosaic',
                            'id' => (string)$mosaic->getRecord()->getId(),
                        ]
                    )
                );
                $output->writeln('<info>Remove mosaic result ' . $res['msg'] . '</info>');
                $em->remove($mosaic);
            } catch (\Exception $e) {
                $output->writeln('<info>Exception remove mosaic id= ' . $mosaic->getId() . '</info>');
                $output->writeln('<info>' . $e->getMessage() . '</info>');
            }
        }
        $em->flush();

    }
}