<?php

namespace FS\CamBundle\Controller;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManager;
use FS\CamBundle\Entity\Alert;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\MosaicQueue;
use FS\CamBundle\Entity\Preset;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Recording;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Entity\Schedule;
use FS\CamBundle\Entity\Wowza;
use FS\UserBundle\Entity\Log;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;


/**
 * @Route("api")
 */
class ApiController extends Controller
{

    const RECORD_CUT_REDIS_CHANNEL = 'record:cut';

    /**
     * @Route(path="/cameras", name="api_cameras")
     */
    public function camerasAction(Request $request)
    {
        $em = $this->getDoctrine()->getManager();
        /** @var Camera[] $cameras */
        $cameras = $em->getRepository('FSCamBundle:Camera')
            ->getWowzaCameras($request->query->get('name'));
        $response = [];

        $controllers = [];
        foreach ($cameras as $camera) {
            $cameraData = $camera->toArray();
            if ($camera->getDeviceType() == Camera::DEVICE_TYPE_CARD_READER) {
                if (!in_array($camera->getIp(), $controllers)) {
                    $controllers[] = $camera->getIp();
                    $response['cameras'][] = $cameraData;
                }
            } else {
                $response['cameras'][] = $cameraData;
            }

        }
        return new JsonResponse($response);
    }

    /**
     * @Route(path="/camera", name="api_camera_view")
     * @param Request $request
     * @return JsonResponse
     * @throws \Exception
     */
    public function cameraReadAction(Request $request)
    {
        $cameraId = $request->query->get('cameraId');
        if (!$cameraId) {
            throw new \Exception('cameraId is not set');
        }
        $em = $this->getDoctrine()->getManager();
        /** @var \FS\CamBundle\Entity\Camera $camera */
        $camera = $em->getRepository('FSCamBundle:Camera')->findOneByIdJoinedToPreset($cameraId);
        if (!$camera) {
            throw $this->createNotFoundException('Unable to find Camera entity.');
        }

        $response = $camera->toArray();

        return new JsonResponse($response);
    }

    /**
     * @Route(path="/sendCamerasState", name="api_camera_set_state")
     * @Method("POST")
     */
    public function sendCamerasStateAction(Request $request)
    {
        $em = $this->getDoctrine()->getManager();

        $content = json_decode($request->getContent());

        if (!isset($content->dateTime) || !isset($content->cameras)) {
            return new Response('dateTime or cameras not set', 500);
        }

        $createdAt = new \DateTime();
        $createdAt->setTimestamp($content->dateTime);


        $cameraStatuses = [];
        foreach ($content->cameras as $camera) {

            $cameraStatuses[$camera->cameraId] = $camera->status;
        }

        /** @var Camera[] $cameras */
        $cameras = $em
            ->createQuery('SELECT c FROM FSCamBundle:Camera c WHERE c.id IN (:cids)')
            ->setParameter('cids', array_keys($cameraStatuses))
            ->getResult();

        /** @var Camera $camera */
        foreach ($cameras as $camera) {
            $camera->setStatusFromWowza($cameraStatuses[$camera->getId()]);
            $em->persist($camera);

            if ($camera->getCameraType() == Camera::TYPE_DEVICE) {
                continue;
            }

            $alert = new Alert();

            $alert
                ->setCreatedAt($createdAt)
                ->setAlertType(Alert::ALERT_TYPE_CAMERA)
                ->setContainer(null)
                ->setCamera($camera);


            switch ($cameraStatuses[$camera->getId()]) {
                case 'online':
                    $alert->setMessage('Camera online.');

                    break;
                case 'offline':
                    $alert->setMessage('Camera offline.');
                    break;
            }
            $em->persist($alert);
        }


        $em->flush();

        return new Response();
    }

    /**
     * @Route(path="/sendWowzaState", name="api_wowza_set_state")
     * @Method("POST")
     */
    public function sendWowzaAlert(Request $request)
    {
        //erverAddress : ..., available
        $em = $this->getDoctrine()->getManager();
        // TODO:parse Content
        $content = json_decode($request->getContent());
        $createdAt = new \DateTime("now");

        $alert = new Alert();

        /** @var Wowza $wowza */
        $wowza = $em->getRepository("FSCamBundle:Wowza")->getWowzaByAddresses([$content->serverAddress]);
        $alert
            ->setCreatedAt($createdAt)
            ->setAlertType(Alert::ALERT_TYPE_WOWZA)
            ->setWowza($wowza);
        $alert->setMessage("Media Server low disk space < " . $content->available . "GB");
        $em->persist($alert);
        $em->flush();

        return new Response();
    }

    /**
     * @Route(path="/preset/create", name="api_preset_create")
     */
    public function presetCreateAction(Request $request)
    {
        $name = $request->request->get('presetName');
        $data = $request->request->get('presetData');
        $cameraId = $request->request->get('cameraId');

        if (!isset($name, $data, $cameraId)) {
            throw new \Exception('presetName, presetData or cameraId is not set');
        }
        $camera = $this->getDoctrine()->getManager()->find('FSCamBundle:Camera', $cameraId);
        if (!$camera) {
            throw $this->createNotFoundException('Unable to find Camera entity.');
        }
        $preset = new Preset();
        $preset->setName($name);
        $preset->setData($data);
        $preset->setCamera($camera);
        $em = $this->getDoctrine()->getManager();
        $em->persist($preset);
        $em->flush();

        return new JsonResponse(array('presetId' => $preset->getId()));
    }

    /**
     * @Route(path="/preset/remove", name="api_preset_remove")
     */
    public function presetRemoveAction(Request $request)
    {
        $presetId = $request->request->get('presetId');
        if (!$presetId) {
            throw new \Exception('presetId is not set');
        }
        $em = $this->getDoctrine()->getManager();
        $preset = $em->find('FSCamBundle:Preset', $presetId);
        if (!$preset) {
            throw $this->createNotFoundException('Unable to find Preset entity.');
        }
        $em = $this->getDoctrine()->getManager();
        $em->remove($preset);
        $em->flush();

        return new Response('TRUE');
    }

    /**
     * @Route(path="/rooms", name="api_rooms")
     */
    public function roomsAction()
    {
        /** @var \FS\UserBundle\Entity\User $user */
        $user = $this->getUser();

        //TODO: SUPER ADMIN!
        $rooms = $user->getUserGroup()->getRooms();
        $rooms = $this->get('fs_user.container_manager')->filterArray($rooms->toArray());
        $sharedRooms = $user->getSharedRooms();

        $rooms = new ArrayCollection(
            array_unique(array_merge(
                $rooms, $sharedRooms->toArray()
            ), SORT_REGULAR)
        );

//        $rooms = $em->getRepository('FSCamBundle:Room')->findAll();
        $data = array();
        /** @var Room $room */
        foreach ($rooms as $id => $room) {
            $data[$id]['name'] = $room->getName();
            $data[$id]['id'] = $room->getId();
            $data[$id]['recording'] = $room->hasRecording();
            if ($room->hasRecording()) {
                $data[$id]['camerasInRecord'] = $room->getRecording()->getRecord()->getCamerasInRecord();
                $data[$id]['recordingTime'] = $this->get('fs_cam.recorder')->getTime(
                    $room->getRecording()->getRecord()
                );
                $data[$id]['recordId'] = $room->getRecording()->getRecord()->getId();
                $user = $this->getUser();
                $accessToRecord = 0;

                if ($user != null && is_object($user)) {
                    $accessToRecord = ($user->hasAccessToRecord($room->getRecording()->getRecord()) == true) ? 1 : 0;
                }
                $data[$id]['recordAuthor'] = $room->getRecording()->getRecord()->getCreatedBy()->getId();
                $data[$id]['accessToView'] = $accessToRecord;
            }

            $camId = 0;
            /** @var Camera $camera */
            foreach ($room->getCameras() as $camera) {
                if ($camera->getDeviceType() == Camera::DEVICE_TYPE_CARD_READER) {
                    continue;
                }
                $data[$id]['cameras'][$camId]['name'] = $camera->getName();
                $data[$id]['cameras'][$camId]['room'] = $room->getId();
                $data[$id]['cameras'][$camId]['id'] = $camera->getId();
                $data[$id]['cameras'][$camId]['wowza_address'] = $camera->getWowza()->getAddress();
                $data[$id]['cameras'][$camId]['wowza_port'] = $camera->getWowza()->getPort();
                $data[$id]['cameras'][$camId]['wowza_ssl'] = $camera->getWowza()->getUseSsl();
                if ($camera->getCameraType() == Camera::TYPE_DEVICE) {
                    $data[$id]['cameras'][$camId]['type'] = Camera::TYPE_DEVICE;
                } else {
                    $data[$id]['cameras'][$camId]['type'] = Camera::TYPE_CAMERA;
                }
                ++$camId;
            }
        }

        return new JsonResponse($data);
    }

    /**
     * @Route(path="/getRecordsInfo", name="api_record")
     */
    public function getRecordsInfoAction(Request $request)
    {
        $recordId = (int)$request->query->get('recordId');
        if (!$recordId) {
            throw new \Exception('recordId is not set');
        }

        $em = $this->getDoctrine()->getManager();
        /** @var Record $record */
        $record = $em->find('FS\CamBundle\Entity\Record', $recordId);

        $result = $this->get('fs_cam.record_files')->getRecordVideos($record);
        /** @var Room $room */
        $room = $record->getRoom();
        $videos = [];
        $i = 0;
        if ($record->getRecordType() == Record::RECORD_TYPE_RECORD || $record->getRecordType() == null) {
            if ($room == null) {
                foreach ($result as $files) {
                    foreach ($files['names'] as $file) {
                        /** @var Wowza $wowza */
                        $wowza = $this->get("fs_cam.wowza_manager")->getLocalWowza();
                        if ($wowza->getUseSsl()) {
                            $videos[$i]['url'] = 'rtmps://' . $wowza->getAddress() . ':' .
                                $wowza->getPort() . '/vod/mp4:' .
                                $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                        } else {
                            $videos[$i]['url'] = 'rtmp://' . $wowza->getAddress() . ':1935/vod/mp4:' .
                                $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                        }
                        $videos[$i++]['name'] = "Video";
                    }
                }
            } else {
                foreach ($result as $files) {
                    foreach ($files['names'] as $file) {
                        $cameraId = preg_split("/[\\s_.]+/", $file->name)[2];
                        $cameras = $record->getOnlyCamerasInRecord();
                        if ($cameras != null) {
                            foreach ($cameras as $camId => $camName) {
                                if ($camId == $cameraId) {
                                    $camera = $em->getRepository('FSCamBundle:Camera')->find($cameraId);
                                    /** @var Wowza $wowza */
                                    $wowza = $this->get('fs_cam.wowza_manager')->findWowzaByRecordAndCamera($record, $camera);
                                    if ($wowza->getUseSsl()) {
                                        $videos[$i]['url'] = 'rtmps://' . $wowza->getAddress() . ':' .
                                            $wowza->getPort() . '/vod/mp4:' .
                                            $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                                    } else {
                                        $videos[$i]['url'] = 'rtmp://' . $wowza->getAddress() . ':1935/vod/mp4:' .
                                            $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                                    }
                                    $videos[$i++]['name'] = $camName;
                                }
                            }
                        } else {
                            $camera = $room->getCameras()->filter(
                                function ($entry) use ($cameraId) {
                                    /** @var Camera $entry */
                                    if ($entry->getId() == $cameraId)
                                        return $entry;
                                })->first();
                            if ($camera) {
                                /** @var Wowza $wowza */
                                $wowza = $this->get('fs_cam.wowza_manager')->findWowzaByRecordAndCamera($record, $camera);
                                if ($wowza->getUseSsl()) {
                                    $videos[$i]['url'] = 'rtmps://' . $wowza->getAddress() . ':' .
                                        $wowza->getPort() . '/vod/mp4:' .
                                        $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                                } else {
                                    $videos[$i]['url'] = 'rtmp://' . $wowza->getAddress() . ':1935/vod/mp4:' .
                                        $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                                }
                                $videos[$i++]['name'] = $camera->getName();
                            } else {
                                $wowza = $files['wowza'];
                                if ($wowza->getUseSsl()) {
                                    $videos[$i]['url'] = 'rtmps://' . $wowza->getAddress() . ':' .
                                        $wowza->getPort() . '/vod/mp4:' .
                                        $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                                } else {
                                    $videos[$i]['url'] = 'rtmp://' . $wowza->getAddress() . ':1935/vod/mp4:' .
                                        $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                                }
                                $videos[$i++]['name'] = "Video";
                            }
                        }
                    }
                }
            }

            sort($videos);

        } else {
            foreach ($result as $files) {
                foreach ($files['names'] as $file) {
                    /** @var Wowza $wowza */
                    $wowza = $this->get("fs_cam.wowza_manager")->getLocalWowza();
                    if ($wowza->getUseSsl()) {
                        $videos[$i]['url'] = 'rtmps://' . $wowza->getAddress() . ':' .
                            $wowza->getPort() . '/vod/mp4:' .
                            $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                        if (isset($file->inverted)) {
                            $videos[$i]['inverted'] = $file->inverted;
                        }
                    } else {
                        $videos[$i]['url'] = 'rtmp://' . $wowza->getAddress() . ':1935/vod/mp4:' .
                            $wowza->getStorageFolder() . '/video/' . $recordId . '/' . $file->name;
                        if (isset($file->inverted)) {
                            $videos[$i]['inverted'] = $file->inverted;
                        }

                    }
                    $videos[$i++]['name'] = "Video";
                }
            }

            sort($videos);
        }
        $markers = [];
        $rawMarkers = $record->getMarkers();
        $rawMarkers = $this->get('fs_user.container_manager')->filterArray($rawMarkers->toArray());
        /** @var \FS\CamBundle\Entity\Marker $marker */
        foreach ($rawMarkers as $marker) {
            $markers[] = [
                'id' => (string)$marker->getId(),
                'time' => (string)$marker->getTime(),
                'note' => $marker->getName(),
            ];
        }


        return new JsonResponse(['videos' => $videos, 'markers' => $markers]);

    }

    /**
     * @Route("/record-time/{room}/", name="record_time", options={"expose"=true})
     */
    public function getRecordTimeAction(Room $room)
    {
        $em = $this->getDoctrine()->getManager();
        /** @var Recording $recording */
        $recording = $em->getRepository('FSCamBundle:Recording')->findOneByRoom($room);

        if ($recording) {
            $result = $this->get('fs_cam.recorder')->getTime($recording->getRecord());

            return new JsonResponse(['time' => $result]);
        } else {
            return new JsonResponse([], 500);
        }
    }

    /**
     * @Route("/get-record-id-by-room-id/{room}/", name="record_by_room", options={"expose"=true})
     * @param Room $room
     * @return JsonResponse
     */
    public function getCurrentRoomRecord(Room $room)
    {
        /** @var EntityManager $em */
        $em = $this->getDoctrine()->getManager();
        /** @var Recording $recording */
        $recording = $em->getRepository('FSCamBundle:Recording')->findOneByRoom($room);

        if ($recording) {
            return new JsonResponse(['id' => $recording->getRecord()->getId()]);
        } else {
            return new JsonResponse([], 500);
        }
    }

    /**
     * @Route(path="/uploadConvertAudioForIPCamera", name="api_convert_file")
     * @deprecated
     */
    public function uploadConvertAudioForIPCameraAction(Request $request)
    {
        if (isset ($GLOBALS["HTTP_RAW_POST_DATA"])) {
            $filePath = realpath(__DIR__ . '/../../../../web/audio/') . '/';

            //the image file name
            $fileName = substr(md5("1asdF6" . microtime() . mt_rand()), 0, 20) . ".wav";
            $newFileName = substr(md5("1aasdf" . microtime() . mt_rand()), 0, 20) . ".au";

            // get the binary stream
            $im = $GLOBALS["HTTP_RAW_POST_DATA"];

            //write it
            $fp = fopen($filePath . $fileName, 'wb');
            fwrite($fp, $im);
            fclose($fp);
            $command = "/usr/local/bin/ffmpeg -i " . $filePath . $fileName . " -acodec pcm_mulaw -ar 8000 -ac 1 -y " . $filePath . $newFileName;
            exec($command);

            //echo the fileName;
            $responseText = 'http://' . $_SERVER['SERVER_NAME'] . ':' . $_SERVER['SERVER_PORT'] . '/audio/' . $newFileName;
            unlink($fileName);
        } else {
            $responseText = 'No data';
        }

        return new Response($responseText);
    }

    /**
     * Clip record method
     * @Route(path="/clipRecord/{id}/{start}/{end}", name="api_clip_record")
     * @param Record $recordToClip
     * @param $start
     * @param $end
     *
     * @return Response
     */
    public function clipRecordAction(Record $recordToClip, $start, $end)
    {
        if (((int)$start >= (int)$end) || (int)$end == 0) {
            return new JsonResponse(['End time must be bigger than start time'], 400);
        }
        $em = $this->getDoctrine()->getManager();

        $clipRecord = clone $recordToClip;
        // $newTime = $clipRecord->getStartedAt()->modify('+1 second');
        //$clipRecord->setStartedAt($newTime);

        $clipRecord->setName($clipRecord->getName() . " (Clip)");
        $clipRecord->setIsUncompleted();
        $clipRecord->setDuration($end - $start);

        $em->persist($clipRecord);
        $em->flush();
        $result = $this->get('fs_cam.recorder')->clipRecord($recordToClip, $clipRecord, $start, $end);

        if (!$result) {
            $em->remove($clipRecord);
            $em->flush();
            return new JsonResponse(['error']);
        }

        return new JsonResponse(['processed']);
    }

    /**
     * @Route(path="/clipRecordResponse", name="api_clip_record_response")
     *
     * {recordId=#, successfulyComplete=1, clipRecordId=#}
     * @param Request $request
     * @return JsonResponse
     */
    public function clipRecordResponseAction(Request $request)
    {
        $recordId = $this->get('request')->request->get('recordId');
        $success = $this->get('request')->request->get('successfulyComplete');
        $clipId = $this->get('request')->request->get('clipRecordId');

        $em = $this->getDoctrine()->getManager();

        $record = $em->getRepository('FSCamBundle:Record')->find($recordId);
        $clip = $em->getRepository('FSCamBundle:Record')->find($clipId);

        if ($success == 1) {
            $clip->setIsCompleted();
            $em->persist($clip);
            $msg = "Clip Saved:
            <a href='" . $this->generateUrl('record_show', ['id' => $clip->getId()]) . "' >" . $clip->getName() . "</a>
                to view please check your search result";
        } else {
            $em->remove($clip);
            $msg = 'Something gone wrong';
        }

        $this->get('snc_redis.default')->publish(
            self::RECORD_CUT_REDIS_CHANNEL,
            json_encode(
                [
                    'record_id' => $record->getId(),
                    'msg' => $msg,
                ]
            )
        );
        $em->flush();

        return new JsonResponse(['processed']);
    }

    /**
     * @Route(path="/cameraIOResponse", name="api_io_camera_response")
     * @param Request $request
     * @deprecated
     * @return JsonResponse
     */
    public function cameraIOResponseAction(Request $request)
    {
        ///Old method not working
        return new JsonResponse(['processed']);
    }

    /**
     * @return \FS\CamBundle\Utils\Recorder
     */
    protected function getRecorder()
    {
        return $this->get('fs_cam.recorder');
    }

    /**
     * @Route(path="/checkUserSession", name="check_user_session")
     */
    public function checkUserSessionAction()
    {
        /** @var User $user */
        $user = $this->getUser();
        if ($user == null) {
            return new JsonResponse('true');
        } else {
            if (!$user->hasAccessNow() || $user->isDeleted()) {
                return new JsonResponse('true');
            }
            return new JsonResponse('false');
        }

    }

    /**
     * @Route(path="/wowza/notification/stop/{record}", name="wowza_notificate_stop_record")
     * @param Request $request
     * @param Record $record
     * @return JsonResponse
     */
    public function wowzaNotificationStopRecordAction(Request $request, Record $record)
    {
        $em = $this->getDoctrine()
            ->getManager();
        $room = $record->getRoom();
        if ($room->hasRecording()) {
            $recording = $room->getRecording();
            $schedule = $recording->getSchedule();
            if ($schedule) {
                $schedule->setStatus(Schedule::STATUS_PENDING);
            }
            $this->getRecorder()->stopRecording($recording->getRecord());
            $recording->getRecord()->setIsCompleted();
            $em->remove($recording);
            $em->flush();
        }

        return new JsonResponse(['processed']);
    }

    /**
     * @Route(path="/wowza/notification/service/stop", name="wowza_notificate_service_stop")
     *
     * not used.
     * @param Request $request
     * @return JsonResponse
     */
    public function wowzaNotificationStop(Request $request)
    {
        $wowzaAddress = $request->request->get('serverAddress');
        $em = $this->getDoctrine()->getManager();

        /** @var Wowza $wowza */
        $wowza = $em->getRepository("FSCamBundle:Wowza")->getWowzaByAddresses([$wowzaAddress]);
        $recordings = $em->getRepository('FSCamBundle:Recording')->findAll();
        foreach ($recordings as $recording) {
            $wowzas = $recording->getRoom()->getWowzas();
            if (in_array($wowza, $wowzas)) {
                $schedule = $recording->getSchedule();
                if ($schedule) {
                    $schedule->setStatus(Schedule::STATUS_PENDING);
                }
                $this->getRecorder()->stopRecording($recording->getRecord());
                $recording->getRecord()->setIsCompleted();
                $em->remove($recording);
                $em->flush();
            }
        }

        return new JsonResponse(['processed']);
    }

    /**
     * @Route(path="/wowza/notification/converted", name="wowza_notificate_converted")
     *
     * not used.
     * @param Request $request
     * @return JsonResponse
     */
    public function wowzaNotificationConverted(Request $request)
    {
        $result = @json_decode($request->getContent());
        if ($result != null) {
            $em = $this->getDoctrine()->getManager();

            $recordId = (int)$result->recordId;
            $record = $em->getRepository('FSCamBundle:Record')->find($recordId);

            if (((int)$result->result) == 1) {
                $record->setIsCompleted();
                $em->persist($record);
            } else {
                $em->remove($record);
            }
            $em->flush();
            try {
                unlink(
                    realpath(
                        $this->get('kernel')->getRootDir() .
                        '/../web/TMPUpload/' .
                        $record->getRecordUploadetSequence() .
                        ".mp4"
                    )
                );
            } catch (\Exception $e) {
                /*NOP*/
            }
            return new JsonResponse('true');
        }

        return new JsonResponse('false');

    }

    /**
     * @Route(path="/wowza/notification/image/image_ready", name="wowza_notificate_image_reay")
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function wowzaNotificationImageReady(Request $request)
    {
        $result = @json_decode($request->getContent());
        if ($result != null) {
            $em = $this->getDoctrine()->getManager();

            $recordId = (int)$result->recordId;
            $record = $em->getRepository('FSCamBundle:Record')->find($recordId);
            $downloadRes = $this->get('fs_cam.wowza_manager')
                ->getImageFromWowza(
                    $record->getId(),
                    $result->server,
                    (int)$result->port
                );
            $record->setRecordPreviewReady($downloadRes);
            $em->persist($record);
            $em->flush();

        }
        return new JsonResponse(['processed']);
    }

    /**
     * @Route(path="/wowza/mosaicResult", name="api_wowza_mosaic_result")
     * @param Request $request
     * @return JsonResponse
     */
    public function mosaicResultAction(Request $request)
    {
        $content = $request->getContent();
        $response = @json_decode($content);
        if ($response == null) {
            return new JsonResponse(['error']);
        }
        $em = $this->getDoctrine()->getManager();
        $mosaicRepository = $em->getRepository("FSCamBundle:MosaicQueue");

        /** @var MosaicQueue $mosaic */
        $mosaic = $mosaicRepository->getMosaicByRecordId($response->id);
        if ($response->result == 1) {
            $mosaic->setStatus(MosaicQueue::STATUS_DONE);
            $mosaic->setProgress(100);
        } else {
            if ($response->error == 1) {
                $mosaic->setStatus(MosaicQueue::STATUS_FAIL_FILES_COUNT);
            } else {
                $mosaic->setStatus(MosaicQueue::STATUS_FAIL);
            }
        }
        $em->persist($mosaic);
        $em->flush();
        return new JsonResponse(['processed']);
    }

    /**
     * @Route(path="/wowza/mosaicProgressUpdate", name="api_wowza_mosaic_update")
     * @param Request $request
     * @return JsonResponse
     */
    public function mosaicProgressUpdateAction(Request $request)
    {
        $content = $request->getContent();
        $response = @json_decode($content);
        if ($response == null) {
            return new JsonResponse(['error']);
        }
        $em = $this->getDoctrine()->getManager();
        $mosaicRepository = $em->getRepository("FSCamBundle:MosaicQueue");

        /** @var MosaicQueue $mosaic */
        $mosaic = $mosaicRepository->getMosaicByRecordId($response->id);
        if ($mosaic) {
            if ($mosaic->getStatus() == MosaicQueue::STATUS_CREATING) {
                $mosaic->setProgress((int)$response->progress);
                $em->persist($mosaic);
            }
        }

        $em->flush();
        return new JsonResponse(['processed']);
    }

    // /controller/card_event/
    /**
     * @Route(path="/controller/card_event", name="api_controller_card_event")
     * @param Request $request
     * @return JsonResponse
     */
    public function cardEventAction(Request $request)
    {
        $content = $request->getContent();
        $response = @json_decode($content);
        if ($response == null) {
            return new JsonResponse(['access' => 'denied']);
        }
        $response = $response->controller_event;
        /** @var EntityManager $em */
        $em = $this->getDoctrine()
            ->getManager();

        switch ($response->type) {
            case "card_detected":
                $reader = $em->getRepository('FSCamBundle:Camera')
                    ->findOneBy(['ip' => $response->controllerIp, 'controller' => $response->readerNumber]);
                $user = $em->getRepository('FSUserBundle:User')
                    ->findOneBy(['cardNumber' => $response->cardNumber]);
                if ($reader != null && $user != null) {
                    $rooms = $reader->getRooms();
                    foreach ($rooms as $room) {
                        if ($user->getSharedRooms()->contains($room)
                            ||
                            $user->getUserGroup()->getRooms()->contains($room)
                        ) {
                            if ($room->numCamerasInRoom() == 0) {
                                continue;
                            }
                            /** @var Recording $recording */
                            $recording = $em->getRepository('FS\CamBundle\Entity\Recording')->findOneByRoom($room);
                            if ($recording) {
                                $schedule = $recording->getSchedule();
                                if ($schedule) {
                                    $schedule->setStatus(Schedule::STATUS_PENDING);
                                }
                                $record = $recording->getRecord();
                                $this->getRecorder()->stopRecording($record);

                                $record->setIsCompleted();
                                $em->persist($record);
                                $em->flush($record);
                                $em->remove($recording);
                                $em->flush($recording);

                                $this->container->get('fs_user.util.logger')->logAction(
                                    Log::OB_TYPE_RECORDING,
                                    $record,
                                    $record->getCreatedBy(),
                                    Log::$actions[Log::OB_TYPE_RECORDING]['stop']);
                            } else {
                                $record = new Record();
                                $record->setCreatedBy($user);

                                $template = $user->getUserGroup()->getInfoTemplate();
                                $record->setTemplate($template);
                                $record->setTemplateData($template->getTemplateDefaultData());

                                $record->setName('Controller Record');
                                $record->setRoom($room);

                                $em->persist($record);
                                $em->flush($record);

                                $recording = new Recording();
                                $recording->setRecord($record);
                                $recording->setRoom($room);
                                $recording->setStartedAt(new \DateTime);

                                $em->persist($recording);

                                $room->setRecording($recording);

                                $startResult = $this->getRecorder()->startRecording($record, [], $response->controllerIp, $response->readerNumber);
                                if (!$startResult) {
                                    $room->setRecording(null);
                                    $em->remove($recording);
                                    $em->remove($record);
                                    $em->persist($room);
                                    $em->flush();
                                    continue;
                                }
                                $em->persist($room);

                                $em->flush($recording);
                                $em->flush($room);
                                $em->flush($record);

                                $this->container->get('fs_user.util.logger')->logAction(
                                    Log::OB_TYPE_RECORDING,
                                    $record,
                                    $record->getCreatedBy(),
                                    Log::$actions[Log::OB_TYPE_RECORDING]['start']);
                            }
                        }
                    }
                    return new JsonResponse(['access' => 'granted']);
                }
                break;
            default:
                break;
        }
        return new JsonResponse(['access' => 'denied']);
    }

    /**
     * @Route(path="/camera/camera_event", name="api_camera_camera_event")
     * @param Request $request
     * @return JsonResponse
     */
    public function cameraEventAction(Request $request)
    {
        $content = $request->getContent();
        $response = @json_decode($content);
        if ($response == null) {
            return new JsonResponse(['false']);
        }

        switch ($response->camera_event->type) {
            case "camera_io_event":
                $this->processIO($response->camera_event->cameraId);
                break;
            default:
                break;
        }
        return new JsonResponse(['processed']);
    }

    private function processIO($cameraId)
    {
        /** @var EntityManager $em */
        $em = $this->getDoctrine()
            ->getManager();

        $cameraIO = $em->getRepository('FSCamBundle:Camera')->find($cameraId);

        if ($cameraIO == null) {
            return false;
        }
        $rooms = $cameraIO->getRooms();
        foreach ($rooms as $room) {
            if (!$room->getIsCameraIO()) {
                return false;
            }

            if ($room->getCameraIO()->getId() != $cameraId) {
                return false;
            }

            if ($room->getUserIO() === null) {
                return false;
            }

            /** @var Recording $recording */
            $recording = $em->getRepository('FS\CamBundle\Entity\Recording')->findOneByRoom($room);
            if ($recording) {
                $record = $recording->getRecord();

                $schedule = $recording->getSchedule();
                if ($schedule) {
                    $schedule->setStatus(Schedule::STATUS_PENDING);
                }
                $this->getRecorder()->stopRecording($record);

                $record->setIsCompleted();
                $em->persist($record);
                $em->flush($record);
                $em->remove($recording);
                $em->flush($recording);

                $this->container->get('fs_user.util.logger')->logAction(
                    Log::OB_TYPE_RECORDING,
                    $record,
                    $record->getCreatedBy(),
                    Log::$actions[Log::OB_TYPE_RECORDING]['stop']);
            } else {
                $record = new Record();
                $ioUser = $room->getUserIO();
                /** @var UserGroup $ioGroup */
                $ioGroup = $ioUser->getUserGroup();

                $ioRecordName = $ioGroup->getIoRecordName();
                $record->setCreatedBy($ioUser);
                if ($ioGroup->getInfoTemplate() != null) {
                    $template = $ioGroup->getInfoTemplate();
                    $record->setTemplate($template);
                    $record->setTemplateData($template->getTemplateDefaultData());
                }
                $record->setName(
                    (
                    ($ioRecordName != "" || $ioRecordName != null) ? $ioRecordName : "IORecord"
                    )
                );
                $record->setContainer($ioGroup->getContainer());
                $record->setParentRetentionRule($ioGroup->getDefaultRetentionRule());
                $record->setRoom($room);

                $em->persist($record);
                $em->flush($record);

                $recording = new Recording();
                $recording->setRecord($record);
                $recording->setRoom($room);
                $recording->setStartedAt(new \DateTime);

                $em->persist($recording);

                $room->setRecording($recording);

                $startResult = $this->getRecorder()->startRecording($record, []);
                if (!$startResult) {
                    $room->setRecording(null);
                    $em->remove($recording);
                    $em->remove($record);
                    $em->persist($room);
                    $em->flush();
                    continue;
                }

                $em->persist($room);
                $em->flush($recording);
                $em->flush($room);
                $em->flush($record);

                $this->container->get('fs_user.util.logger')->logAction(
                    Log::OB_TYPE_RECORDING,
                    $record,
                    $record->getCreatedBy(),
                    Log::$actions[Log::OB_TYPE_RECORDING]['start']);
            }
        }
        return true;
    }

}
