<?php

namespace FS\CamBundle\Controller;

use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Utils\LicenseChecker;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class CameraAdminController
 * @Route("/admin/camera")
 * @Security(expression="user.hasRight('admin')  || is_granted('ROLE_ADMIN')")
 */
class CameraAdminController extends Controller
{
    /**
     * @Route(path="/", name="admin_camera")
     * @Template()
     */
    public function indexAction()
    {
        return [];
    }


    /**
     * Displays a form to create a new Camera entity.
     *
     * @Route("/new", name="admin_camera_new")
     * @Template()
     */
    public function newAction()
    {
        $camera = new Camera();
        $count = (int)$this->getDoctrine()->getManager()
            ->getRepository("FSCamBundle:Camera")
            ->countCameras();
        $limit = LicenseChecker::getCameraLimit();
        $disabled = ($count >= $limit);
        $form = $this->createForm('camera', $camera, ['disabled' => $disabled]);

        return array(
            'camera' => $camera,
            'form' => $form->createView(),
            'cameralimit' => $limit,
            'disabled' => $disabled
        );
    }

    /**
     *
     * @Route("/new/device", name="admin_device_new")
     * @Template()
     */
    public function newDeviceAction()
    {
        $camera = new Camera();
        $count = (int)$this->getDoctrine()->getManager()
            ->getRepository("FSCamBundle:Camera")
            ->countCameras();
        $limit = LicenseChecker::getCameraLimit();
        $disabled = ($count >= $limit);
        $form = $this->createForm('device', $camera, ['disabled' => $disabled]);

        return array(
            'camera' => $camera,
            'form' => $form->createView(),
            'cameralimit' => $limit,
            'disabled' => $disabled
        );
    }

    /**
     * Creates a new Camera entity.
     *
     * @Route("/create/device", name="admin_device_create")
     * @Method("POST")
     * @Template("FSCamBundle:CameraAdmin:newDevice.html.twig")
     * @param Request $request
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function createDeviceAction(Request $request)
    {
        $camera = new Camera();
        $count = (int)$this->getDoctrine()->getManager()
            ->getRepository("FSCamBundle:Camera")
            ->countCameras();
        $limit = LicenseChecker::getCameraLimit();
        $disabled = ($count >= $limit);
        $form = $this->createForm('device', $camera);
        $wowza = $this->get('fs_cam.wowza_manager')->getLocalWowza();
        $camera->setBrand(1)
            ->setModel(1)
            ->setWowza($wowza)
            ->setCameraType(Camera::TYPE_DEVICE)
            ->setDeviceUID(uniqid())
            ->setIp('0.0.0.0')
            ->setUsername('NOP')
            ->setPort(Camera::CAMERA_STANDARD_HTTP_PORT)
            ->setPassword('NOP');

        $form->submit($request);

        if ($request->isXmlHttpRequest()) {
            $form = $this->createForm('device', $camera);
            $form->setData($camera);

            return array(
                'camera' => $camera,
                'form' => $form->createView(),
                'cameralimit' => $limit,
                'disabled' => $disabled
            );
        }

        if (!$disabled && $form->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($camera);
            $em->flush();

            return $this->redirect($this->generateUrl('admin_camera'));
        }


        return array(
            'camera' => $camera,
            'form' => $form->createView(),
            'cameralimit' => $limit,
            'disabled' => $disabled
        );
    }

    /**
     * Creates a new Camera entity.
     *
     * @Route("/create", name="admin_camera_create")
     * @Method("POST")
     * @Template("FSCamBundle:CameraAdmin:new.html.twig")
     */
    public function createAction(Request $request)
    {
        $camera = new Camera();
        $count = (int)$this->getDoctrine()->getManager()
            ->getRepository("FSCamBundle:Camera")
            ->countCameras();
        $limit = LicenseChecker::getCameraLimit();
        $disabled = ($count >= $limit);
        $form = $this->createForm('camera', $camera);

        if (!$disabled && $form->submit($request)->isValid()) {
            $camera->setCameraType(Camera::TYPE_CAMERA);
            $em = $this->getDoctrine()->getManager();
            $em->persist($camera);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'New camera has been successfully added'
            );

            return $this->redirect($this->generateUrl('admin_camera_edit', ['id' => $camera->getId()]));
        }

        return array(
            'camera' => $camera,
            'form' => $form->createView(),
            'cameralimit' => $limit,
            'disabled' => $disabled
        );
    }

    /**
     * Displays a form to edit an existing Camera entity.
     *
     * @Route("/edit/device/{id}", name="admin_device_edit", defaults={"{id}" = null})
     * @Template()
     */
    public function editDeviceAction(Request $request, Camera $camera = null)
    {
        if ($camera == null) {
            return $this->render('FSCamBundle:CameraAdmin:deleted.html.twig');
        }

        $form = $this->createForm('device', $camera);

        return array(
            'camera' => $camera,
            'form' => $form->createView(),
        );
    }

    /**
     * Edits an existing Camera entity.
     *
     * @Route("/update/device/{id}", name="admin_device_update")
     * @Method("POST")
     * @Template("FSCamBundle:CameraAdmin:editDevice.html.twig")
     */
    public function updateDeviceAction(Camera $camera, Request $request)
    {
        $camera->setDeviceType($request->request->get('device')['deviceType']);
        $form = $this->createForm('device', $camera);

        $form->submit($request);
        if ($request->isXmlHttpRequest()) {
            $form = $this->createForm('device', $camera);
            $form->setData($camera);

            return array(
                'camera' => $camera,
                'form' => $form->createView(),
            );
        }

        if ($form->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $camera->setStatus(Camera::STATUS_OFFLINE);
            $em->flush();
        }

        return array(
            'camera' => $camera,
            'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to edit an existing Camera entity.
     *
     * @Route("/edit/{id}", name="admin_camera_edit", defaults={"{id}" = null})
     * @Template()
     * @param Camera $camera
     * @return array
     */
    public function editAction(Request $request, Camera $camera = null)
    {

        if ($camera == null) {
            return $this->render('FSCamBundle:CameraAdmin:deleted.html.twig');
        }
        $editForm = $this->createForm('camera', $camera);

        return array(
            'camera' => $camera,
            'form' => $editForm->createView(),
        );
    }

    /**
     * Edits an existing Camera entity.
     *
     * @Route("/update/{id}", name="admin_camera_update")
     * @Method("POST")
     * @Template("FSCamBundle:CameraAdmin:edit.html.twig")
     */
    public function updateAction(Camera $camera, Request $request)
    {
        $editForm = $this->createForm('camera', $camera);
        if ($editForm->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->flush();
        }

        return array(
            'camera' => $camera,
            'form' => $editForm->createView(),
        );
    }

    /**
     * Displays a form to delete a Camera entity.
     *
     * @Route("/{id}/remove", name="admin_camera_remove")
     * @Template()
     */
    public function removeAction(Camera $camera, Request $request)
    {
        $form = $this->createDeleteForm($camera->getId());

        return array('camera' => $camera, 'form' => $form->createView());
    }

    /**
     * Deletes a Camera entity.
     *
     * @Route("/{id}/delete", name="admin_camera_delete")
     * @Method("POST")
     */
    public function deleteAction(Camera $camera, Request $request)
    {
        $form = $this->createDeleteForm($camera->getId());
        if ($form->bind($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($camera);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'Camera has been successfully deleted'
            );
        }

        return $this->redirect($this->generateUrl('admin_camera'));
    }


    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }


} 