<?php

namespace FS\CamBundle\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use FS\CamBundle\Entity\Exception;
use FS\CamBundle\Form\Type\ExceptionType;

/**
 * Exception controller.
 *
 */
class ExceptionController extends Controller
{
    /**
     * Lists all Exception entities.
     *
     * @Route("/exception", name="exception")
     * @Template()
     */
    public function indexAction(Request $request)
    {
        $em = $this->getDoctrine()->getManager();

        $exceptions = $em
            ->getRepository('FSCamBundle:Exception')
            ->createQueryBuilder('e')
            ->orderBy('e.id', 'DESC')
            ->getQuery()
            ->getResult();

        $exception = new Exception();
        $form = $this->createForm(new ExceptionType(), $exception);

        return array(
            'exceptions' => $exceptions,
            'new_exception' => $exception,
            'form' => $form->createView(),
        );
    }

    private function renderIndexAction()
    {
        return $this->forward('FSCamBundle:Exception:index')->getContent();
    }

    /**
     * Creates a new Exception entity.
     *
     * @Route("/exception/create", name="exception_create")
     * @Method("POST")
     */
    public function createAction(Request $request)
    {
        $exception = new Exception();
        $form = $this->createForm(new ExceptionType(), $exception);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($exception);
            $em->flush();

            return new JsonResponse(['processed', $this->renderIndexAction()]);
        }

        return new JsonResponse([
            'error',
            $this->renderView(
                "FSCamBundle:Exception:new.html.twig",
                ['exception' => $exception, 'form' => $form->createView()]
            )
        ]);
    }

    /**
     * Displays a form to delete a Exception entity.
     *
     * @Route("/exception/{id}/remove", name="exception_remove")
     * @Template()
     */
    public function removeAction(Exception $exception)
    {
        $form = $this->createDeleteForm($exception->getId());

        return array('exception' => $exception, 'form' => $form->createView());
    }

    /**
     * Deletes a Exception entity.
     *
     * @Route("/exception/{id}/delete", name="exception_delete")
     * @Method("POST")
     */
    public function deleteAction(Exception $exception, Request $request)
    {
        $form = $this->createDeleteForm($exception->getId());
        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($exception);
            $em->flush();
        }

        return new JsonResponse(['processed', $this->renderIndexAction()]);
    }

    /**
     * Create Delete form
     *
     * @param  integer     $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }
}
