<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 28.04.2015
 * Time: 20:56
 */

namespace FS\CamBundle\Controller;


use Doctrine\Common\Collections\ArrayCollection;
use FS\CamBundle\Entity\Alert;
use FS\CamBundle\Entity\Filter;
use FS\CamBundle\Entity\FilterTemplate;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Wowza;
use FS\CamBundle\Form\Type\FilterType;
use FS\UserBundle\Entity\UserGroup;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Preset;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Entity\Recording;
use Symfony\Component\Form\FormBuilder;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * @Route("filter")
 *
 * //No permissions need to add
 */
class FilterController extends Controller
{
    /**
     * @Route(path="/create", name="filter_create")
     * @Template("@FSCam/Filter/newFilter.html.twig")
     * @param Request $request
     * @return JsonResponse
     */
    public function createFilterAction(Request $request)
    {
        $filter = new Filter();
        $filter->setOwner($this->getUser());
        $form = $this->createForm(new FilterType(), $filter);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $filterData = $filter->getFilterInfoData();
            /** @var FilterTemplate $item */
            foreach ($filterData as $item) {
                $item->setFilter($filter);
                $em->persist($item);
            }
            if($filter->getFilterDateType() == Filter::FILTER_DATE_TYPE_RANGE){
                if($filter->getRangeStart() == null){
                    $filter->setRangeStart(new \DateTime('now'));
                }
                if($filter->getRangeEnd() == null){
                    $filter->setRangeEnd(new \DateTime('now'));
                }
            }
            $em->persist($filter);
            $em->flush();
            return new JsonResponse(['processed']);
        }
        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    "@FSCam/Filter/newFilter.html.twig",
                    [
                        'form' => $form->createView(),
                    ]
                )
            ]
        );
    }

    /**
     * @Route(path="/new", name="filter_new")
     * @Template()
     * @param Request $request
     * @return array
     */
    public function newFilterAction(Request $request)
    {
        $filter = new Filter();
        $filter->setOwner($this->getUser());

        $form = $this->createForm(new FilterType(), $filter);
        return [
            'form' => $form->createView(),
        ];
    }

    /**
     * @Route(path="/edit/{filter}", name="filter_edit")
     * @Template()
     * @param Request $request
     * @param Filter $filter
     * @return array
     */
    public function editFilterAction(Request $request, Filter $filter)
    {
        $form = $this->createForm(new FilterType(), $filter);

        return [
            'form' => $form->createView(),
            'filter' => $filter
        ];
        // do you believe in magic?
    }

    /**
     * @Route(path="/update/{filter}", name="filter_update")
     * @param Request $request
     * @param Filter $filter
     * @return array
     */
    public function updateFilterAction(Request $request, Filter $filter)
    {
        $form = $this->createForm(new FilterType(), $filter);
        if ($form->submit($request)->isValid()) {
            if($filter->getFilterDateType() == Filter::FILTER_DATE_TYPE_RANGE){
                if($filter->getRangeStart() == null){
                    $filter->setRangeStart(new \DateTime('now'));
                }
                if($filter->getRangeEnd() == null){
                    $filter->setRangeEnd(new \DateTime('now'));
                }
            }
            $em = $this->getDoctrine()->getManager();
            $em->flush();

            return new JsonResponse(['processed']);
        }

        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    "FSCamBundle:Filter:editFilter.html.twig",
                    [
                        'filter' => $filter,
                        'form' => $form->createView()
                    ]
                )
            ]
        );
    }

    /**
     * @Route(path="/remove/{filter}", name="filter_remove")
     * @Template()
     * @param Request $request
     * @param Filter $filter
     * @return array
     */
    public function removeFilterAction(Request $request, Filter $filter)
    {
        $form = $this->createDeleteForm($filter->getId());
        return array('filter' => $filter, 'form' => $form->createView());
    }

    /**
     * Deletes a Schedule entity.
     *
     * @Route("/filter/delete/{filter}", name="filter_delete")
     * @Method("POST")
     * @param Filter $filter
     * @param Request $request
     * @return JsonResponse
     */
    public function deleteAction(Filter $filter, Request $request)
    {
        $form = $this->createDeleteForm($filter->getId());
        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($filter);
            $em->flush();
        }

        return new JsonResponse(['processed']);
    }

    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }
}