<?php

namespace FS\CamBundle\Controller;

use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use FS\CamBundle\Entity\HelpTopic;
use FS\CamBundle\Form\Type\HelpTopicType;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class HelpAdminController
 * @Route("/admin/help")
 * @Security(expression="user.hasRight('admin--help') || is_granted('ROLE_ADMIN')")
 */
class HelpAdminController extends Controller
{

    /**
     * @Route(path="/", name="admin_help")
     * @Template()
     */
    public function indexAction()
    {
        return [];
    }

    /**
     * @Template()
     */
    public function listAction()
    {
        $em = $this->getDoctrine()->getManager();

        $topics = $em->getRepository('FSCamBundle:HelpTopic')->findAll();

        return ['topics' => $topics];
    }

    /**
     * Displays a form to create a new HelpTopic entity.
     *
     * @Route("/new", name="admin_help_new")
     * @Template()
     */
    public function newAction()
    {
        $topic = new HelpTopic();
        $form = $this->createForm(new HelpTopicType(), $topic);

        return array(
            'topic' => $topic,
            'form' => $form->createView(),
        );
    }

    /**
     * Creates a new HelpTopic entity.
     *
     * @Route("/create", name="admin_help_create")
     * @Method("POST")
     * @Template("FSCamBundle:HelpAdmin:new.html.twig")
     */
    public function createAction(Request $request)
    {
        $topic = new HelpTopic();
        $form = $this->createForm(new HelpTopicType(), $topic);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($topic);
            $em->flush();

            return $this->redirect($this->generateUrl('admin_help'));
        }

        return array(
            'topic' => $topic,
            'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to edit an existing HelpTopic entity.
     *
     * @Route("/edit/{id}", name="admin_help_edit")
     * @Template()
     */
    public function editAction(HelpTopic $topic)
    {

        $form = $this->createForm(new HelpTopicType(), $topic);

        return array(
            'topic' => $topic,
            'form' => $form->createView(),
        );
    }

    /**
     * Edits an existing HelpTopic entity.
     *
     * @Route("/update/{id}", name="admin_help_update")
     * @Method("POST")
     * @Template("FSCamBundle:HelpAdmin:edit.html.twig")
     */
    public function updateAction(HelpTopic $topic, Request $request)
    {
        $editForm = $this->createForm(new HelpTopicType(), $topic);

        if ($editForm->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($topic);
            $em->flush();
        }

        return array(
            'topic' => $topic,
            'form' => $editForm->createView(),
        );
    }

    /**
     * Displays a form to delete a HelpTopic entity.
     *
     * @Route("/{id}/remove", name="admin_help_remove")
     * @Template()
     */
    public function removeAction(HelpTopic $topic, Request $request)
    {
        $form = $this->createDeleteForm($topic->getId());

        return array('topic' => $topic, 'form' => $form->createView());
    }

    /**
     * Deletes a HelpTopic entity.
     *
     * @Route("/{id}/delete", name="admin_help_delete")
     * @Method("POST")
     */
    public function deleteAction(HelpTopic $topic, Request $request)
    {
        $form = $this->createDeleteForm($topic->getId());
        if ($form->bind($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($topic);
            $em->flush();
        }

        return $this->redirect($this->generateUrl('admin_help'));
    }

    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }
}