<?php

namespace FS\CamBundle\Controller;

use FS\CamBundle\Form\Type\MarkerType;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\Form\FormBuilder;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use FS\CamBundle\Entity\Marker;
use FS\CamBundle\Entity\Record;

/**
 * Marker controller.
 *
 *
 */
///@Route("api")
class MarkerController extends Controller
{

    /**
     * @Route(path="/markers/{record}", name="api_markers")
     */
    public function indexAction(Record $record)
    {
        $em = $this->getDoctrine()->getManager();
        $markers = $em->getRepository('FSCamBundle:Marker')->findAllByRecord($record);
        $markers = $this->get('fs_user.container_manager')->filterArray($markers);
        $data = [];
        /** @var \FS\CamBundle\Entity\Marker $marker */
        foreach ($markers as $id => $marker) {
            //$data[] = $marker->toArray();
            $data[] = $marker->toArray();
        }


        return new JsonResponse($data);
    }

    /**
     * Displays a form to create a new Marker entity.
     *
     * @Route("/marker/{record}/new/{time}", name="marker_new", options={"expose"=true})
     */
    public function newAction(Request $request, Record $record, $time)
    {
        $isModal = $request->query->get('isModal', false);
        $marker = new Marker();
        $marker->setRecord($record);
        $marker->setTime($time);

        $form = $this->createForm(new MarkerType(), $marker);

        $template = "FSCamBundle:Marker:new.html.twig";

        if ($isModal == 1) {
            $template = "FSCamBundle:Marker:new_modal.html.twig";
        }
        if ($isModal == 2) {
            $template = "FSCamBundle:Marker:new_on_observe.html.twig";
        }

        return $this->render(
            $template,
            [
                'marker' => $marker,
                'form' => $form->createView(),
                'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
            ]
        );
    }

    /**
     * Creates a new Marker entity.
     *
     * @Route("/marker/{record}/create/{time}", name="marker_create",options={"expose"=true})
     * @Method("POST")
     */
    public function createAction(Request $request, Record $record, $time)
    {
        $isModal = $request->query->get('isModal', false);

        $marker = new Marker();
        $marker->setRecord($record);
        $marker->setTime($time);


        $form = $this->createForm(new MarkerType(), $marker);
        $type = 'error';
        $template = "FSCamBundle:Marker:new.html.twig";
        if ($isModal == 1) {
            $template = "FSCamBundle:Marker:new_modal.html.twig";
        }
        if ($isModal == 2) {
            $template = "FSCamBundle:Marker:new_on_observe.html.twig";
        }
        if ($form->submit($request)->isValid()) {
            $type = 'processed';
            $template = "FSCamBundle:Marker:edit.html.twig";
            $em = $this->getDoctrine()->getManager();
            $marker->setContainer($this->get('fs_user.container_manager')->getUserContainer());
            $em->persist($marker);
            $em->flush();
        }


        return $this->jsonResponse(
            $type,
            $template,
            [
                'marker' => $marker,
                'form' => $form->createView(),
                'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
            ],
            $marker
        );
    }

    /**
     * Displays a form to edit an existing Marker entity.
     *
     * @Route("/marker/{marker}/edit", name="marker_edit")
     * @Template()
     */
    public function editAction(Marker $marker)
    {

        $editForm = $this->createForm(new MarkerType(), $marker);

        return [
            'marker' => $marker,
            'form' => $editForm->createView(),
            'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza(),
        ];
    }

    /**
     * Edits an existing Marker entity.
     *
     * @Route("/marker/update/{marker}", name="marker_update")
     * @Method("POST")
     */
    public function updateAction(Marker $marker, Request $request)
    {
        $form = $this->createForm(new MarkerType(), $marker);
        $type = 'error';
        if ($form->submit($request)->isValid()) {
            $type = 'processed';
            $em = $this->getDoctrine()->getManager();
            $em->flush();
        }

        return $this->jsonResponse(
            $type,
            "FSCamBundle:Marker:edit.html.twig",
            [
                'marker' => $marker,
                'form' => $form->createView(),
                'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
            ],
            $marker
        );
    }

    // @Security(expression="user.hasAccessToRecord(record) && (user.hasRight('review--delete-recordings') || is_granted('ROLE_ADMIN'))")
    /**
     * Displays a form to delete a Record entity.
     *
     * @Route("/marker/{marker}/remove", name="marker_remove")
     * @Template()
     *
     */
    public function removeAction(Marker $marker)
    {
        $form = $this->createDeleteForm($marker->getId());

        return array('marker' => $marker, 'form' => $form->createView());
    }


    /**
     *
     * @Route("/marker/{marker}/delete", name="marker_delete")
     * @Security(expression="user.hasAccessToRecord(marker.getRecord()) && (user.hasRight('review--delete-markers') || is_granted('ROLE_ADMIN'))")
     * @param Request $request
     * @param Marker $marker
     * @return JsonResponse
     */
    public function deleteAction(Request $request, Marker $marker)
    {
        //$form = $this->createDeleteForm($marker->getId());
        $removedData = $marker->toArray();
        if ($marker != null) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($marker);
            $em->flush();
        }
        return new JsonResponse(['processed', ['marker' => $removedData]]);
    }

    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }

    private function jsonResponse($type, $template, $templateVars, Marker $marker)
    {

        $view = $this->renderView(
            $template,
            $templateVars
        );
        if ($type == 'error') {
            return new JsonResponse(
                [
                    'error',
                    $view
                ]
            );
        } else {
            return new JsonResponse(
                [
                    $type,
                    [
                        'view' => $view,
                        'marker' => $marker->toArray(),
                    ]
                ]
            );
        }
    }
}
