<?php

namespace FS\CamBundle\Controller;

use Doctrine\ORM\QueryBuilder;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Marker;
use FS\CamBundle\Entity\MosaicQueue;
use FS\CamBundle\Entity\Recording;
use FS\CamBundle\Entity\TemplateField;
use FS\CamBundle\Entity\Wowza;
use FS\CamBundle\Form\Type\RecordInfoType;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use FS\UserBundle\Entity\UserGroupRight;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\Form\Form;
use Symfony\Component\Form\FormBuilder;
use Symfony\Component\HttpFoundation\File\Exception\AccessDeniedException;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use FS\CamBundle\Entity\Record;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Class RecordController
 * @Route("/record")
 */
class RecordController extends Controller
{
    /**
     * Displays a form to create a new Record entity.
     *
     * @Route("/new/{roomId}", name="record_new", options={"expose"=true})
     * @Security(expression="user.hasRight('recording--start-recording')  || is_granted('ROLE_ADMIN')")
     * @param Request $request
     * @param $roomId
     * @return array
     */
    public function newAction(Request $request, $roomId)
    {
        $record = new Record();
        $isModal = $request->query->get('isModal', true);
        $room = $this->getDoctrine()->getManager()->find('FS\CamBundle\Entity\Room', $roomId);
        $record->setRoom($room);

        $form = $this->createForm('record', $record);
        //$room = $this->getDoctrine()->getManager()->find('FS\CamBundle\Entity\Room', $roomId);

        $template = "FSCamBundle:Record:new.html.twig";

        if ($isModal == true) {
            $template = "FSCamBundle:Record:new.html.twig";
        }
        if ($isModal == false) {
            $template = "FSCamBundle:Record:new_no_modal.html.twig";
        }

        return $this->render(
            $template, [
                'form' => $form->createView(),
                'room' => $room,
                'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
            ]
        );
    }

    /**
     * Creates a new Record entity.
     *
     * @Route("/create/{roomId}", name="record_create", options={"expose"=true})
     * @Method("POST")
     * @Security(expression="user.hasRight('recording--start-recording')  || is_granted('ROLE_ADMIN')")
     */
    public function createAction(Request $request, $roomId)
    {
        /** @var \FS\CamBundle\Entity\Room $room */
        $room = $this->getDoctrine()->getManager()->find('FS\CamBundle\Entity\Room', $roomId);
        if (!$room) {
            throw $this->createNotFoundException(
                'No room found for id ' . $roomId
            );
        }

        $isModal = $request->query->get('isModal', true);

        $record = new Record();
        $record->setRoom($room);
        $form = $this->createForm('record', $record);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();

            // todo check if recording exists send another response.
            /** @var \FS\CamBundle\Entity\Recording $recording */
            $recording = $em->getRepository('FS\CamBundle\Entity\Recording')->findOneByRoom($room);
            if ($recording) {
                return new JsonResponse(['processed', 'already started']);
            }

            $record->setContainer($this->get('fs_user.container_manager')->getUserContainer());
            $em->persist($record);
            $em->flush();

            $recording = new Recording();
            $recording->setRecord($record);
            $recording->setRoom($record->getRoom());
            $recording->setStartedAt(new \DateTime);
            $em->persist($recording);
            $em->flush();

            $startResult = $this->getRecorder()->startRecording($record);
            if (!$startResult) {
                $em->remove($recording);
                $em->remove($record);
                $em->flush();
                return new JsonResponse(['processed', 'error start']);
            }
            return new JsonResponse(['processed']);

        }

        $template = "FSCamBundle:Record:new.html.twig";

        if ($isModal == 1) {
            $template = "FSCamBundle:Record:new.html.twig";
        }
        if ($isModal == false) {
            $template = "FSCamBundle:Record:new_no_modal.html.twig";
        }

        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    $template,
                    [
                        'room' => $room,
                        'form' => $form->createView(),
                        'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
                    ]
                )
            ]
        );

    }

    /**
     * Stop live recording.
     *
     * @Route("/live/stop/{roomId}", name="record_stop", options={"expose"=true})
     * @Security(expression="user.hasRight('recording')  || is_granted('ROLE_ADMIN')")
     */
    public function stopRecord($roomId, Request $request)
    {
        $em = $this->getDoctrine()->getManager();
        /** @var \FS\CamBundle\Entity\Room $room */
        $room = $em->find('FS\CamBundle\Entity\Room', $roomId);
        if (!$room) {
            throw $this->createNotFoundException(
                'No room found for id ' . $roomId
            );
        }

        /** @var \FS\CamBundle\Entity\Recording $recording */
        $recording = $em->getRepository('FS\CamBundle\Entity\Recording')->findOneByRoom($room);

        if ($recording->getRecord()->getCreatedBy() != $this->getUser() && !$this->getUser()->hasRight(UserGroupRight::GRANT_RECORDING__STOP_ALL_AUTHORS)) {
            return new AccessDeniedException('Access denied.');
        }
        if ($recording) {
            $schedule = $recording->getSchedule();
            if ($schedule) {
                $schedule->setStatus(\FS\CamBundle\Entity\Schedule::STATUS_PENDING);
            }
            $this->getRecorder()->stopRecording($recording->getRecord());
            $recording->getRecord()->setIsCompleted();
            $em->remove($recording);
            $em->flush();
            if ($request->isXmlHttpRequest()) {
                return new JsonResponse(
                    [
                        'recordId' => $recording->getRecord()->getId()
                    ]
                );
            }
        }

        return new Response();
    }

    /**
     * @Route("/review/", name="record")
     * @Template()
     * @Security(expression="user.hasRight('review')  || is_granted('ROLE_ADMIN')")
     * @param Request $request
     * @return array
     */
    public function indexAction(Request $request)
    {
        $user = $this->getUser();

        if(null != $userGroup = $user->getUserGroup() ){
            /** @var UserGroup $userGroup */
            if($userGroup->getDefaultReviewView() == UserGroup::USER_GROUP_REVIEW_SHOW__LIST){
                return new RedirectResponse($this->generateUrl('record_list'));
            }
            if($userGroup->getDefaultReviewView() == UserGroup::USER_GROUP_REVIEW_SHOW__THUMBNAIL){
                return new RedirectResponse($this->generateUrl('record_thumbnail'));
            }
        }
        return new RedirectResponse($this->generateUrl('record_list'));
    }

    /**
     * @Route("/review/list", name="record_list")
     * @Template()
     * @Security(expression="user.hasRight('review')  || is_granted('ROLE_ADMIN')")
     * @param Request $request
     * @return array
     */
    public function listRecordsAction(Request $request)
    {
        $form = $this->createForm('record_filter');
        $sortArray = [
            'name' => 'up',
            'date' => 'up',
            'duration' => 'up',
            'markers' => 'up',
            'room' => 'up',
            'author' => 'up'
        ];
        $sortType = $request->query->get('type', 'up') == 'up' ? 'ASC' : 'DESC';
        $sortField = $request->query->get('sort', 'id');

        $hasFilter = $this->get('request')->query->has('record_filter');
        $q = $this->processRecordViewRequest($request, $form);

        switch ($sortField) {
            case 'name':
                $q->orderBy('record.name', $sortType);
                break;
            case 'date':
                $q->orderBy('record.startedAt', $sortType);
                break;
            case 'duration':
                $q->orderBy('record.duration', $sortType);
                break;
            case 'markers':
                $q->orderBy('markers.name', $sortType);
                break;
            case 'room':
                $q->orderBy('room.name', $sortType);
                break;
            case 'author':
                $q->orderBy('showingName', $sortType);
                break;
            case 'id':
            default:
                break;
        }

        $q = $q->getQuery()->execute();
        if ($sortType == 'ASC') {
            $sortArray[$sortField] = 'down';
        }
        $q = array_unique($q, SORT_REGULAR);

        $q = $this->get('fs_user.container_manager')->filterArray($q);

        $paginator = $this->get('knp_paginator')->paginate($q, $request->query->get('page', 1), 20);

        /** @var User $user */
        $user = $this->getUser();
        $showFields = [];

        if ($user->getUserGroup()) {
            $showFields = $user->getUserGroup()->getSearchResults();
        }

        return array(
            'form' => $form->createView(),
            'paginator' => $paginator,
            'hasFilter' => $hasFilter,
            'userFilters' => $this->getUser()->getFilters(),
            'sortArray' => $sortArray,
            'sortField' => $sortField,
            'showFields' => $showFields
        );
    }

    /**
     * @Route("/review/thumbnail", name="record_thumbnail")
     * @Template()
     * @Security(expression="user.hasRight('review')  || is_granted('ROLE_ADMIN')")
     * @param Request $request
     * @return array
     */
    public function thumbnailRecordsAction(Request $request)
    {
        $form = $this->createForm('record_filter');

        /** @var User $user */
        $user = $this->getUser();
        $showFields = [];

        if ($user->getUserGroup()) {
            $showFields = $user->getUserGroup()->getSearchResults();
        }
        
        $hasFilter = $this->get('request')->query->has('record_filter');

        $q = $this->processRecordViewRequest($request, $form);
        $q = $q->getQuery()->execute();

        $q = array_unique($q, SORT_REGULAR);

        $q = $this->get('fs_user.container_manager')->filterArray($q);

        $paginator = $this->get('knp_paginator')->paginate($q, $request->query->get('page', 1), 12);

        return array(
            'form' => $form->createView(),
            'hasFilter' => $hasFilter,
            'paginator' => $paginator,
            'userFilters' => $this->getUser()->getFilters(),
            'showFields' => $showFields
        );
    }

    /**
     * @param Request $request
     * @param Form $form
     * @return QueryBuilder
     */
    private function processRecordViewRequest(Request $request, Form $form)
    {
        $em = $this->getDoctrine()->getManager();

        /** @var QueryBuilder $q */
        $q = $em
            ->getRepository('FSCamBundle:Record')
            ->createQueryBuilderAll($this->getUser());

        $q->addSelect('(CASE WHEN (creator.displayName is NULL) 
                        THEN creator.username ELSE creator.displayName END) AS HIDDEN showingName');
        $form->submit($request);


        $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $q);

        return $q;
    }

    /**
     * Displays a Record entity.
     *
     * @Route("/{id}", name="record_show")
     * @Template()
     * @Security(expression="user.hasAccessToRecord(record)")
     * @param Record $record
     * @param Marker $marker Optional marker to start record from
     * @return array
     */
    public function showAction(Record $record, Request $request)
    {
        $markerID = $request->query->get('marker', null);
        $audioIds = [];
        $recordInfo = [];
        $audioInfo = [];
        $templateData = null;
        if ($record->getTemplate() !== null && count($record->getTemplateData()) !== 0) {
            $templateFields = $record->getTemplate()->getFields();
            $templateData = $record->getTemplateData();
            foreach ($templateFields as $templateField) {
                if ($templateField->getEnabled() == false)
                    continue;
                if ($templateField->getType() == TemplateField::TYPE_AUDIO) {
                    if (array_key_exists($templateField->getId(), $templateData)) {
                        $audioInfo['audio_hidden_field_' . $templateField->getId()] = $templateField->getTitle();
                        $audioIds[] = $templateField->getId();
                        $audioInfo[$templateField->getTitle()] = $templateData[$templateField->getId()];
                    }

                }
                if (!empty($templateData[$templateField->getId()]) && $templateData[$templateField->getId()] != null) {
                    if ($templateField->getType() != TemplateField::TYPE_AUDIO) {
                        $recordInfo[$templateField->getTitle()] = $templateData[$templateField->getId()];
                    }
                }
            }
        }

        if ($templateData != null) {
            foreach ($audioIds as $aId) {
                if (array_key_exists("audio_id_hidden_" . $aId, $templateData)) {
                    $audioInfo['audio_hidden_id_' . $aId] = $templateData['audio_id_hidden_' . $aId];
                }
            }
        }
        return array(
            'record' => $record,
            'recordInfo' => $recordInfo,
            'selectedMarker' => $markerID,
            'audioIds' => $audioIds,
            'audioInfo' => $audioInfo,
            'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
        );
    }

    /**
     * Displays a form to edit an existing Record entity.
     *
     * @Route("/{id}/edit-info", name="record_edit_info")
     * @Template()
     * @Security(expression="user.hasAccessToRecord(record)")
     */
    public function editInfoAction(Record $record, Request $request)
    {

        $form = $this->createForm(new RecordInfoType(), $record);

        return array(
            'record' => $record,
            'form' => $form->createView(),
            'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
        );
    }

    /**
     * Edits an existing Record entity.
     *
     * @Route("/{id}/update-info", name="record_update_info")
     * @Method("POST")
     * @Security(expression="user.hasAccessToRecord(record)")
     */
    public function updateInfoAction(Record $record, Request $request)
    {
        $form = $this->createForm(new RecordInfoType(), $record);
        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->flush();

            return new JsonResponse(['processed']);
        }

        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    "FSCamBundle:Record:editInfo.html.twig",
                    ['record' => $record, 'form' => $form->createView(), 'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()]
                )
            ]
        );
    }

    /**
     * Displays a form to edit an existing Record entity.
     *
     * @Route("/{id}/edit", name="record_edit")
     * @Template()
     * @Security(expression="user.hasAccessToRecord(record)")
     */
    public function editAction(Record $record, Request $request)
    {
        $isModal = $request->query->get('isModal', true);

        $form = $this->createForm('record', $record);

        return array(
            'isModal' => $isModal,
            'activeTab' => $request->query->get('tab', 'info'),
            'record' => $record,
            'form' => $form->createView(),
            'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
        );
    }

    /**
     * Edits an existing Record entity.
     *
     * @Route("/{id}/update", name="record_update")
     * @Method("POST")
     * @Security(expression="user.hasAccessToRecord(record)")
     */
    public function updateAction(Record $record, Request $request)
    {
        $isModal = $request->query->get('isModal', true);
        $form = $this->createForm('record', $record);
        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->flush();

            return new JsonResponse(['processed']);
        }

        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    "FSCamBundle:Record:edit.html.twig",
                    [
                        'record' => $record,
                        'form' => $form->createView(),
                        'isModal' => $isModal,
                        'activeTab' => $request->query->get('tab', 'info'),
                        'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
                    ]
                )
            ]
        );
    }

    /**
     * Displays a form to delete a Record entity.
     *
     * @Route("/{id}/remove", name="record_remove")
     * @Template()
     * @Security(expression="user.hasAccessToRecord(record) && (user.hasRight('review--delete-recordings') || is_granted('ROLE_ADMIN'))")
     */
    public function removeAction(Record $record)
    {
        $form = $this->createDeleteForm($record->getId());

        return array('record' => $record, 'form' => $form->createView());
    }

    /**
     * Deletes a Record entity.
     *
     * @Route("/{id}/delete", name="record_delete")
     * @Security(expression="user.hasAccessToRecord(record) && (user.hasRight('review--delete-recordings') || is_granted('ROLE_ADMIN'))")
     */
    public function deleteAction(Record $record, Request $request)
    {
        $form = $this->createDeleteForm($record->getId());
        if ($form->submit($request)->isValid() || $request->isXmlHttpRequest()) {
            $em = $this->getDoctrine()->getManager();

            $buzz = $this->get('buzz');
            if ($record->getRoom() != null) {
                $wowzas = $record->getRoom()->getWowzas();

            } else {
                $wowzas = $this->get('fs_cam.wowza_manager')->getAllWowzas();
            }
            if ($wowzas != null) {
                /** @var Wowza $wowza */
                foreach ($wowzas as $wowza) {
                    $buzz->post(
                        'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                        ['Content-Type' => 'application/json'],
                        json_encode([
                                'command' => 'deleteRecord',
                                'recordId' => (string)$record->getId(),
                                'type' => 'live',
                                'userId' => (string)$this->getUser()->getId(),
                            ]
                        )
                    );
                }
            }


            $em->remove($record);
            $em->flush();
        }

        return new JsonResponse(['processed']);
    }

    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }

    /**
     * @return \FS\CamBundle\Utils\Recorder
     */
    protected function getRecorder()
    {
        return $this->get('fs_cam.recorder');
    }

    /**
     * @Route("/{record}/pdf", name="record_pdf")
     *
     * @param Request $request
     * @param Record $record
     *
     * @return Response
     */
    public function getRecordPdfAction(Request $request, Record $record)
    {
        $recordInfo = [];
        if ($record->getTemplate() !== null && count($record->getTemplateData()) !== 0) {
            $templateFields = $record->getTemplate()->getFields();
            $templateData = $record->getTemplateData();
            foreach ($templateFields as $templateField) {
                if ($templateData[$templateField->getId()] != null) {
                    $recordInfo[$templateField->getTitle()] = $templateData[$templateField->getId()];
                }
            }
        }
        /** @var Marker $marker */
        $markers = $record->getMarkers();

        $markersInfo = [];
        foreach ($markers as $marker) {
            if ($marker->getTemplate() !== null) {
                $templateFields = $marker->getTemplate()->getFields();
                $templateData = $marker->getTemplateData();
                foreach ($templateFields as $templateField) {
                    if ($templateData[$templateField->getId()] != null) {
                        $markersInfo[$marker->getId()][] = [
                            $templateField->getTitle() => $templateData[$templateField->getId()]
                        ];
                    }
                }
            }
        }

        $html = $this->renderView('FSCamBundle:Record/Pdf:single.html.twig', [
            'record' => $record,
            'recordInfo' => $recordInfo,
            'markers' => $markers,
            'markersInfo' => $markersInfo,
        ]);

        $pdfGenerator = $this->get('knp_snappy.pdf');

        return new Response($pdfGenerator->getOutputFromHtml($html, ['encoding' => 'utf-8']),
            200,
            [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' =>
                    'attachment; filename="' .
                    $record->getName() .
                    "_" .
                    $record->getId() .
                    '.pdf"'
            ]
        );
    }

    /**
     * @Route("/search-result/export/csv", name="record_search_result_export_csv")
     */
    public function exportSearchResultToCSVAction(Request $request)
    {
        $form = $this->createForm('record_filter');


        $hasFilter = $this->get('request')->query->has('record_filter');
        if ($hasFilter) {
            $em = $this->getDoctrine()->getManager();

            $q = $em
                ->getRepository('FSCamBundle:Record')
                ->createQueryBuilderAll($this->getUser());

            $form->submit($request);


            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $q);

            $q = $q->getQuery()->execute();

        } else {
            $q = [];
        }

        $q = array_unique($q, SORT_REGULAR);
        $q = $this->get('fs_user.container_manager')->filterArray($q);
        $response = new StreamedResponse();
        $response->setCallback(function () use ($q) {

            $handle = fopen('php://output', 'w+');

            // Add a row with the names of the columns for the CSV file
            fputcsv($handle, array('record-id', 'record-name', 'room-name', 'date-time'), ',');
            /** @var Record $record */
            foreach ($q as $record) {
                fputcsv($handle, array(
                    $record->getId(),
                    $record->getName(),
                    $record->getRoomName(),
                    $record->getStartedAt()->format('g:i:s A m/d/Y')
                ), ',');
            }
            fclose($handle);
        });

        $response->setStatusCode(200);
        $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
        $response->headers->set('Content-Disposition', 'attachment; filename="Recordings_result.csv"');

        return $response;
    }

    /**
     * @Route("/search-result/export/pdf", name="record_search_result_export_pdf")
     */
    public function exportSearchResultToPDFAction(Request $request)
    {
        $form = $this->createForm('record_filter');


        $hasFilter = $this->get('request')->query->has('record_filter');
        if ($hasFilter) {
            $em = $this->getDoctrine()->getManager();

            $q = $em
                ->getRepository('FSCamBundle:Record')
                ->createQueryBuilderAll($this->getUser());

            $form->submit($request);


            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $q);

            $q = $q->getQuery()->execute();

        } else {
            $q = [];
        }

        $q = array_unique($q, SORT_REGULAR);
        $q = $this->get('fs_user.container_manager')->filterArray($q);
        $html = '';
        /** @var Record $record */
        foreach ($q as $record) {
            $recordInfo = [];
            if ($record->getTemplate() !== null && count($record->getTemplateData()) !== 0) {
                $templateFields = $record->getTemplate()->getFields();
                $templateData = $record->getTemplateData();
                foreach ($templateFields as $templateField) {
                    if (!empty($templateData[$templateField->getId()]) && $templateData[$templateField->getId()] != null) {
                        $recordInfo[$templateField->getTitle()] = $templateData[$templateField->getId()];
                    }
                }
            }
            /** @var Marker $marker */
            $markers = $record->getMarkers();

            $markersInfo = [];
            foreach ($markers as $marker) {
                if ($marker->getTemplate() !== null && count($marker->getTemplateData()) !== 0) {
                    $templateFields = $marker->getTemplate()->getFields();
                    $templateData = $marker->getTemplateData();
                    foreach ($templateFields as $templateField) {
                        if (!empty($templateData[$templateField->getId()])) {
                            $markersInfo[$marker->getId()][] = [
                                $templateField->getTitle() => $templateData[$templateField->getId()]
                            ];
                        }
                    }
                }
            }

            $html .= $this->renderView('FSCamBundle:Record/Pdf:single.html.twig', [
                'record' => $record,
                'recordInfo' => $recordInfo,
                'markers' => $markers,
                'markersInfo' => $markersInfo,
            ]);
            $html .= "<hr/>";
        }
        $pdfGenerator = $this->get('knp_snappy.pdf');

        return new Response($pdfGenerator->getOutputFromHtml($html, ['encoding' => 'utf-8']),
            200,
            [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' =>
                    'attachment; filename="' .
                    'Recordings' . ///// something
                    "_" .
                    'result' . /////side
                    '.pdf"'
            ]
        );
    }

    /**
     * @Route("/share/activate/{record}", name="record_share_activate")
     * @param Request $request
     * @param Record $record
     * @return JsonResponse
     */
    public function shareActivateRecordAction(Request $request, Record $record)
    {
        $em = $this->getDoctrine()->getManager();
        $now = new \DateTime();
        $sequence = sha1(uniqid() . $record->getName() . $now->getTimestamp());

        $record->setShared(true)
            ->setRecordShareSequence($sequence);

        $em->persist($record);
        $em->flush();

        $url = $this->generateUrl('record_share', ['sequence' => $sequence], true);

        return new JsonResponse(['processed', 'url' => $url]);
    }

    /**
     * @Route("/share/deactivate/{record}", name="record_share_deactivate")
     * @param Request $request
     * @param Record $record
     * @return JsonResponse
     */
    public function shareDeactivateRecordAction(Request $request, Record $record)
    {
        $em = $this->getDoctrine()->getManager();

        $record->setShared(false)
            ->setRecordShareSequence(null);

        $em->persist($record);
        $em->flush();

        return new JsonResponse(['processed']);
    }

    /**
     * Shared Record
     * @Route("/share/{sequence}", name="record_share")
     * @Template()
     * @param Request $request
     * @param $sequence
     * @return array|void
     */
    public function shareRecordAction(Request $request, $sequence)
    {
        $em = $this->getDoctrine()->getManager();


        $record = $em->getRepository('FSCamBundle:Record')->getRecordBySequence($sequence);

        if ($record == null) {
            throw new AccessDeniedHttpException('Access denied.');
        }
        return ['record' => $record];
    }

    /**
     * Upload Record
     * @Route("/upload/new", name="record_upload_new")
     * @Security(expression="(user.hasRight('upload') || is_granted('ROLE_ADMIN'))")
     * @Template()
     * @param Request $request
     * @return array
     */
    public function uploadRecordAction(Request $request)
    {
        $record = new Record();
        $record->setRoom(null);

        $form = $this->createForm('record', $record, ['csrf_protection' => false]);
        $form->add('file', 'file', [
                'mapped' => false,
                'label' => 'Video file',
                'constraints' => [
                    new Assert\File(['mimeTypes' => ['video/mp4']]),
                    new Assert\NotBlank(),
                ]]
        );

        return [
            'record' => $record,
            'form' => $form->createView(),
            'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
        ];

    }

    /**
     * Upload Record
     * @Route("/upload/create", name="record_upload_create")
     * @Security(expression="(user.hasRight('upload') || is_granted('ROLE_ADMIN'))")
     * @param Request $request
     * @return array
     */
    public function uploadCreateRecordAction(Request $request)
    {
        $record = new Record();
        $record->setRoom(null);
        $form = $this->createForm('record', $record, ['csrf_protection' => false]);
        $form->add('file', 'file', [
                'mapped' => false,
                'label' => 'Video file',
                'constraints' => [
                    new Assert\File(
                        [
                            'mimeTypes' =>
                                [
                                    'video/mp4',
                                    'application/octet-stream',
                                    'video/quicktime',
                                    'video/x-ms-asf'
                                ]
                        ]
                    ),
                    new Assert\NotBlank(),
                ]]
        );

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $record->setContainer($this->get('fs_user.container_manager')->getUserContainer());
            $record->setRecordType(Record::RECORD_TYPE_UPLOAD);
            $em->persist($record);
            $em->flush();

            $ext = @pathinfo(
                $form['file']
                    ->getData()
                    ->getClientOriginalName(),
                PATHINFO_EXTENSION
            );

            $sequence = false;
            switch ($ext) {
                case 'asf':
                    $sequence = $this->get('fs_cam.wowza_manager')
                        ->sendConvertRequestToWowza(
                            $record,
                            $form['file']->getData()
                        );

                    break;
                case 'mp4':
                    $sequence = $this->get('fs_cam.wowza_manager')
                        ->moveFileToLocalWowza(
                            $record,
                            $form['file']->getData()
                        );
                    $record->setIsCompleted();
                    break;
                default:
                    break;
            }
            if ($sequence != null) {
                $record->setRecordUploadetSequence($sequence);

                $em->persist($record);
                $em->flush();
                
                $this->get('fs_cam.wowza_manager')->makeThumbnailLocalWowza($record);
                
                return new JsonResponse(['processed']);
            }

        }

        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    "@FSCam/Record/uploadRecord.html.twig",
                    [
                        'form' => $form->createView(),
                        'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
                    ]
                )
            ]
        );
    }

    /**
     * Download a Record.
     * @Route("/{record}/download/{camera}", name="record_download")
     * @Security(expression="user.hasRight('review--download-recordings')  || is_granted('ROLE_ADMIN')")
     * @param Request $request
     * @param Record $record
     * @param $camera
     * @return Response
     */
    public function downloadAction(Request $request, Record $record, $camera)
    {
        $em = $this->getDoctrine()->getManager();
        $cameraEntity = $em->getRepository('FSCamBundle:Camera')->findOneBy(['id' => $camera]);
        if ($cameraEntity) {
            try {
                /** @var \SplFileInfo $file */
                $file = $this->get('fs_cam.record_files')->getCameraVideo($record, $cameraEntity);
            } catch (\Exception $e) {
                return $this->downloadError();
            }
            $filename = preg_replace(
                    '/[^A-Za-z0-9_. -]/',
                    '',
                    $record->getName() . '  ' . $cameraEntity->getName()
                ) . ' ' . $record->getStartedAt()->format('m-d-Y g-i-A') . '.mp4';
        } else {
            try {
                /** @var \SplFileInfo $file */
                $file = $this->get('fs_cam.record_files')->getFindCameraVideo($record, $camera);
                if ($file == null) {
                    throw new \Exception("bad file");
                }
                $filename = preg_replace(
                        '/[^A-Za-z0-9_. -]/',
                        '',
                        $record->getName() . '  ' . $record->getCamerasInRecord()[$camera]
                    ) . ' ' . $record->getStartedAt()->format('m-d-Y g-i-A') . '.mp4';
            } catch (\Exception $e) {
                return $this->downloadError();
            }
        }

        return $this->downloadFile($file, $filename);
    }

    /**
     * Download a Record.
     * @Route("/{record}/downloadUploaded", name="record_download_uploaded")
     * @Security(expression="user.hasRight('review--download-recordings')  || is_granted('ROLE_ADMIN')")
     * @param Request $request
     * @param Record $record
     * @return Response
     */
    public function downloadUploadedRecordAction(Request $request, Record $record)
    {
        if (
            $record->getRecordType() == Record::RECORD_TYPE_UPLOAD
            ||
            $record->getRecordType() == Record::RECORD_TYPE_DEVICE
        ) {
            /** @var Wowza $wowza */
            $wowza = $this->get('fs_cam.wowza_manager')->getLocalWowza();
            $port = $this->get('fs_cam.wowza_manager')->getLocalWowzaFileserverPort();
            if ($port == null) {
                return $this->downloadError();
            }
            $file = (object)[
                'url' => "http://" . $wowza->getAddress() . ":" . $port . "/" . $record->getId() . "/" . $record->getRecordUploadetSequence() . ".mp4",
                'contentLength' => 0,
            ];
            return $this->downloadFile($file, $record->getName() . ".mp4");
        } else {
            return $this->downloadError();
        }
    }

    private function downloadError()
    {
        return $this->render("FSCamBundle:Record:download_error.html.twig");
    }

    /**
     * @param $file
     * @param $filename
     */
    private function downloadFile($file, $filename)
    {
        // Close session
        session_write_close();
        $savename = str_replace(' ', '_', $filename);
        header('Content-Description: File Transfer');
        header('Content-Type: video/mp4');
        header('Content-Disposition: attachment; filename=' . $savename);
        header('Content-Transfer-Encoding: binary');
        header('Expires: 0');
        header('Cache-Control: must-revalidate');
        header('Pragma: public');

        $ch = curl_init($file->url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); //not necessary unless the file redirects (like the PHP example we're using here)
        $data = curl_exec($ch);
        curl_close($ch);
        $contentLength = 'unknown';
        if (preg_match('/Content-Length: (\d+)/', $data, $matches)) {
            $contentLength = (int)$matches[1];
        }
        header('Content-Length:' . $contentLength);

        $done = false;
        $start = 0;
        $end = $contentLength;
        while (!$done) {
            $info = $this->curlGetContent($file->url, $start, $end);
            $start += $info['size_download'];
            if ($start == $end) {
                $done = true;
            }
        }
        exit;
    }

    public function curlGetContent($url, $range_start = 0, $range_end = 0)
    {
        $writefn = function ($ch, $chunk) {
            echo $chunk;
            $len = strlen($chunk);
            return $len;
        };

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_BINARYTRANSFER, 1);
        curl_setopt($ch, CURLOPT_BUFFERSIZE, 8096);

        if ($range_start != 0 && $range_end != 0) {
            curl_setopt($ch, CURLOPT_HTTPHEADER, array("range:bytes=$range_start-$range_end"));
        }
        curl_setopt($ch, CURLOPT_WRITEFUNCTION, $writefn);
        $result = curl_exec($ch);

        $info = curl_getinfo($ch);
        curl_close($ch);
        return $info;
    }


    /**
     * @Route("/checkAccessToRecord/{record}", name="record_check_access", options={"expose"=true})
     * @param Request $request
     * @param Record $record
     * @return JsonResponse
     */
    public function checkAccessToRecord(Request $request, Record $record)
    {

        /** @var User $user */
        $user = $this->getUser();
        $accessToRecord = 0;

        if ($user != null && is_object($user)) {
            $accessToRecord = ($user->hasAccessToRecord($record) == true) ? 1 : 0;
        }

        return new JsonResponse([$accessToRecord]);
    }

    /**
     * @Route("/mosaic/status/{record}", name="record_mosaic", options={"expose"=true})
     * @param Request $request
     * @param Record $record
     * @return JsonResponse|Response
     */
    public function getMosaicAction(Request $request, Record $record)
    {
        $em = $this->getDoctrine()
            ->getManager();

        /** @var MosaicQueue $mosaic */
        $mosaic = $em->getRepository('FSCamBundle:MosaicQueue')->getMosaicByRecordId($record->getId());
        if ($mosaic == null) {
            $mosaic = new MosaicQueue();
            $mosaic->setStatus(MosaicQueue::STATUS_WAITING);
            $mosaic->setRecord($record);
            $em->persist($mosaic);
            $em->flush();

            return new JsonResponse(['created']);
        } else {
            if ($mosaic->getStatus() == MosaicQueue::STATUS_DONE) {
                return new JsonResponse(['done']);
            } else {
                if ($mosaic->getStatus() == MosaicQueue::STATUS_FAIL_FILES_COUNT) {
                    return new JsonResponse(['file fail']);
                }
                return new JsonResponse(['not done']);
            }
        }
    }

    /**
     * @Route(path="/{record}/mosaic/download/", name="record_mosaic_download")
     * @param Request $request
     * @param Record $record
     * @return Response
     */
    public function downloadMosaicAction(Request $request, Record $record)
    {
        $em = $this->getDoctrine()
            ->getManager();

        /** @var MosaicQueue $mosaic */
        $mosaic = $em->getRepository('FSCamBundle:MosaicQueue')->getMosaicByRecordId($record->getId());
        if ($mosaic == null) {
            return $this->downloadError();
        }
        if ($mosaic->getStatus() != MosaicQueue::STATUS_DONE) {
            return $this->downloadError();
        }

        $wowza = $this->get('fs_cam.wowza_manager')->getLocalWowza();
        $port = $this->get('fs_cam.wowza_manager')->getLocalWowzaFileserverPort();
        if ($port == null) {
            return $this->downloadError();
        }
        $file = (object)[
            'url' => "http://" . $wowza->getAddress() . ":" . $port . "/mosaic/" . $record->getId() . "/" . $record->getId() . ".mp4",
            'contentLength' => 0,
        ];

        $this->downloadFile($file, $record->getName() . ".mp4");
    }
    
}
