<?php

namespace FS\CamBundle\Controller;

use FS\UserBundle\Entity\User;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Form\Type\RoomType;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class RoomAdminController
 * @Route("/admin/room")
 * @Security(expression="user.hasRight('admin--rooms')  || is_granted('ROLE_ADMIN')")
 */
class RoomAdminController extends Controller
{

    /**
     * @Template()
     */
    public function listAction()
    {

        $em = $this->getDoctrine()->getManager();

        $rooms = $em->getRepository('FSCamBundle:Room')->findAll();
        $cameras = $em->getRepository('FSCamBundle:Camera')->getCamerasWithoutRoom();
        $rooms = $this->get('fs_user.container_manager')->filterArray($rooms);
        return [
            'rooms' => $rooms,
            'cameras' => $cameras,
        ];
    }

    /**
     * Displays a form to create a new Room entity.
     *
     * @Route("/new", name="admin_room_new")
     * @Template()
     */
    public function newAction()
    {
        $room = new Room();
        $form = $this->createForm('room', $room);

        return array(
            'room' => $room,
            'form' => $form->createView(),
        );
    }

    /**
     * Creates a new Room entity.
     *
     * @Route("/create", name="admin_room_create")
     * @Method("POST")
     * @Template("FSCamBundle:RoomAdmin:new.html.twig")
     */
    public function createAction(Request $request)
    {
        $room = new Room();
        $form = $this->createForm('room', $room);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($room);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'New room has been successfully added'
            );

            return $this->redirect($this->generateUrl('admin_room_edit', ['id' => $room->getId()]));
        }

        return array(
            'room' => $room,
            'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to edit an existing Room entity.
     *
     * @Route("/edit/{id}", name="admin_room_edit")
     * @Template()
     */
    public function editAction(Room $room)
    {

        $editForm = $this->createForm('room', $room);

        return array(
            'room' => $room,
            'form' => $editForm->createView(),
        );
    }

    /**
     * Edits an existing Room entity.
     *
     * @Route("/update/{id}", name="admin_room_update")
     * @Method("POST")
     * @Template("FSCamBundle:RoomAdmin:edit.html.twig")
     */
    public function updateAction(Room $room, Request $request)
    {
        $editForm = $this->createForm('room', $room);
        if ($editForm->submit($request)->isValid()) {

            $camera = $room->getCameraIO();
            if ($camera != null) {
                $camera->setIoConfigured(true);
                //$this->get('fs_cam.io_manager')->configureIO($camera);
            }
            $em = $this->getDoctrine()->getManager();
            $em->flush();
        }

        if ($request->isXmlHttpRequest()) {
            $form = $this->createForm('room', $room);

            return array(
                'room' => $room,
                'form' => $form->createView()
            );
        }

        return array(
            'room' => $room,
            'form' => $editForm->createView(),
        );
    }

    /**
     * Displays a form to delete a Room entity.
     *
     * @Route("/{id}/remove", name="admin_room_remove")
     * @Template()
     */
    public function removeAction(Room $room, Request $request)
    {
        $form = $this->createDeleteForm($room->getId());

        return array('room' => $room, 'form' => $form->createView());
    }

    /**
     * Deletes a Room entity.
     *
     * @Route("/{id}/delete", name="admin_room_delete")
     * @Method("POST")
     */
    public function deleteAction(Room $room, Request $request)
    {
        $form = $this->createDeleteForm($room->getId());
        if ($form->bind($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $users = $room->getUsers();
            /** @var User $user */
            foreach ($users as $user) {
                $user->removeSharedRooms($room);
            }
            $cameras = $room->getCameras();
            foreach($cameras as $camera)
            {
                $camera->removeRoom($room);
            }

            $em->remove($room);
            $em->flush($room);

            $this->get('session')->getFlashBag()->add(
                'note',
                'Room has been successfully deleted'
            );
        }

        return $this->redirect($this->generateUrl('admin_camera'));
    }


    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }
}