<?php

namespace FS\CamBundle\Controller;

use Doctrine\ORM\QueryBuilder;
use FS\CamBundle\Entity\Recur;
use FS\UserBundle\Entity\UserGroupRight;
use Symfony\Component\Form\Form;
use Symfony\Component\Form\FormBuilder;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use FS\CamBundle\Entity\Schedule;
use FS\CamBundle\Form\Type\ScheduleType;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Schedule controller.
 *
 */
class ScheduleController extends Controller
{
    /**
     * Lists all Schedule entities.
     * @Security(expression="user.hasRight('schedule')")
     *
     * @Route("/schedule", name="schedule")
     * @Template()
     * @param Request $request
     * @return array
     */
    public function indexAction(Request $request)
    {
        $form = $this->createForm('schedule_filter');
        $em = $this->getDoctrine()->getManager();

        $exceptions = $em
            ->getRepository('FSCamBundle:Exception')
            ->createQueryBuilder('e')
            ->orderBy('e.id', 'DESC')
            ->getQuery()
            ->getResult();

        $sortArray = [
            'name' => 'up',
            'room' => 'up',
            'start' => 'up',
            'end' => 'up',
            'user' => 'up',
            'status' => 'up',
            'recur' => 'up',
        ];

        $sortType = $request->query->get('type', 'up') == 'up' ? 'ASC' : 'DESC';
        $sortField = $request->query->get('sort', 'id');
        $qb = $this->processScheduleViewRequest($request, $form);

        switch ($sortField) {
            case 'name':
                $qb->orderBy('s.name', $sortType);
                break;
            case 'room':
                $qb->orderBy('room.name', $sortType);
                break;
            case 'status':
                $qb->orderBy('s.status', $sortType);
                break;
            case 'recur':
                $qb->orderBy('recur.frequency', $sortType);
                break;
            case 'user':
                $qb->orderBy('showingName', $sortType);
                break;
            case 'id':
            default:
                break;
        }

        if ($sortType == 'ASC') {
            $sortArray[$sortField] = 'down';
        }

        $q = $qb->getQuery()->execute();
        switch ($sortField) {
            case 'start':
                $q = Schedule::sortSchedulesByStartDate($q, $sortType);
                break;
            case 'end':
                $q = Schedule::sortSchedulesByEndDate($q, $sortType);
                break;
            default:
                break;
        }
        $q = array_unique($q, SORT_REGULAR);
        $q = $this->get('fs_user.container_manager')->filterArray($q);
        $conflictSchedules = [];
        //here i find conflict schedules
        /** @var Schedule $schedule */
        foreach ($q as $schedule) {
            /** @var Schedule $scheduleIns */
            foreach ($q as $scheduleIns) {
                if (
                    ($schedule->getId() == $scheduleIns->getId())
                    ||
                    ($schedule->getRoom() != $scheduleIns->getRoom())
                ) {
                    continue;
                } else {
                    if (
                        (
                            $schedule->getNextStartAt() <= $scheduleIns->getNextStartAt()
                            &&
                            $schedule->getNextStartAt() <= $scheduleIns->getNextStopAt()
                        )
                        &&
                        (
                            $schedule->getNextStopAt() >= $scheduleIns->getNextStartAt()
                            &&
                            $schedule->getNextStopAt() >= $scheduleIns->getNextStopAt()

                        )
                    ) {
                        $conflictSchedules[] = $schedule;
                        $conflictSchedules[] = $scheduleIns;
                    }
                }
            }
        }

        $blockedSchedules = [];
        $now = new \DateTime();
        $schedules = $em->createQuery(
            'SELECT s FROM FSCamBundle:Schedule s WHERE s.startAt <= :now AND s.status = :status
                AND s.room IN(
                SELECT room FROM FSCamBundle:Room room LEFT JOIN room.recording recording WHERE recording.room = room.id
                )'
        )
            ->setParameter('now', $now)
            ->setParameter('status', Schedule::STATUS_PENDING)
            ->getResult();
        /** @var Schedule $blockedSchedule */
        foreach ($schedules as $blockedSchedule) {
            if($blockedSchedule->getRealStartAt() != null){
                if($blockedSchedule->getRealStartAt()->format('d-m-Y') != $now->format('d-m-Y')){
                    continue;
                }
                $stop = clone $blockedSchedule->getRealStartAt();
                $stop->modify('+ ' . $blockedSchedule->getDuration() . ' seconds');
                if (
                    !($blockedSchedule->getRealStartAt()->format('d-m-Y H:i') < $now->format('d-m-Y H:i'))
                    &&
                    !($stop->format('d-m-Y H:i') < $now->format('d-m-Y H:i'))
                ) {
                    continue;
                }
            }else {
                if($blockedSchedule->getStartAt()->format('d-m-Y') != $now->format('d-m-Y')){
                    continue;
                }
                if (!$blockedSchedule->timeInRange($now)) {
                    continue;
                }
            }
            $blockedSchedules[] = $blockedSchedule;
        }

        $paginator = $this->get('knp_paginator')->paginate($q, $request->query->get('page', 1), 20);

        return array(
            'paginator' => $paginator,
            'form' => $form->createView(),
            'userFilters' => $this->getUser()->getFilters(),
            'sortArray' => $sortArray,
            'sortField' => $sortField,
            'exceptions' => $exceptions,
            'conflictSchedules' => $conflictSchedules,
            'blockedSchedules' =>$blockedSchedules
        );
    }

    /**
     * @param Request $request
     * @param Form $form
     * @return QueryBuilder
     */
    private function processScheduleViewRequest(Request $request, Form $form) {
        $em = $this->getDoctrine()->getManager();
        $q = $em->getRepository('FSCamBundle:Schedule')->getQBAllOrderedById();
        $q->addSelect('(CASE WHEN (user.displayName is NULL) 
                        THEN user.username ELSE user.displayName END) AS HIDDEN showingName');

        $form->submit($request);

        $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $q);

        return $q;
    }

    /**
     * Displays a form to create a new Schedule entity.
     * @Security(expression="user.hasRight('schedule--add-schedule')")
     *
     * @Route("/schedule/new", name="schedule_new")
     * @Template()
     */
    public function newAction()
    {
        $schedule = new Schedule();

        $form = $this->createForm('schedule', $schedule);

        return array(
            'schedule' => $schedule,
            'form' => $form->createView(),
            'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
        );
    }

    /**
     * Creates a new Schedule entity.
     * @Security(expression="user.hasRight('schedule--add-schedule')")
     *
     * @Route("schedule/create", name="schedule_create")
     * @Method("POST")
     */
    public function createAction(Request $request)
    {
        $schedule = new Schedule();
        $form = $this->createForm('schedule', $schedule);

        $form->submit($request);
        $user = $this->getUser();
        if ($user->hasRight(UserGroupRight::GRANT_SCHEDULE__CONTROL)) {
            foreach ($form['control']['cameras'] as $presetForm) {
                $preset = $presetForm->getData();
                if (null !== $preset) {
                    $schedule->addPreset($preset);
                }
            }
        }


        if ($form->isValid()) {
            $schedule->setContainer($this->get('fs_user.container_manager')->getUserContainer());
            $em = $this->getDoctrine()->getManager();

            $em->persist($schedule);
            $em->flush();

            return new JsonResponse(['processed']);
        }

        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    "FSCamBundle:Schedule:new.html.twig",
                    [
                        'schedule' => $schedule,
                        'form' => $form->createView(),
                        'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
                    ]
                )
            ]
        );
    }

    /**
     * Displays a form to edit an existing Schedule entity.
     * @Security(expression="user.hasRight('schedule--edit-schedules')")
     *
     * @Route("/schedule/{id}/edit", name="schedule_edit")
     * @Template()
     */
    public function editAction(Request $request, Schedule $schedule)
    {
        $form = $this->createForm('schedule', $schedule);

        if (isset($form['control'])) {
            foreach ($form['control']['cameras'] as $presetForm) {
                /** @var ArrayCollection $presets */
                $presets = $presetForm->getConfig()->getOptions()['choices'];
                if (!empty($presets)) {
                    foreach ($schedule->getPresets() as $preset) {
                        if ($presets->contains($preset)) {
                            $presetForm->setData($preset);
                        }
                    }
                }
            }
        }

        return array(
            'schedule' => $schedule,
            'form' => $form->createView(),
            'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
        );
    }

    /**
     * Edits an existing Schedule entity.
     * @Security(expression="user.hasRight('schedule--edit-schedules')")
     *
     * @Route("/schedule/{id}/update", name="schedule_update")
     * @Method("POST")
     */
    public function updateAction(Schedule $schedule, Request $request)
    {
        $form = $this->createForm('schedule', $schedule);

        if (isset($form['control'])) {
            foreach ($form['control']['cameras'] as $presetForm) {
                /** @var ArrayCollection $presets */
                $presets = $presetForm->getConfig()->getOptions()['choices'];
                if (!empty($presets)) {
                    foreach ($schedule->getPresets() as $preset) {
                        if ($presets->contains($preset)) {
                            $presetForm->setData($preset);
                        }
                    }
                }
            }
        }
        $startAt = null;
        if ($schedule->getStartAt()) {
            $startAt = clone $schedule->getStartAt();
        }

        $form->submit($request);
        if ($schedule->getStartAt() == null) {
            $schedule->setStartAt($startAt);
        }

        /*
         * If schedules is different clear all variables
         */
        if ($startAt != $schedule->getStartAt())
        {
            $schedule->setRealStartAt(null);
            $schedule->setNextStartAt(null);
            //$schedule->setDoNotStartUntil(null);
        }

        $schedule->removePresets();
        if (isset($form['control'])) {
            foreach ($form['control']['cameras'] as $presetForm) {
                $preset = $presetForm->getData();
                if (null !== $preset) {
                    $schedule->addPreset($preset);
                }
            }
        }

        if ($form->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->flush();

            return new JsonResponse(['processed']);
        }

        return new JsonResponse(
            [
                'error',
                $this->renderView(
                    "FSCamBundle:Schedule:edit.html.twig",
                    [
                        'schedule' => $schedule,
                        'form' => $form->createView(),
                        'wowza' => $this->get('fs_cam.wowza_manager')->getLocalWowza()
                    ]
                )
            ], 200
        );
    }

    /**
     * Displays a form to delete a Schedule entity.
     * @Security(expression="user.hasRight('schedule--delete-and-remove-schedules')")
     *
     * @Route("/schedule/{id}/remove", name="schedule_remove")
     * @Template()
     */
    public function removeAction(Schedule $schedule)
    {
        $form = $this->createDeleteForm($schedule->getId());

        return array('schedule' => $schedule, 'form' => $form->createView());
    }

    /**
     * Deletes a Schedule entity.
     * @Security(expression="user.hasRight('schedule--delete-and-remove-schedules')")
     *
     * @Route("/schedule/{id}/delete", name="schedule_delete")
     * @Method("POST")
     */
    public function deleteAction(Schedule $schedule, Request $request)
    {
        $form = $this->createDeleteForm($schedule->getId());
        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($schedule);
            $em->flush();
        }

        return new JsonResponse(['processed']);
    }

    /**
     * Deletes a Schedule entity.
     * @Security(expression="user.hasRight('schedule--delete-and-remove-schedules')")
     *
     * @Route("/schedule/{id}/remove-next", name="schedule_remove_next")
     * @Method("GET")
     */
    public function removeNextAction(Schedule $schedule, Request $request)
    {
        $em = $this->getDoctrine()->getManager();

        $schedule->skipNextStart();
        //$schedule->getNextStartAt();
        $recur = $schedule->getRecur();
        $schedule->setRealStartAt($schedule->getNextStartAt());

        $now = new \DateTime();

        $em->merge($schedule);

        $recur = $schedule->getRecur();
        if ($recur && $recur->getRange() == Recur::RANGE_END_BY_OCCURENCES) {
            $recur->setOccurrences($recur->getOccurrences() - 1);
            if ($recur->getOccurrences() == 0) {
                $em->remove($schedule);
            }
        }

        if ($recur && $recur->getRange() == Recur::RANGE_END_BY_DATE && $now > $recur->getEndAt()) {
            $em->remove($schedule);
        }

        $em->flush();

        return new RedirectResponse($this->generateUrl('schedule'));
    }


    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }

    /**
     * Deletes a Schedule entity.
     *
     * @Route("/schedule/{id}/stop", name="schedule_stop")
     */
    public function stopAction(Request $request, Schedule $schedule)
    {
        $em = $this->getDoctrine()->getManager();

        $recording = $schedule->getRecording();
        $recording->getRecord()->setIsCompleted();
        $schedule->setStatus(Schedule::STATUS_PENDING);
        // тут если она рекурентная то ставим новую дату
        $recur = $schedule->getRecur();
        if ($recur) {
            //$schedule->skipNextStart();
            $schedule->setRealStartAt($schedule->getNextStartAt());
            $em->persist($schedule);
        } else {
            // если одиночная то удаляем
            $em->remove($schedule);
        }

        if ($recording) {
            $recording = $schedule->getRecording();
            /** @var \FS\CamBundle\Utils\Recorder $recorder */
            $recorder = $this->get('fs_cam.recorder');
            $recorder->stopRecording($recording->getRecord());
            $em->remove($recording);
        }
        $em->flush();

        if ($request->isXmlHttpRequest()) {
            return new JsonResponse(['processed']);
        };

        return new RedirectResponse($this->generateUrl('schedule'));
    }

    /**
     * @Route("/search-result/export/pdf", name="schedule_search_result_export_pdf")
     */
    public function exportSearchResultToPDFAction(Request $request)
    {
        $form = $this->createForm('schedule_filter');

        $em = $this->getDoctrine()->getManager();
        $qb = $em
            ->getRepository('FSCamBundle:Schedule')->getQBAllOrderedById();

        $hasFilter = $this->get('request')->query->has('schedule_filter');
        if ($hasFilter) {
            $form->submit($request);

            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $qb);

            $q = $qb->getQuery()->execute();

        } else {
            $q = $qb->getQuery()->execute();
        }
        $q = array_unique($q, SORT_REGULAR);
        $q = $this->get('fs_user.container_manager')->filterArray($q);
        $html = '';
        /** @var Schedule $schedule */
        foreach ($q as $schedule) {
            $scheduleInfo = [];
            if ($schedule->getTemplate() !== null && count($schedule->getTemplateData()) !== 0) {
                $templateFields = $schedule->getTemplate()->getFields();
                $templateData = $schedule->getTemplateData();
                foreach ($templateFields as $templateField) {
                    if ($templateData[$templateField->getId()] != null) {
                        $scheduleInfo[$templateField->getTitle()] = $templateData[$templateField->getId()];
                    }
                }
            }
            $html .= $this->renderView('FSCamBundle:Schedule/Pdf:single.html.twig', [
                'schedule' => $schedule,
                'scheduleInfo' => $scheduleInfo,
            ]);

        }
        $pdfGenerator = $this->get('knp_snappy.pdf');

        return new Response($pdfGenerator->getOutputFromHtml($html, ['encoding' => 'utf-8']),
            200,
            [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' =>
                    'attachment; filename="' .
                    'Schedule' . ///// something
                    "_" .
                    'result' . /////side
                    '.pdf"'
            ]
        );
    }

    /**
     * @Route("/search-result/export/csv", name="schedule_search_result_export_csv")
     */
    public function exportSearchResultToCSVAction(Request $request)
    {
        $form = $this->createForm('schedule_filter');

        $em = $this->getDoctrine()->getManager();
        $qb = $em
            ->getRepository('FSCamBundle:Schedule')->getQBAllOrderedById();

        $hasFilter = $this->get('request')->query->has('schedule_filter');
        if ($hasFilter) {
            $form->submit($request);

            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $qb);

            $q = $qb->getQuery()->execute();

        } else {
            $q = $qb->getQuery()->execute();
        }
        $q = array_unique($q, SORT_REGULAR);
        $q = $this->get('fs_user.container_manager')->filterArray($q);
        $response = new StreamedResponse();
        $response->setCallback(function () use ($q) {

            $handle = fopen('php://output', 'w+');

            // Add a row with the names of the columns for the CSV file
            fputcsv($handle, array('schedule-id', 'schedule-name', 'room-name', 'author', 'next-start-at', 'next-stop-at'), ',');
            /** @var Schedule $schedule */
            foreach ($q as $schedule) {
                fputcsv($handle, array(
                    $schedule->getId(),
                    $schedule->getName(),
                    $schedule->getRoom()->getName(),
                    $schedule->getCreatedBy()->getUsername(),
                    $schedule->getNextStartAt()->format('M j, g:i A'),
                    $schedule->getNextStopAt()->format('M j, g:i A'),
                ), ',');
            }
            fclose($handle);
        });

        $response->setStatusCode(200);
        $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
        $response->headers->set('Content-Disposition', 'attachment; filename="Schedule_result.csv"');

        return $response;
    }

}
