<?php

namespace FS\CamBundle\Controller;

use Doctrine\Common\Collections\ArrayCollection;
use FS\CamBundle\Form\Type\TemplateFieldType;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use FS\CamBundle\Entity\Template as CamTemplate;
use FS\CamBundle\Form\Type\TemplateType;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class TemplateAdminController
 * @Route("/admin/template")
 * @Security(expression="user.hasRight('admin--templates') || is_granted('ROLE_ADMIN')")
 */
class TemplateAdminController extends Controller
{

    /**
     * @Route(path="/", name="admin_template")
     * @Template()
     */
    public function indexAction()
    {
        return [];
    }

    /**
     * @Template()
     */
    public function listAction()
    {

        $em = $this->getDoctrine()->getManager();

        $templates = $em->getRepository('FSCamBundle:Template')->findAll();
        $templates = $this->get('fs_user.container_manager')->filterArray($templates);

        return ['templates' => $templates];
    }

    /**
     * Displays a form to create a new CamTemplate entity.
     *
     * @Route("/new", name="admin_template_new")
     * @Template()
     */
    public function newAction()
    {
        $template = new CamTemplate();
        $form = $this->createForm('fs_template', $template);

//        $form = $this->createForm(new TemplateFieldType());

        return array(
          'template' => $template,
          'form' => $form->createView(),
        );
    }

    /**
     * Creates a new CamTemplate entity.
     *
     * @Route("/create", name="admin_template_create")
     * @Method("POST")
     * @Template("FSCamBundle:TemplateAdmin:new.html.twig")
     */
    public function createAction(Request $request)
    {
        $template = new CamTemplate();
        $form = $this->createForm('fs_template', $template);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($template);
            $em->flush();

            return $this->redirect($this->generateUrl('admin_template'));
        }

        return array(
          'template' => $template,
          'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to edit an existing CamTemplate entity.
     *
     * @Route("/view/{id}", name="admin_template_view")
     * @Template()
     */
    public function viewAction(CamTemplate $template)
    {

        $form = $this->createForm('fs_template', $template, ['editable' => true]);

        return array(
          'template' => $template,
          'form' => $form->createView(),
        );
    }

//    /**
//     * Displays a form to edit an existing CamTemplate entity.
//     *
//     * @Route("/edit/{id}", name="admin_template_edit")
//     * @Template()
//     */
//    public function editAction(CamTemplate $template)
//    {
//
//        $editForm = $this->createForm(new TemplateType(), $template, ['editable' => false]);
//
//        return array(
//          'template' => $template,
//          'form' => $editForm->createView(),
//        );
//    }

    /**
     * Edits an existing CamTemplate entity.
     *
     * @Route("/update/{id}", name="admin_template_update")
     * @Method("POST")
     * @Template("FSCamBundle:TemplateAdmin:edit.html.twig")
     */
    public function updateAction(CamTemplate $template, Request $request)
    {
        $editForm = $this->createForm('fs_template', $template);

        $originalFields = new ArrayCollection();

        // Create an ArrayCollection of the current Tag objects in the database
        foreach ($template->getFields() as $field) {
            $originalFields->add($field);
        }

        $editForm->handleRequest($request);
        if ($editForm->isValid()) {
            $em = $this->getDoctrine()->getManager();

            foreach ($originalFields as $field) {
                if (false === $template->getFields()->contains($field)) {
                    $template->getFields()->removeElement($field);

                    $em->persist($field);
                    $em->remove($field);
                }
            }

            $em->flush();
        }

        return array(
          'template' => $template,
          'form' => $editForm->createView(),
        );
    }

    /**
     * Displays a form to delete a CamTemplate entity.
     *
     * @Route("/{id}/remove", name="admin_template_remove")
     * @Template()
     */
    public function removeAction(CamTemplate $template, Request $request)
    {
        $form = $this->createDeleteForm($template->getId());

        return array('template' => $template, 'form' => $form->createView());
    }

    /**
     * Deletes a CamTemplate entity.
     *
     * @Route("/{id}/delete", name="admin_template_delete")
     * @Method("POST")
     */
    public function deleteAction(CamTemplate $template, Request $request)
    {
        $form = $this->createDeleteForm($template->getId());
        if ($form->bind($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($template);
            $em->flush();
        }

        return $this->redirect($this->generateUrl('admin_template'));
    }

    /**
     * Hides a CamTemplate entity.
     *
     * @Route("/{id}/hide", name="admin_template_hide")
     */
    public function hideAction(CamTemplate $template, Request $request)
    {
        $template->setHidden(true);
        $em = $this->getDoctrine()->getManager();
        $em->persist($template);
        $em->flush();

        return $this->redirect($this->generateUrl('admin_template'));
    }

    /**
     * Unhides a CamTemplate entity.
     *
     * @Route("/{id}/unhide", name="admin_template_unhide")
     */
    public function unhideAction(CamTemplate $template, Request $request)
    {
        $template->setHidden(false);
        $em = $this->getDoctrine()->getManager();
        $em->persist($template);
        $em->flush();

        return $this->redirect($this->generateUrl('admin_template'));
    }

    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
          ->add('id', 'hidden')
          ->getForm();
    }
}