<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 27.03.2015
 * Time: 18:15
 */

namespace FS\CamBundle\Controller;

use FS\CamBundle\Entity\Wowza;
use FS\CamBundle\Form\Type\WowzaType;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\Form\FormBuilder;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class WowzaAdminController
 * @Route("/admin/wowza")
 * @Security(expression="is_granted('ROLE_ADMIN') || user.hasRight('admin--wowza')")
 */
class WowzaAdminController extends Controller
{

    /**
     * @Route(path="/", name="admin_wowza")
     * @Template()
     */
    public function indexAction()
    {
        return [];
    }

    /**
     * @Template()
     * @return array
     */
    public function listAction()
    {

        $em = $this->getDoctrine()->getManager();

        $wowzas = $em->getRepository('FSCamBundle:Wowza')->findAll();

        return ['wowzas' => $wowzas];
    }

    /**
     * Displays a form to create a new Room entity.
     *
     * @Route("/new", name="admin_wowza_new")
     * @Template()
     */
    public function newAction()
    {
        $wowza = new Wowza();
        $form = $this->createForm(new WowzaType(), $wowza);

        return array(
            'wowza' => $wowza,
            'form' => $form->createView(),
        );
    }

    /**
     * Creates a new Room entity.
     *
     * @Route("/create", name="admin_wowza_create")
     * @Method("POST")
     * @Template("FSCamBundle:WowzaAdmin:new.html.twig")
     * @param Request $request
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function createAction(Request $request)
    {
        $wowza = new Wowza();
        $form = $this->createForm(new WowzaType(), $wowza);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($wowza);
            $em->flush();

            return $this->redirect($this->generateUrl('admin_wowza'));
        }

        return array(
            'wowza' => $wowza,
            'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to edit an existing Room entity.
     *
     * @Route("/edit/{id}", name="admin_wowza_edit")
     * @Template()
     * @param Wowza $wowza
     * @return array
     */
    public function editAction(Wowza $wowza)
    {

        $editForm = $this->createForm(new WowzaType(), $wowza);

        return array(
            'wowza' => $wowza,
            'form' => $editForm->createView(),
        );
    }

    /**
     * Edits an existing Room entity.
     *
     * @Route("/update/{id}", name="admin_wowza_update")
     * @Method("POST")
     * @Template("FSCamBundle:WowzaAdmin:edit.html.twig")
     */
    public function updateAction(Wowza $wowza, Request $request)
    {
        $editForm = $this->createForm(new WowzaType(), $wowza);
        if ($editForm->submit($request)->isValid()) {

            $em = $this->getDoctrine()->getManager();
            $em->flush();
        }

        return array(
            'wowza' => $wowza,
            'form' => $editForm->createView(),
        );
    }

    /**
     * Displays a form to delete a Room entity.
     *
     * @Route("/{id}/remove", name="admin_wowza_remove")
     * @Template()
     */
    public function removeAction(Wowza $wowza, Request $request)
    {
        $form = $this->createDeleteForm($wowza->getId());

        return array('wowza' => $wowza, 'form' => $form->createView());
    }

    /**
     * Deletes a Room entity.
     *
     * @Route("/{id}/delete", name="admin_wowza_delete")
     * @Method("POST")
     */
    public function deleteAction(Wowza $wowza, Request $request)
    {
        $form = $this->createDeleteForm($wowza->getId());
        if ($form->bind($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();

            $em->remove($wowza);
            $em->flush();
        }

        return $this->redirect($this->generateUrl('admin_wowza'));
    }


    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }
}