<?php

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Model\ContainerizedInterface;

/**
 * Alert
 *
 * @ORM\Table(name="alerts")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\AlertRepository")
 */
class Alert implements ContainerizedInterface
{
    use FSContainerTrait;

    const ALERT_TYPE_WOWZA = 'wowza';
    const ALERT_TYPE_CAMERA = 'camera';

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime")
     */
    private $createdAt;

    /**
     * @var string
     *
     * @ORM\Column(type="text")
     */
    private $message;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean")
     */
    private $viewed = false;

    /**
     * @var Camera
     *
     * @ORM\ManyToOne(targetEntity="Camera")
     * @ORM\JoinColumn(name="camera_id", referencedColumnName="id", onDelete="CASCADE", nullable=true)
     */
    private $camera;

    /**
     * @var Camera
     *
     * @ORM\ManyToOne(targetEntity="Wowza")
     * @ORM\JoinColumn(name="wowza_id", referencedColumnName="id", onDelete="CASCADE", nullable=true)
     */
    private $wowza;

    /**
     * @var string
     *
     * @ORM\Column(type="text")
     *
     */
    private $alertType;

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set createdAt
     *
     * @param  \DateTime $createdAt
     * @return Alert
     */
    public function setCreatedAt($createdAt)
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    /**
     * Get createdAt
     *
     * @return \DateTime
     */
    public function getCreatedAt()
    {
        return $this->createdAt;
    }

    /**
     * Set message
     *
     * @param  string $message
     * @return Alert
     */
    public function setMessage($message)
    {
        $this->message = $message;

        return $this;
    }

    /**
     * Get message
     *
     * @return string
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * Set camera
     *
     * @param  \FS\CamBundle\Entity\Camera $camera
     * @return Alert
     */
    public function setCamera(\FS\CamBundle\Entity\Camera $camera = null)
    {
        $this->camera = $camera;

        return $this;
    }

    /**
     * Get camera
     *
     * @return \FS\CamBundle\Entity\Camera
     */
    public function getCamera()
    {
        return $this->camera;
    }

    /**
     * Set camera
     *
     * @param  \FS\CamBundle\Entity\Wowza $wowza
     * @return Alert
     */
    public function setWowza(Wowza $wowza = null)
    {
        $this->wowza = $wowza;

        return $this;
    }

    /**
     * Get camera
     *
     * @return \FS\CamBundle\Entity\Wowza
     */
    public function getWowza()
    {
        return $this->wowza;
    }

    /**
     * Set viewed
     *
     * @param boolean $viewed
     * @return Alert
     */
    public function setViewed($viewed)
    {
        $this->viewed = $viewed;

        return $this;
    }

    /**
     * Get viewed
     *
     * @return boolean
     */
    public function getViewed()
    {
        return $this->viewed;
    }

    /**
     * @param $alertType
     * @return $this
     */
    public function setAlertType($alertType)
    {
        $this->alertType = $alertType;

        return $this;
    }

    /**
     * @return string
     */
    public function getAlertType()
    {
        return $this->alertType;
    }

    public function getName()
    {
        if($this->alertType == self::ALERT_TYPE_CAMERA && $this->camera != null){
            return $this->camera->getName();
        }
        if($this->alertType == self::ALERT_TYPE_WOWZA && $this->wowza != null){
            return $this->wowza->getName();
        }
        if($this->alertType == "" || $this->alertType == null){
            if($this->camera != null){
                return $this->camera->getName();
            }
            elseif($this->wowza != null){
                return $this->wowza->getName();
            }
            return "Undefined";
        }
        return "Undefined";
    }
}
