<?php

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use FS\BasicBundle\Model\Loggable;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Camera
 *
 * @ORM\Table(name="cameras")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\CameraRepository")
 */
class Camera implements Loggable
{

    const STATUS_ONLINE = 'online';
    const STATUS_OFFLINE = 'offline';

    const TYPE_CAMERA = 'camera';
    const TYPE_DEVICE = 'device';

    const DEVICE_TYPE_IPHONE = 'iphone';
    const DEVICE_TYPE_IPAD = 'ipad';
    const DEVICE_TYPE_CARD_READER = 'controller';


    const CAMERA_STANDARD_HTTP_PORT = 80;
    const CAMERA_STANDARD_RTSP_PORT = 554;


    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=128)
     * @Assert\NotBlank()
     */
    private $name;

    /**
     * @var int
     *
     * @ORM\Column(name="model", type="integer")
     * @Assert\NotBlank()
     */
    private $model;

    /**
     * @var int
     *
     * @ORM\Column(name="brand", type="integer")
     * @Assert\NotBlank()
     */
    private $brand;

    /**
     * @var string
     *
     * @ORM\Column(name="ip", type="string")
     * @Assert\NotBlank()
     */
    private $ip;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $cameraType;


    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $deviceType;

    /**
     * @var int
     *
     * @ORM\Column(type="integer", nullable=true)
     */
    private $controller;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $deviceUID;

    /**
     * @var string
     *
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $linkedDevice = 0;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $ioConfigured;

    /** @var  boolean
     *
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $ioSettings;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $cameraIoH;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $cameraIoE;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $cameraIoA;
    /**
     * @var int
     *
     * @ORM\Column(name="port", type="smallint", options={"default":Camera::CAMERA_STANDARD_HTTP_PORT})
     * @Assert\NotBlank()
     * @Assert\Type(type="integer", message="The value {{ value }} is not a valid {{ type }}.")
     */
    private $port;

    /**
     * @var int
     *
     * @ORM\Column(name="rtspPort", type="smallint", options={"default":Camera::CAMERA_STANDARD_RTSP_PORT})
     * @Assert\NotBlank()
     * @Assert\Type(type="integer", message="The value {{ value }} is not a valid {{ type }}.")
     */
    private $rtspPort;

    /**
     * @var string
     *
     * @ORM\Column(name="username", type="string", length=128)
     * @Assert\NotBlank()
     */
    private $username;

    /**
     * @var string
     *
     * @ORM\Column(name="password", type="string", length=128)
     * @Assert\NotBlank()
     */
    private $password;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="Preset", mappedBy="camera", cascade={"persist", "remove"})
     */
    private $presets;

    /**
     * var Room
     * @ORM\ManyToOne(targetEntity="Room", inversedBy="camerasOld")
     * @ORM\JoinColumn(name="room_id", referencedColumnName="id", nullable=true)
     *
     **/
    private $roomOld;

    /**
     * var ArrayCollection
     * @ORM\ManyToMany(targetEntity="Room", inversedBy="cameras")
     * @ORM\JoinTable(name="devices_rooms")
     * @Assert\NotBlank()
     *
     **/
    private $rooms;


    /**
     * @var Wowza
     * @ORM\ManyToOne(targetEntity="FS\CamBundle\Entity\Wowza", inversedBy="cameras")
     * @ORM\JoinColumn(name="wowza_id", referencedColumnName="id", onDelete="SET NULL")
     * @Assert\NotBlank()
     *
     **/
    private $wowza;

    /**
     * @var string
     * @ORM\Column(type="string", options={"default":"offline"})
     */
    private $status = self::STATUS_OFFLINE;

    public function __construct()
    {
        //Default type is camera.
        $this->cameraType = self::TYPE_CAMERA;
        $this->presets = new ArrayCollection();
        $this->rooms = new ArrayCollection();
        $this->status = self::STATUS_OFFLINE;
        $this->port = self::CAMERA_STANDARD_HTTP_PORT;
        $this->rtspPort = self::CAMERA_STANDARD_RTSP_PORT;
//        $this->model = 1;
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param  string $name
     * @return Camera
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set model
     *
     * @param  integer $model
     * @return Camera
     */
    public function setModel($model)
    {
        $this->model = $model;

        return $this;
    }

    /**
     * Get model
     *
     * @return integer
     */
    public function getModel()
    {
        return $this->model;
    }

    /**
     * @return int
     */
    public function getBrand()
    {
        return $this->brand;
    }

    /**
     * @param int $brand
     */
    public function setBrand($brand)
    {
        $this->brand = $brand;

        return $this;
    }

    /**
     * Set ip
     *
     * @param  string $ip
     * @return Camera
     */
    public function setIp($ip)
    {
        $this->ip = $ip;

        return $this;
    }

    /**
     * Get ip
     *
     * @return string
     */
    public function getIp()
    {
        return $this->ip;
    }

    /**
     * Set port
     *
     * @param  integer $port
     * @return Camera
     */
    public function setPort($port)
    {
        $this->port = $port;

        return $this;
    }

    /**
     * Get port
     *
     * @return integer
     */
    public function getPort()
    {
        return $this->port;
    }

    /**
     * Set port
     *
     * @param  integer $port
     * @return Camera
     */
    public function setRtspPort($port)
    {
        $this->rtspPort = $port;

        return $this;
    }

    /**
     * Get port
     *
     * @return integer
     */
    public function getRtspPort()
    {
        return $this->rtspPort;
    }

    /**
     * Set username
     *
     * @param  string $username
     * @return Camera
     */
    public function setUsername($username)
    {
        $this->username = $username;

        return $this;
    }

    /**
     * Get username
     *
     * @return string
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * Set password
     *
     * @param  string $password
     * @return Camera
     */
    public function setPassword($password)
    {
        $this->password = $password;

        return $this;
    }

    /**
     * Get password
     *
     * @return string
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Add presets
     *
     * @param  \FS\CamBundle\Entity\Preset $presets
     * @return Camera
     */
    public function addPreset(\FS\CamBundle\Entity\Preset $presets)
    {
        $this->presets[] = $presets;

        return $this;
    }

    /**
     * Remove presets
     *
     * @param \FS\CamBundle\Entity\Preset $presets
     */
    public function removePreset(\FS\CamBundle\Entity\Preset $presets)
    {
        $this->presets->removeElement($presets);
    }

    /**
     * Get presets
     *
     * @return \Doctrine\Common\Collections\Collection | Preset[]
     */
    public function getPresets()
    {
        return $this->presets;
    }

    /**
     * Set room
     *
     * @param  \FS\CamBundle\Entity\Room $room
     * @return Camera
     */
    public function setRoomOld(\FS\CamBundle\Entity\Room $room = null)
    {
        $this->roomOld = $room;

        return $this;
    }

    /**
     * Get room
     *
     * @return \FS\CamBundle\Entity\Room
     */
    public function getRoomOld()
    {
        return $this->roomOld;
    }

    /**
     * Add room
     *
     * @param Room $room
     * @return Room
     */
    public function addRoom(\FS\CamBundle\Entity\Room $room)
    {
        $room->addCamera($this);
        $this->rooms[] = $room;

        return $this;
    }
    
    /**
     * Remove room
     *
     * @param Room $room
     */
    public function removeRoom(\FS\CamBundle\Entity\Room $room)
    {
        $this->rooms->removeElement($room);
    }

    /**
     * Get rooms
     *
     * @return \Doctrine\Common\Collections\Collection | Room[]
     */
    public function getRooms()
    {
        return $this->rooms;
    }

    /**
     * @param Wowza $wowza
     * @return $this
     */
    public function setWowza(\FS\CamBundle\Entity\Wowza $wowza = null)
    {
        $this->wowza = $wowza;

        return $this;
    }

    /**
     * @return Wowza
     */
    public function getWowza()
    {
        return $this->wowza;
    }

    /**
     * @param $ioConfigured
     * @return $this
     */
    public function setIoConfigured($ioConfigured)
    {
        $this->ioConfigured = $ioConfigured;

        return $this;
    }

    /**
     * @return string
     */
    public function getIoConfigured()
    {
        return $this->ioConfigured;
    }

    public function __toString()
    {
        return $this->name;
    }

    public function toArray()
    {
        $array['cameraId'] = $this->getId();

        if ($this->getCameraType() == self::TYPE_DEVICE) {
            //uid for device
            $array['modelId'] = $this->getDeviceUID();
            // if device send device

            if ($this->getDeviceType() == Camera::DEVICE_TYPE_CARD_READER) {
                $array['brandId'] = $this->getController(); // Type device
            } else {
                $array['brandId'] = 4; // Type device
            }
            $array['deviceType'] = $this->getDeviceType();
            $array['controller'] = $this->getController();
        } else {
            $array['deviceType'] = 'camera';
            $array['modelId'] = $this->getModel();
            $array['brandId'] = $this->getBrand();
        }

        $array['roomsIds'] = $this->getRoomsIds();
        $array['ip'] = $this->getIp();
        $array['httpPort'] = $this->getPort();

        if ($this->getRtspPort() == 0) {
            $array['rtspPort'] = self::CAMERA_STANDARD_RTSP_PORT;
        } else {
            $array['rtspPort'] = $this->getRtspPort();
        }

        $array['username'] = $this->getUsername();
        $array['password'] = $this->getPassword();
        $array['ioConfigured'] = $this->getIoConfigured();
        if($this->getIoConfigured()){
            $array['io'] = true;
        }


        if (!$this->getPresets()->isEmpty()) {
            /** @var Preset $preset */
            foreach ($this->getPresets() as $preset) {
                $array['presets'][] = array(
                    'presetId' => $preset->getId(),
                    'presetName' => $preset->getName(),
                    'presetData' => $preset->getData(),
                );
            }
        } else {
            $array['presets'] = array();
        }

        return $array;
    }

    public function getRoomsIds(){
        $result = [];
        foreach($this->getRooms() as $room)
        {
            $result[] = $room->getId();
        }
        return $result;
    }
    public function getObjectName()
    {
        return $this->name;
    }

    /**
     * Set ioSettings
     *
     * @param boolean $ioSettings
     * @return Camera
     */
    public function setIoSettings($ioSettings)
    {
        $this->ioSettings = $ioSettings;

        return $this;
    }

    /**
     * Get ioSettings
     *
     * @return boolean
     */
    public function getIoSettings()
    {
        return $this->ioSettings;
    }

    /**
     * Set cameraIoH
     *
     * @param string $cameraIoH
     * @return Camera
     */
    public function setCameraIoH($cameraIoH)
    {
        $this->cameraIoH = $cameraIoH;

        return $this;
    }

    /**
     * Get cameraIoH
     *
     * @return string
     */
    public function getCameraIoH()
    {
        return $this->cameraIoH;
    }

    /**
     * Set cameraIoE
     *
     * @param string $cameraIoE
     * @return Camera
     */
    public function setCameraIoE($cameraIoE)
    {
        $this->cameraIoE = $cameraIoE;

        return $this;
    }

    /**
     * Get cameraIoE
     *
     * @return string
     */
    public function getCameraIoE()
    {
        return $this->cameraIoE;
    }

    /**
     * Set cameraIoA
     *
     * @param string $cameraIoA
     * @return Camera
     */
    public function setCameraIoA($cameraIoA)
    {
        $this->cameraIoA = $cameraIoA;

        return $this;
    }

    /**
     * Get cameraIoA
     *
     * @return string
     */
    public function getCameraIoA()
    {
        return $this->cameraIoA;
    }

    /**
     * @return string
     */
    public function getCameraType()
    {
        return $this->cameraType;
    }

    public function setCameraType($cameraType)
    {
        $this->cameraType = $cameraType;

        return $this;
    }

    public function getDeviceUID()
    {
        return $this->deviceUID;
    }

    public function setDeviceUID($deviceUID)
    {
        $this->deviceUID = $deviceUID;

        return $this;
    }

    public function setLinkedDevice($linkedDevice)
    {
        $this->linkedDevice = $linkedDevice;
        return $this;
    }

    public function getLinkedDevice()
    {
        return $this->linkedDevice;
    }

    /**
     * @return string
     */
    public function getDeviceType()
    {
        if ($this->deviceType != null) {
            return $this->deviceType;
        }
        if ($this->cameraType != null) {
            if ($this->cameraType == self::TYPE_DEVICE) {
                return self::DEVICE_TYPE_IPAD;
            }
        }
        return self::TYPE_CAMERA;
    }

    /**
     * @param $deviceType
     * @return $this
     */
    public function setDeviceType($deviceType)
    {
        $this->deviceType = $deviceType;

        return $this;
    }

    /**
     * @return int
     */
    public function getController()
    {
        return $this->controller;
    }

    /**
     * @param int $controller
     * @return $this
     */
    public function setController($controller)
    {
        $this->controller = $controller;

        return $this;
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param string $status
     * @return $this
     */
    public function setStatus($status)
    {
        $this->status = $status;
        return $this;
    }

    /**
     * @param $status
     * @return $this
     */
    public function setStatusFromWowza($status)
    {
        switch ($status) {
            case 'online':
               $this->status = self::STATUS_ONLINE;
                break;
            case 'offline':
                $this->status = self::STATUS_OFFLINE;
                break;
            default:
                $this->status = self::STATUS_OFFLINE;
                break;
        }
        return $this;
    }
}
