<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 28.04.2015
 * Time: 12:54
 */

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use FS\CamBundle\Template\TemplateAwareTrait;
use FS\CamBundle\Template\TemplateDataTrait;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Record
 *
 * @ORM\Table(name="filters")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\FilterRepository")
 */
class Filter
{

    const FILTER_DATE_TYPE_DAYS = "day";
    const FILTER_DATE_TYPE_WEEKS = "weeks";
    const FILTER_DATE_TYPE_RANGE = "range";
    const FILTER_DATE_TYPE_NONE = "none";


    /** @deprecated */
    const FILTER_RANGE_DAY_START = -90;
    /** @deprecated */
    const FILTER_RANGE_DAY_END = 90;
    /** @deprecated */
    const FILTER_RANGE_WEEK_START = -52;
    /** @deprecated */
    const FILTER_RANGE_WEEK_END = 52;


    public static $filterDayRange = [
        -90 => -90,
        -60 => -60,
        -30 => -30,
        -10 => -10,
        -5 => -5,
        -4 => -4,
        -3 => -3,
        -2 => -2,
        -1 => -1,
        0 => 'Today',
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4,
        5 => 5,
        10 => 10,
        30 => 30,
        60 => 60,
        90 => 90,

    ];

    public static $filterWeekRange = [
        -52 => -52,
        -26 => -26,
        -10 => -10,
        -5 => -5,
        -4 => -4,
        -3 => -3,
        -2 => -2,
        -1 => -1,
        0 => 'This Week',
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4,
        5 => 5,
        10 => 10,
        26 => 26,
        52 => 52,

    ];
    //use TemplateDataTrait;

    // something something like thiiiiiiis
    //use TemplateAwareTrait;

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=128)
     * @Assert\NotBlank()
     */
    private $name;

    /**
     * @var ArrayCollection | \FS\CamBundle\Entity\Room[]
     * @ORM\ManyToMany(targetEntity="FS\CamBundle\Entity\Room", inversedBy="filters")
     * @ORM\JoinTable(name="user_filter_rooms",
     *      joinColumns={@ORM\JoinColumn(name="filter_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="room_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $filterRooms;

    /**
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\User", inversedBy="filters")
     * @ORM\JoinColumn(name="user_id", referencedColumnName="id")
     *
     **/
    private $owner;

    /**
     * @var ArrayCollection | \FS\UserBundle\Entity\User[]
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\User")
     * @ORM\JoinTable(name="user_filter_users",
     *      joinColumns={@ORM\JoinColumn(name="filter_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="user_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $filteredUsers;

    /**
     * @var ArrayCollection | \FS\UserBundle\Entity\UserGroup[]
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\UserGroup")
     * @ORM\JoinTable(name="user_filter_userGroups",
     *      joinColumns={@ORM\JoinColumn(name="filter_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="userGroup_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $filteredUserGroups;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection | FilterTemplate[]
     * @ORM\OneToMany(targetEntity="FS\CamBundle\Entity\FilterTemplate", mappedBy="filter", cascade={"all"},orphanRemoval=true)
     */
    private $filterInfoData;

    /**
     * @var string
     *
     * @ORM\Column(type="string",options={"default" = "none"})
     */
    private $filterDateType = 'none';

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=true,options={"default" = 0})
     */
    private $days = 0;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=true,options={"default" = 0})
     */
    private $weeks = 0;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $rangeStart;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $rangeEnd;


    /**
     * Constructor
     */
    public function __construct()
    {
        $this->filterRooms = new \Doctrine\Common\Collections\ArrayCollection();
        $this->fiteredUsers = new \Doctrine\Common\Collections\ArrayCollection();
        $this->filteredUserGroups = new \Doctrine\Common\Collections\ArrayCollection();
        $this->filterInfoData = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param string $name
     * @return Filter
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set filterDateType
     *
     * @param string $filterDateType
     * @return Filter
     */
    public function setFilterDateType($filterDateType)
    {
        $this->filterDateType = $filterDateType;

        return $this;
    }

    /**
     * Get filterDateType
     *
     * @return string
     */
    public function getFilterDateType()
    {
        if ($this->filterDateType != null)
            return $this->filterDateType;
        else
            return self::FILTER_DATE_TYPE_NONE;
    }

    /**
     * Set days
     *
     * @param integer $days
     * @return Filter
     */
    public function setDays($days)
    {
        $this->days = $days;

        return $this;
    }

    /**
     * Get days
     *
     * @return integer
     */
    public function getDays()
    {
        return $this->days;
    }

    /**
     * Set weeks
     *
     * @param integer $weeks
     * @return Filter
     */
    public function setWeeks($weeks)
    {
        $this->weeks = $weeks;

        return $this;
    }

    /**
     * Get weeks
     *
     * @return integer
     */
    public function getWeeks()
    {
        return $this->weeks;
    }

    /**
     * Set rangeStart
     *
     * @param \DateTime $rangeStart
     * @return Filter
     */
    public function setRangeStart($rangeStart)
    {
        $this->rangeStart = $rangeStart;

        return $this;
    }

    /**
     * Get rangeStart
     *
     * @return \DateTime
     */
    public function getRangeStart()
    {
        return $this->rangeStart;
    }

    /**
     * Set rangeEnd
     *
     * @param \DateTime $rangeEnd
     * @return Filter
     */
    public function setRangeEnd($rangeEnd)
    {
        $this->rangeEnd = $rangeEnd;

        return $this;
    }

    /**
     * Get rangeEnd
     *
     * @return \DateTime
     */
    public function getRangeEnd()
    {
        return $this->rangeEnd;
    }


    /**
     * Add filterRooms
     *
     * @param \FS\CamBundle\Entity\Room $filterRooms
     * @return Filter
     */
    public function addFilterRoom(\FS\CamBundle\Entity\Room $filterRooms)
    {
        $this->filterRooms[] = $filterRooms;

        return $this;
    }

    /**
     * Remove filterRooms
     *
     * @param \FS\CamBundle\Entity\Room $filterRooms
     */
    public function removeFilterRoom(\FS\CamBundle\Entity\Room $filterRooms)
    {
        $this->filterRooms->removeElement($filterRooms);
    }

    /**
     * Get filterRooms
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getFilterRooms()
    {
        return $this->filterRooms;
    }

    /**
     * Set owner
     *
     * @param \FS\UserBundle\Entity\User $owner
     * @return Filter
     */
    public function setOwner(\FS\UserBundle\Entity\User $owner = null)
    {
        $this->owner = $owner;

        // Set template
        // $this->setTemplate($owner->getUserGroup()->getInfoTemplate());


        return $this;
    }

    /**
     * Get owner
     *
     * @return \FS\UserBundle\Entity\User
     */
    public function getOwner()
    {
        return $this->owner;
    }

    /**
     * Add filteredUsers
     *
     * @param \FS\UserBundle\Entity\User $filteredUsers
     * @return Filter
     */
    public function addFilteredUser(\FS\UserBundle\Entity\User $filteredUsers)
    {
        $this->filteredUsers[] = $filteredUsers;

        return $this;
    }

    /**
     * Remove fiteredUsers
     *
     * @param \FS\UserBundle\Entity\User $filteredUsers
     */
    public function removeFilteredUser(\FS\UserBundle\Entity\User $filteredUsers)
    {
        $this->filteredUsers->removeElement($filteredUsers);
    }

    /**
     * Get filteredUsers
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getFilteredUsers()
    {
        return $this->filteredUsers;
    }

    /**
     * Add filteredUserGroups
     *
     * @param \FS\UserBundle\Entity\UserGroup $filteredUserGroups
     * @return Filter
     */
    public function addFilteredUserGroup(\FS\UserBundle\Entity\UserGroup $filteredUserGroups)
    {
        $this->filteredUserGroups[] = $filteredUserGroups;

        return $this;
    }

    /**
     * Remove filteredUserGroups
     *
     * @param User|UserGroup $filteredUserGroups
     */
    public function removeFilteredUserGroup(\FS\UserBundle\Entity\UserGroup $filteredUserGroups)
    {
        $this->filteredUserGroups->removeElement($filteredUserGroups);
    }

    /**
     * Get filteredUserGroups
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getFilteredUserGroups()
    {
        return $this->filteredUserGroups;
    }


    /**
     * @return ArrayCollection
     */
    public function getFilteredUsersAndGroups()
    {
        $users = !empty($this->filteredUsers) ? $this->filteredUsers->toArray() : [];
        $groups = !empty($this->filteredUserGroups) ? $this->filteredUserGroups->toArray() : [];

        return new ArrayCollection(array_merge($users, $groups));
    }

    /**
     * @param ArrayCollection $usersAndGroups
     */
    public function setFilteredUsersAndGroups(ArrayCollection $usersAndGroups = null)
    {
        $this->filteredUsers = new ArrayCollection();
        $this->filteredUserGroups = new ArrayCollection();

        if (null === $usersAndGroups) {
            return;
        }

        foreach ($usersAndGroups->toArray() as $entity) {
            if ($entity instanceof User) {
                $this->filteredUsers[] = $entity;

            } elseif ($entity instanceof UserGroup) {
                $this->filteredUserGroups[] = $entity;
            }
        }
    }

    public static function getDateFilterTypeList()
    {
        return [
            self::FILTER_DATE_TYPE_NONE => 'None',
            self::FILTER_DATE_TYPE_DAYS => 'Days',
            self::FILTER_DATE_TYPE_WEEKS => 'Weeks',
            self::FILTER_DATE_TYPE_RANGE => 'Range',
        ];
    }

    /**
     * Add filterInfoData
     *
     * @param \FS\CamBundle\Entity\FilterTemplate $filterInfoData
     * @return Filter
     */
    public function addFilterInfoData(\FS\CamBundle\Entity\FilterTemplate $filterInfoData)
    {
        //$filterInfoData->setFilter($this);
        $this->filterInfoData[] = $filterInfoData;
        $filterInfoData->setFilter($this);

        return $this;
    }

    /**
     * Add filterInfoData
     *
     * @param \FS\CamBundle\Entity\FilterTemplate $filterInfoData
     * @return Filter
     */
    public function addFilterInfoDatum(\FS\CamBundle\Entity\FilterTemplate $filterInfoData)
    {
        //$filterInfoData->setFilter($this);
        $this->filterInfoData[] = $filterInfoData;
        $filterInfoData->setFilter($this);

        return $this;
    }

    /**
     * Remove filterInfoData
     *
     * @param \FS\CamBundle\Entity\FilterTemplate $filterInfoData
     */
    public function removeFilterInfoDatum(\FS\CamBundle\Entity\FilterTemplate $filterInfoData)
    {
        $filterInfoData->setFilter(null);
        $this->filterInfoData->removeElement($filterInfoData);

    }

    /**
     * Remove filterInfoData
     *
     * @param \FS\CamBundle\Entity\FilterTemplate $filterInfoData
     */
    public function removeFilterInfoData(\FS\CamBundle\Entity\FilterTemplate $filterInfoData)
    {
        $filterInfoData->setFilter(null);
        $this->filterInfoData->removeElement($filterInfoData);

    }

    /**
     * Get filterInfoData
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getFilterInfoData()
    {
        return $this->filterInfoData;
    }


    public function __toString()
    {
        return $this->getName();
    }

    public function getNameForView()
    {
        $name = $this->getName();
        if (strlen($name) > 16) {
            $name = substr($name, 0, 13) . "...";
            return $name;
        } else {
            return $name;
        }
    }
}
