<?php

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Model\ContainerizedInterface;
use Symfony\Component\Validator\Constraints as Assert;
use FS\CamBundle\Template\TemplateDataTrait;
use FS\CamBundle\Template\TemplateAwareTrait;
use FS\CamBundle\Template\TemplateAwareInterface;

/**
 * Marker
 *
 * @ORM\Table(name="markers")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\MarkerRepository")
 */
class Marker implements TemplateAwareInterface, ContainerizedInterface
{
    use TemplateDataTrait;
    use TemplateAwareTrait;

    use FSContainerTrait;
    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=128)
     * @Assert\NotBlank
     *
     */
    private $name;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     */
    private $time;

    /**
     * @var Record
     *
     * @ORM\ManyToOne(targetEntity="Record", inversedBy="markers")
     * @ORM\JoinColumn(name="record_id", referencedColumnName="id", onDelete="CASCADE")
     */
    private $record;

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param  string $name
     * @return Marker
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set time
     *
     * @param  string $time
     * @return Marker
     */
    public function setTime($time)
    {
        $this->time = $time;

        return $this;
    }

    /**
     * Get time
     *
     * @return integer
     */
    public function getTime()
    {
        return $this->time;
    }

    public function getTimeFormatted()
    {
        $hours = floor($this->time / 3600);
        $minutes = floor(($this->time / 60) % 60);
        $seconds = $this->time % 60;

        return "$hours:$minutes:$seconds";
    }

    /**
     * @param \FS\CamBundle\Entity\Record $record
     */
    public function setRecord($record)
    {
        $this->record = $record;
    }

    /**
     * @return \FS\CamBundle\Entity\Record
     */
    public function getRecord()
    {
        return $this->record;
    }

    public function toArray()
    {
        return [
          'id' => $this->getId(),
          'name' => $this->getName(),
          'time' => $this->getTime(),
        ];
    }
}
