<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 29.10.2015
 * Time: 16:40
 */

namespace FS\CamBundle\Entity;


use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Mosaic
 *
 * @ORM\Table(name="mosaic_queue")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\MosaicQueueRepository")
 */
class MosaicQueue
{

    const STATUS_WAITING = 0;
    const STATUS_DONE = 1;
    const STATUS_DOWNLOADING = 2;
    const STATUS_CREATING = 3;
    const STATUS_FAIL = 4;
    const STATUS_FAIL_FILES_COUNT = 5;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime")
     */
    private $created;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     */
    private $status;

    /**
     * @var Record
     *
     * @ORM\OneToOne(targetEntity="Record")
     * @ORM\JoinColumn(name="record_id", referencedColumnName="id", onDelete="CASCADE")
     */
    private $record;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", options={"default":0})
     */
    private $progress = 0;


    /**
     * Constructor
     */
    public function __construct()
    {
        $this->created = new \DateTime();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set created
     *
     * @param \DateTime $created
     * @return MosaicQueue
     */
    public function setCreated($created)
    {
        $this->created = $created;

        return $this;
    }

    /**
     * Get created
     *
     * @return \DateTime
     */
    public function getCreated()
    {
        return $this->created;
    }

    /**
     * Set status
     *
     * @param integer $status
     * @return MosaicQueue
     */
    public function setStatus($status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * Get status
     *
     * @return integer
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * Set record
     *
     * @param \FS\CamBundle\Entity\Record $record
     * @return MosaicQueue
     */
    public function setRecord(\FS\CamBundle\Entity\Record $record = null)
    {
        $this->record = $record;

        return $this;
    }

    /**
     * Get record
     *
     * @return \FS\CamBundle\Entity\Record
     */
    public function getRecord()
    {
        return $this->record;
    }

    /**
     * @return int
     */
    public function getProgress()
    {
        return $this->progress;
    }

    /**
     * @param int $progress
     * @return $this
     */
    public function setProgress($progress)
    {
        $this->progress = $progress;
        return $this;
    }

    public static function getMarkedStatusesList()
    {
        return [
            self::STATUS_WAITING,
            self::STATUS_CREATING,
            self::STATUS_DOWNLOADING,
        ];
    }

    public function isMarkedStatus()
    {
        return in_array($this->status, self::getMarkedStatusesList());
    }
}
