<?php

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * Recording
 *
 * @ORM\Table(name="recordings")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\RecordingRepository")
 */
class Recording // repositoryClass="FS\CamBundle\Repository\RecordingRepository"
{

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var Record
     *
     * @ORM\OneToOne(targetEntity="Record")
     */
    private $record;

    /**
     * @var Schedule
     *
     * @ORM\OneToOne(targetEntity="Schedule", inversedBy="recording", cascade={"persist"})
     * @ORM\JoinColumn(name="schedule_id", referencedColumnName="id")
     */
    private $schedule;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime")
     */
    private $startedAt;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $stopAt;

    /**
     * @var Room
     *
     * @ORM\OneToOne(targetEntity="Room", inversedBy="recording", cascade={"persist"})
     * @ORM\JoinColumn(name="room_id", referencedColumnName="id")
     */
    private $room;

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set record
     *
     * @param  \FS\CamBundle\Entity\Record $room
     * @return Camera
     */
    public function setRecord(\FS\CamBundle\Entity\Record $record = null)
    {
        $this->record = $record;

        $this->setRoom($record->getRoom());

        return $this;
    }

    /**
     * Get record
     *
     * @return \FS\CamBundle\Entity\Record
     */
    public function getRecord()
    {
        return $this->record;
    }

    /**
     * @param \FS\CamBundle\Entity\Schedule $schedule
     * @return $this
     */
    public function setSchedule($schedule)
    {
        $this->schedule = $schedule;

        return $this;
    }

    /**
     * @return \FS\CamBundle\Entity\Schedule
     */
    public function getSchedule()
    {
        return $this->schedule;
    }

    /**
     * Set startedAt
     *
     * @param  \DateTime $startedAt
     * @return Recording
     */
    public function setStartedAt($startedAt)
    {
        $this->startedAt = $startedAt;

        return $this;
    }

    /**
     * Get startedAt
     *
     * @return \DateTime
     */
    public function getStartedAt()
    {
        return $this->startedAt;
    }

    /**
     * Set stopAt
     *
     * @param  \DateTime $stopAt
     * @return Recording
     */
    public function setStopAt($stopAt)
    {
        $this->stopAt = $stopAt;

        return $this;
    }

    /**
     * Get stopAt
     *
     * @return \DateTime
     */
    public function getStopAt()
    {
        return $this->stopAt;
    }

    /**
     * @param \FS\CamBundle\Entity\Room $room
     * @return $this
     */
    public function setRoom($room)
    {
        $this->room = $room;

        return $this;
    }

    /**
     * @return \FS\CamBundle\Entity\Room
     */
    public function getRoom()
    {
        return $this->room;
    }
/*

    public function __toString()
    {
        return "Recoding[id=" .
        $this->id . ", Recording#Room[name=" .
        $this->getRoom()->getName() . "]Recording#Schedule[name=" .
        $this->getSchedule()->getName() .
        "]startedAt=".$this->getStartedAt()->format("Y-m-d H:i").
        ",".
        "stopAt=".$this->getStopAt()->format("Y-m-d H:i")."]";
    }*/
}
