<?php

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\ExecutionContextInterface;

/**
 * Schedule
 *
 * @ORM\Table(name="recurs")
 * @ORM\Entity
 */
class Recur
{
    const FREQ_DAILY = 'daily';
    const FREQ_WEEKLY = 'weekly';
    const FREQ_MONTHLY = 'monthly';

    const RANGE_NO_END_DATE = 'no_end_date';
    const RANGE_END_BY_DATE = 'end_by_date';
    const RANGE_END_BY_OCCURENCES = 'end_by_occurrences';

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="range_type", type="string", nullable=true)
     */
    private $range;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="date", nullable=true)
     */
    private $endAt;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=true)
     */
    private $occurrences;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $frequency;

    /**
     * @var array
     * @Assert\Count(min = 1, minMessage = "At least one item must be selected")
     * @ORM\Column(type="array", nullable=true)
     *
     */
    private $weekly;

    public function __construct()
    {
        $this->range = self::RANGE_NO_END_DATE;
    }

    /**
     * @param \DateTime $endAt
     */
    public function setEndAt($endAt)
    {
        $this->endAt = $endAt;
    }

    /**
     * @return \DateTime
     */
    public function getEndAt()
    {
        return $this->endAt;
    }

    /**
     * @param string $frequency
     */
    public function setFrequency($frequency)
    {
        $this->frequency = $frequency;
    }

    /**
     * @return string
     */
    public function getFrequency()
    {
        return $this->frequency;
    }

    public static function getFrequencyList()
    {
        return [
            self::FREQ_DAILY => 'Daily',
            self::FREQ_WEEKLY => 'Weekly',
            self::FREQ_MONTHLY => 'Monthly',
        ];
    }

    /**
     * @param int $id
     * @return $this
     */
    public function setId($id)
    {
        $this->id = $id;

        return $this;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param string $occurrences
     * @return $this
     */
    public function setOccurrences($occurrences)
    {
        $this->occurrences = $occurrences;

        return $this;
    }

    /**
     * @return string
     */
    public function getOccurrences()
    {
        return $this->occurrences;
    }

    /**
     * @param array $weekly
     * @return $this
     */
    public function setWeekly($weekly)
    {
        $this->weekly = $weekly;

        return $this;
    }

    /**
     * @return array
     */
    public function getWeekly()
    {
        return $this->weekly;
    }

    /**
     * @param int $range
     * @return $this
     */
    public function setRange($range)
    {
        $this->range = $range;

        return $this;
    }

    /**
     * @return int
     */
    public function getRange()
    {
        return $this->range;
    }

    public static function getRangeList()
    {
        return [
            self::RANGE_NO_END_DATE => 'No End Date',
            self::RANGE_END_BY_OCCURENCES => 'End After',
            self::RANGE_END_BY_DATE => 'End By',
        ];
    }

    public static function getWeekList()
    {
        return [
            0 => 'Sunday',
            1 => 'Monday',
            2 => 'Tuesday',
            3 => 'Wednesday',
            4 => 'Thursday',
            5 => 'Friday',
            6 => 'Saturday'
        ];
    }

    public function isEndAtValid()
    {
        $now = new \DateTime();
        $now->modify('midnight');
        return $this->getEndAt() >= $now;
    }
    /**
     * @Assert\Callback
     */
    public function validate(ExecutionContextInterface $context)
    {

        if (!$this->isEndAtValid()) {
            $context->addViolationAt(
                null, // 'recur.endAt',
                'The End by date must be greater than Now',
                array(),
                null
            );
        }
    }

}
