<?php

namespace FS\CamBundle\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Gedmo\Mapping\Annotation as Gedmo;
use Doctrine\ORM\Mapping as ORM;
use FS\CamBundle\RecordAccess\AccessedByUsersAndGroupsInterface;
use FS\CamBundle\RecordAccess\AccessedByUsersAndGroupsTrait;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\ExecutionContextInterface;
use FS\CamBundle\Template\TemplateDataTrait;
use FS\CamBundle\Template\TemplateAwareTrait;
use FS\CamBundle\Template\TemplateAwareInterface;
use FS\Userbundle\Entity\User;

/**
 * RetentionRule
 *
 * @ORM\Table(name="retention_rules")
 * @ORM\Entity
 */
class RetentionRule
{
    const KEEP_FOR_DAYS = 0;
    const KEEP_FOR_HOURS = 1;
    const KEEP_FOR_MINUTES = 2;
    const KEEP_UNTIL = 3;
    const KEEP_INFINITELY = 4;

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var integer
     *
     * @ORM\Column(type="smallint")
     */
    private $type = self::KEEP_INFINITELY;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=true)
     */
    private $keepTime = 0;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime",nullable=true)
     */
    private $keepUntil;


    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set type
     *
     * @param integer $type
     * @return RetentionRule
     */
    public function setType($type)
    {
        $this->type = $type;

        return $this;
    }

    /**
     * Get type
     *
     * @return integer
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * Set keepTime
     *
     * @param integer $keepTime
     * @return RetentionRule
     */
    public function setKeepTime($keepTime)
    {
        $this->keepTime = $keepTime;

        return $this;
    }

    /**
     * Get keepTime
     *
     * @return integer
     */
    public function getKeepTime()
    {
        return $this->keepTime;
    }

    /**
     * Set keepUntil
     *
     * @param \DateTime $keepUntil
     * @return RetentionRule
     */
    public function setKeepUntil($keepUntil)
    {
        $this->keepUntil = $keepUntil;

        return $this;
    }

    /**
     * Get keepUntil
     *
     * @return \DateTime
     */
    public function getKeepUntil()
    {
        return $this->keepUntil;
    }

    public function __toString()
    {
        return "ID: " . $this->id . "; "
        . "Type: " . $this->type . "; "
        . "KeepTime: " . $this->keepTime . "; "
        . "KeepUntil: " . (is_null($this->keepUntil) ? "<NULL>" : $this->keepUntil->format('Y-m-d H:i:s'));
    }
}
