<?php

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;

/**
 * Exception
 *
 * @ORM\Table(name="template_fields")
 * @ORM\Entity
 */
class TemplateField
{
    const TYPE_DROPDOWN = 'dropdown';
    const TYPE_TEXT = 'text';
    const TYPE_AUDIO = 'audio';
    const TYPE_NOTE = 'note';

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=128)
     * @Assert\NotBlank(message="Title should not be blank.")
     */
    private $title;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=128)
     * @Assert\NotBlank(message="Type should not be blank")
     */
    private $type;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean")
     */
    private $required;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean")
     */
    private $enabled;

    /**
     * @var string
     *
     * @ORM\Column(type="text", nullable=true)
     */
    private $defaultData;

    /**
     * var Template
     * @ORM\ManyToOne(targetEntity="FS\CamBundle\Entity\Template", inversedBy="fields")
     * @ORM\JoinColumn(name="template_id", referencedColumnName="id")
     **/
    private $template;

    public function __construct()
    {
        $this->enabled = true;
        $this->required = false;
        $this->setType(self::TYPE_TEXT);
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getEnabled()
    {
        return $this->enabled;
    }

    /**
     * @param string $enabled
     */
    public function setEnabled($enabled)
    {
        $this->enabled = $enabled;

        return $this;
    }

    /**
     * @return string
     */
    public function getRequired()
    {
        return $this->required;
    }

    /**
     * @param string $required
     */
    public function setRequired($required)
    {
        $this->required = $required;

        return $this;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;

        return $this;
    }

    /**
     * @return Template
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param Template $type
     */
    public function setType($type)
    {
        $this->type = $type;

        return $this;
    }

    /**
     * @return string
     */
    public function getDefaultData()
    {
        if ($this->type == self::TYPE_DROPDOWN && $this->defaultData != null) {
            $arr = explode(',', $this->defaultData);
            $arr = array_unique($arr);
            $arr = array_filter ($arr);
            $defaultData = implode(",", $arr);
            return $defaultData;
        }
        return $this->defaultData;
    }

    /**
     * @param string $defaultData
     * @return $this
     */
    public function setDefaultData($defaultData)
    {
        if ($this->type == self::TYPE_DROPDOWN) {
            $arr = explode(',', $defaultData);
            $arr = array_unique($arr);
            $arr = array_filter ($arr);
            $defaultData = implode(",", $arr);
            $this->defaultData = $defaultData;
        } else {
            $this->defaultData = $defaultData;
        }


        return $this;
    }

    /**
     * @return Template
     */
    public function getTemplate()
    {
        return $this->template;
    }

    /**
     * @param Template $template
     */
    public function setTemplate($template)
    {
        $this->template = $template;

        return $this;
    }

    /**
     * @return int
     */
    public function getVersion()
    {
        return $this->version;
    }

    /**
     * @param int $version
     */
    public function setVersion($version)
    {
        $this->version = $version;

        return $this;
    }


    public function getIdTitle()
    {
        return [
            $this->getId() => $this->getTitle()
        ];
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $result = [];
        foreach ($this as $k => $v) {
            $result[$k] = $v;
        }
        return $result;
    }

    public function toApiArray()
    {
        $result = [];
        foreach ($this as $k => $v) {
            if (
                $k == 'enabled' ||
                $k == 'template'
            )
                continue;
            if ($k == 'defaultData' && $v != null) {
                $data = explode(',', $v);
                $data = array_unique($data);
                foreach ($data as $value) {
                    if($value != ''){
                        $result['data'][] = $value;
                    }

                }
                continue;
            } elseif ($v == null) {
                $result['data'] = [];
                continue;
            }
            $result[$k] = $v;
        }
        // if required this is not optional field
        $result['optional'] = ($this->required == true) ? 0 : 1;
        return $result;
    }


}
