<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 25.03.2015
 * Time: 12:33
 */

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use FS\BasicBundle\Model\Loggable;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Camera
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\WowzaRepository")
 * @ORM\Table(name="wowza")
 */
class Wowza implements Loggable
{
    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string")
     * @Assert\NotBlank()
     */
    private $name;

    /**
     * @var int
     *
     * @ORM\Column(type="integer", nullable=true)
     */
    private $port;

    /**
     * @var string
     *
     * @ORM\Column(type="string")
     * @Assert\NotBlank()
     */
    private $address;

    /**
     * @var string
     *
     * @ORM\Column(type="string")
     * @Assert\NotBlank()
     */
    private $storageFolder;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $useSsl;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="Camera", mappedBy="wowza")
     */
    private $cameras;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->cameras = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer 
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param string $name
     * @return Wowza
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string 
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set port
     *
     * @param integer $port
     * @return Wowza
     */
    public function setPort($port)
    {
        $this->port = $port;

        return $this;
    }

    /**
     * Get port
     *
     * @return integer 
     */
    public function getPort()
    {
        return $this->port;
    }

    /**
     * Set address
     *
     * @param string $address
     * @return Wowza
     */
    public function setAddress($address)
    {
        $this->address = $address;

        return $this;
    }

    /**
     * Get address
     *
     * @return string 
     */
    public function getAddress()
    {
        return $this->address;
    }

    /**
     * Set storageFolder
     *
     * @param string $storageFolder
     * @return Wowza
     */
    public function setStorageFolder($storageFolder)
    {
        $this->storageFolder = $storageFolder;

        return $this;
    }

    /**
     * Get storageFolder
     *
     * @return string 
     */
    public function getStorageFolder()
    {
        return $this->storageFolder;
    }

    /**
     * Set ssl
     *
     * @param boolean $useSsl
     * @return Wowza
     */
    public function setUseSsl($useSsl)
    {
        $this->useSsl = $useSsl;

        return $this;
    }

    /**
     * Get ssl
     *
     * @return boolean 
     */
    public function getUseSsl()
    {
        return $this->useSsl;
    }

    /**
     * Add cameras
     *
     * @param \FS\CamBundle\Entity\Camera $cameras
     * @return Wowza
     */
    public function addCamera(\FS\CamBundle\Entity\Camera $cameras)
    {
        $this->cameras[] = $cameras;

        return $this;
    }

    /**
     * Remove cameras
     *
     * @param \FS\CamBundle\Entity\Camera $cameras
     */
    public function removeCamera(\FS\CamBundle\Entity\Camera $cameras)
    {
        $this->cameras->removeElement($cameras);
    }

    /**
     * Get cameras
     *
     * @return \Doctrine\Common\Collections\Collection 
     */
    public function getCameras()
    {
        return $this->cameras;
    }

    public function getObjectName()
    {
        return $this->name;
    }
}
