<?php

namespace FS\CamBundle\EventListener;

use Buzz\Exception\ClientException;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\Event\LifecycleEventArgs;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Preset;
use Buzz\Browser;

class CameraUpdateListener implements EventSubscriber
{
    private $buzz;

    public function __construct(Browser $buzz)
    {
        $this->buzz = $buzz;
    }

    public function notify($command, LifecycleEventArgs $args)
    {
        $entity = $args->getEntity();
        if ($entity instanceof Camera) {
            $camera = $entity;
            $address = $camera->getWowza()->getAddress();
            $cameraData = $camera->toArray();

            try {
                $this->buzz
                    ->post(
                        'http://' . $address . ':8086/dustinUpdateCamera',
                        ['Content-Type' => 'application/json'],
                        json_encode(['command' => $command, 'camera' => $cameraData])
                    );
            } catch (ClientException $e) {
                /*NOP*/
            }

            if ($command == 'delete' && $camera->getIoConfigured() && $camera->getIoSettings()) {

                $host = 'http://' . $camera->getUsername() . ':' . $camera->getPassword() . '@' . $camera->getIp() . ':' . $camera->getPort();

                $response = new \Buzz\Message\Response();
                $request = new \Buzz\Message\Request('GET', '/', $host);

                $client = $this->buzz->getClient();

                $resource = '/axis-cgi/param.cgi?action=remove&group=EventServers.HTTP.' . $camera->getCameraIoH();
                $request->setResource($resource);
                try {
                    $client->send($request, $response);
                } catch (ClientException $e) {
                    /*NOP*/
                }

                $resource = '/axis-cgi/param.cgi?action=remove&group=Event.' .
                    $camera->getCameraIoE() . '.Actions.' . $camera->getCameraIoA();
                $request->setResource($resource);
                try {
                    $client->send($request, $response);
                } catch (ClientException $e) {
                    /*NOP*/
                }

                $resource = '/axis-cgi/param.cgi?action=remove&group=Event.' . $camera->getCameraIoE();
                $request->setResource($resource);
                try {
                    $client->send($request, $response);
                } catch (ClientException $e) {
                    /*NOP*/
                }
            }
        } else {
            return;
        }

    }

    public function getSubscribedEvents()
    {
        return array(
            'postPersist',
            'postUpdate',
            'preRemove'
        );
    }

    public function postUpdate(LifecycleEventArgs $args)
    {
        $this->notify('update', $args);
    }

    public function postPersist(LifecycleEventArgs $args)
    {
        $this->notify('create', $args);
    }

    public function preRemove(LifecycleEventArgs $args)
    {
        $this->notify('delete', $args);
    }

}