<?php

namespace FS\CamBundle\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use FS\CamBundle\Entity\MosaicQueue;
use Predis\Client as RedisClient;


class MosaicQueueUpdateListener
{
    const MOSAIC_QUEUE__UPDATE_REDIS_CHANNEL = 'mosaic:update';

    /** @var \Predis\Client $redis */
    private $redis;

    public function __construct(RedisClient $redis)
    {
        $this->redis = $redis;
    }

    public function postUpdate(LifecycleEventArgs $args)
    {
        $entity = $args->getEntity();
        if ($entity instanceof MosaicQueue) {
            if(in_array($entity->getStatus(), self::getSubscribeStatusList())
            ){
                $this->redis->publish(
                    self::MOSAIC_QUEUE__UPDATE_REDIS_CHANNEL,
                    json_encode(
                        [
                            'id' => $entity->getId(),
                            'status' => $entity->getStatus(),
                            'progress' => $entity->getProgress(),
                            'recordId' => $entity->getRecord()->getId(),
                        ]
                    )
                );
            }
        }
    }

    private static function getSubscribeStatusList()
    {
        return [
            MosaicQueue::STATUS_DONE,
            MosaicQueue::STATUS_CREATING,
            MosaicQueue::STATUS_FAIL,
            MosaicQueue::STATUS_FAIL_FILES_COUNT,
        ];
    }


}