<?php

namespace FS\CamBundle\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use FS\CamBundle\Entity\Schedule;
use FS\UserBundle\Entity\Log;
use Predis\Client as RedisClient;
use Symfony\Component\DependencyInjection\ContainerInterface;


class ScheduleUpdateListener
{

    const SCHEDULE_UPDATE_REDIS_CHANNEL = 'schedule:update';
    const SCHEDULE_DELETE_REDIS_CHANNEL = 'schedule:delete';

    private $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    public function postPersist(LifecycleEventArgs $args)
    {

        $entity = $args->getEntity();

        if ($entity instanceof Schedule) {
            $user = $this->container->get('security.context')->getToken()->getUser();
            $extraFields = function(Log $log) use ($entity) {
                $log->setContainer($entity->getContainer());
            };

            $this->container->get('fs_user.util.logger')->logAction(
                Log::OB_TYPE_RECORDING,
                $entity,
                $user,
                Log::$actions[Log::OB_TYPE_RECORDING]['schedule'],
                $extraFields);
        }
    }

    public function preRemove(LifecycleEventArgs $args)
    {
        $entity = $args->getEntity();

        if ($entity instanceof Schedule) {
            $this->container->get('snc_redis.default')->publish(
                self::SCHEDULE_DELETE_REDIS_CHANNEL,
                json_encode(
                    [
                        'status' => 'remove',
                        'id' => (string)$entity->getId(),
                    ]
                )
            );
        }

    }

    public function postUpdate(LifecycleEventArgs $args)
    {
        $entity = $args->getEntity();

        if ($entity instanceof Schedule) {
            $token = $this->container->get('security.context')->getToken();

            if ($token != null) {
                $this->container->get('snc_redis.default')->publish(
                    self::SCHEDULE_UPDATE_REDIS_CHANNEL,
                    json_encode(
                        [
                            'id' => $entity->getId(),
                            'status' => 'update',
                            'content' => $this->container->get('templating')->render(
                                'FSCamBundle:Schedule:index_item.html.twig',
                                ['schedule' => $entity, 'conflict' => [], 'block'=>[]]
                            )
                        ]
                    )
                );
            } else {
                $this->container->get('snc_redis.default')->publish(
                    self::SCHEDULE_UPDATE_REDIS_CHANNEL,
                    json_encode(
                        [
                            'id' => $entity->getId(),
                            'status' => 'update',
                            'content' => $this->container->get('templating')->render(
                                'FSCamBundle:Schedule:index_item_listener.html.twig',
                                ['schedule' => $entity, 'conflict' => [], 'block'=>[]]
                            )
                        ]
                    )
                );
            }
        }
    }
}