<?php

namespace FS\CamBundle\Form\Builder;

use FS\CamBundle\Entity\TemplateField;
use Symfony\Component\Form\FormInterface;
use FS\CamBundle\Entity\Template;
use Symfony\Component\Validator\Constraints\NotBlank;

// todo change to template aware interface
use FS\CamBundle\Entity\Marker;

class TemplateFieldsFormBuilder
{

    /**
     * @var FormInterface
     */
    protected $form;

    /**
     * @var  Template
     */
    protected $template;

    protected $data;

    /**
     * @param $data
     * @param Template $template
     * @param FormInterface $form
     */
    public function __construct($data, Template $template = null, FormInterface $form)
    {
        $this->form = $form;
        $this->template = $template;
        $this->data = $data;
    }


    public function build()
    {
        $template = $this->template;


        if (null === $template || $template->getFields()->isEmpty()) {
            return;
        }
        $fields = $template->getFields();

        foreach ($fields as $field) {
            if (!$field->getEnabled()) {
                continue;
            }

            $options = [
                'label' => $field->getTitle(),
                'required' => $field->getRequired(),
                'by_reference' => false,
            ];

//            $options['mapped'] = false;

            if ($field->getRequired()) {
                $options['constraints'][] = new NotBlank();
            }

            $type = 'text';
            switch ($field->getType()) {
                case TemplateField::TYPE_TEXT:
                    $type = 'text';
                    $options['empty_data'] = $field->getDefaultData();
                    $options['attr']['placeholder'] = $field->getDefaultData();
                    $options['attr']['pattern'] = '.+[.\S]+(.)*';
                    break;
                case TemplateField::TYPE_DROPDOWN:
                    $type = 'choice';
                    $choices = explode(',', $field->getDefaultData());
                    $options['choices'] = array_combine($choices, $choices);
                    array_map(
                        function ($value) {
                            return trim($value);
                        },
                        $options['choices']
                    );
                    break;
                case TemplateField::TYPE_AUDIO:
                    $uniqId = uniqid('audio');
                    if(
                        $this->data != null
                        &&
                        array_key_exists('audio_id_hidden_' . $field->getId(), $this->data)
                    ){
                        $uniqId = $this->data['audio_id_hidden_' . $field->getId()];
                    }
                    $type = 'text';
                    $options['attr']['placeholder'] = "Notes about activity";
                    $options['attr']['class'] = 'audio-template ' . $uniqId;
                    $options['attr']['data-uid'] = $uniqId;
                    $options['attr']['pattern'] = '.+[.\S]+(.)*';
                    $this->form->add('audio_id_hidden_' . $field->getId(), 'hidden', [
                        'data' => $uniqId,
                        'attr' => [
                            'class' => 'audio-uid ' . $uniqId
                        ],
                    ]);
                    break;
                case TemplateField::TYPE_NOTE:
                    $data = $field->getDefaultData();
                    $type = 'textarea';
                    $options['empty_data'] = $data;
                    $options['attr']['placeholder'] = $data;
                    $options['attr']['rows'] = 4;
                    break;
                default:
                    break;
            }


            $this->form->add($field->getId(), $type, $options);
        }

    }


} 