<?php
namespace FS\CamBundle\Form\EventListener;

use FS\CamBundle\Template\TemplateAwareInterface;
use FS\CamBundle\Entity\TemplateField;
use FS\CamBundle\Template\TemplateTypes;
use FS\UserBundle\Entity\User;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Security\Core\SecurityContext;
use FS\CamBundle\Entity\Template;
use FS\CamBundle\Form\Builder\TemplateFieldsFormBuilder;
use Symfony\Component\PropertyAccess\PropertyAccess;

class TemplateFieldsSubscriber implements EventSubscriberInterface
{
    private $user;

    public function __construct(User $user, $templateAware)
    {
        $this->user = $user;
        $this->templateAwareEntity = $templateAware;
    }

    public static function getSubscribedEvents()
    {
        // Tells the dispatcher that you want to listen on the form.pre_set_data
        // event and that the preSetData method should be called.
        return array(FormEvents::PRE_SET_DATA => 'preSetData', FormEvents::PRE_SUBMIT => 'postSubmit');
    }

    public function preSetData(FormEvent $event)
    {
        $data = $event->getData();
        $form = $event->getForm();


        $templateBuilder = new TemplateFieldsFormBuilder($data, $this->getTemplate(), $form);
        $templateBuilder->build();
    }

    public function postSubmit(FormEvent $event)
    {
        $form = $event->getForm();
        $data = $event->getData();


        $templateBuilder = new TemplateFieldsFormBuilder($data, $this->getTemplate(), $form);
        $templateBuilder->build();
    }

    /**
     * @return Template|null
     */
    public function getTemplate()
    {
        if (null === $this->templateAwareEntity) {
            return null;
        }
        if (null === $this->templateAwareEntity->getId()) {
            $template = TemplateTypes::getTemplate($this->templateAwareEntity, $this->user);
            if ($template !== null) {
                $this->templateAwareEntity->setTemplate($template);
            }
        }

        if (null !== $template = $this->templateAwareEntity->getTemplate()) {
            return $template;
        }

        return null;
    }
}