<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 05.02.2016
 * Time: 10:32
 */

namespace FS\CamBundle\Form\Transformer;


use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\Expiration;
use Symfony\Component\Form\DataTransformerInterface;

class ExpirationTransformer implements DataTransformerInterface
{
    /** @var ObjectManager */
    private $om;

    public function __construct(ObjectManager $om)
    {
        $this->om = $om;
    }

    /**
     * @param Expiration $value
     * @return mixed|void
     */
    public function transform($value)
    {
        if (is_null($value)){
            return [
                'type' => Expiration::NO_EXPIRATION,
                'expiration_from' => null,
                'id' => null,
            ];
        }
        $expirationFormData = [
            'type' => $value->getType(),
            'id' => $value->getId(),
            'expiration_from' => null
        ];

        switch ($value->getType()) {
            case Expiration::ACCESS_EXPIRATION:
                $expirationFormData['expiration_from'] = $value->getExpirationFrom();
                break;
            default:
                break;
        }
        return $expirationFormData;
    }

    /**
     * @param array $value
     * @return Expiration
     */
    public function reverseTransform($value)
    {
        if (!$value)
            return null;

        $id = $value['id'];

        if (!is_null($id))
            $expiration = $this->om->getRepository("FSCamBundle:Expiration")->find($id);
        else
            $expiration = new Expiration();

        $type = $value['type'];
        if (is_null($type))
            return null;

        $expiration->setType($type);

        switch ($type) {
            case Expiration::ACCESS_EXPIRATION:
                $expirationFrom = $value['expiration_from'];
                    if(!is_null($expirationFrom)){
                        $expiration->setExpirationFrom($expirationFrom);
                    }
                break;
            default:
                break;
        }
        return $expiration;
    }
}