<?php

namespace FS\CamBundle\Form\Transformer;

use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\RetentionRule;
use Symfony\Component\Form\DataTransformerInterface;


class RetentionRuleTransformer implements DataTransformerInterface
{
    /** @var ObjectManager */
    private $om;

    public function __construct(ObjectManager $om)
    {
        $this->om = $om;
    }

    /**
     * Transforms a RetentionRule (retentionRule) to an array.
     *
     * @param  RetentionRule $retentionRule
     * @return array
     */
    public function transform($retentionRule)
    {
        if (is_null($retentionRule))
            return ['inherit' => true];
        $retentionFormData = [
            'type' => $retentionRule->getType(),
            'id' => $retentionRule->getId(),
            'keep_for_days' => null,
            'keep_for_hours' => null,
            'keep_for_minutes' => null,
            'keep_until' => null
        ];
        switch ($retentionRule->getType()) {
            case RetentionRule::KEEP_FOR_DAYS:
                $retentionFormData['keep_for_days'] = $retentionRule->getKeepTime();
                break;
            case RetentionRule::KEEP_FOR_HOURS:
                $retentionFormData['keep_for_hours'] = $retentionRule->getKeepTime();
                break;
            case RetentionRule::KEEP_FOR_MINUTES:
                $retentionFormData['keep_for_minutes'] = $retentionRule->getKeepTime();
                break;
            case RetentionRule::KEEP_UNTIL:
                $retentionFormData['keep_until'] = $retentionRule->getKeepUntil();
                break;
        }
        return $retentionFormData;
    }

    /**
     * Transforms an array (retentionRule) to a RetentionRule.
     *
     * @param  array $retentionFormData
     * @return RetentionRule
     */
    public function reverseTransform($retentionFormData)
    {
        if (!$retentionFormData)
            return null;
        $inherit = $retentionFormData['inherit'];
        if($inherit)
            return null;
        $id = $retentionFormData['id'];
        if (!is_null($id))
            $retentionRule = $this->om->getRepository("FSCamBundle:RetentionRule")->find($id);
        else
            $retentionRule = new RetentionRule();
        $type = $retentionFormData['type'];
        if (is_null($type))
            return null;
        $retentionRule->setType($type);
        switch ($type) {
            case RetentionRule::KEEP_FOR_DAYS:
                $keepForDays = $retentionFormData['keep_for_days'];
                if (is_null($keepForDays))
                    $keepForDays = 0;
                $retentionRule->setKeepTime($keepForDays);
                break;
            case RetentionRule::KEEP_FOR_HOURS:
                $keepForHours = $retentionFormData['keep_for_hours'];
                if (is_null($keepForHours))
                    $keepForHours = 0;
                $retentionRule->setKeepTime($keepForHours);
                break;
            case RetentionRule::KEEP_FOR_MINUTES:
                $keepForMinutes = $retentionFormData['keep_for_minutes'];
                if (is_null($keepForMinutes))
                    $keepForMinutes = 0;
                $retentionRule->setKeepTime($keepForMinutes);
                break;
            case RetentionRule::KEEP_UNTIL:
                $keepUntil = $retentionFormData['keep_until'];
                if (!is_null($keepUntil))
                    $retentionRule->setKeepUntil($keepUntil);
                break;
        }
        return $retentionRule;
    }
}
