<?php

namespace FS\CamBundle\Form\Transformer;

use Doctrine\Common\Collections\ArrayCollection;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception\TransformationFailedException;
use Doctrine\Common\Persistence\ObjectManager;


class UsersAndGroupsArrayTransformer implements DataTransformerInterface
{

    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @param ObjectManager $om
     */
    public function __construct(ObjectManager $om)
    {
        $this->om = $om;
    }

    /**
     *
     * @return mixed An array of entities
     *
     * @throws TransformationFailedException
     */
    public function transform($collection)
    {
        if (null === $collection) {
            return array();
        }

        $array = [];
        foreach ($collection as $entity) {

            $prefix = '';
            if ($entity instanceof User) {
                $prefix = 'user_';
            } elseif ($entity instanceof UserGroup) {
                $prefix = 'group_';
            }
            $array[] = $prefix . $entity->getId();
        }

        return $array;
    }

    /**
     * Transforms choice keys into entities.
     *
     * @param mixed $array An array of entities
     *
     * @return ArrayCollection   A collection of entities
     */
    public function reverseTransform($array)
    {

        if ('' === $array || null === $array) {
            $array = array();
        } else {
            $array = (array) $array;
        }

        $userIds = [];
        $groupIds = [];
        foreach ($array as $value) {
            list($prefix, $id) = explode('_', $value);
            if ($prefix == 'user') {
                $userIds[] = $id;
            } else {
                $groupIds[] = $id;
            }
        }

        $users = $this->om->getRepository('FSUserBundle:User')->findBy(['id' => $userIds]);
        $groups = $this->om->getRepository('FSUserBundle:UserGroup')->findBy(['id' => $groupIds]);

        return new ArrayCollection(array_merge($groups, $users));
    }

}
