<?php

namespace FS\CamBundle\Form\Type;

use Doctrine\Common\Persistence\ObjectManager;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Form\FormInterface;
use FS\CamBundle\Utils\CameraModelManager;

class CameraType extends AbstractType
{

    /**
     * @var CameraModelManager
     */
    protected $cameraModelManager;
    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var ContainerManager
     */
    private $containerManager;

    public function __construct(
        CameraModelManager $cameraModelManager,
        ObjectManager $om,
        ContainerManager $containerManager
    )
    {
        $this->cameraModelManager = $cameraModelManager;
        $this->om = $om;
        $this->containerManager = $containerManager;
    }

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $modelManager = $this->cameraModelManager;
        $brands = $modelManager->getBrands();

        $builder
            ->add('name', null, ['label' => 'Camera name'])
            ->add('ip')
            ->add('port', null, ['label' => 'http Port'])
            ->add('rtspPort')
            ->add('username')
            ->add('password', 'password',
                [
                    'always_empty' => false,
                ]
            )
            ->add(
                 'rooms',
                 'genemu_jqueryselect2_entity',
                 [
                     'class' => 'FS\CamBundle\Entity\Room',
                     'property' => 'name',
                     'multiple' => true,
                     'required' => true,
                     'label' => 'Rooms',
                     'attr' => ['placeholder' => 'Add Room', 'class' => 'col-xs-12'],
                     'choices' => $this->containerManager->filterArray(
                         $this->om->getRepository('FSCamBundle:Room')->findAll()
                     )
                 ]
             )
            ->add(
                'wowza',
                'entity',
                [
                    'class' => 'FS\CamBundle\Entity\Wowza',
                    'property' => 'name',
                    'multiple' => false,
                    'required' => true,
                    'label' => 'Media Server',
                ]
            )
            ->add(
                'brand',
                'choice',
                [
                    'empty_value' => 'Select brand',
                    'choices' => $brands
                ]
            );


        $formModifier = function (FormInterface $form, $brand = null) use ($modelManager) {
            $models = null === $brand ? array() : $modelManager->getModels($brand);
            $form->add(
                'model',
                'choice',
                [
                    'choices' => $models,
                    'required' => true
                ]
            );
        };

        $builder->addEventListener(
            FormEvents::PRE_SET_DATA,
            function (FormEvent $event) use ($formModifier) {
                // this would be your entity, i.e. brandMeetup
                $data = $event->getData();

                $formModifier($event->getForm(), $data->getbrand());
            }
        );

        $builder->get('brand')->addEventListener(
            FormEvents::POST_SUBMIT,
            function (FormEvent $event) use ($formModifier) {
                // It's important here to fetch $event->getForm()->getData(), as
                // $event->getData() will get you the client data (that is, the ID)
                $brand = $event->getForm()->getData();

                // since we've added the listener to the child, we'll have to pass on
                // the parent to the callback functions!
                $formModifier($event->getForm()->getParent(), $brand);
            }
        );
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Camera',
                'disabled' => false
            )
        );
    }

    /**
     * @return string
     */
    public function getName()
    {
        return 'camera';
    }
}
