<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 07.12.2015
 * Time: 19:38
 */

namespace FS\CamBundle\Form\Type;


use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Utils\CameraModelManager;
use FS\CamBundle\Utils\DeviceTypeManager;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class DeviceType extends CameraType
{
    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var DeviceTypeManager
     */
    protected $deviceTypeManager;

    /**
     * @var ContainerManager
     */
    private $containerManager;

    public function __construct(CameraModelManager $cameraModelManager,
                                DeviceTypeManager $deviceTypeManager,
                                ObjectManager $om,
                                ContainerManager $containerManager
    )
    {
        parent::__construct($cameraModelManager,$om, $containerManager);
        $this->deviceTypeManager = $deviceTypeManager;
        $this->om = $om;
        $this->containerManager = $containerManager;
    }

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {

        $typeManager = $this->deviceTypeManager;
        $builder
            ->add(
                'rooms',
                'genemu_jqueryselect2_entity',
                [
                    'class' => 'FS\CamBundle\Entity\Room',
                    'property' => 'name',
                    'multiple' => true,
                    'required' => true,
                    'label' => 'Rooms',
                    'attr' => ['placeholder' => 'Add Room', 'class' => 'col-xs-12'],
                    'choices' => $this->containerManager->filterArray(
                        $this->om->getRepository('FSCamBundle:Room')->findAll()
                    )
                ]
            )
            ->add('name', null, ['label' => 'Device name'])
            ->add('deviceType', 'choice', [
                'choices' => $this->deviceTypeManager->getDevices(),
                'required' => true
            ]);


        $formModifier = function (FormInterface $form, $deviceType = null) use ($typeManager) {
            if ($deviceType == Camera::DEVICE_TYPE_CARD_READER) {
                $form
                    ->add(
                        'controller',
                        'choice',
                        [
                            'choices' => $this->deviceTypeManager->getControllers(),
                            'required' => true
                        ]
                    )
                    ->add('ip', null,
                        [
                            'required' => true
                        ]
                    )
                    ->add('port', null,
                        [
                            'label' => 'http port',
                            //'data' => 80,
                            'empty_data' => Camera::CAMERA_STANDARD_HTTP_PORT,
                            'required' => true
                        ]
                    )
                    ->add('rtspPort', null,
                        [
                            'label' => 'rtsp port',
                            //'data' => 554,
                            'empty_data' => Camera::CAMERA_STANDARD_RTSP_PORT,
                            'required' => true
                        ]
                    )
                    ->add('username', null,
                        [
                            'required' => true
                        ]
                    )
                    ->add('password', 'password',
                        [
                            'always_empty' => false,
                            'required' => true
                        ]
                    );
            }

        };
        $builder->addEventListener(
            FormEvents::PRE_SET_DATA,
            function (FormEvent $event) use ($formModifier) {
                /** @var Camera $data */
                $data = $event->getData();
                $formModifier($event->getForm(), $data->getDeviceType());
            }
        );

        $builder->get('deviceType')->addEventListener(
            FormEvents::POST_SUBMIT,
            function (FormEvent $event) use ($formModifier) {
                $data = $event->getForm()->getData();
                $formModifier($event->getForm()->getParent(), $data);
            }
        );


    }
    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Camera',
                'disabled' => false,
                'csrf_protection' => false,
            )
        );
    }

    /**
     * @return string
     */
    public function getName()
    {
        return 'device';
    }
}