<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 30.04.2015
 * Time: 17:38
 */

namespace FS\CamBundle\Form\Type;


use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\FilterTemplate;
use FS\CamBundle\Entity\TemplateField;
use FS\CamBundle\Template\TemplateTypes;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Security\Core\SecurityContext;

class FilterTemplateType extends AbstractType
{

    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var SecurityContext
     */
    private $securityContext;

    /**
     * @param ObjectManager $om
     * @param SecurityContext $securityContext
     */
    public function __construct(ObjectManager $om, SecurityContext $securityContext)
    {
        $this->om = $om;
        $this->securityContext = $securityContext;
    }

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {

        $templateFields = $this->om->getRepository('FSCamBundle:TemplateField')->findAll();
        $templateChoices = [];

        /**
         * Forming template
         */
        $templateChoices[FilterTemplate::FILTER_TEMPLATE_TYPE_INFORMATION] = 'Information';
        $templateChoices[FilterTemplate::FILTER_TEMPLATE_TYPE_RECORD] = 'Record Name';
        /** @var TemplateField $field */
        foreach ($templateFields as $field) {
            if($field->getTemplate()->getType() == TemplateTypes::TYPE_INFO){
                $templateChoices[$field->getId()] = $field->getTitle();
            }
        }
        $templateChoices[FilterTemplate::FILTER_TEMPLATE_TYPE_MARKER] = 'Marker';
        $templateChoices[FilterTemplate::FILTER_TEMPLATE_TYPE_EVENT] = 'Event';
        /** @var TemplateField $field */
        foreach ($templateFields as $field) {
            if($field->getTemplate()->getType() == TemplateTypes::TYPE_MARKER){
                $templateChoices[$field->getId()] = $field->getTitle();
            }
        }

        $builder
            ->add(
                'enabled',
                'checkbox',
                [
                    'label' => 'On',
                    'widget_checkbox_label' => 'widget',
                    'horizontal' => false,
                    'error_bubbling' => true,
                ]
            )
            ->add(
                'fieldId',
                'choice',
                [
                    'choices' => $templateChoices,
                    'label' => false,
                    'widget_type' => "inline",
                    'horizontal' => false,
                    'label_render' => false,
                    'error_bubbling' => true,
                    'attr' => ['placeholder' => 'Template field'],
                ]
            )
            ->add(
                'filterText',
                'text',
                [
                    'label' => false,
                    'horizontal' => false,
                    'label_render' => false,
                    'error_bubbling' => true,
                    'attr' => ['placeholder' => 'Matching Information'],
                ]
            );

        $builder->addEventListener(
            FormEvents::SUBMIT,
            function (FormEvent $event) {
                /** @var FilterTemplate $filterTemplate */
                $filterTemplate = $event->getData();
                switch ($filterTemplate->getFieldId()) {
                    case FilterTemplate::FILTER_TEMPLATE_TYPE_INFORMATION:
                        $filterTemplate->setFilterType(FilterTemplate::FILTER_TEMPLATE_TYPE_INFORMATION);
                        $filterTemplate->setFieldId(0);
                        break;
                    case FilterTemplate::FILTER_TEMPLATE_TYPE_RECORD:
                        $filterTemplate->setFilterType(FilterTemplate::FILTER_TEMPLATE_TYPE_RECORD);
                        $filterTemplate->setFieldId(0);
                        break;
                    case FilterTemplate::FILTER_TEMPLATE_TYPE_MARKER:
                        $filterTemplate->setFilterType(FilterTemplate::FILTER_TEMPLATE_TYPE_MARKER);
                        $filterTemplate->setFieldId(0);
                        break;
                    case FilterTemplate::FILTER_TEMPLATE_TYPE_EVENT:
                        $filterTemplate->setFilterType(FilterTemplate::FILTER_TEMPLATE_TYPE_EVENT);
                        $filterTemplate->setFieldId(0);
                        break;
                    default:
                        $filterTemplate->setFilterType(FilterTemplate::FILTER_TEMPLATE_TYPE_REGULAR);
                        break;
                }
            }
        );
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\FilterTemplate',
                'render_fieldset' => false,
                'label_render' => false,
                'show_legend' => false,
                'attr' => ['class' => 'form-inline'],
                'error_bubbling' => false,
            )
        );
    }

    /**
     * Returns the name of this type.
     *
     * @return string The name of this type
     */
    public function getName()
    {
        return 'user_filter_information';
    }
}