<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 28.04.2015
 * Time: 19:47
 */

namespace FS\CamBundle\Form\Type;


use FS\CamBundle\Entity\Filter;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class FilterType extends AbstractType
{

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        /** @var Filter $filter */
        $filter = $builder->getData();

        $builder->add('name', null, ['label' => 'Filter Name']);
        $infoTab = $builder->create(
            'info',
            'tab',
            array(
                'label' => 'Information',
                'inherit_data' => true,
            )
        );
        $roomTab = $builder->create(
            'room',
            'tab',
            array(
                'label' => 'Room',
                'inherit_data' => true,
            )
        );
        $authorTab = $builder->create(
            'author',
            'tab',
            array(
                'label' => 'Author',
                'inherit_data' => true,
            )
        );

        $dateTab = $builder->create(
            'date',
            'tab',
            array(
                'label' => 'Date',
                'inherit_data' => true,
            )
        );

        $roomTab->add(
            'filterRooms',
            'genemu_jqueryselect2_entity',
            [
                'class' => 'FS\CamBundle\Entity\Room',
                'property' => 'name',
                'multiple' => true,
                'required' => false,
                'label' => 'Rooms',
                'attr' => ['placeholder' => 'Add Rooms', 'style'=>"width:450px"]
            ]
        );


        $authorTab->add('filtered_users_and_groups', 'users_and_groups', [
            'label' => 'Users and UserGroups filter',
            'required' => false,
        ]);

        $infoTab->add(
            'filterInfoData',
            'collection',
            [
                'type' => 'user_filter_information',
                'label' => " ",
                'allow_add' => true,
                'allow_delete' => true,
                'attr' => array('class'=>'filter'),
                'prototype' => true,
                'widget_add_btn' => array('label' => "Add", 'icon' => 'plus', 'attr' => ['class' => 'btn btn-default btn-sm', 'id'=>'add-field']),
                'show_legend' => false, // dont show another legend of subform
                'options' => array(
                    'label_render' => false,
                    'widget_remove_btn' => array('label' => false, 'icon' => 'trash-o', 'attr' => array('class' => 'btn btn-default btn-sm')),
                    'horizontal' => false,
                ),
                'disabled' => false,
                'by_reference' => false,
            ]
        );
        $dateTab
            ->add(
                'filterDateType',
                'choice',
                [
                    'choices' => Filter::getDateFilterTypeList(),
                    'label' => false,
                    'expanded' => true,
                    'multiple' => false,
                    'required' => true,
                    'data' => $filter->getFilterDateType(),
                ]
            );

        $dateTab
            ->add(
                'days',
                'choice',
                [
                    "choices" => Filter::$filterDayRange,
                    'label' => false,
                    'data' => $filter->getDays()
                ]
            );
        $dateTab
            ->add(
                'weeks',
                'choice',
                [
                    "choices" => Filter::$filterWeekRange,
                    'label' => false,
                    'data' => $filter->getWeeks()
                ]
            );

        $dateTab
            ->add(
                'rangeStart',
                'genemu_jquerydate',
                [
                    'label' => 'Start:',
                    'widget' => 'single_text',
                    'format' => 'MM/dd/yyyy',
                    'widget_type' => 'inline',
                    'attr' => ['readonly' => true],
                    'data'=> $filter->getRangeStart()
                ]
            );

        $dateTab
            ->add(
                'rangeEnd',
                'genemu_jquerydate',
                [
                    'label' => 'End:',
                    'widget' => 'single_text',
                    'format' => 'MM/dd/yyyy',
                    'attr' => ['readonly' => true],
                    'data'=> $filter->getRangeEnd()
                ]
            );


        $builder->add($infoTab)
            ->add($roomTab)
            ->add($authorTab)
            ->add($dateTab);
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Filter',
                'cascade_validation' => true,
            )
        );
    }

    /**
     * Returns the name of this type.
     *
     * @return string The name of this type
     */
    public function getName()
    {
        return "user_filter";
    }
}