<?php

namespace FS\CamBundle\Form\Type;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Persistence\ObjectManager;
use FS\BasicBundle\Utils\ArrayCollectionFilter;
use FS\CamBundle\Entity\AccessedByUsersAndGroupsInterface;
use FS\CamBundle\Template\TemplateTypes;
use FS\CamBundle\Entity\TemplateField;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroupRight;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use FS\CamBundle\Entity\Record;
use Symfony\Component\Security\Core\SecurityContext;

class RecordType extends AbstractType
{
    /** @var SecurityContext */
    private $security;

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $user = $this->getUser();
        $record = $builder->getData();
        $review = $record->getId() !== null;
        $observe = !$review;
        if ($review && $user->hasRight(UserGroupRight::GRANT_REVIEW__EDIT_INFORMATION) || $observe) {
            $infoTab = $builder->create(
                'info',
                'tab',
                array(
                    'label' => 'Information',
                    'inherit_data' => true,
                )
            );

            $infoTab
                ->add(
                    'info',
                    new RecordInfoType(),
                    [
                        'inherit_data' => true,
                        'label' => false,
                        'data' => $builder->getData()
                    ]
                );
            $builder->add($infoTab);
        }

        if ($review && $user->hasRight(UserGroupRight::GRANT_REVIEW__EDIT_SHARING) || $observe && $user->hasRight(UserGroupRight::GRANT_RECORDING__SHARING)) {
            $shareTab = $builder->create(
                'share',
                'tab',
                array(
                    'label' => 'Sharing',
                    'inherit_data' => true,
                )
            );

            $shareTab->add('accessedByUsersAndGroups', 'users_and_groups',
                [
                    'owner' => $builder->getData()->getCreatedBy(),
                    'label' => "Video Access:",
                ]
            );
            $builder->add($shareTab);

        }
        if ($review && $user->hasRight(UserGroupRight::GRANT_REVIEW__EDIT_RETENTION) || $observe && $user->hasRight(UserGroupRight::GRANT_RECORDING__RETENTION)) {
            $retentionTab = $builder->create(
                'retention',
                'tab',
                array(
                    'label' => 'Retention',
                    'inherit_data' => true,
                )
            );
            $retentionTab->add('retentionRule', 'retention', ['label' => false]);
            $builder->add($retentionTab);
        }
    }

    public function finishView(FormView $view, FormInterface $form, array $options)
    {

    }


    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Record',
            )
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'record';
    }

    public function __construct(SecurityContext $securityContext)
    {
        $this->security = $securityContext;
    }

    /**
     * @return User
     */
    private function getUser()
    {
        return $this->security->getToken()->getUser();
    }
}
