<?php

namespace FS\CamBundle\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use FS\CamBundle\Entity\Recur;
use Symfony\Component\Validator\Constraints\Count;

class RecurType extends AbstractType
{

    /** @var  Recur */
    private $recur;

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $weeklyConstraints = [];
        if (isset($_POST['schedule']['schedule']['recur'])) {
            $recur = $_POST['schedule']['schedule']['recur'];
            if ($recur['frequency'] == 'weekly' && (!array_key_exists('weekly', $recur) || count($recur['weekly']) === 0)) {
                $weeklyConstraints [] = new Count(['min' => 1, 'minMessage' => 'Please select day(s)']);
            }
        }

        $builder
            ->add(
                'frequency',
                'choice',
                [
                    'choices' => Recur::getFrequencyList(),
                    'label' => false,
                    'expanded' => true,
                    'multiple' => false,
                    'required' => true,
                    'data' => Recur::FREQ_DAILY,
                ]
            )
            ->add(
                'weekly',
                'choice',
                [
                    'choices' => Recur::getWeekList(),
                    'expanded' => true,
                    'multiple' => true,
                    'label' => false,
                    'required' => true,
                    'widget_type' => 'inline',
                    'data' => [],
                    'constraints' => $weeklyConstraints
                ]
            )
            ->add(
                'range',
                'choice',
                [
                    'choices' => Recur::getRangeList(),
                    'expanded' => true,
                    'required' => true,
                    'data' => Recur::RANGE_NO_END_DATE,
                ]
            )
            ->add(
                'endAt',
                'genemu_jquerydate',
                [
                    'label' => 'End By',
                    'widget' => 'single_text',
                    'format' => 'MM/dd/yyyy',
                    'attr' => ['readonly' => true],
                    'view_timezone' => 'UTC',
                    'model_timezone' => 'UTC'
                ]
            )
            ->add(
                'occurrences',
                'choice',
                [
                    'label' => 'End After',
                    'choices' => [2 => 2, 3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10],
                ]
            );

        $builder->addEventListener(
            FormEvents::PRE_SET_DATA,
            function (FormEvent $event) {
                $this->recur = $data = $event->getData();
            }
        );

    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Recur',
                'cascade_validation' => true,
            )
        );
    }

    public function finishView(FormView $view, FormInterface $form, array $options)
    {
        if ($this->recur != null) {
            $recur = $this->recur;

            switch ($recur->getFrequency()) {
                case $recur::FREQ_DAILY:
                    $view->children['frequency']->children[0]->vars['checked'] = true;
                    $view->children['frequency']->children[1]->vars['checked'] = false;
                    $view->children['frequency']->children[2]->vars['checked'] = false;
                    break;
                case $recur::FREQ_WEEKLY:
                    $view->children['frequency']->children[0]->vars['checked'] = false;
                    $view->children['frequency']->children[1]->vars['checked'] = true;
                    $view->children['frequency']->children[2]->vars['checked'] = false;

                    foreach ($view->children['weekly']->children as $weekDay) {
                        $weekDay->vars['checked'] = false;
                    }
                    foreach ($recur->getWeekly() as $item) {
                        $view->children['weekly']->children[$item]->vars['checked'] = true;
                    }
                    break;

                case $recur::FREQ_MONTHLY:
                    $view->children['frequency']->children[0]->vars['checked'] = false;
                    $view->children['frequency']->children[1]->vars['checked'] = false;
                    $view->children['frequency']->children[2]->vars['checked'] = true;
                    break;
            }
            //$view->children['range']

            switch ($recur->getRange()) {
                case $recur::RANGE_NO_END_DATE:
                    $view->children['range']->children[0]->vars['checked'] = true;
                    $view->children['range']->children[1]->vars['checked'] = false;
                    $view->children['range']->children[2]->vars['checked'] = false;
                    break;
                case $recur::RANGE_END_BY_OCCURENCES:
                    $view->children['range']->children[0]->vars['checked'] = false;
                    $view->children['range']->children[1]->vars['checked'] = true;
                    $view->children['range']->children[2]->vars['checked'] = false;
                    break;
                case $recur::RANGE_END_BY_DATE:
                    $view->children['range']->children[0]->vars['checked'] = false;
                    $view->children['range']->children[1]->vars['checked'] = false;
                    $view->children['range']->children[2]->vars['checked'] = true;
                    break;
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'recur';
    }
}
