<?php

namespace FS\CamBundle\Form\Type;

use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Form\Transformer\RetentionRuleTransformer;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class RetentionType extends AbstractType
{
    /** @var ObjectManager */
    private $om;

    public function __construct(ObjectManager $om)
    {
        $this->om = $om;
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add("type", "choice", [
                "choices" => [" "/*Days*/, " "/*Hours*/, " "/*Minutes*/, "Keep Until", "Keep Infinitely"],
                "expanded" => true
            ])
            ->add("keep_for_days", "number", [
                "label" => "Days",
                'attr' => [
                    'size' => 3
                ]
            ])
            ->add("keep_for_hours", "number", [
                "label" => "Hours",
                'attr' => [
                    'min' => 0,
                    'size' => 3
                ]
            ])
            ->add("keep_for_minutes", "number", [
                "label" => "Minutes",
                'attr' => [
                    'min' => 0,
                    'size' => 3
                ]
            ])
            ->add("keep_until", "genemu_jquerydate", [
                'label' => 'Keep Until',
                'widget' => 'single_text',
                'format' => 'MM/dd/yyyy',
                'attr' => ['class' => "keep-until-date"]
            ])
            ->add("id", "hidden")
            ->addModelTransformer(new RetentionRuleTransformer($this->om));

        if ($options['may_inherit'])
            $builder->add("inherit", "checkbox");
        else
            $builder->add("inherit", "hidden", ['attr' => ['value' => false]]);
    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(['may_inherit' => true]);
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'retention';
    }

}
