<?php

namespace FS\CamBundle\Form\Type;

use FS\CamBundle\Entity\Room;
use FS\UserBundle\Repository\UserRepository;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class RoomType extends AbstractType
{

    /**
     * @var ContainerManager
     */
    protected $containerManager;

    public function __construct(ContainerManager $containerManager)
    {
        $this->containerManager = $containerManager;
    }

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $room = $builder->getData();

        $builder
            ->add('name', null, ['label' => 'Room name']);
        $builder->add(
            'container',
            'entity',
            [
                'class' => 'FS\UserBundle\Entity\Container',
                'multiple' => false,
                'required' => true,
                'choices' => $this->containerManager->getContainersList(),
            ]
        );
        $builder->add('isCameraIO', 'checkbox', ['label' => 'Use I/O', 'required' => false]);

        $formModifier = function(FormInterface $form, $container = null, $isCameraIO = true) use ($room) {
            if ($isCameraIO) {
                $form->add(
                    'cameraIO',
                    'entity',
                    [
                        'class' => 'FS\CamBundle\Entity\Camera',
                        'property' => 'name',
                        'choices' => $room->getCameras(),
                        'empty_value' => false,
                        'multiple' => false,
                        'required' => false,
                        'label' => 'I/O Camera',
                    ]
                );

                $form->add(
                    'userIO',
                    'entity',
                    [
                        'class' => 'FS\UserBundle\Entity\User',
                        'query_builder' => function (UserRepository $repository) use ($room, $container) {
                            return $repository->findUsersForRoom($room, $container);
                        },
                        'empty_value' => false,
                        'multiple' => false,
                        'required' => false,
                        'label' => 'I/O User',
                    ]
                );
            }
        };

        $builder->addEventListener(
            FormEvents::PRE_SET_DATA,
            function (FormEvent $event) use ($formModifier) {
                $data = $event->getData();
                $formModifier($event->getForm(), $data->getContainer(), $data->getIsCameraIO());
            }
        );

        $builder->get('container')->addEventListener(
            FormEvents::POST_SUBMIT,
            function (FormEvent $event) use ($formModifier) {
                $data = $event->getForm()->getData();
                $formModifier($event->getForm()->getParent(), $data);
            }
        );
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Room'
            )
        );
    }

    /**
     * @return string
     */
    public function getName()
    {
        return 'room';
    }
}
