<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 06.03.2015
 * Time: 19:46
 */

namespace FS\CamBundle\Form\Type;

use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\Filter;
use FS\CamBundle\Entity\FilterTemplate;
use FS\CamBundle\Entity\Room;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use Lexik\Bundle\FormFilterBundle\Filter\FilterOperands;
use Lexik\Bundle\FormFilterBundle\Filter\Query\QueryInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Security\Core\SecurityContext;

class ScheduleFilterType extends AbstractType
{

    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var SecurityContext
     */
    private $securityContext;

    /**
     * @param ObjectManager $om
     * @param SecurityContext $securityContext
     */
    public function __construct(ObjectManager $om, SecurityContext $securityContext)
    {
        $this->om = $om;
        $this->securityContext = $securityContext;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {

        $builder->add('userFilter', 'filter_entity',
            [
                'class' => 'FSCamBundle:Filter',
                'property' => 'id',
                'apply_filter' => function (QueryInterface $filterQuery, $field, $values) {
                    $qb = $filterQuery->getQueryBuilder();
                    if (!empty($values['value'])) {
                        /** @var Filter $filter */
                        $filter = $values['value'];
                        $rooms = $filter->getFilterRooms();
                        if ($rooms->count() > 0) {
                            $roomsId = [];
                            /** @var Room $room */
                            foreach ($rooms as $room) {
                                $roomsId[] = $room->getId();
                            }
                            $qb->andWhere(
                                $filterQuery->getExpr()->in('room.id', ':fRooms')
                            )
                                ->setParameter(':fRooms', $roomsId);
                        }
                        $userGroups = $filter->getFilteredUserGroups();
                        if ($userGroups->count() > 0) {
                            $userGroupsId = [];
                            /** @var UserGroup $userGroup */
                            foreach ($userGroups as $userGroup) {
                                $userGroupsId[] = $userGroup->getId();
                            }
                            $qb->andWhere(
                                $filterQuery->getExpr()->in('user.userGroup', ':fUserGroup')
                            )
                                ->setParameter(':fUserGroup', $userGroupsId);
                        }

                        $users = $filter->getFilteredUsers();

                        if ($users->count() > 0) {
                            $usersId = [];
                            /** @var User $userGroup */
                            foreach ($users as $user) {
                                $usersId[] = $user->getId();
                            }
                            $qb->andWhere(
                                $filterQuery->getExpr()->in('user.id', ':fUsers')
                            )
                                ->setParameter(':fUsers', $usersId);
                        }

                        $info = $filter->getFilterInfoData();

                        if ($info->count() > 0) {
                            /** @var FilterTemplate $filterTemplate */
                            foreach ($info as $filterTemplate) {
                                if ($filterTemplate->getEnabled()) {
                                    $value = $filterQuery->getExpr()->literal('%' . $filterTemplate->getFilterText() . '%');
                                    switch ($filterTemplate->getFilterType()) {
                                        case FilterTemplate::FILTER_TEMPLATE_TYPE_RECORD:
                                            $qb->andWhere(
                                                $filterQuery->getExpr()->like('s.name', $value)
                                            );
                                            break;
                                        case FilterTemplate::FILTER_TEMPLATE_TYPE_MARKER:
                                        case FilterTemplate::FILTER_TEMPLATE_TYPE_EVENT:
                                            /*NOP*/
                                            break;
                                        case FilterTemplate::FILTER_TEMPLATE_TYPE_INFORMATION:
                                        case FilterTemplate::FILTER_TEMPLATE_TYPE_REGULAR:
                                        default:
                                            $qb->andWhere(
                                                $filterQuery->getExpr()->like('s.templateData', $value)
                                            );
                                            break;
                                    }
                                }

                            }
                        }

                        $from = new \DateTime("now");
                        $to = new \DateTime("now");
                        switch ($filter->getFilterDateType()) {
                            /**
                             * Interest thing we have dayStartAt its when schedule created
                             * and RealStartAt
                             * if schedule started once we add RealStartAt
                             */
                            case Filter::FILTER_DATE_TYPE_NONE:
                                /*NOP*/
                                break;
                            case Filter::FILTER_DATE_TYPE_DAYS:
                                /**
                                 * Filter working process for DAYS
                                 * if days == 0
                                 *
                                 * we search between start of this day and end of this day
                                 * if days > 0 we find from now to future
                                 * if days < 0 only now schedules
                                 *
                                 */
                                $qb->andWhere(
                                    $filterQuery->getExpr()->orx(
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNotNull("s.realStartAt"),
                                            $filterQuery->getExpr()->gte("s.realStartAt", ':fromD')
                                        ),
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNull("s.realStartAt"),
                                            $filterQuery->getExpr()->gte("s.startAt", ':fromD')
                                        )
                                    ));
                                $qb->andWhere(
                                    $filterQuery->getExpr()->orx(
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNotNull("s.realStartAt"),
                                            $filterQuery->getExpr()->lte("s.realStartAt", ':toD')
                                        ),
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNull("s.realStartAt"),
                                            $filterQuery->getExpr()->lte("s.startAt", ':toD')
                                        )
                                    ));
                                if ($filter->getDays() < 0 || $filter->getDays() == 0) {
                                    $from->modify('today');
                                    $to->modify('tomorrow')->modify('-1 second');
                                    $qb->setParameter('fromD', $from->format('Y-m-d H:i:s'));
                                    $qb->setParameter('toD', $to->format('Y-m-d H:i:s'));
                                }
                                if ($filter->getDays() > 0) {
                                    $from->modify('today'); // set for day Start
                                    $to->modify("+ " . $filter->getDays() . "day");
                                    $qb->setParameter('fromD', $from->format('Y-m-d H:i:s'));
                                    $qb->setParameter('toD', $to->format('Y-m-d H:i:s'));
                                }
                                break;
                            case Filter::FILTER_DATE_TYPE_WEEKS:
                                /**
                                 * We find in this week
                                 * if < 0 or 0 it`s this week schedules
                                 * if > 0 we modify weeks from this to next week;
                                 */
                                $qb->andWhere(
                                    $filterQuery->getExpr()->orx(
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNotNull("s.realStartAt"),
                                            $filterQuery->getExpr()->gte("s.realStartAt", ':fromD')
                                        ),
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNull("s.realStartAt"),
                                            $filterQuery->getExpr()->gte("s.startAt", ':fromD')
                                        )
                                    ));
                                $qb->andWhere(
                                    $filterQuery->getExpr()->orx(
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNotNull("s.realStartAt"),
                                            $filterQuery->getExpr()->lte("s.realStartAt", ':toD')
                                        ),
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNull("s.realStartAt"),
                                            $filterQuery->getExpr()->lte("s.startAt", ':toD')
                                        )
                                    ));
                                if ($filter->getWeeks() < 0 || $filter->getWeeks() == 0) {
                                    $from->modify("monday this week");
                                    $to->modify("monday this week")->modify("+ 1 week")->modify('-1 second');
                                    $qb->setParameter('fromD', $from->format('Y-m-d H:i:s'));
                                    $qb->setParameter('toD', $to->format('Y-m-d H:i:s'));

                                }
                                if ($filter->getWeeks() > 0) {
                                    $to->modify("+ " . $filter->getWeeks() . "week");
                                    $from->modify("monday this week");
                                    $qb->setParameter('fromD', $from->format('Y-m-d H:i:s'));
                                    $qb->setParameter('toD', $to->format('Y-m-d H:i:s'));
                                }
                                break;
                            case Filter::FILTER_DATE_TYPE_RANGE:
                                $qb->andWhere(
                                    $filterQuery->getExpr()->orx(
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNotNull("s.realStartAt"),
                                            $filterQuery->getExpr()->gte("s.realStartAt", ':fromD')
                                        ),
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNull("s.realStartAt"),
                                            $filterQuery->getExpr()->gte("s.startAt", ':fromD')
                                        )
                                    ));
                                $qb->andWhere(
                                    $filterQuery->getExpr()->orx(
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNotNull("s.realStartAt"),
                                            $filterQuery->getExpr()->lte("s.realStartAt", ':toD')
                                        ),
                                        $filterQuery->getExpr()->andx(
                                            $filterQuery->getExpr()->isNull("s.realStartAt"),
                                            $filterQuery->getExpr()->lte("s.startAt", ':toD')
                                        )
                                    ));
                                $qb->setParameter('fromD',
                                    $filter->getRangeStart()
                                        ->modify('today')
                                        ->format('Y-m-d H:i:s')
                                );
                                $qb->setParameter('toD',
                                    $filter->getRangeEnd()
                                        ->modify('tomorrow')
                                        ->modify('-1 second')
                                        ->format('Y-m-d H:i:s')
                                );
                                break;
                            default:
                                break;
                        }
                    }
                }]);
    }

    public
    function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'csrf_protection' => false,
                'validation_groups' => array('filtering') // avoid NotBlank() constraint-related message
            )
        );
    }

    /**
     * Returns the name of this type.
     *
     * @return string The name of this type
     */
    public
    function getName()
    {
        return "schedule_filter";
    }
}