<?php

namespace FS\CamBundle\Form\Type;

use FS\CamBundle\Entity\TemplateField;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class TemplateFieldType extends AbstractType
{
    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add(
                'enabled',
                'checkbox',
                [
                    'label' => 'On',
                    'widget_checkbox_label' => 'widget',
                    'horizontal' => false,
                    'error_bubbling' => true,
                ]
            )
            ->add(
                'title',
                null,
                [
                    'label' => 'Title',
                    'attr' => ['placeholder' => 'Title'],
                    'widget_type' => "inline",
                    'horizontal' => false,
                    'label_render' => false,
                    'error_bubbling' => true

                ]
            )
            ->add(
                'required',
                'checkbox',
                [
                    'label' => 'Req',
                    'widget_checkbox_label' => 'widget',
                    'widget_type' => "inline",
                    'horizontal' => false,
                    'error_bubbling' => true
                ]
            )
            ->add(
                'type',
                'choice',
                [
                    'choices' => [
                        TemplateField::TYPE_TEXT => 'Text',
                        TemplateField::TYPE_DROPDOWN => 'Drop Down',
                        TemplateField::TYPE_AUDIO => 'Audio',
                        TemplateField::TYPE_NOTE => 'Note'
                    ],
                    'horizontal' => false,
                    'label_render' => false,
                    'error_bubbling' => true
                ]
            )
            ->add(
                'defaultData',
                'text',
                [
                    'label' => 'Pre-Filed Data',
                    'label_render' => false,
                    'attr' => ['placeholder' => 'Pre-Filed Data'],
                    'horizontal' => false,
                    'error_bubbling' => true
                ]
            );
        $builder->addEventListener(
            FormEvents::PRE_SUBMIT,
            function (FormEvent $event) {
                $form = $event->getForm();
                /** @var TemplateField $data */
                $data = $event->getData();
                if($data['type'] == TemplateField::TYPE_DROPDOWN && $data['defaultData'] != null){
                    $arr = explode(',',$data['defaultData']);
                    $arr = array_unique($arr);
                    $arr = array_filter ($arr);
                    $data['defaultData'] = implode(",", $arr);
                    $event->setData($data);
                }
            }
        );
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\TemplateField',
                'render_fieldset' => false,
                'label_render' => false,
                'show_legend' => false,
                'attr' => ['class' => 'form-inline'],
                'error_bubbling' => false
            )
        );
    }

    /**
     * @return string
     */
    public function getName()
    {
        return 'camera';
    }
}
