<?php

namespace FS\CamBundle\Form\Type;

use FS\CamBundle\Entity\Template;
use FS\CamBundle\Template\TemplateTypes;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class TemplateType extends AbstractType
{

    /**
     * @var ContainerManager
     */
    private $containerManager;

    /**
     * @param ContainerManager $containerManager
     */
    public function __construct(ContainerManager $containerManager)
    {
        $this->containerManager = $containerManager;
    }

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add(
                'name',
                null,
                [
                    'label' => 'Template name',
                    'disabled' => !$options['editable'],
                ]
            )
            ->add(
                'type',
                'choice',
                [
                    'multiple' => false,
                    'expanded' => false,
                    'choices' => [
                        TemplateTypes::TYPE_INFO => 'Information',
                        TemplateTypes::TYPE_MARKER => 'Marker'
                    ],
                    'disabled' => !$options['editable'],
                ]
            )
            ->add('container',
                'entity',
                [
                    'class' => 'FS\UserBundle\Entity\Container',
                    'multiple' => false,
                    'required' => true,
                    'choices' => $this->containerManager->getContainersList(),
                    'disabled' => !$options['editable'],
                    'validation_groups' => 'container'
                ]
            )
            ->add(
                'hidden',
                null,
                [
                    'disabled' => !$options['editable'],
                ]
            )
            ->add(
                'fields',
                'collection',
                [
                    'type' => new TemplateFieldType(),
                    'allow_add' => $options['editable'],
                    'allow_delete' => $options['editable'], // should render default button, change text with widget_remove_btn
                    'prototype' => $options['editable'],
                    'widget_add_btn' => $options['editable'] ? array('label' => "Add Field", 'icon' => 'plus', 'attr' => ['class' => 'btn btn-default btn-sm', 'id' => 'add-collection-field']) : null,
                    'show_legend' => false, // dont show another legend of subform
                    'options' => array(
                        'label_render' => false,
                        'widget_remove_btn' => $options['editable'] ? array('label' => false, 'icon' => 'trash-o', 'attr' => array('class' => 'btn btn-default btn-sm')) : null,
                        'horizontal' => false,
                    ),
                    'by_reference' => false,
                    'disabled' => !$options['editable'],
                ]
            );
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Template',
                'cascade_validation' => true,
                'editable' => true,
            )
        );
    }

    /**
     * @return string
     */
    public function getName()
    {
        return 'fs_template';
    }
}
