<?php
/**
 * @author Andrey Yatsenco <andrey@fora-soft.com>
 */

namespace FS\CamBundle\Form\Type;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Persistence\ObjectManager;
use FS\BasicBundle\Utils\ArrayCollectionFilter;
use FS\CamBundle\Form\Transformer\UsersAndGroupsArrayTransformer;
use FS\CamBundle\Template\TemplateTypes;
use FS\CamBundle\Entity\TemplateField;
use FS\UserBundle\Entity\UserGroup;
use FS\UserBundle\Twig\Extension\UserDisplayNameExtension;
use FS\UserBundle\Twig\Extension\TruncateStringExtension;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Bridge\Doctrine\Form\DataTransformer\CollectionToArrayTransformer;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use FS\CamBundle\Entity\Record;
use FS\UserBundle\Entity\User;
use Symfony\Component\Security\Core\SecurityContext;


class UsersAndGroupsType extends AbstractType
{

    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var SecurityContext
     */
    private $securityContext;

    /**
     * @var ContainerManager
     */
    private $containerManager;

    /**
     * @var UserDisplayNameExtension
     */
    private $displayNameExtension;

    /**
     * @var TruncateStringExtension
     */
    private $truncateStringExtension;

    /**
     * @param ObjectManager $om
     * @param SecurityContext $securityContext
     * @param ContainerManager $containerManager
     * @param UserDisplayNameExtension $displayNameExtension
     * @param TruncateStringExtension $truncateStringExtension
     */
    public function __construct(
        ObjectManager $om,
        SecurityContext $securityContext,
        ContainerManager $containerManager,
        UserDisplayNameExtension $displayNameExtension,
        TruncateStringExtension $truncateStringExtension
    )
    {
        $this->om = $om;
        $this->securityContext = $securityContext;
        $this->containerManager = $containerManager;
        $this->displayNameExtension = $displayNameExtension;
        $this->truncateStringExtension = $truncateStringExtension;
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder->addModelTransformer(new UsersAndGroupsArrayTransformer($this->om), true);
    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $usersAndGroups = [];
        /** @var \FS\UserBundle\Entity\UserGroup[] $groups */
        $groups = $this->om->getRepository('FSUserBundle:UserGroup')->findAllExceptOneUserJoinedToUser();
        $groups = $this->containerManager->filterArray($groups);
        $users = new ArrayCollection();
        /** @var \FS\UserBundle\Entity\UserGroup $group */
        foreach ($groups as $group) {
            $usersAndGroups['group_' . $group->getId()] = $this->truncateStringExtension->truncateString($group->getName(), 40);
            foreach ($group->getUsers() as $user) {
                if ($user->getType() != User::TYPE_IO && !$user->isDeleted()) {
                    $usersAndGroups['user_' . $user->getId()] = ' --- ' . $this->displayNameExtension->displayName($user);
                    $users[] = $user;
                }

            }
        }

        $resolver->setDefaults(
            [
                'multiple' => true,
                'expanded' => true,
                'choices' => $usersAndGroups,
                'by_reference' => false,
                'owner' => false
            ]
        );

//        $resolver->setAllowedTypes(
//          [
//            'owner' => ['false', 'null', 'FS\UserBundle\Entity\User']
//          ]
//        );
    }

    public function finishView(FormView $view, FormInterface $form, array $options)
    {

        /** @var User $user */
        $user = $options['owner'];

        /** @var Record $record */
        $record = $view->parent->vars['data'];

        if ($record instanceof Record) {
            $room = $record->getRoom();
        } else {
            $room = null;
        }

        $fullAccess = false;
        if (false == $user || null === $user) {
            $user = $this->securityContext->getToken()->getUser();
        }
        if (in_array("ROLE_ADMIN", $user->getRoles()) || $user->getUserGroup()->getId() == 1) {
            $fullAccess = true;
        }

        $om = $this->om;
        $users = $om->getRepository("FSUserBundle:User")
            ->findAccessedToUser($user);
        //$users[] = $user;
        $groups = $om->getRepository("FSUserBundle:UserGroup")
            ->findAccessedToUser($user);
        //$groups[] = $user->getUserGroup();
        if ($record->getId() != null) {
            if ($record instanceof Record) {
                if ($user->hasAccessToRecord($record)) {
                    $users[] = $user;
                    $groups[] = $user->getUserGroup();
                }
            }else {
                $users[] = $user;
                $groups[] = $user->getUserGroup();
            }

        }

//        $transformer = new UsersAndGroupsArrayTransformer($this->om);
//        $disabled = array_merge($users, $groups);
//
//        $finalDes = [];
//        foreach ($disabled as $item) {
//            if ($item instanceof User) {
//                if ($item->getUserGroup()) {
//                    if ($item->getUserGroup()->getRooms()->contains($room)) {
//                        $finalDes[] = $item;
//                    }
//                    if($room == null){
//                        $finalDes[] = $item;
//                    }
//                }
//
//            } elseif ($item instanceof UserGroup) {
//                if ($item->getRooms()->contains($room)) {
//                    $finalDes[] = $item;
//                }
//                if($room == null){
//                    $finalDes[] = $item;
//                }
//            }
//        }


//        $disabled = $transformer->transform($finalDes);
//        foreach ($view->children as $item) {
//            if (in_array($item->vars['value'], $disabled, true) || $options['disabled'] == true) {
//                $item->vars['attr']['disabled'] = 'disabled';
//                $item->vars['checked'] = true;
//                if ($room != null) {
//
//                }
//
//            }
//        }
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'users_and_groups';
    }

    public function getParent()
    {
        return 'choice';
    }
}