<?php
/**
 * @author Andrey Yatsenco <andrey@fora-soft.com>
 */

namespace FS\CamBundle\RecordAccess;


use Doctrine\Common\Collections\ArrayCollection;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;

trait AccessedByUsersAndGroupsTrait
{
    /**
     * @var ArrayCollection
     *
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\User", cascade={"persist"})
     * @ORM\JoinTable(
     *      joinColumns={@ORM\JoinColumn(name="entity_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="accessed_by_user_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $accessedByUsers;

    /**
     * @var ArrayCollection
     *
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\UserGroup", cascade={"persist"})
     * @ORM\JoinTable(
     *      joinColumns={@ORM\JoinColumn(name="entity_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="accessed_by_user_group_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $accessedByGroups;

    private $accessedByUsersAndGroups;

    /**
     * @return mixed
     */
    public function getAccessedByGroups()
    {
        return $this->accessedByGroups;
    }

    /**
     * @param mixed $accessedByGroups
     * @return $this
     */
    public function setAccessedByGroups($accessedByGroups = null)
    {
        $this->accessedByGroups = $accessedByGroups;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getAccessedByUsers()
    {
        return $this->accessedByUsers;
    }

    /**
     * @param mixed $accessedByUsers
     * @return $this
     */
    public function setAccessedByUsers($accessedByUsers = null)
    {
        $this->accessedByUsers = $accessedByUsers;

        return $this;
    }

    public function getAccessedByUsersAndGroups()
    {
        $users = !empty($this->accessedByUsers) ? $this->accessedByUsers->toArray() : [];
        $groups = !empty($this->accessedByGroups) ? $this->accessedByGroups->toArray() : [];

        return new ArrayCollection(array_merge($users, $groups));
    }

    public function setAccessedByUsersAndGroups(ArrayCollection $usersAndGroups = null)
    {
        $accessedByUsers = new ArrayCollection();
        $accessedByGroups = new ArrayCollection();

        if (null === $usersAndGroups) {
            return;
        }

        foreach ($usersAndGroups->toArray() as $entity) {
            if ($entity instanceof User) {
                $accessedByUsers[] = $entity;

            } elseif ($entity instanceof UserGroup) {
                $accessedByGroups[] = $entity;
            }
        }
        $this->setAccessedByUsers($accessedByUsers);
        $this->setAccessedByGroups($accessedByGroups);
    }
} 