<?php
/**
 * @author Andrey Yatsenco <andrey@fora-soft.com>
 */

namespace FS\CamBundle\RecordAccess;


use Doctrine\Common\Collections\ArrayCollection;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;

trait AccessedUsersAndGroupsTrait
{

    /**
     * @var ArrayCollection
     *
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\User", cascade={"persist"})
     * @ORM\JoinTable(
     *      joinColumns={@ORM\JoinColumn(name="entity_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="accessed_user_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $accessedUsers;

    /**
     * @var ArrayCollection
     *
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\UserGroup", cascade={"persist"})
     * @ORM\JoinTable(
     *      joinColumns={@ORM\JoinColumn(name="entity_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="accessed_user_group_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $accessedGroups;

    /**
     * @return mixed
     */
    public function getAccessedUsers()
    {
        return $this->accessedUsers;
    }

    /**
     * @param mixed $accessedUsers
     * @return $this
     */
    public function setAccessedUsers($accessedUsers)
    {
        $this->accessedUsers = $accessedUsers;

        return $this;
    }

    /**
     * @return $this
     */
    public function addSelfToAccessedUsers()
    {
        if (!is_a($this, 'FS\UserBundle\Entity\User')) return false;
        if ($this->accessedUsers === null) $this->accessedUsers = [$this];
        elseif (!$this->accessedUsers->contains($this)) $this->accessedUsers->add($this);

        return $this;
    }

    /**
     * @return $this
     */
    public function addSelfToAccessedGroups()
    {
        if (!is_a($this, 'FS\UserBundle\Entity\UserGroup')) return false;
        if ($this->accessedGroups === null) $this->accessedGroups = [$this];
        elseif (!$this->accessedGroups->contains($this)) $this->accessedGroups->add($this);

        return $this;
    }

    /**
     * @return mixed
     */
    public function getAccessedGroups()
    {
        return $this->accessedGroups;
    }

    /**
     * @param mixed $accessedGroups
     * @return $this
     */
    public function setAccessedGroups($accessedGroups)
    {
        $this->accessedGroups = $accessedGroups;

        return $this;
    }

    public function getAccessedUsersAndGroups()
    {
        $users = !empty($this->accessedUsers) ? $this->accessedUsers->toArray() : [];
        $groups = !empty($this->accessedGroups) ? $this->accessedGroups->toArray() : [];

        return new ArrayCollection(array_merge($users, $groups));
    }

    public function setAccessedUsersAndGroups(ArrayCollection $usersAndGroups = null)
    {
        $this->accessedUsers = new ArrayCollection();
        $this->accessedGroups = new ArrayCollection();

        if (null === $usersAndGroups) {
            return;
        }

        foreach ($usersAndGroups->toArray() as $entity) {
            if ($entity instanceof User) {
                $this->accessedUsers[] = $entity;

            } elseif ($entity instanceof UserGroup) {
                $this->accessedGroups[] = $entity;
            }
        }
    }

} 