<?php

namespace FS\CamBundle\Repository;

use Doctrine\ORM\EntityRepository;
use FS\CamBundle\Entity\Camera;

class CameraRepository extends EntityRepository
{
    public function findOneByIdJoinedToPreset($id)
    {
        $query = $this->getEntityManager()
            ->createQuery('SELECT c, p FROM FSCamBundle:Camera c left JOIN c.presets p WHERE c.id = :id')
            ->setParameter('id', $id);
        try {
            return $query->getSingleResult();
        } catch (\Doctrine\ORM\NoResultException $e) {
            return null;
        }
    }

    public function countCameras()
    {
        $query = $this->getEntityManager()
            ->createQuery('SELECT COUNT(c) FROM FSCamBundle:Camera c');

        try {
            return $query->getSingleResult()[1];
        } catch (\Doctrine\ORM\NoResultException $e) {
            return 0;
        }
    }

    public function getWowzaCameras($wowzaName)
    {
        $qb = $this->createQueryBuilder('c');
        $qb->select('c,w,r,p')
            ->leftJoin('c.wowza', 'w')
            ->leftJoin('c.presets', 'p')
            ->leftJoin('c.rooms', 'r')
            ->where('w.name = :wName')
            ->orWhere('w.address = :wAddress')
            ->setParameter('wName', $wowzaName)
            ->setParameter('wAddress', $wowzaName);

        return $qb->getQuery()->getResult();
    }

    public function getCamerasWithoutRoom()
    {
        $qb = $this->createQueryBuilder('c');
        $qb->select('c,w,r,p')
            ->leftJoin('c.wowza', 'w')
            ->leftJoin('c.presets', 'p')
            ->leftJoin('c.rooms', 'r');
        $cameras = $qb->getQuery()->getResult();
        $result = [];
        /** @var Camera $camera */
        foreach ($cameras as $camera) {
            if ($camera->getRooms()->count() == 0) {
                $result[] = $camera;
            }
        }

        return $result;
    }
}
