<?php

namespace FS\CamBundle\Repository;

use Doctrine\ORM\EntityRepository;
use FS\CamBundle\Entity\Record;
use FS\UserBundle\Entity\User;

class RecordRepository extends EntityRepository
{

    public function getRecordBySequence($sequence)
    {
        $qb = $this->createQueryBuilder('record')
            ->select('record')
            ->where('record.recordShareSequence = :sequence')
            ->setParameter('sequence', $sequence);

        return $qb->getQuery()->getOneOrNullResult();
    }

    public function createQueryBuilderAll(User $user)
    {
        $qb = $this
            ->createQueryBuilder('record');
        $adminGroup = false;
        if($user->getUserGroup()){
            if($user->getUserGroup()->getId() == 1){
                $adminGroup = true;
            }
        }
        if (in_array("ROLE_ADMIN", $user->getRoles()) ||  $adminGroup) {
            $qb->select('record, room, user, markers')
                ->leftJoin('record.room', 'room')
                ->leftJoin('record.createdBy', 'creator')
                ->leftJoin('record.accessedByUsers', 'accessedByUsers')
                ->leftJoin('record.accessedByGroups', 'accessedByGroups')
                ->leftJoin('record.markers', 'markers')
                ->leftJoin('creator.userGroup', 'creatorGroup')
                ->leftJoin('room.cameras', 'camera')
                ->leftJoin('record.createdBy', 'user')
                ->where('record.status = :status')
                ->setParameter('status', Record::STATUS_COMPLETED)
                ->orderBy('record.startedAt', 'DESC')
                ->orderBy('record.id', 'DESC');
        } else {
            $qb
                ->select('record, room, camera, markers')
                ->leftJoin('record.room', 'room')
                ->leftJoin('record.createdBy', 'creator')
                ->leftJoin('record.markers', 'markers')
                ->leftJoin('record.accessedByUsers', 'accessedByUsers')
                ->leftJoin('record.accessedByGroups', 'accessedByGroups')
                ->leftJoin('creator.userGroup', 'creatorGroup')
                ->leftJoin('room.cameras', 'camera')
                ->leftJoin('record.createdBy', 'user')
                ->where('record.status = :status')
                ->andWhere(
                    $qb->expr()->orX(
                    //$qb->expr()->eq('creator', ':userId'),

                        $qb->expr()->in('creatorGroup', ':groupAccessedGroups'),
                        $qb->expr()->in('creatorGroup', ':userAccessedGroups'),
                        $qb->expr()->in('creator', ':groupAccessedUsers'),
                        $qb->expr()->in('creator', ':userAccessedUsers'),

                        //$qb->expr()->in('record.room', ':userGroupRooms'),//Update 277

                        $qb->expr()->eq('accessedByUsers.id', ':userId'),
                        $qb->expr()->eq('accessedByGroups.id', ':userGroupId')
                    )
                )
                ->andWhere($qb->expr()->orX(
                    $qb->expr()->in('record.room', ':userGroupRooms'), //Update 277
                    $qb->expr()->in('record.room', ':userSharedRooms'), //Update 277
                    $qb->expr()->isNull('record.room'), //Update 277
                    $qb->expr()->eq('accessedByUsers.id', ':userId'),
                    $qb->expr()->eq('accessedByGroups.id', ':userGroupId')
                )
                )
                ->setParameter('status', Record::STATUS_COMPLETED)
                ->setParameter('userId', $user->getId())
                ->setParameter('userGroupId', $user->getUserGroup()->getId())
                ->setParameter('groupAccessedGroups', $user->getUserGroup()->getAccessedGroups()->toArray())
                ->setParameter('userAccessedGroups', $user->getAccessedGroups()->toArray())
                ->setParameter('groupAccessedUsers', $user->getUserGroup()->getAccessedUsers()->toArray())
                ->setParameter('userAccessedUsers', $user->getAccessedUsers()->toArray())
                ->setParameter('userGroupRooms', $user->getUserGroup()->getRooms()->toArray())//Update 277
                ->setParameter('userSharedRooms', $user->getSharedRooms()->toArray())//Update 608
                ->orderBy('record.startedAt', 'DESC')
                ->orderBy('record.id', 'DESC');
        }
        return $qb;
    }

    /**
     * @param User $user
     * @return array
     * @deprecated
     */
    public function getAuthors(User $user)
    {
        $qb = $this
            ->createQueryBuilder('record');
        $adminGroup = false;
        if($user->getUserGroup()){
            if($user->getUserGroup()->getId() == 1){
                $adminGroup = true;
            }
        }
        if (in_array("ROLE_ADMIN", $user->getRoles()) || $adminGroup) {
            $qb->select('record, room, user, markers')
                ->leftJoin('record.room', 'room')
                ->leftJoin('record.createdBy', 'creator')
                ->leftJoin('record.markers', 'markers')
                ->leftJoin('record.accessedByUsers', 'accessedByUsers')
                ->leftJoin('record.accessedByGroups', 'accessedByGroups')
                ->leftJoin('creator.userGroup', 'creatorGroup')
                ->leftJoin('room.cameras', 'camera')
                ->leftJoin('record.createdBy', 'user')
                ->where('record.status = :status')
                ->setParameter('status', Record::STATUS_COMPLETED)
                ->orderBy('record.startedAt', 'DESC');
        } else {
            $qb
                ->select('record, room, camera, markers')
                ->leftJoin('record.room', 'room')
                ->leftJoin('record.markers', 'markers')
                ->leftJoin('record.createdBy', 'creator')
                ->leftJoin('record.accessedByUsers', 'accessedByUsers')
                ->leftJoin('record.accessedByGroups', 'accessedByGroups')
                ->leftJoin('creator.userGroup', 'creatorGroup')
                ->leftJoin('room.cameras', 'camera')
                ->leftJoin('record.createdBy', 'user')
                ->where('record.status = :status')
                ->andWhere(
                    $qb->expr()->orX(
                        $qb->expr()->in('creatorGroup', ':groupAccessedGroups'),
                        $qb->expr()->in('creatorGroup', ':userAccessedGroups'),
                        $qb->expr()->in('creator', ':groupAccessedUsers'),
                        $qb->expr()->in('creator', ':userAccessedUsers'),
                        $qb->expr()->eq('accessedByUsers.id', ':userId'),
                        $qb->expr()->eq('accessedByGroups.id', ':userGroupId')
                    )
                )
                ->andWhere($qb->expr()->orX(
                    $qb->expr()->in('record.room', ':userGroupRooms'),
                    $qb->expr()->eq('accessedByUsers.id', ':userId'),
                    $qb->expr()->eq('accessedByGroups.id', ':userGroupId')
                )
                )
                ->setParameter('status', Record::STATUS_COMPLETED)
                ->setParameter('userId', $user->getId())
                ->setParameter('userGroupId', $user->getUserGroup()->getId())
                ->setParameter('groupAccessedGroups', $user->getUserGroup()->getAccessedGroups()->toArray())
                ->setParameter('userAccessedGroups', $user->getAccessedGroups()->toArray())
                ->setParameter('groupAccessedUsers', $user->getUserGroup()->getAccessedUsers()->toArray())
                ->setParameter('userAccessedUsers', $user->getAccessedUsers()->toArray())
                ->setParameter('userGroupRooms', $user->getUserGroup()->getRooms()->toArray())
                ->orderBy('record.startedAt', 'DESC');
        }
        $resultRaw = $qb->getQuery()->getResult();
        $result = [];
        /** @var Record $record */
        foreach ($resultRaw as $record) {
            if ($record->getCreatedBy() != null) {
                $result[$record->getCreatedBy()->getId()] = $record->getCreatedBy()->getUsername();
            }

        }
        return $result;
    }

}
