'use strict';


var app = angular.module('camLive', ['ngRoute', 'ngSanitize', 'ngDragDrop', 'FBAngular']);

app.config(function ($routeProvider) {
    $routeProvider
        .when('/', {
            controller: 'RoomListCtrl',
            reloadOnSearch: true
        });
});

app.config(['$interpolateProvider', function ($interpolateProvider) {
    $interpolateProvider.startSymbol('[[');
    $interpolateProvider.endSymbol(']]');
}]);

app.factory('recordFactory', function ($http) {
    return {
        getCurrentRoomRecord: function (roomId) {
            return $http.get('/api/get-record-id-by-room-id/' + roomId);
        }
    };
});

app.controller('RoomListCtrl', function ($scope, $location, $sce, $timeout, $route, $routeParams, $http, $compile, Fullscreen, markerFactory, recordFactory) {
        $scope.$location = $location;
        $scope.routeParams = $routeParams;

        $scope.userId = settings.userId;


        $scope.rooms = [];

        var getRooms = $http.get('/api/rooms').success(function (data) {
            $scope.rooms = $.map(data, function(value, index) {
                return [value];
            });

            for (var i = 0; i < $scope.rooms.length; i++) {
                $scope.rooms[i].expanded = false;
            }
            // update recording time
            $scope.recordingTimeCounter();
        });

        $scope.isRoomOpened = function (room) {
            var isCameraSelected = false;
            if (room.cameras !== undefined) {
                room.cameras.forEach(function(camera) {
                    if (camera.id == $scope.matrix.selectedCameraId
                        && $scope.routeParams.cameraRoom == camera.room) isCameraSelected = true;
                });
            }
            return (room.id == $scope.matrix.selectedRoomId) || isCameraSelected;
        };

        $scope.markerFocus = function () {
            $('#full-screen-container')
                .parent()
                .append('<div id="overflow-marker" ' +
                'style="' +
                'background: #000;' +
                'position: fixed;' +
                'left: 0;' +
                'right: 0;' +
                'top: 0;' +
                'bottom: 0;' +
                'opacity: 0.7;' +
                'z-index: 100;' +
                '"></div>'
            );
            $('#full-screen-container').attr('style',
                'z-index: 102;' +
                'width:100%;' +
                'position: absolute;' +
                '').removeClass('full-screen-size');
            $('.wowza-logo').hide();
        };
        $scope.markersUnFocus = function () {
            $('#overflow-marker').remove();
            $('#full-screen-container').addClass('full-screen-size');
            $('.wowza-logo').show();
        };
        $scope.getAddMarkerForm = function (href) {
            $scope.markerTime = $scope.matrix.selectedRoom.recordingTime;
            $http.get(href).success(function (data) {
                $scope.markerFocus();
                $('#marker-form-section').html('');
                $compile($('#marker-form-section').append(data).attr(
                    'style',
                    'height:' + (+$('#marker-form-section')[0].scrollHeight + 20) + 'px;' +
                    'top:0;' +
                    'bottom:0'))($scope);
                $("#marker-form").find("*:input[type!=hidden]:first").focus()
            });

        };
        $scope.closeModal = function () {
            $scope.markersUnFocus();
            $('#marker-form-section').html('').removeAttr('style');
        };

        $scope.markerSubmit = function () {
            var formData = new FormData();
            var params = $('#marker-form').serializeArray();
            var data = {};
            $.each(params, function (i, val) {
                data[val.name] = val.value;
                formData.append(val.name, val.value);
            });
            $.ajax({
                type: "POST",
                url: $scope.submitMarkerLink(),
                data: formData,
                cache: false,
                contentType: false,
                processData: false,
                success: function (data) {
                    var status = data[0];
                    switch (status) {
                        case 'error':
                            humane.log('Error');
                            break;
                        case 'processed':
                            humane.log('Marker added.');
                    }
                    $('#marker-form-section').html('').removeAttr('style');
                    $scope.markersUnFocus();
                },
                error: function (xhr) {
                    humane.log('Error');
                    $('#marker-form-section').html('').removeAttr('style');
                    $scope.markersUnFocus();
                }
            });
        };


        /* Recording --------------------- */
        $scope.stopRecording = function () {
            $scope.closeModal();
            var recordToStop = $scope.matrix.selectedRoomId;

            var url = Routing.generate('record_stop', {roomId: $scope.matrix.selectedRoomId});
            $.get(url, function (data) {
                $scope.logger('log_record_stop', data.recordId);
                humane.log('Recording stopped.');
                $scope.$apply(function () {
                    if (recordToStop == $scope.matrix.selectedRoomId) {
                        $scope.matrix.selectedRoom.recording = false;
                    }
                });
            });
        };

        // Log function
        $scope.logger = function(type, id) {
            var url = Routing.generate(type, {id: id});
            $.get(url, function(data) {});
        };

        $scope.matrix = {};
        $scope.matrix.grid = [null];

        // Function to be sure that grid at least could show all available cameras
        $scope.matrix.minCeilsCount = function () {
            return $scope.matrix.grid.filter(function(val) {
                return val !== null;
            }).length;
        };

        $scope.maximized = false;
        $scope.clearResize = function () {
            var $cameras = $('#matrix-grid').find('.camera-component');
            $scope.maximized = false;
            $cameras.each(function (ind, el) {
                $(el).attr('style', 'top:0;left:0;position:relative');
                var cssId = 'camera-component-' + ind;
                var swf = document.getElementById(cssId);
                if (swf && swf.isResizable && swf.minimize) {
                    if ($scope.matrix.grid.length == 4) {
                        swf.isResizable('true');
                    } else {
                        swf.isResizable('false');
                    }
                    swf.minimize()
                }
            });
        };

        $scope.matrix.changeMatrixSize = function (size) {
            if ($scope.matrix.minCeilsCount() > size) {
                $location.search('gridSize', $scope.matrix.grid.length);
                return;
            }
            if ($scope.roomDragEnable == true) {
                $scope.changeRoomLocation()
            }
            $scope.clearResize();

            for (var i = 0; i < $scope.matrix.grid.length; i++) {
                var cssId = 'camera-component-' + i;
                var swf = document.getElementById(cssId);
                if (swf && swf.isResizable && swf.minimize) {
                    if (((size > 2 && size <= 4)) || ((size == 1) && ($scope.matrix.minCeilsCount() > size))) {
                        swf.isResizable('true');
                    } else {
                        swf.isResizable('false');
                    }
                    swf.minimize()
                }
            }

            if (1 >= size) {
                $scope.roomDragEnable = false;
                size = 1;
            } else if (size > 1 && size <=2) {
                $scope.matrix.selectedCameraId = null;
                size = 2;
            } else if (size > 2 && size <= 4) {
                $scope.matrix.selectedCameraId = null;
                size = 4;
            } else if (size > 4) {
                $scope.matrix.selectedCameraId = null;
                size = 9;
            }
            if (size != $location.search()['gridSize']) {
                $location.search('gridSize', size);
            }
            if (size < $scope.matrix.minCeilsCount()) {
                $timeout(function () {
                    $scope.matrix.changeMatrixSize($scope.matrix.minCeilsCount());
                });
                return;
            }

            var sizeDiff = size - $scope.matrix.grid.length;
            if (sizeDiff > 0) {
                for (var i = 0; i < sizeDiff; i++) {
                    $scope.matrix.grid.push(null);
                }
            }
            else if (sizeDiff < 0) {
                for (var i = 0; i < $scope.matrix.grid.length; i++) {
                    if ($scope.matrix.grid[i] == null) {
                        $scope.matrix.grid.splice(i, 1);
                        i--;
                    }
                }
                sizeDiff = size - $scope.matrix.grid.length;
                for (var i = 0; i < sizeDiff; i++) {
                    $scope.matrix.grid.push(null);
                }
                for (var i = 0; i < size; ++i) {
                    if ($scope.matrix.grid[i] != null) {
                        $scope.matrix.setCeilContent(i, $scope.matrix.grid[i]);
                    } else {
                        var cssId = 'camera-component-' + i;
                        $('#' + cssId).replaceWith('<div id="' + cssId + '"></div>');
                    }
                }
            }
        };

        /////////////-----------------////////////

        $scope.matrix.clearGrid = function () {
            for (var i = 0; i < $scope.matrix.grid.length; i++) {
                $scope.matrix.grid[i] = null;
                $scope.matrix.setCeilContent(i, null);
            }
        };

        $scope.matrix.addCamera = function (cameraId, index) {
            $timeout(function () {
                $scope.matrix.grid[index] = cameraId;
                $scope.matrix.setCeilContent(index, cameraId);
            });
        };

        // this function called from flash
        // mute all cameras except camera index in grid
        window.muteAll = function (index) {
            index = parseInt(index[0]);
            for (var i = 0; i < $scope.matrix.grid.length; i++) {
                if (i !== index) {
                    var cssId = 'camera-component-' + i;
                    var swf = document.getElementById(cssId);
                    if (swf && swf.mute) {
                        swf.mute();
                    }
                }
            }

        };
        window.muteAllCameras = function () {
            for (var i = 0; i < $scope.matrix.grid.length; i++) {
                var cssId = 'camera-component-' + i;
                var swf = document.getElementById(cssId);
                if (swf && swf.mute) {
                    swf.mute();
                }
            }
        };

        //this function will be called from flash
        $scope.resizedCamera = false;
        window.maximizeView = function (index) {
            $scope.resizedCamera = true;
            index = parseInt(index[0]);
            console.log('maxi, index = ', index);
            $scope.maximized = true;
            //some magic

            var $cameras = $('#matrix-grid').find('.camera-component');
            if ($scope.isFullscreen) {
                var height = $('#matrix-grid').outerHeight();
                var width = $('#matrix-grid').outerWidth();

                var bottom = 51;
                $cameras.each(function (elem, item) {
                    if (elem === index) {
                        $(item).attr('style', 'width:' + width / 2 + 'px;height:' + height + 'px;position:absolute');
                    } else {
                        $(item).attr('style',
                            'width:' + width / 2 + ';height:' + height / 3 + 'px;position:absolute;right:0;top:' + bottom + 'px;margin-right:10px');
                        bottom += height / 3;
                    }
                });

            } else {
                $('#matrix-grid').attr('style', 'height:400px;');
                //add styles
                var bottom = 15;
                $cameras.each(function (elem, item) {
                    if (elem === index) {
                        $(item).attr('style', 'width:500px;height:387px;position:absolute');

                    } else {
                        $(item).attr('style',
                            'width:300px;height:122px;position:absolute;right:0;top:' + bottom + 'px;margin-right:10px');
                        bottom += 132;
                    }
                });
            }

            for (var i = 0; i < $scope.matrix.grid.length; i++) {
                console.log('tryResize', i);
                if (i !== index) {
                    var cssId = 'camera-component-' + i;
                    var swf = document.getElementById(cssId);
                    if (swf && swf.minimize) {
                        swf.minimize();
                    }
                }
            }
        };

        window.minimizeView = function (index) {
            $scope.resizedCamera = false;
            index = parseInt(index[0]);
            console.log('mini, index = ', index);
            $scope.maximized = false;
            var $cameras = $('#matrix-grid').find('.camera-component');
            if ($scope.isFullscreen) {
                var matrixGrid = $('#matrix-grid');
                var size = Math.sqrt($scope.matrix.grid.length);

                $cameras.removeAttr("style")
                    .outerWidth((matrixGrid.width() - 11 * (size - 1)) / size)
                    .outerHeight((matrixGrid.height() - 11 * (size - 1)) / size)
            } else {
                $('#matrix-grid').removeAttr("style");
                $cameras.each(function (elem, item) {
                    $(item).removeAttr("style");
                });
            }
        };

        $scope.matrix.setCeilContent = function (index, cameraId) {
            var cssId = 'camera-component-' + index;
            if (null === cameraId) {
                $('#' + cssId).replaceWith('<div id="' + cssId + '"></div>');
            } else {
                // For version detection, set to min. required Flash Player version, or 0 (or 0.0.0), for no version detection.
                var swfVersionStr = "11.1.0";
                // To use express install, set to playerProductInstall.swf, otherwise the empty string.
                var xiSwfUrlStr = "/flash/playerProductInstall.swf";
                var isResize = ($scope.matrix.gridSize == 4) && (!$scope.roomDragEnable);
                var camera = $scope.getCameraById(cameraId);
                var wowzaURL = '';
                if (camera.wowza_ssl) {
                    wowzaURL = 'rtmps://' + camera.wowza_address + ':' + camera.wowza_port;
                } else {
                    wowzaURL = 'rtmp://' + camera.wowza_address;
                }
                var blocked = 0;
                if ($scope.matrix.selectedRoom != null) {
                    if ($scope.matrix.selectedRoom.recording && !$scope.matrix.selectedRoom.accessToView) {
                        blocked = 1
                    }
                } else {
                    var room = $scope.getRoomById(
                        camera.room
                    );
                    if (room.recording && !room.accessToView) {
                        blocked = 1;
                    }
                }

                var flashvars = {
                    cameraId: cameraId,
                    userId: $scope.userId,
                    cameraName: camera.name,
                    isDevice: (camera.type == 'camera') ? 0 : 1,
                    muted: index !== 0, // all cameras should be muted by default except the first
                    index: index,
                    wowzaURL: wowzaURL,
                    access__talkback: settings.access__talkback,
                    access__ptz_control: settings.access__ptz_control,
                    access__edit_presets: settings.access__edit_presets,
                    isResizeAvailable: isResize,
                    blocked: blocked
                };
                var params = {};
                params.quality = "high";
                params.bgcolor = "#ffffff";
                params.allowscriptaccess = "sameDomain";
                params.allowfullscreen = "true";
                params.wmode = "opaque";
                var attributes = {};
                attributes.id = cssId;
                attributes.name = "V3CamerasLive";
                attributes.align = "middle";
                swfobject.embedSWF(
                    "/flash/V3CamerasLive.swf", cssId,
                    '100%', '100%',
                    swfVersionStr, xiSwfUrlStr,
                    flashvars, params, attributes);
                // JavaScript enabled so display the flashContent div in case it is not replaced with a swf object.
                swfobject.createCSS("#flashContent", "display:block;text-align:left;");
            }
        };

        $scope.selectRoom = function (room) {
            $scope.logger('log_room_view', room.id);
            if (typeof room.cameras == 'undefined' ||
                room.cameras == null ||
                room.cameras.length == 0
            ) {
                humane.log("Room is empty");
                return;
            }
            $scope.selectedRoom = room.id;
            if (room.id == $scope.matrix.selectedRoomId) {
                return;
            }
            window.muteAllCameras();
            $scope.matrix.dirty = false;
            $location.search({'room': room.id, gridSize: room.cameras.length});
        };
        $scope.selectCamera = function (camera) {
            $scope.logger('log_camera_view', camera.id);
            if (camera.id == $scope.matrix.selectedCameraId && camera.room == $scope.routeParams.cameraRoom) {
                return;
            }
            $location.search({'camera': camera.id, gridSize: 1, 'cameraRoom':camera.room});
        };

        $scope.dragged = false;
        $scope.cameraOnClick = function (camera) {
            if (!$scope.dragged) {
                $scope.selectCamera(camera);
            }
            $scope.dragged = false;
        };
        $scope.startDragging = function () {
            $scope.dragItemRoom = false;
            $scope.dragged = true;
        };
        $scope.stopDragging = function () {
            $scope.dragged = false;
        };
        $scope.matrix.dirty = false;

        $scope.cameraDragEnabled = function (camera) {
            if ($scope.getRoomsOnAir($scope.matrix.selectedRoomId)) {
                return false;
            }
            return camera.id != $scope.matrix.selectedCameraId;

        };

        $scope.roomDragEnable = false;

        $scope.changeRoomLocation = function () {
            if ($scope.getRoomsOnAir($scope.matrix.selectedRoomId)) {
                return false;
            }
            if ($scope.matrix.grid.length == 1) {
                return false;
            }
            $scope.clearResize();

            for (var i = 0; i < $scope.matrix.grid.length; i++) {
                var cssId = 'camera-component-' + i;
                var swf = document.getElementById(cssId);
                if (swf && swf.isResizable && swf.minimize) {
                    if ($scope.roomDragEnable && $scope.matrix.grid.length == 4)
                        swf.isResizable('true');
                    else
                        swf.isResizable('false');
                    swf.minimize()
                }
            }
            $scope.roomDragEnable = !$scope.roomDragEnable;
        };

        ///DragRoomWithCameras$
        $scope.selectedRoom = false;
        $scope.videoRoomDraged = false;
        $scope.startDraggingVideoRoom = function () {
            $scope.videoRoomDraged = true;
        };

        $scope.videoRoomDragEnable = function (id) {
            if ($scope.getRoomsOnAir($scope.matrix.selectedRoomId)) {
                return false;
            }
            return $scope.matrix.selectedRoomId != id;

        };

        ///Drad view
        $scope.roomDragged = false;
        $scope.dragItemRoom = false;

        //это пушка, первая комната не хочет нивкакую ставить на себя стили
        //пришлось решить таким путём
        $scope.startDraggingRoom = function (event, ui, item) {
            $scope.dragItemRoom = item;
            $scope.roomDragged = true;
            if (item === 0) {
                var cssId = '#camera-component-0';
                if ($scope.resizedCamera) {
                    $(cssId).parent().css('z-index', '99999')
                } else {
                    $(cssId).parent().attr('style', 'position:relative;z-index:99999;')
                }

            }
        };
        $scope.stopDraggingRoom = function (event, ui, item) {
            $scope.roomDragged = false;
            if (item === 0) {
                var cssId = '#camera-component-0';
                if ($scope.resizedCamera) {
                    $(cssId).parent().css('z-index', '99999')
                } else {
                    $(cssId).parent().attr('style', 'left:0px;top:0px;position:relative;z-index:auto;')
                }
            }
        };

        $scope.roomDragEnabled = function (index) {
            if ($scope.getRoomsOnAir($scope.matrix.selectedRoomId)) {
                return false;
            }
            if ($scope.matrix.grid.length == 1) {
                return false;
            }
            return $scope.roomDragEnable;
        };

        $scope.dropRoom = function (cameraId, $index, room) {
            var camera = $scope.matrix.grid[$index];
            $scope.matrix.grid[$index] = null;
            $scope.matrix.addCamera(camera, room);
            $scope.matrix.addCamera(cameraId, $index);
        };

        $scope.dropCamera = function (cameraId, $index) {
            $scope.logger('log_camera_view', cameraId);

            $scope.matrix.addCamera(cameraId, $index);
            $scope.matrix.dirty = true;
        };

        $scope.dropRoomVideo = function (roomId) {
            var room = $.grep($scope.rooms, function (e) {
                return e.id == roomId;
            });

            if (room[0].recording == true && room[0].accessToView == 0) {
                humane.log("You do not have access to the current session");
                return false;
            }

            $scope.logger('log_room_view', room[0].id);

            var cameraIndex = 0;
            $scope.matrix.dirty = true;
            $scope.matrix.grid.forEach(function (item, index) {
                if (item === null) {
                    $scope.matrix.addCamera(room[0].cameras[cameraIndex++].id, index);
                }
            })
        };

        $scope.drop = function (event, ui, cameraId, $index) {
            if ($scope.videoRoomDraged) {
                $scope.videoRoomDraged = false;
                $scope.dropRoomVideo(cameraId);
            }
            else if ($scope.dragItemRoom !== false) {
                var roomId = $scope.dragItemRoom;
                $scope.dragItemRoom = false;
                $scope.dropRoom(cameraId, $index, roomId);
            } else {
                $scope.dropCamera(cameraId, $index)
            }
        };

        $scope.isCeilEmpty = function (index) {
            return $scope.matrix.grid[index] == null;
        };

        $scope.isCameraNotOnRecord = function (index) {
            var cell = $scope.matrix.grid[index];
            if(cell != null){
                var camera = $scope.getCameraById(cell);
                var room = $scope.getRoomById(camera.room);
                if(room.recording == true){
                    if(typeof room.camerasInRecord == "undefined" ){
                        return false;
                    }
                    if(!(cell in room.camerasInRecord)){
                        return true;
                    }
                }
            }
            return false;
        };

        $scope.getRoomsOnAir = function (id) {
            var result = $.grep($scope.rooms, function (e) {
                return e.id == id;
            });
            if (result.length == 0) {
                return false;
            }
            return result[0].recording;
        };
        $scope.$on('$routeChangeStart', function (event, next, current) {
            var newPage = false;
            if (current == undefined) {
                newPage = true;
            }

            if (undefined !== next.params.room) {
                if (newPage || next.params.room != current.params.room) {
                    getRooms.then(function () {
                        $scope.matrix.selectedRoomId = parseInt(next.params.room);
                    });
                }
            }

            if (undefined !== next.params.camera) {
                if (newPage || next.params.camera != current.params.camera) {
                    $scope.matrix.selectedCameraId = parseInt(next.params.camera);
                }
            }

            if (undefined !== next.params.gridSize) {
                if (newPage || next.params.gridSize != current.params.gridSize) {
                    $scope.matrix.gridSize = parseInt(next.params.gridSize);
                }
            }
        });

        // Selected room
        $scope.matrix.selectedRoomId = null;
        $scope.matrix.selectedRoom = null;
        $scope.$watch('matrix.selectedRoomId', function (newRoomId, oldRoomId) {
            if (newRoomId == oldRoomId) {
                return;
            }
            if (newRoomId != null) {
                $scope.matrix.selectedCameraId = null;
            }

            if (undefined == $scope.rooms) {
                return;
            }
            $scope.matrix.selectedRoom = $scope.getRoomById(newRoomId);
            $scope.matrix.clearGrid();
            if ($scope.matrix.selectedRoom == null) {
                return;
            }

            // add cameras
            if ($scope.matrix.selectedRoom.cameras.length > $scope.matrix.grid.length) {
                $location.search('gridSize', $scope.matrix.selectedRoom.cameras.length);
            }
            $scope.matrix.selectedRoom.cameras.forEach(function (camera, index) {
                $scope.matrix.addCamera(camera.id, index);
            });
        });

        // Selected camera
        $scope.matrix.selectedCameraId = null;
        $scope.$watch('matrix.selectedCameraId', function (newCameraId, oldCameraId) {
            if (newCameraId == oldCameraId || null == newCameraId) {
                return;
            }
            $scope.matrix.selectedRoomId = null;

            $scope.matrix.clearGrid();
            $location.search('gridSize', 1);

            $scope.matrix.addCamera(newCameraId, 0);
        });

        // Grid size
        $scope.matrix.gridSize = 1;
        $scope.$watch('matrix.gridSize', function (newgridSize, oldgridSize) {
            if (newgridSize == oldgridSize || null == newgridSize) {
                return;
            }
            $scope.matrix.changeMatrixSize(newgridSize);
        });

        $scope.getRoomById = function (roomId) {
            if (null == $scope.rooms) {
                return null;
            }
            for (var i = 0; i < $scope.rooms.length; i++) {
                if ($scope.rooms[i].id == roomId) {
                    return $scope.rooms[i];
                }
            }
            return null;
        };

        $scope.getCameraById = function (cameraId) {
            if (null == $scope.rooms) {
                return null;
            }
            var cameras = [];
            var camera = null;
            for (var i = 0; i < $scope.rooms.length; i++) {
                if ($scope.rooms[i].cameras) {
                    cameras = $.merge(cameras, $scope.rooms[i].cameras);
                }
            }

            if (cameras.length) {
                for (var j = 0; j < cameras.length; j++) {
                    if (cameras[j].id == cameraId) {
                        camera = cameras[j];
                    }
                }
            }

            return camera;
        };

        $scope.pageTitle = function () {
            if ($scope.matrix.selectedRoomId != null) {
                var room = $scope.getRoomById($scope.matrix.selectedRoomId);
                return room.name;
            } else if ($scope.matrix.selectedCameraId != null) {
                var camera = $scope.getCameraById($scope.matrix.selectedCameraId);

                if (camera != null) {
                    return camera.name;
                } else {
                    return '';
                }

            } else if ($scope.matrix.gridSize == 1) {
                return '1x1 Matrix';
            } else if ($scope.matrix.gridSize == 2) {
                return '2x1 Matrix';
            } else if ($scope.matrix.gridSize == 4) {
                return '2x2 Matrix';
            } else if ($scope.matrix.gridSize == 9) {
                return '3x3 Matrix';
            }
            return 'Matrix';

        };

        Fullscreen.$on('FBFullscreen.change', function (object, isEn) {
            if (!isEn) {
                $scope.clearResize();
                $("#full-screen-container").removeAttr('style').attr('style', "position:absolute");
            } else {
                $("#full-screen-container").removeAttr('style').attr('style', "height:100%");
            }

        });

        $(window).keydown(function (eventObject) {
            if (eventObject.which == 122) {
                if (document.msFullscreenEnabled == true) {
                    return false;
                }
                $scope.goFullscreen();
                return false;
            }
        });

        $scope.goFullscreen = function () {
            var container = document.getElementById('full-screen-container');
            if (Fullscreen.isEnabled()) {
                $scope.clearResize();
                Fullscreen.cancel();
            } else {
                if ($scope.roomDragEnable == true) {
                    $scope.changeRoomLocation()
                }
                Fullscreen.enable(document.getElementById('full-screen-container'));
                $scope.matrix.scaleMatrixSize();
            }
        };

        $scope.fullscreenEnabled = function () {
            if (document.msFullscreenEnabled == true) {
                return false;
            }
            return document.fullscreenEnabled || document.mozFullScreenEnabled || document.webkitFullscreenEnabled;
        };

        $scope.isFullscreen = false;

        $scope.matrix.scaleMatrixSize = function () {
            var container = $('#full-screen-container');
            var content = $('#full-screen-content');
            var matrixGrid = $('#matrix-grid');
            var cameraComponents = $('.camera-component');

            if ($scope.maximized == true) {
                $scope.clearResize();
            }

            var rePos = function () {
                if (!Fullscreen.isEnabled()) {
                    container.removeClass('fullscreen');
                    $scope.isFullscreen = false;
                    content.css({width: "", height: ""});
                    matrixGrid.css({width: "", height: ""});
                    cameraComponents.css({width: "", height: ""});
                } else {
                    container.addClass('fullscreen');
                    $scope.isFullscreen = true;
                    content.width('100%').height(window.innerHeight);
                    matrixGrid.height("100%");
                    if ($scope.matrix.grid.length == 1) {
                        content.height('100%');
                        cameraComponents.outerWidth('100%').outerHeight('100%');
                    } else if($scope.matrix.grid.length == 2) {
                        cameraComponents.outerWidth((matrixGrid.width()/2)-5).outerHeight(matrixGrid.height());
                    } else {
                        var size = Math.sqrt($scope.matrix.grid.length);
                        cameraComponents
                            .outerWidth((matrixGrid.width() - 11 * (size - 1)) / size)
                            .outerHeight((matrixGrid.height() - 11 * (size - 1)) / size);
                    }
                }
            };

            $(window).resize(rePos);
            rePos();
        };


        (function ($) {
            $.ajaxModalFormOps.startRecording = function (modal, target, response) {
                modal.modal('hide');
                $scope.startRecording(target, response);
            };
        })(jQuery);

        $scope.startRecording = function (target, response) {
            if (response == 'error start') {
                humane.log('There is no active cameras. Recording is impossible.');
                return;
            }
            $scope.$apply(function () {
                $scope.matrix.selectedRoom.recording = true;
                $scope.matrix.selectedRoom.recordingTime = 0;
                recordFactory.getCurrentRoomRecord($scope.matrix.selectedRoomId).success(function (data) {
                    $scope.matrix.selectedRoom.recordId = data.id;
                    $scope.logger('log_record_start', data.id);
                });
            });
            if (response == 'already started') {
                humane.log('Sorry, recording already started.');
                location.reload();
            } else {
                humane.log('Recording started.');
            }
        };

        $scope.blockRoom = function () {
            if ($scope.matrix.selectedRoom.recording && !$scope.matrix.selectedRoom.accessToView) {
                for (var i = 0; i < $scope.matrix.selectedRoom.cameras.length; ++i) {
                    console.log('block');
                    var cssId = 'camera-component-' + i;
                    var swf = document.getElementById(cssId);
                    if (swf && swf.blocked) {
                        console.log('blocked', i);
                        swf.blocked(1);
                    }
                }
            }
        };

        $scope.unBlockRoom = function () {
            if ($scope.matrix.selectedRoom) {
                for (var i = 0; i < $scope.matrix.selectedRoom.cameras.length; ++i) {
                    console.log('unblock');
                    var cssId = 'camera-component-' + i;
                    var swf = document.getElementById(cssId);
                    if (swf && swf.blocked) {
                        console.log('unblocked', i);
                        swf.blocked(0);
                    }
                }
            }
        };

        $scope.selectedRoomRecording = function () {
            if ($scope.matrix.selectedRoom == null || settings.recordAccess == 0) {
                return false;
            }
            return $scope.matrix.selectedRoom.recording;
        };

        $scope.selectedRoomAccessRecording = function () {
            if ($scope.matrix.selectedRoom == null) {
                return false;
            }
            return $scope.recordingAccess($scope.matrix.selectedRoom.id);
        };

        $scope.selectedRoomAccessStop = function () {
            if ($scope.matrix.selectedRoom == null) {
                return false;
            }
            if (settings.viewerId == $scope.matrix.selectedRoom.recordAuthor) {
                return true;
            }
            if (!$scope.recordingAccess($scope.matrix.selectedRoom.id) || settings.recordStopAllAuthors == 0) {
                return false;
            }
            return $scope.matrix.selectedRoom.recording;
        };

        $scope.recordingAccess = function (roomId) {
            var result = false;
            if (typeof roomId != 'undefined' && settings.recordAccess != 0) {
                var room = $scope.getRoomById(roomId);
                if (room.accessToView != null && room.accessToView == 1) {
                    result = true;
                }
            }
            return result;
        };
        var recordingTimer;

        $scope.recordingTimeCounter = function () {
            recordingTimer = $timeout(function () {
                angular.forEach($scope.rooms, function (room, key) {
                    if (room.recording) {
                        $scope.$apply(function () {
                            $scope.rooms[key].recordingTime++;
                        });
                    }
                });
                $scope.recordingTimeCounter();
            }, 983);
        };

        $scope.recordingTime = function (time) {
            if (time == null) {
                return '';
            }
            return toHHMMSS(time);
        };

        $scope.stopRecordingTimeCounter = function () {
            $timeout.cancel(recordingTimer);
        };

        // MARKER
        $scope.markerTime = 0;
        $scope.addMarkerLink = function () {
            return Routing.generate('marker_new', {
                record: $scope.matrix.selectedRoom.recordId,
                time: $scope.matrix.selectedRoom.recordingTime,
                isModal: 2
            });
        };

        $scope.submitMarkerLink = function () {
            var time = $scope.markerTime;
            $scope.markerTime = 0;
            return Routing.generate('marker_create', {
                record: $scope.matrix.selectedRoom.recordId,
                time: time,
                isModal: 2
            });
        };

        $scope.submitRecordLink = function () {
            return Routing.generate('record_create', {roomId: $scope.matrix.selectedRoomId, isModal: 1});
        };
        $scope.startRecordingLink = function () {
            return Routing.generate('record_new', {roomId: $scope.matrix.selectedRoomId, isModal: 1});
        };
        (function ($) {
            $.ajaxModalFormOps.markerAdded = function (modal, target, response) {
                modal.modal('hide');
                humane.log('Marker added.');
            };
        })(jQuery);

        // ROOM STATUS UPDATE
        (function ($) {
            $('body').on('hide', '#ajax-modal', function () {
                console.log('hide');
            });

            var recorderChannel = 'recording:status';
            var options = settings.node;

            var socket = io.connect(options.host);

            socket.on(recorderChannel, function (json) {
                var obj = $.parseJSON(json);
                console.log(obj);

                if (settings.recordAccess == 1) {
                    angular.forEach($scope.rooms, function (room, key) {
                        if (room.id == obj.room_id) {
                            $scope.$apply(function () {
                                $scope.rooms[key].recording = obj.action == 'start';
                                $scope.rooms[key].recordingTime = 0;
                                $scope.rooms[key].recordId = obj.record_id;
                                if (obj.action == 'start') {
                                    $scope.rooms[key].recordAuthor = obj.recordAuthor;
                                    $.get(Routing.generate('record_check_access', {
                                        record: obj.record_id
                                    }), function (data) {
                                        if (data == 1) {
                                            $scope.rooms[key].accessToView = 1;
                                        } else {
                                            $scope.rooms[key].accessToView = 0;
                                            $scope.blockRoom();
                                        }
                                    })
                                } else {
                                    $scope.unBlockRoom();
                                }
                            });
                        }
                    });
                }
            });

            console.log('roomStatusUpdate Init');
        })(jQuery);
    }
);
