'use strict';

var app = angular.module('camMarkers', ['ngRoute', 'ngSanitize']);

app.config(function ($routeProvider) {
    $routeProvider
        .when('/', {
            controller: 'MarkersCtrl',
            reloadOnSearch: true
        });
});

app.config(['$interpolateProvider', function ($interpolateProvider) {
    $interpolateProvider.startSymbol('[[');
    $interpolateProvider.endSymbol(']]');
}]);

app.directive('optionsClass', function ($parse) {
    return {
        require: 'select',
        link: function (scope, elem, attrs, ngSelect) {
            // get the source for the items array that populates the select.
            var optionsSourceStr = attrs.ngOptions.split(' ').pop(),
            // use $parse to get a function from the options-class attribute
            // that you can use to evaluate later.
                getOptionsClass = $parse(attrs.optionsClass);

            scope.$watch(optionsSourceStr, function (items) {
                // when the options source changes loop through its items.
                angular.forEach(items, function (item, index) {
                    // evaluate against the item to get a mapping object for
                    // for your classes.
                    var classes = getOptionsClass(item),
                    // also get the option you're going to need. This can be found
                    // by looking for the option with the appropriate index in the
                    // value attribute.
                        option = elem.find('option[value=' + index + ']');

                    // now loop through the key/value pairs in the mapping object
                    // and apply the classes that evaluated to be truthy.
                    angular.forEach(classes, function (add, className) {
                        if (add) {
                            angular.element(option).addClass(className);
                        }
                    });
                });
            });
        }
    };
});
app.directive("btnLoading", function () {
    return function (scope, element, attrs) {
        scope.$watch(function () {
            return scope.$eval(attrs.ngDisabled);
        }, function (newVal) {
            if (newVal) {
                return;
            } else {
                return scope.$watch(function () {
                        return scope.$eval(attrs.btnLoading);
                    },
                    function (loading) {
                        if (loading)
                            return element.button("loading");
                        element.button("reset");
                    });
            }
        });
    };
});

app.controller('MarkersCtrl', function ($scope, $http, $filter, markerFactory, $timeout) {


    $scope.selectedMarker = null;
    $scope.selectedMarkerId = null;

    $scope.loadingState = false;

    $scope.selectedMarkerTimeString = function () {
        if ($scope.selectedMarker === null || $scope.selectedMarker.time === undefined) {
            return '';
        }

        return toHHMMSS($scope.selectedMarker.time);
    };
    var $markerForm = $('#marker-form');
    var markerSelectedCallback = function (newValue, oldValue) {

        if (newValue === null) {
            // we don't need selectedMarker
            $scope.selectedMarker = null;
        }
        else if ($scope.newMarker) {
            $scope.newMarker = false;
        }
        else if (newValue === false) {
            // create new marker
            var time = getSWF().getCurrentTime();
//            var time = 0;
            $scope.selectedMarker = {time: time};


            $markerForm.html($scope.spinner);
            markerFactory.getAddMarkerForm($scope.selectedMarker).success(function (data, status, headers, config) {
                $markerForm.html(data);

            });
        }
        else {
            // go to existing marker
            if (!(oldValue === false))
                getSWF().goToMarker(newValue.toString());

            var found = $filter('filter')($scope.markers, {id: newValue}, false);

            if (found.length) {
                $scope.selectedMarker = JSON.parse(JSON.stringify(found[0]));
                $scope.selectedMarkerId = newValue;

                $markerForm.html($scope.spinner);
                markerFactory.getUpdateMarkerForm($scope.selectedMarker).success(function (data, status, headers, config) {
                    $markerForm.html(data);
                });
            }
            $scope.selectedMarkerId = 'none';
        }

    };

    $scope.$watch('selectedMarkerId', markerSelectedCallback);

    $scope.markers = [];
    $scope.addMode = function () {
        return $scope.selectedMarkerId == false;
    };

    var getMarkersSuccessCallback = function (data, status) {
        $scope.markers = data;
    };

    var successCallback = function () {
        $scope.loadingState = false;
        return markerFactory.getMarkers(settings.recordId).success(getMarkersSuccessCallback);

    };

    markerFactory.getMarkers(settings.recordId).success(getMarkersSuccessCallback);

    // Callback function that ensures that swf is fully loaded to launch record from marker
    settings.loaded = function () {
        if (settings.selectedMarker) markerSelectedCallback(settings.selectedMarker);
    };


    /********************************************************************************************/
    $scope.spinner = '<div class="well">Loading...</div>';

    (function ($) {
        $.ajaxFormOps.updateMarker = function (form, target, response) {
            $scope.$apply(function () {
                successCallback().success(function () {
                    var preState = $scope.selectedMarkerId;

                    $scope.selectedMarker = response.marker;
                    $scope.selectedMarkerId = response.marker.id;
                    form.html(response.view);
                    var data = $scope.selectedMarker;
                    if (preState == null || preState == false) {
                        getSWF().addMarker(data.id, data.time, data.name);
                    }

                });
            });
        };

        $('.video-info').on('click', "#remove-marker", function () {
            $.get($(this).attr('href')).success(function (data, status, headers, config) {
                $scope.$apply(function () {
                    if (data[0] == "processed") {
                        var marker = $scope.selectedMarker;
                        $scope.selectedMarker = null;
                        $scope.selectedMarkerId = null;
                        $("#marker-form").html("");
                        var index = $scope.markers.indexOf(marker);
                        var indexes = $.map($scope.markers, function (obj, index) {
                            if (obj.id == marker.id) {
                                return index;
                            }
                        });
                        var firstIndex = indexes[0];
                        if (firstIndex != -1) {
                            $scope.markers.splice(firstIndex, 1);
                            getSWF().removeMarker(marker.id);
                        }
                    }
                });
            })
            return false;
        })
    })(jQuery);

    $scope.createMarker = function () {
        $scope.selectedMarkerId = false;
        markerSelectedCallback($scope.selectedMarkerId);

    };
    $scope.cancelSelectMarker = function () {
        $scope.selectedMarkerId = null;
        markerSelectedCallback($scope.selectedMarkerId);
    };

    $scope.newMarker = false;
    $scope.addMarker = function () {
        $scope.newMarker = true;
        $scope.loadingState = true;
        var data = $scope.selectedMarker;
        getSWF().addMarker(data.id, data.time, data.name);
    };

    $scope.updateMarker = function (marker) {
        $scope.loadingState = true;
        markerFactory.updateMarker(marker).success(successCallback);
    };

});