<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 01.02.2016
 * Time: 12:00
 */

namespace FS\CamBundle\RestrictAccess;

use Doctrine\ORM\Mapping as ORM;
use FS\CamBundle\Entity\RestrictAccessWeekDay;
use Symfony\Component\Validator\Constraints as Assert;
use Gedmo\Mapping\Annotation as Gedmo;
use FS\CamBundle\Entity\RestrictAccess;

trait RestrictAccessTrait
{
    /**
     * @var RestrictAccess
     *
     * @ORM\OneToOne(targetEntity="FS\CamBundle\Entity\RestrictAccess", cascade={"persist"})
     * @ORM\JoinColumn(name="r_access_id", referencedColumnName="id", onDelete="SET NULL")
     **/
    private $restrictAccess;

    /**
     * @return RestrictAccess
     */
    public function getRestrictAccess()
    {
        if($this->restrictAccess == null){
            $this->restrictAccess = RestrictAccessBuilder::buildRestrictAccess();
        }

        return $this->restrictAccess;
    }

    /**
     * @param RestrictAccess $restrictAccess
     * @return $this
     */
    public function setRestrictAccess(RestrictAccess $restrictAccess)
    {
        $this->restrictAccess = $restrictAccess;
        return $this;
    }

    /**
     * @param $number
     * @return RestrictAccessWeekDay
     */
    public function getAccessByWeekDayNumber($number)
    {
        return $this->restrictAccess->getWeekDays()->get($number);
    }

    /**
     * @param $number
     * @return RestrictAccessWeekDay
     */
    public function getWeekDayByDayNumber($number)
    {
        if($number == 7)
        {
            return $this->getRestrictAccess()->getWeekDays()[0];
        } else {
            return $this->getRestrictAccess()->getWeekDays()[$number];
        }
    }

    private function isAccessToday()
    {
        $now = new \DateTime();

        $hours = (int)$now->format('H');
        $minutes = (int)$now->format('i');
        $seconds = (($hours * 60) + $minutes)*60;

        $weekDay = $now->format('N');
        if($this->getRestrictAccess() == null || $this->getRestrictAccess()->getWeekDays() == null){
            return true;
        }
        /** @var RestrictAccessWeekDay $restrictAccess */
        $restrictAccess = $this->getWeekDayByDayNumber($weekDay);
        $restrictAccessStart = $restrictAccess->getStartTime();
        $restrictAccessEnd = $restrictAccess->getEndTime();
        if ($restrictAccess->isIsEnable()) {
            return false;
        } else {
            if ($restrictAccess->getStartTime() != null || $restrictAccess->getEndTime()) {
                for ($i = 0; $i < count($restrictAccessStart); $i++) {
                    if ($restrictAccessStart[$i] <= $seconds && $restrictAccessEnd[$i] >= $seconds) return true;
                }
                return false;
            }
            return true;
        }
    }
}