<?php
/**
 * @author Andrey Yatsenco <andrey@fora-soft.com>
 */

namespace FS\CamBundle\Utils;


use Doctrine\Common\Proxy\Exception\InvalidArgumentException;
use Symfony\Component\Config\Util\XmlUtils;

class CameraModelManager
{
    /**
     * @var array
     */
    private $brands;

    /**
     * @var array
     */
    private $models;

    public function __construct($filePath)
    {
        $this->parseFile($filePath);
    }

    protected function parseFile($file)
    {
        try {
            $dom = XmlUtils::loadFile($file);
        } catch (\InvalidArgumentException $e) {
            throw new InvalidArgumentException(sprintf('Unable to parse file "%s".', $file), $e->getCode(), $e);
        }
        /** @var \SimpleXMLElement $rawData */
        $rawData = simplexml_import_dom($dom);

        /** @var \SimpleXMLElement $brand */
        foreach ($rawData->children() as $brand) {
            $brandAttributes = $brand->attributes();
            $models = [];
            /** @var \SimpleXMLElement $model */
            foreach ($brand->children() as $model) {
                $modelAttributes = $model->attributes();
                $models[(string) $modelAttributes->id] = (string) $modelAttributes->name;
            }
            $this->models[(string) $brandAttributes->id] = $models;
            $this->brands[(string) $brandAttributes->id] = (string) $brandAttributes->name;

        }
    }

    public function getBrands()
    {
        return $this->brands;
    }

    public function getModels($brandId)
    {
        return isset($this->models[$brandId]) ? $this->models[$brandId] : [];
    }
} 