<?php

namespace FS\CamBundle\Utils;

use Buzz\Browser;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Wowza;

class RecordFiles
{
    const ARCHIVE_NAME = 'videos.zip';

    private $client;
    private $wowzaManager;

    public function __construct(WowzaSenderClient $client, WowzaManager $wowzaManager)
    {
        $this->client = $client;
        $this->wowzaManager = $wowzaManager;
    }

    /**
     * @param  Record $record
     * @return array
     */
    public function getRecordVideos(Record $record)
    {

        $wowzas = $this->wowzaManager->getAllWowzas();

        $resultRaw = [];
        $i = 0;
        foreach ($wowzas as $wowza) {

            $content = $this
                ->client->post(
                    'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                    ['Content-Type' => 'application/json'],
                    json_encode([
                            'command' => 'getFilesList',
                            'recordId' => (string)$record->getId(),
                        ]
                    )
                );

            if($content['content']!= null){
                $content = @json_decode($content['content']);
                if($content->namesArray != null ){
                    $resultRaw[] = $content;
                    $resultRaw[0]->wowza = $wowza;
                }

            }
        }
        $result = [];
        $i = 0;
        foreach ($resultRaw as $namesArray) {
            $result[$i]['names'] = array_merge($result, $namesArray->namesArray);
            $result[$i++]['wowza'] = $namesArray->wowza;
        }
        return $result;
    }

    /**
     * @param Record $record
     * @param Camera $camera
     * @return mixed
     */
    public function getCameraVideo(Record $record, Camera $camera)
    {

        $wowza = $camera->getWowza();

        $rawRontent = $this->client->post(
            'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
            ['Content-Type' => 'application/json'],
            json_encode([
                    'command' => 'getRecordCameraVideo',
                    'recordId' => (string)$record->getId(),
                    'cameraId' => (string)$camera->getId(),
                ]
            )
        );
        return @json_decode($rawRontent['content']);
    }

    /**
     * @param Record $record
     * @param $cameraId
     * @return mixed
     */
    public function getFindCameraVideo(Record $record, $cameraId)
    {

        $wowzas = $this->wowzaManager->getAllWowzas();
        /** @var Wowza $wowza */
        foreach($wowzas as $wowza){
            $rawRontent = $this->client->post(
                'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                ['Content-Type' => 'application/json'],
                json_encode([
                        'command' => 'getRecordCameraVideo',
                        'recordId' => (string)$record->getId(),
                        'cameraId' => (string)$cameraId,
                    ]
                )
            );
            $response = @json_decode($rawRontent['content']);
            if($response != null ){
                return $response;
            }
        }
    }
}
