<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 12.05.2015
 * Time: 19:07
 */

namespace FS\CamBundle\Utils;


use Buzz\Browser;
use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Wowza;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\Filesystem\Exception\IOException;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Finder\Finder;
use Symfony\Component\Finder\SplFileInfo;
use Symfony\Component\HttpFoundation\File\Exception\FileException;
use Symfony\Component\HttpFoundation\File\File;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Kernel;

class WowzaManager
{
    private $kernel;
    private $om;
    private $request;
    private $filesystem;
    private $client;

    private $finder;

    private $siteTMPDir;
    private $siteImagesDir;
    private $siteImagesRecordDir;
    private $mosaicDir;


    public function __construct
    (
        Kernel $kernel,
        ObjectManager $objectManager,
        RequestStack $requestStack,
        Filesystem $filesystem,
        WowzaSenderClient $client
    )
    {
        $this->kernel = $kernel;
        $this->om = $objectManager;
        $this->request = $requestStack->getCurrentRequest();
        $this->filesystem = $filesystem;
        $this->client = $client;
        $this->siteTMPDir = $this->kernel->getRootDir() . '/../web/TMPUpload/';
        $this->siteImagesDir = $this->kernel->getRootDir() . '/../web/images/';
        $this->siteImagesRecordDir = $this->kernel->getRootDir() . '/../web/images/record/';
        $this->mosaicDir = $this->kernel->getRootDir() . '/../web/video/mosaic/';

        try {
            if (!$this->filesystem->exists($this->mosaicDir)) {
                $this->filesystem->mkdir($this->mosaicDir);
            }
            if (!$this->filesystem->exists($this->siteImagesDir)) {
                $this->filesystem->mkdir($this->siteImagesDir);
            }
            if (!$this->filesystem->exists($this->siteImagesRecordDir)) {
                $this->filesystem->mkdir($this->siteImagesRecordDir);
            }
        } catch (IOException $e) {
            /* NOP */
        } catch (FileException $e) {
            /* NOP */
        }
    }

    private function getIp()
    {
        preg_match_all('/inet' . (false ? '6?' : '') . ' addr: ?([^ ]+)/', `ifconfig`, $ips);
        return $ips[1];
    }

    private function getHost()
    {

    }

    /**
     * @return mixed
     */
    public function getLocalWowza()
    {

        $localWowza = null;
        try {
            if ($this->request != null) {
                $ips = $this->getIp();
                $ips [] = $this->request->getHost();
                $ips [] = $this->request->server->get("SERVER_ADDR");
                $localWowza = $this->om->getRepository("FSCamBundle:Wowza")
                    ->getWowzaByAddresses($ips);
            } else {
                $localWowza = $this->om->getRepository("FSCamBundle:Wowza")
                    ->getWowzaByAddresses(
                        $this->getIp()
                    );
            }
        } catch (\Exception $e) {
            /* NOP */
        }


        return $localWowza;
    }

    public function getLocalWowzaFileserverPort()
    {
        $wowza = $this->getLocalWowza();
        $responseRaw = $this->client
            ->post(
                'http://' . $wowza->getAddress() . ':8086/dustinSystemData',
                ['Content-Type' => 'application/json'],
                json_encode(['command' => "getWebServerPort"])
            );
        $response = json_decode($responseRaw['content']);

        if ($response == null) {
            return null;
        }
        return $response->webServerPort;
    }

    public function getWowzaFileserverPort(Wowza $wowza)
    {
        $responseRaw = $this->client
            ->post(
                'http://' . $wowza->getAddress() . ':8086/dustinSystemData',
                ['Content-Type' => 'application/json'],
                json_encode(['command' => "getWebServerPort"])
            );
        $response = json_decode($responseRaw['content']);

        if ($response == null) {
            return null;
        }
        return $response->webServerPort;
    }

    public function getImageFromWowza($recordId, $serverAddress, $port)
    {
        try {
            $this->filesystem->mkdir($this->siteImagesRecordDir . $recordId);
            $fileOut = $this->siteImagesRecordDir . $recordId . "/prev.jpeg";
            $fileIn = "http://" . $serverAddress . ":" . $port . "/" . $recordId . "/" . $recordId . ".jpeg";
            $file = file_get_contents($fileIn);
            $res = file_put_contents($fileOut, $file);
            return $res;
        } catch (\Exception $e) {
            return false;
        }


    }

    /**
     * @param UploadedFile $file
     * @param $fileUID
     * @return bool
     */
    public function moveFileToTMPDir(UploadedFile $file, $fileUID)
    {
        $dirName = $this->kernel->getRootDir() . '/../web/TMPUpload/';
        $fileName = $fileUID;
        $extension = 'mp4';
        $result = false;
        try {
            $file->move($dirName, $fileName . '.' . $extension);
            $result = true;
        } catch (FileException $e) {
            /*NOP*/
        }
        return $result;
    }

    /**
     * @param Record $record
     */
    public function makeThumbnailLocalWowza(Record $record)
    {
        $wowza = $this->getLocalWowza();
        $this->client
            ->post(
                'http://' . $wowza->getAddress() . ':8086/dustinSystemData',
                ['Content-Type' => 'application/json'],
                json_encode([
                    'command' => "makeThumbnail",
                    'id' => (string)$record->getId(),
                ])
            );
    }

    public function makeThumbnailForRecord(Record $record)
    {
        $cameraRepository = $this->om->getRepository('FSCamBundle:Camera');

        foreach ($record->getCamerasInRecord() as $cameraId => $cameraName) {
            /** @var Camera $camera */
            $camera = $cameraRepository->findOneBy(['id' => $cameraId]);
            if($camera == null){
                $wowza = $this->findWowzaByRecordAndCameraId($record, $cameraId);
            } else {
                $wowza = $camera->getWowza();
            }

            if($wowza != null)
            {
                $this->client
                    ->post(
                        'http://' . $wowza->getAddress() . ':8086/dustinSystemData',
                        ['Content-Type' => 'application/json'],
                        json_encode([
                            'command' => "makeThumbnail",
                            'id' => (string)$record->getId(),
                        ])
                    );
                break;
            }
        }

        return;

    }

    public function moveTMPFileToLocalWowza(Record $record, $fileUID)
    {
        $tmpDir = $this->kernel->getRootDir() . '/../web/TMPUpload/';
        $dirName = $this->kernel->getRootDir() . '/../web/video/' . $record->getId() . '/';
        $result = false;
        $finder = new Finder();
        $finder->files()->name($fileUID . '.mp4');
        $fileToMove = null;
        /** @var SplFileInfo $file */
        foreach ($finder->in($tmpDir) as $file) {
            $fileToMove = $file;
            break;
        }
        if ($fileToMove != null) {
            try {
                $this->filesystem->mkdir($dirName, 0777);
                /** @var File $file */
                $file = new File($fileToMove->getRealPath());
                $file->move($dirName, $fileToMove->getFilename());
                $result = true;
            } catch (IOException $e) {
                return false;
            } catch (FileException $e) {
                return false;
            }
        }
        return $result;
    }

    /**
     * @param Record $record
     * @param UploadedFile $file
     * @return string
     */
    public function moveFileToLocalWowza(Record $record, UploadedFile $file)
    {
        $dirName = $this->kernel->getRootDir() . '/../web/video/' . $record->getId() . '/';
        $this->filesystem->mkdir($dirName, 0777);

        $fileName = sha1($record->getName() . $file->getClientOriginalName() . uniqid());

        $extension = 'mp4';

        $file->move($dirName, $fileName . '.' . $extension);

        return $fileName;
    }

    public function sendConvertRequestToWowza(Record $record, UploadedFile $file)
    {
        $fileName = sha1($record->getName() . $file->getClientOriginalName() . uniqid());

        $wowza = $this->getLocalWowza();
        $res = $this->moveFileToTMPDir($file, $fileName);
        if ($res) {
            $responseRaw = $this->client
                ->post(
                    'http://' . $wowza->getAddress() . ':8086/dustinSystemData',
                    ['Content-Type' => 'application/json'],
                    json_encode(
                        [
                            'command' => "convert",
                            'record' => (string)$record->getId(),
                            'name' => $fileName,
                            'path' => realpath($this->siteTMPDir . $fileName . ".mp4")
                        ]
                    )
                );
            $response = json_decode($responseRaw['content']);
        } else {
            $fileName = null;
        }

        return $fileName;
    }

    public function prepareToMosaic(Record $record)
    {

        $recordDir = $this->kernel->getRootDir() . '/../web/video/' . $record->getId() . '/';
        // create dir for mosaic videos
        $cameraRepository = $this->om->getRepository('FSCamBundle:Camera');
        /** @var Wowza $localWowza */
        $localWowza = $this->getLocalWowza();
        $recordFolderCopied = false;
        try {
            $dirName = $this->mosaicDir . $record->getId() . '/';
            $this->filesystem->mkdir($dirName, 0777);
            foreach ($record->getCamerasInRecord() as $cameraId => $cameraName) {
                /** @var Camera $camera */
                $camera = $cameraRepository->findOneBy(['id' => $cameraId]);
                if ($camera == null) {
                    continue;
                }
                $wowza = $camera->getWowza();
                // need to change
                if ($wowza == null || $localWowza == null) {
                    continue;
                }
                if ($wowza->getId() == $localWowza->getId()) {
                    //copies all folder
                    //just copies it
                    if (!$recordFolderCopied) {
                        /** @var SplFileInfo $file */
                        foreach ($this->getFinder()->in($recordDir) as $file) {
                            $this->filesystem->copy($file->getPathname(), $dirName . $file->getFilename(), true);
                        }
                        $recordFolderCopied = true;
                    }
                } else {
                    //download
                    $videoFile = $this->findCameraVideo($record, $camera->getId());
                    if (!$videoFile) {
                        continue;
                    }
                    $this->downloadVideo($videoFile, $dirName . uniqid("video") . ".mp4");
                }
            }

        } catch (\Exception $e) {
            return false;
        }
        return true;

    }

    /**
     * @return Finder
     */
    private function getFinder()
    {
        return new Finder();
    }

    private function downloadVideo($file, $destination)
    {
        if (!is_writable($destination)) {
            $this->filesystem->touch($destination);
        }

        $ch = curl_init($file->url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); //not necessary unless the file redirects (like the PHP example we're using here)
        $data = curl_exec($ch);
        curl_close($ch);
        $contentLength = 'unknown';
        if (preg_match('/Content-Length: (\d+)/', $data, $matches)) {
            $contentLength = (int)$matches[1];
        }
        $done = false;
        $start = 0;
        $end = $contentLength;
        while (!$done) {
            $info = $this->curlGetContent($file->url, $start, $end, $destination);
            $start += $info['size_download'];
            if ($start == $end) {
                $done = true;
            }
        }
        return $done;
    }

    public function curlGetContent($url, $range_start = 0, $range_end = 0, $file = "video.mp4")
    {
        $writefn = function ($ch, $chunk) use ($file) {

            //echo $chunk; $file chunk here
            $fileWriter = fopen($file, 'a+');
            fwrite($fileWriter, $chunk);
            fclose($fileWriter);

            $len = strlen($chunk);
            return $len;
        };

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_BINARYTRANSFER, 1);
        curl_setopt($ch, CURLOPT_BUFFERSIZE, 8096);

        if ($range_start != 0 && $range_end != 0) {
            curl_setopt($ch, CURLOPT_HTTPHEADER, array("range:bytes=$range_start-$range_end"));
        }
        curl_setopt($ch, CURLOPT_WRITEFUNCTION, $writefn);
        $result = curl_exec($ch);

        $info = curl_getinfo($ch);
        curl_close($ch);
        return $info;
    }

    public function getAllWowzas()
    {
        return $this->om->getRepository('FSCamBundle:Wowza')->findAll();
    }

    public function getWowzaSpaceInformation(Wowza $wowza)
    {
        $responseRaw = $this->client
            ->post(
                'http://' . $wowza->getAddress() . ':8086/dustinSystemData',
                ['Content-Type' => 'application/json'],
                json_encode(['command' => "getFreeSpace"])
            );
        $response = json_decode($responseRaw['content']);
        if ($response != null) {
            $proc = 100;
            if ($response->available != 0) {
                $proc -= floor(($response->available * 100) / $response->total);
            }
            return [
                'total' => (int)$response->total,
                'available' => (int)$response->available,
                'proc' => $proc,
                'totalGB' => (float)number_format(($response->total * (1 / 1024)), 2, '.', ''),
                'totalTB' => (float)number_format(($response->total * (1 / 1024) / 1000), 2, '.', ''),
                'availableGB' => (float)number_format(($response->available * (1 / 1024)), 2, '.', ''),
                'availableTB' => (float)number_format(($response->available * (1 / 1024) / 1000), 2, '.', ''),
            ];
        } else {
            return [
                'total' => 0,
                'available' => 0,
                'proc' => 0,
                'totalGB' => 0,
                'totalTB' => 0,
                'availableGB' => 0,
                'availableTB' => 0,
            ];
        }

    }

    /**
     * @param Record $record
     * @param $cameraId
     * @return mixed
     */
    public function findCameraVideo(Record $record, $cameraId)
    {

        $wowzas = $this->getAllWowzas();
        /** @var Wowza $wowza */
        foreach ($wowzas as $wowza) {
            $rawRontent = $this->client->post(
                'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                ['Content-Type' => 'application/json'],
                json_encode([
                        'command' => 'getRecordCameraVideo',
                        'recordId' => (string)$record->getId(),
                        'cameraId' => (string)$cameraId,
                    ]
                )
            );
            $response = @json_decode($rawRontent['content']);
            if ($response != null) {
                return $response;
            }
        }
        return false;
    }


    public function findWowzaByRecordAndCamera(Record $record, Camera $camera)
    {
        $wowzas = $this->getAllWowzas();

        foreach ($wowzas as $wowza) {
            $rawRontent = $this->client->post(
                'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                ['Content-Type' => 'application/json'],
                json_encode([
                        'command' => 'getRecordCameraVideo',
                        'recordId' => (string)$record->getId(),
                        'cameraId' => (string)$camera->getId(),
                    ]
                )
            );
            $response = @json_decode($rawRontent['content']);
            if ($response != null) {
                return $wowza;
            }
        }
        return $this->getLocalWowza();

    }

    public function findWowzaByRecordAndCameraId(Record $record, $cameraId)
    {
        $wowzas = $this->getAllWowzas();

        foreach ($wowzas as $wowza) {
            $rawRontent = $this->client->post(
                'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                ['Content-Type' => 'application/json'],
                json_encode([
                        'command' => 'getRecordCameraVideo',
                        'recordId' => (string)$record->getId(),
                        'cameraId' => (string)$cameraId,
                    ]
                )
            );
            $response = @json_decode($rawRontent['content']);
            if ($response != null) {
                return $wowza;
            }
        }
        return $this->getLocalWowza();

    }
}