<?php

namespace FS\MobileBundle\Controller;

use Doctrine\ORM\EntityManager;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Template;
use FS\CamBundle\Entity\TemplateField;
use FS\MobileBundle\Entity\FileState;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * @Route("/mobile/api")
 */
class MobileController extends Controller
{
    /**
     * @Route(path="/login", name="mobile_login")
     * @param Request $request
     * @return JsonResponse
     */
    public function loginAction(Request $request)
    {
        $username = $request->request->get("username");
        $password = $request->request->get("password");
        $deviceUID = $request->request->get("deviceUID");
        $bool = false;

        $userManager = $this->get('fos_user.user_manager');
        $factory = $this->get('security.encoder_factory');
        $em = $this->getDoctrine()->getManager();
        /** @var User $user */
        $user = $userManager->findUserByUsername($username);
        // find user
        if ($user || is_object($user)) {
            $encoder = $factory->getEncoder($user);
            // first check
            $bool = $encoder->isPasswordValid($user->getPassword(), $password, $user->getSalt());
            if ($bool) {
                $bool = $user->hasRight('admin');
                if (!$bool) {
                    $bool = $this->get('security.context')->isGranted('ROLE_ADMIN');
                }
            }
        }
        if ($bool) {
            $device = $em->getRepository("FSCamBundle:Camera")->findOneBy(['deviceUID' => $deviceUID]);
            if (($device || is_object($device)) && $deviceUID != null) {
                $device->setLinkedDevice(true);
                $em->persist($device);
                $em->flush();
            } else {
                $bool = false;
            }
        }
        if ($bool) {
            return new JsonResponse(['response' => 'device Linked']);
        } else {
            return new JsonResponse(['code' => 400, 'message' => 'Server address, device UID, username or password is not correct. \n Please, try again'], 400);
        }
    }

    /**
     * @Route(path="/syncUserData", name="mobile_sync_user_data")
     * @param Request $request
     * @return JsonResponse
     */
    public function syncUserDataAction(Request $request)
    {
        $deviceUID = $request->request->get("deviceUID");
        $em = $this->getDoctrine()->getManager();
        $result = false;
        if ($deviceUID != null) {
            /** @var Camera $device */
            $device = $em->getRepository("FSCamBundle:Camera")->findOneBy(['deviceUID' => $deviceUID]);
            if (($device || is_object($device)) && $device->getLinkedDevice()) {
                $result = true;
            }
        }
        if ($result) {
            $users = $em->getRepository("FSUserBundle:User")->findAll();
            $usersResult = [];
            /** @var User $user */
            foreach ($users as $user) {
                $usersResult[] = $user->getUserSimpleData();
            }
            $userGroups = $em->getRepository("FSUserBundle:UserGroup")->findAll();
            $templates = [];
            /** @var UserGroup $userGroup */
            foreach ($userGroups as $userGroup) {
                $templates[] = $userGroup->getGroupTemplateInfo();
            }
            $response = [];
            $response['users'] = $usersResult;
            $response['templates'] = $templates;

            return new JsonResponse(['response' => $response]);
        } else {
            return new JsonResponse(['code' => 400, 'message' => 'device unlinked'], 400);
        }
    }

    /**
     * @Route(path="/uploadRecord", name="mobile_upload_file")
     * @param Request $request
     * @return JsonResponse
     */
    public function uploadRecorderFileAction(Request $request)
    {
        $fileUID = $request->query->get('fileUID');
        $deviceUID = $request->query->get('deviceUID');
        $contentLength = $request->query->get('length');

        $fileReceiver = $this->get('fs_mobile.file_receiver');
        $fileSize = $fileReceiver->filesize($fileUID);
        $moveResult = false;
        /** @var EntityManager $em */
        $em = $this->getDoctrine()->getManager();
        $fileState = $em->getRepository('FSMobileBundle:FileState')->findOneBy(['uid' => $fileUID]);
        if ($fileState != null) {
            $fileState->setState(FileState::STATE_WAIT);
        } else {
            $fileState = new FileState();
            $fileState->setState(FileState::STATE_WAIT);
            $fileState->setUid($fileUID);
        }
        $em->persist($fileState);
        $em->flush();


        if ($fileSize != false && ((string)$fileSize == (string)$contentLength)) {
            $moveResult = true;
        } elseif ($fileUID != null && $deviceUID != null) {
            $append = $fileSize == false ? false : true;
            $moveResult = $fileReceiver->receive($fileUID, $contentLength, $append);
        }

        if ($moveResult) {
            $em->remove($fileState);
            $em->flush();
            return new JsonResponse(['response' => 'done'], 200);
        } else {
            $fileState->setState(FileState::STATE_WRITE);
            $em->flush();
            return new JsonResponse(
                [
                    'code' => 400,
                    'message' => 'File not Uploaded',
                ], 400);
        }
    }

    /**
     * @Route(path="/uploadStatus", name="mobile_upload_status")
     * @param Request $request
     * @return JsonResponse
     */
    public function getUploadStatusAction(Request $request)
    {
        $fileUID = $request->query->get('fileUID');
        $fileReceiver = $this->get('fs_mobile.file_receiver');
        $fileSize = $fileReceiver->filesize($fileUID);
        $em = $this->getDoctrine()->getManager();

        $returnState = 1;

        /** @var FileState $fileState */
        $fileState = $em->getRepository('FSMobileBundle:FileState')->findOneBy(['uid' => $fileUID]);
        if ($fileState != null) {
            $returnState = $fileState->getState();
        }

        if ($fileSize != false) {
            return new JsonResponse(
                [
                    'offset' => $fileSize,
                    'status' => $returnState
                ]
            );
        } else {
            return new JsonResponse(
                [
                    'offset' => "0",
                    'status' => $returnState
                ]
            );
        }
    }

    /**
     * @Route(path="/saveRecordedData", name="mobile_save_data")
     * @param Request $request
     * @return JsonResponse
     */
    public function saveRecordedDataAction(Request $request)
    {
        $fileUID = $request->request->get('fileUID');
        $name = $request->request->get('name');
        $deviceUID = $request->request->get('deviceUID');
        $templateData = $request->request->get('templateData');
        $username = $request->request->get("username");
        $duration = $request->request->get("duration",0);

        $userManager = $this->get('fos_user.user_manager');

        if($fileUID == "" || $name == "" || $username == ''){
            return new JsonResponse(['code' => 400, 'message' => 'Bad Request, some data is empty'], 400);
        }

        /** @var User $user */
        $user = $userManager->findUserByUsername($username);
        $em = $this->getDoctrine()->getManager();
        $result = false;
        /** @var Camera $device */
        $device = $em->getRepository("FSCamBundle:Camera")->findOneBy(['deviceUID' => $deviceUID]);
        if (
            ($user != null && is_object($user)) &&
            ($device || is_object($device)) && $device->getLinkedDevice()
        ) {
            $record = new Record();
            $record->setRoom($device->getRooms()->first());
            $record->setStartedAt(new \DateTime());
            $record->setName($name);
            $record->setRecordUploadetSequence($fileUID);
            $record->setCreatedBy($user);
            $record->setTemplate($user->getUserGroup()->getInfoTemplate());
            $record->setContainer($user->getUserGroup()->getContainer());
            $record->setTemplateData($templateData);
            $record->setDuration((int)$duration);
            $record->setRecordType(Record::RECORD_TYPE_UPLOAD);
            $record->setIsCompleted();
            $record->isCompleted();

            $em->persist($record);
            $em->flush();

            $moveResult = $this->get('fs_cam.wowza_manager')->moveTMPFileToLocalWowza($record, $fileUID);
            if ($moveResult) {
                $result = true;
                $this->get('fs_cam.wowza_manager')->makeThumbnailLocalWowza($record);
            } else {
                $em->remove($record);
                $em->flush();
            }
        }
        if ($result) {
            return new JsonResponse(['response' => 'done']);
        } else {
            return new JsonResponse(['code' => 400, 'message' => 'Recording can not save'], 400);
        }

    }
}
