<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 22.08.2015
 * Time: 14:58
 */

namespace FS\UserBundle\Controller;


use Doctrine\DBAL\DBALException;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use FS\UserBundle\Form\Type\ContainerType;
use FS\UserBundle\Form\Type\ImportUsersType;
use FS\UserBundle\Model\ImportUsers;
use PHPExcel;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\Debug\Exception\ContextErrorException;
use Symfony\Component\Form\FormBuilder;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class ContainerController
 * @package FS\UserBundle\Controller
 * @Route("/admin/container")
 * @Security(expression="user.hasRight('admin--containers') || is_granted('ROLE_ADMIN')")
 */
class ContainerController extends Controller
{
    /**
     * @Route("/", name="admin_container")
     * @Template()
     */
    public function indexAction(Request $request)
    {
        return [];
    }

    /**
     * @Route("/list", name="admin_container_list")
     * @Template()
     * @param Request $request
     * @return array
     */
    public function listAction(Request $request)
    {
        //$em = $this->getDoctrine()->getManager();
        //$$container = $em->getRepository("FSUserBundle:Container")->getContainerHead();
        /** @var UserGroup $userGroup */
        $userGroup = $this->getUser()->getUserGroup();
        return [
            'container' => $userGroup->getContainer(),
        ];
    }

    /**
     * @Route("/list/children/{id}", name="admin_container_list_children")
     * @Template()
     * @param Container $container
     * @return array
     */
    public function listChildrenAction(Container $container)
    {
        return [
            'container' => $container
        ];
    }

    /**
     * @Route("/new", name="admin_container_new")
     * @Template()
     * @return array
     */
    public function newAction()
    {
        $container = new Container();
        $form = $this->createForm('fs_container', $container);

        return [
            'container' => $container,
            'form' => $form->createView(),
        ];
    }

    /**
     * @Route("/create", name="admin_container_create")
     * @Template("FSUserBundle:Container:new.html.twig")
     * @param Request $request
     * @return array|RedirectResponse
     */
    public function createAction(Request $request)
    {
        $container = new Container();
        $form = $this->createForm('fs_container', $container);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($container);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'New container has been successfully added'
            );

            return $this->redirect($this->generateUrl('admin_container_edit', ['id' => $container->getId()]));
        }

        return array(
            'container' => $container,
            'form' => $form->createView(),
        );
    }

    /**
     * @Route("/edit/{id}", name="admin_container_edit")
     * @Template()
     * @param Container $container
     * @param Request $request
     * @return array
     */
    public function editAction(Container $container, Request $request)
    {
        $form = $this->createForm('fs_container', $container);

        return [
            'container' => $container,
            'form' => $form->createView(),
        ];
    }

    /**
     * @Route("/update/{id}", name="admin_container_update")
     * @Template("FSUserBundle:Container:edit.html.twig")
     * @param Container $container
     * @param Request $request
     * @return array|RedirectResponse
     */
    public function updateAction(Container $container, Request $request)
    {
        $form = $this->createForm('fs_container', $container);

        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->persist($container);
            $em->flush();

            return $this->redirect($this->generateUrl('admin_container'));
        }

        return array(
            'container' => $container,
            'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to delete a Room entity.
     *
     * @Route("/{id}/remove", name="admin_container_remove")
     * @Template()
     * @param Container $container
     * @param Request $request
     * @return array
     */
    public function removeAction(Container $container, Request $request)
    {
        $form = $this->createDeleteForm($container->getId());

        return array('container' => $container, 'form' => $form->createView());
    }

    /**
     * Deletes a Room entity.
     *
     * @Route("/{id}/delete", name="admin_container_delete")
     * @Method("POST")
     * @param Container $container
     * @param Request $request
     * @return RedirectResponse
     */
    public function deleteAction(Container $container, Request $request)
    {
        $form = $this->createDeleteForm($container->getId());
        if ($form->bind($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();

            $em->remove($container);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'Container has been successfully deleted'
            );
        }

        return $this->redirect($this->generateUrl('admin_container'));
    }

    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }

}