<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 22.08.2015
 * Time: 14:58
 */

namespace FS\UserBundle\Controller;


use Doctrine\DBAL\DBALException;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Form\Type\ImportUsersType;
use FS\UserBundle\Model\ImportUsers;
use PHPExcel;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\Debug\Exception\ContextErrorException;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class LDAPController
 * @package FS\UserBundle\Controller
 * @Route("/admin/user-import")
 * @Security(expression="user.hasRight('admin--users-and-groups') || is_granted('ROLE_ADMIN')")
 */
class ImportController extends Controller
{
    /**
     * @Route("/", name="admin_import")
     * @Template()
     */
    public function indexAction(Request $request)
    {
        $importUsers = new ImportUsers();

        $form = $this->createForm('fs_import_users', $importUsers);
        return ['form' => $form->createView()];
    }

    /**
     * @Route("/update", name="admin_import_update")
     */
    public function importAction(Request $request)
    {
        $importUsers = new ImportUsers();

        $form = $this->createForm('fs_import_users', $importUsers);

        $form->handleRequest($request);

        if ($form->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $message = [];
            // check for extension of file
            $fileResult = false;

            $ext = @pathinfo($importUsers->getFile()->getClientOriginalName(), PATHINFO_EXTENSION);
            $users = [];
            switch ($ext) {
                case 'xls':
                case 'xlsx':
                case'csv':
                    /** @var PhpExcel $phpExcelObject */
                    $phpExcelObject = $this->get('phpexcel')
                        ->createPHPExcelObject($importUsers->getFile()->getRealPath());
                    $aSheet = $phpExcelObject->getActiveSheet();
                    $counter = 0;
                    $headers = [];
                    $badInput = false;
                    foreach ($aSheet->getRowIterator() as $row) {
                        //header
                        if ($counter++ == 0) {
                            $cellIterator = $row->getCellIterator();
                            foreach ($cellIterator as $cell) {
                                $headers[] = $cell->getValue();
                            }
                            continue;
                        }

                        $headers = array_map('strtolower', $headers);
                        if (!in_array("username", $headers) || !in_array("password", $headers)) {
                            $badInput = true;
                            break;
                        }


                        $cellIterator = $row->getCellIterator();
                        $userToAdd = new User();
                        $userToAdd->addSelfToAccessedUsers();
                        $userToAdd->setEnabled(true);
                        $userToAdd->setType(User::TYPE_LOCAL);
                        if ($importUsers->getUserGroup()) {
                            $userToAdd->setUserGroup($importUsers->getUserGroup());
                        }
                        $cellNum = 0;
                        foreach ($cellIterator as $cell) {
                            if (strtolower($headers[$cellNum]) == 'username')
                                $userToAdd->setUsername($cell->getValue());
                            else if (strtolower($headers[$cellNum]) == 'password')
                                $userToAdd->setPlainPassword($cell->getValue());
                            else if (strtolower($headers[$cellNum]) == 'description')
                                $userToAdd->setDescription($cell->getValue());
                            $cellNum++;
                        }
                        if(!($userToAdd->getUsername() == null || $userToAdd->getPlainPassword() == null)){
                            $users[] = $userToAdd;
                        }
                    }
                    $fileResult = !$badInput;
                    break;
                case 'txt':
                    $handle = fopen($importUsers->getFile()->getRealPath(), "r");
                    if ($handle) {
                        $counter = 0;
                        $badInput = false;
                        while (($line = fgets($handle)) !== false) {
                            if ($counter++ == 0) {
                                $headers = preg_split("/(\:|\,|\;|\.)/", $line);
                                continue;
                            }
                            $headers = array_map(function ($header) {
                                $header = trim(preg_replace('/\s\s+/', ' ', $header));
                                $header = strtolower($header);
                                return $header;
                            }, $headers);

                            if (!in_array("username", $headers) || !in_array("password", $headers)) {
                                $badInput = true;
                                break;
                            }
                            $userToAdd = new User();
                            $userToAdd->addSelfToAccessedUsers();
                            $userToAdd->setEnabled(true);
                            $userToAdd->setType(User::TYPE_LOCAL);
                            if ($importUsers->getUserGroup()) {
                                $userToAdd->setUserGroup($importUsers->getUserGroup());
                            }
                            // process the line read.
                            $vars = preg_split("/(\:|\,|\;|\.)/", $line);
                            $cellNum = 0;
                            try {
                                foreach ($vars as $var) {
                                    $var = trim(preg_replace('/\s\s+/', ' ', $var));
                                    $header = trim(preg_replace('/\s\s+/', ' ', @$headers[$cellNum]));
                                    if ($header == 'username')
                                        $userToAdd->setUsername($var);
                                    else if ($header == 'password')
                                        $userToAdd->setPlainPassword($var);
                                    else if ($header == 'description')
                                        $userToAdd->setDescription($var);
                                    $cellNum++;
                                }
                                if(!($userToAdd->getUsername() == null || $userToAdd->getPlainPassword() == null)){
                                    $users[] = $userToAdd;
                                }
                            } catch (\Exception $e) {
                                /*NOP*/
                            }
                        }

                        $fileResult = !$badInput;
                        fclose($handle);
                    } else {
                        /*NOP*/
                    }
                    break;
                default:
                    break;
            }
            $usersAdd = 0;
            if (count($users) > 0) {
                $usersFromDB = $em->getRepository('FSUserBundle:User')->findAll();
                foreach ($users as $user) {
                    if (!($this->arraySearchUsers($user->getUsername(), $usersFromDB))) {
                        try {
                            $this->container->get('fos_user.user_manager')->updateUser($user);
                        } catch (DBALException $e) {
                            $fileResult = false;
                        }
                        $usersAdd += 1;
                    }
                }

            }
            if ($fileResult == false) {
                $message['type'] = 'error';
                $message['body'] = 'Wrong file format';
            } elseif (count($users) == 0) {
                $message['type'] = 'error';
                $message['body'] = 'Users not found';
            } else {
                $message['type'] = 'note';
                if ($usersAdd == 1) $message['body'] = $usersAdd . ' user added';
                else $message['body'] = $usersAdd . ' users added';
            }
            $this->get('session')->getFlashBag()->add(
                $message['type'],
                $message['body']
            );
            return new RedirectResponse($this->generateUrl('admin_import'));
        }
        return $this->render(
            'FSUserBundle:Import:index.html.twig',
            [
                'form' => $form->createView()
            ]
        );
    }

    private function arraySearchUsers($username, $usersArray)
    {

        foreach ($usersArray as $key => $value) {
            if ($value->getUsername() === $username && !$value->isDeleted()) {
                return true;
            }
        }
        return false;
    }
}