<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 01.06.2015
 * Time: 17:24
 */

namespace FS\UserBundle\Controller;

use FS\UserBundle\Entity\LdapServer;
use FS\UserBundle\Entity\LdapSynchronize;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Model\LdapUser;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class LDAPController
 * @package FS\UserBundle\Controller
 * @Route("/admin/user-ldap")
 * @Security(expression="user.hasRight('admin--users-and-groups') || is_granted('ROLE_ADMIN')")
 */
class LDAPController extends Controller
{
    /**
     * @Route("/", name="admin_ldap")
     * @Template()
     * @param Request $request
     * @return array
     */
    public function indexAction(Request $request)
    {
        $showAll = $request->query->get('show_all', false);
        return
            [
                'form_show_all' => $showAll,
                'edit_ldap_sync' => false,
                'ldapServerForm' => $this->createForm('fs_ldap_server', $this->getLdapServer())->createView(),
                'ldapUserForm' => $this->createForm('fs_ldap_user', new LdapUser())->createView(),
                'ldapSynchronizeForm' => $this->createForm('fs_ldap_sync', new LdapSynchronize())->createView(),
                'ldapSyncSchedules' => $this->getLdapSyncSchedules(),
            ];
    }

    /**
     * @return LdapServer
     */
    private function getLdapServer()
    {
        $em = $this->getDoctrine()
            ->getManager();

        if (null === $ldapServer = $em->getRepository('FSUserBundle:LdapServer')->getLdapServer()) {
            $ldapServer = new LdapServer();
        }

        return $ldapServer;
    }

    private function getLdapSyncSchedules()
    {
        return $this->get('fs_user.container_manager')
            ->filterArray(
                $this->getDoctrine()->getManager()->getRepository("FSUserBundle:LdapSynchronize")->findAll()
            );

    }

    /**
     * @param Request $request
     *
     * @Route("/ldap-server", name="admin_ldap_server")
     * @return RedirectResponse
     */
    public function ldapServerAction(Request $request)
    {
        //Here i update ldap server
        $ldapServer = $this->getLdapServer();
        $ldapServerForm = $this->createForm('fs_ldap_server', $ldapServer);

        if ($ldapServerForm->submit($request)->isValid()) {
            $result = $this->checkLdapConnect($ldapServer);

            if ($result['status']) {
                $em = $this->getDoctrine()
                    ->getManager();
                $ldapServer->setDn($result['dn']);
                $em->persist($ldapServer);
                $em->flush();
            }
            $this->get('session')->getFlashBag()->add(
                $result['type'],
                $result['body']
            );
            return new RedirectResponse($this->generateUrl('admin_ldap', ['show_all' => $result['status']]));
        }

        return $this->render(
            'FSUserBundle:LDAP:index.html.twig',
            [
                'form_show_all' => true,
                'edit_ldap_sync' => false,
                'ldapServerForm' => $ldapServerForm->createView(),
                'ldapUserForm' => $this->createForm('fs_ldap_user', new LdapUser())->createView(),
                'ldapSynchronizeForm' => $this->createForm('fs_ldap_sync', new LdapSynchronize())->createView(),
                'ldapSyncSchedules' => $this->getLdapSyncSchedules(),
            ]
        );
    }

    /**
     * @param LdapServer $ldapServer
     * @return mixed
     */
    private function checkLdapConnect(LdapServer $ldapServer)
    {
        $result['status'] = false;
        $result['type'] = 'server_error';
        $conn = @ldap_connect($ldapServer->getServer(), '389');
        if ($conn) {
            @ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3);
            $ldapBind = @ldap_bind($conn, $ldapServer->getUsername(), $ldapServer->getPassword());
            if ($ldapBind) {
                $dn = '';
                if(strpos($ldapServer->getUsername(), "@") !== false){
                    $domain = substr(strrchr($ldapServer->getUsername(), "@"), 1);
                    $dnArr = explode('.',$domain);
                    foreach ($dnArr as $item) {
                        $dn .= ',dc='.$item;
                    }
                } else {
                    $nameArray = $this->parseLdapDn($ldapServer->getUsername());
                    foreach ($nameArray['dc'] as $dc)
                        $dn .= ',dc=' . $dc;;
                }
                $dn = substr($dn, 1);

                $result['status'] = true;
                $result['type'] = 'server_note';
                $result['body'] = 'Connected';
                $result['dn'] = $dn;
            } else {
                $result['body'] = 'Wrong credentials';
            }
        } else {
            $result['body'] = 'Unable to connect LDAP server';
        }
        @ldap_close($conn);

        return $result;
    }

    /**
     * @author gabriel at hrz dot uni-marburg dot de 05-Aug-2003 02:27 (part of the character replacement)
     * @author Renoir Boulanger
     * @param $dn
     * @return array
     */
    private function parseLdapDn($dn)
    {
        $parsr = ldap_explode_dn($dn, 0);
        $out = array();
        foreach ($parsr as $key => $value) {
            if (false !== strstr($value, '=')) {
                list($prefix, $data) = explode("=", $value);
                $prefix = strtolower($prefix);
                if (isset($current_prefix) && $prefix == $current_prefix) {
                    $out[$prefix][] = $data;
                } else {
                    $current_prefix = $prefix;
                    $out[$prefix][] = $data;
                }
            }
        }
        return $out;
    }

    /**
     * @Route("/ldap-user", name="admin_ldap_user")
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function ldapUserAction(Request $request)
    {
        $ldapServer = $this->getLdapServer();
        $ldapUser = new LdapUser();

        $ldapAddUserForm = $this->createForm('fs_ldap_user', $ldapUser);

        if ($ldapAddUserForm->submit($request)->isValid()) {
            $result['type'] = 'user_import_error';
            $result['body'] = 'Unable to connect LDAP server';

            $conn = @ldap_connect($ldapServer->getServer(), '389');
            if ($conn) {
                @ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3);
                $ldapBind = @ldap_bind($conn, $ldapServer->getUsername(), $ldapServer->getPassword());
                if ($ldapBind) {
                    $fields = ["ou", "sn", "cn", "samaccountname", "givenname", "mail", "dn", "uid"];
                    $search = @ldap_search(
                        $conn,
                        $ldapUser->getBindScript() . "," . $ldapServer->getDn(),
                        'cn=*',
                        $fields
                    );
                    if ($search) {
                        $entry = @ldap_first_entry($conn, $search);
                        $attrs = @ldap_get_attributes($conn, $entry);
                        if (array_key_exists('uid', $attrs)) {
                            $username = $attrs['uid'][0];
                        } else if (array_key_exists('samaccountname', $attrs[0])) {
                            $username = $attrs[0]['samaccountname'][0];
                        } else {
                            $username = $attrs['cn'][0];
                        }
                        $userDn = @ldap_get_dn($conn, $entry);

                        $em = $this->getDoctrine()
                            ->getManager();

                        $user = $em->getRepository('FSUserBundle:User')->findLDAPUserByDN($userDn);
                        if ($user) {
                            $result['type'] = 'user_import_note';
                            $result['body'] = 'User ' . $username . ' already imported';
                        } else {
                            $userToAdd = new User();
                            $userToAdd->addSelfToAccessedUsers();
                            $userToAdd->setEnabled(true);
                            $userToAdd->setUsername($username);
                            $userToAdd->setPlainPassword('LdapUser' . $username);

                            if ($ldapUser->getUserGroup()) {
                                $userToAdd->setUserGroup($ldapUser->getUserGroup());
                            }

                            $userToAdd->setType(User::TYPE_LDAP);
                            $userToAdd->setLdapDomain($ldapServer->getDn());
                            $userToAdd->setLdapDN($userDn);
                            $userToAdd->setLdapScript($ldapUser->getBindScript());

                            $usersFromDB = $em->getRepository('FSUserBundle:User')->findAll();
                            if (!($this->arraySearchUsers($userToAdd->getUsername(), $usersFromDB))) {
                                $this->container->get('fos_user.user_manager')->updateUser($userToAdd);

                                $result['type'] = 'user_import_note';
                                $result['body'] = 'User ' . $username . ' imported';
                            } else {
                                $result['type'] = 'user_import_note';
                                $result['body'] = 'User ' . $username . ' already registered';
                            }
                        }

                    } else {
                        $result['body'] = 'User not found';
                    }
                }
            }

            $this->get('session')->getFlashBag()->add(
                $result['type'],
                $result['body']
            );
            return new RedirectResponse($this->generateUrl('admin_ldap'));
        }

        return $this->render(
            'FSUserBundle:LDAP:index.html.twig',
            [
                'form_show_all' => true,
                'edit_ldap_sync' => false,
                'ldapServerForm' => $this->createForm('fs_ldap_server', $ldapServer)->createView(),
                'ldapUserForm' => $ldapAddUserForm->createView(),
                'ldapSynchronizeForm' => $this->createForm('fs_ldap_sync', new LdapSynchronize())->createView(),
                'ldapSyncSchedules' => $this->getLdapSyncSchedules(),
            ]
        );
    }

    /**
     * @Route("/ldap-sync", name="admin_ldap_sync")
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function ldapSyncAction(Request $request)
    {
        $ldapServer = $this->getLdapServer();

        $ldapSynchronize = new LdapSynchronize();

        $ldapSynchronizeForm = $this->createForm('fs_ldap_sync', $ldapSynchronize);

        $em = $this->getDoctrine()
            ->getManager();

        if ($ldapSynchronizeForm->submit($request)->isValid()) {
            $ldapSynchronize->setStatus(LdapSynchronize::LDAP_SYNCHRONIZE__STATUS_NOT_SYNCHRONIZED);
            $em->persist($ldapSynchronize);
            $em->flush();

            if ($request->request->has('import')) {
                $syncRes = $this->get('fs_user.util.ldap_synchronizer')->synchronize($ldapSynchronize);
                if ($syncRes['status'] == true) {
                    if ($ldapSynchronize->getSyncInterval() == LdapSynchronize::LDAP_SYNCHRONIZE__SYNC_INTERVAL_NONE) {
                        $ldapSynchronize->setStatus(LdapSynchronize::LDAP_SYNCHRONIZE__STATUS_SYNCHRONIZED);
                    } else {
                        $ldapSynchronize->setStatus(LdapSynchronize::LDAP_SYNCHRONIZE__STATUS_RECURRENCE_SYNCHRONIZATION);
                    }
                }
                $this->get('session')->getFlashBag()->add(
                    $syncRes['type'],
                    $syncRes['body']
                );
            }


            return new RedirectResponse($this->generateUrl('admin_ldap', ['show_all' => true]));
        }

        return $this->render(
            'FSUserBundle:LDAP:index.html.twig',
            [
                'form_show_all' => true,
                'edit_ldap_sync' => false,
                'ldapServerForm' => $this->createForm('fs_ldap_server', $ldapServer)->createView(),
                'ldapUserForm' => $this->createForm('fs_ldap_user', new LdapUser())->createView(),
                'ldapSynchronizeForm' => $ldapSynchronizeForm->createView(),
                'ldapSyncSchedules' => $this->getLdapSyncSchedules(),
            ]
        );
    }

    /**
     * @Route("/ldap-sync-edit/{id}", name="admin_ldap_sync_edit")
     * @param Request $request
     * @param LdapSynchronize $ldapSynchronize
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function ldapSyncEditAction(Request $request, LdapSynchronize $ldapSynchronize)
    {
        $ldapServer = $this->getLdapServer();

        $ldapSynchronizeForm = $this->createForm('fs_ldap_sync', $ldapSynchronize);

        return $this->render(
            'FSUserBundle:LDAP:index.html.twig',
            [
                'form_show_all' => true,
                'edit_ldap_sync' => true,
                'ldapsync' => $ldapSynchronize,
                'ldapServerForm' => $this->createForm('fs_ldap_server', $ldapServer)->createView(),
                'ldapUserForm' => $this->createForm('fs_ldap_user', new LdapUser())->createView(),
                'ldapSynchronizeForm' => $ldapSynchronizeForm->createView(),
                'ldapSyncSchedules' => $this->getLdapSyncSchedules(),
            ]
        );
    }

    /**
     * @Route("/ldap-sync-update/{id}", name="admin_ldap_sync_update")
     * @param Request $request
     * @param LdapSynchronize $ldapSynchronize
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function ldapSyncUpdateAction(Request $request, LdapSynchronize $ldapSynchronize)
    {
        $ldapServer = $this->getLdapServer();

        $ldapSynchronizeForm = $this->createForm('fs_ldap_sync', $ldapSynchronize);

        $em = $this->getDoctrine()
            ->getManager();

        if ($ldapSynchronizeForm->submit($request)->isValid()) {
            $ldapSynchronize->setDateUpdate(new \DateTime());
            $em->persist($ldapSynchronize);
            $em->flush();

            if ($request->request->has('import')) {
                $syncRes = $this->get('fs_user.util.ldap_synchronizer')->synchronize($ldapSynchronize);
                if ($syncRes['status'] == true) {
                    if ($ldapSynchronize->getSyncInterval() == LdapSynchronize::LDAP_SYNCHRONIZE__SYNC_INTERVAL_NONE) {
                        $ldapSynchronize->setStatus(LdapSynchronize::LDAP_SYNCHRONIZE__STATUS_SYNCHRONIZED);
                    } else {
                        $ldapSynchronize->setStatus(LdapSynchronize::LDAP_SYNCHRONIZE__STATUS_RECURRENCE_SYNCHRONIZATION);
                    }
                }
                $this->get('session')->getFlashBag()->add(
                    $syncRes['type'],
                    $syncRes['body']
                );
            }

            return new RedirectResponse($this->generateUrl('admin_ldap', ['show_all' => true]));
        }

        return $this->render(
            'FSUserBundle:LDAP:index.html.twig',
            [
                'form_show_all' => true,
                'edit_ldap_sync' => true,
                'ldapsync' => $ldapSynchronize,
                'ldapServerForm' => $this->createForm('fs_ldap_server', $ldapServer)->createView(),
                'ldapUserForm' => $this->createForm('fs_ldap_user', new LdapUser())->createView(),
                'ldapSynchronizeForm' => $ldapSynchronizeForm->createView(),
                'ldapSyncSchedules' => $this->getLdapSyncSchedules(),
            ]
        );
    }

    /**
     * @param $username
     * @param $usersArray
     * @return bool|int|string
     */
    private function arraySearchUsers($username, $usersArray)
    {
        foreach ($usersArray as $key => $value) {
            /** @var User $value */
            if ($value->getUsername() === $username && !$value->isDeleted()) {
                return true;
            }
        }
        return false;
    }
}