<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 06.04.2015
 * Time: 13:28
 */

namespace FS\UserBundle\Controller;

use FS\CamBundle\Entity\Alert;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Form\Type\AlertFilterType;
use FS\UserBundle\Entity\Log;
use FS\UserBundle\Form\Type\LogFilterType;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Class LogController
 * @package FS\UserBundle\Controller
 *
 * @Route("/log")
 */
class LogController extends Controller
{
    private static $activeLinks = [
        Log::OB_TYPE_ADMIN => "log_admin",
        Log::OB_TYPE_ROOM => "log_camera_room",
        Log::OB_TYPE_CAMERA => "log_camera_room",
        Log::OB_TYPE_RECORDING => "log_recording",
        Log::OB_TYPE_VIDEO => "log_video",
        Log::OB_TYPE_USER => "log_login",
    ];

    /**
     * @Route(path="/", name="admin_log")
     * @Template()
     * @Security(expression="is_granted('ROLE_ADMIN') || user.hasRight('admin--logs')")
     */
    public function indexAction()
    {
        return $this->redirect($this->generateUrl('admin_log_show', ['type' => 'User']));
    }

    /**
     * @Route(path="/show/{type}", name="admin_log_show")
     * @Template()
     * @Security(expression="is_granted('ROLE_ADMIN') || user.hasRight('admin--logs')")
     * @param Request $request
     * @param $type
     * @return array
     */
    public function showAction(Request $request, $type)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $form = $this->createForm(new LogFilterType());

        $hasFilter = $this->get('request')->query->has('log_filter');

        $logs = $em->getRepository("FSUserBundle:Log")->getLogsByType($type);
        $form->submit($request);

        if ($hasFilter) {
            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $logs);
        }
        $logs = $logs->getQuery()->execute();

        $logs = array_unique($logs, SORT_REGULAR);
        $logs = $this->get('fs_user.container_manager')->filterArray($logs);
        $paginator = $this->get('knp_paginator')->paginate($logs, $request->query->get('page', 1), 20);

        return [
            'activeLink' => LogController::$activeLinks[$type],
            'type' => $type,
            'form' => $form->createView(),
            'paginator' => $paginator,
            'hasFilter' => $hasFilter
        ];
    }

    /**
     * @Route(path="/show/alert/{type}", name="admin_log_alerts_show")
     * @Template()
     * @Security(expression="is_granted('ROLE_ADMIN') || user.hasRight('admin--logs')")
     * @param Request $request
     * @param $type
     * @return array
     */
    public function showAlertsAction(Request $request, $type)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $form = $this->createForm(new AlertFilterType());

        $hasFilter = $this->get('request')->query->has('alert_filter');

        $logs = $em->getRepository("FSCamBundle:Alert")->getAllAlertsForFiltering();
        $form->submit($request);

        if ($hasFilter) {
            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $logs);
        }
        $logs = $logs->getQuery()->execute();

        $logs = array_unique($logs, SORT_REGULAR);
        $logs = $this->get('fs_user.container_manager')->filterArray($logs);
        $paginator = $this->get('knp_paginator')->paginate($logs, $request->query->get('page', 1), 20);

        return [
            'activeLink' => 'log_alerts',
            'type' => $type,
            'form' => $form->createView(),
            'paginator' => $paginator,
            'hasFilter' => $hasFilter
        ];
    }

    /**
     * @Route("/room/{id}", name="log_room_view", options={"expose"=true})
     *
     * @param Room $room
     * @return JsonResponse
     */
    public function roomViewedAction(Room $room)
    {
        $this->container->get('fs_user.util.logger')->logAction(
            Log::OB_TYPE_ROOM,
            $room,
            $this->getUser(),
            Log::$actions[Log::OB_TYPE_ROOM]['view']);
        return new JsonResponse(['processed']);
    }

    /**
     * @Route("/camera/{id}", name="log_camera_view", options={"expose"=true})
     *
     * @param Camera $camera
     * @return JsonResponse
     */
    public function cameraViewedAction(Camera $camera)
    {
        $this->container->get('fs_user.util.logger')->logAction(
            Log::OB_TYPE_CAMERA,
            $camera,
            $this->getUser(),
            Log::$actions[Log::OB_TYPE_CAMERA]['view']);
        return new JsonResponse(['processed']);
    }

    /**
     * @Route("/record/{id}/start", name="log_record_start", options={"expose"=true})
     *
     * @param Record $record
     * @return JsonResponse
     */
    public function logStartRecordingAction(Record $record)
    {
        $this->container->get('fs_user.util.logger')->logAction(
            Log::OB_TYPE_RECORDING,
            $record,
            $this->getUser(),
            Log::$actions[Log::OB_TYPE_RECORDING]['start']);
        return new JsonResponse(['processed']);
    }

    /**
     * @Route("/record/{id}/stop", name="log_record_stop", options={"expose"=true})
     *
     * @param Record $record
     * @return JsonResponse
     */
    public function logStopRecordingAction(Record $record)
    {
        $this->container->get('fs_user.util.logger')->logAction(
            Log::OB_TYPE_RECORDING,
            $record,
            $this->getUser(),
            Log::$actions[Log::OB_TYPE_RECORDING]['stop']);
        return new JsonResponse(['processed']);
    }

    /**
     * @Route("/search-result/export/pdf/{type}", name="logs_search_result_export_pdf")
     */
    public function exportPDFLogsAction(Request $request, $type)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $form = $this->createForm(new AlertFilterType());

        $hasFilter = $this->get('request')->query->has('log_filter');
        if ($hasFilter) {

            $logs = $em->getRepository("FSUserBundle:Log")->getLogsByType($type);

            $form->submit($request);

            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $logs);

            $logs = $logs->getQuery()->execute();


        } else {
            $logs = [];
        }

        $logs = array_unique($logs, SORT_REGULAR);

        $html = $this->renderView('FSUserBundle:Log/Pdf:singleLog.html.twig', [
            'logs' => $logs,
        ]);

        $pdfGenerator = $this->get('spraed.pdf.generator');
        return new Response($pdfGenerator->generatePDF($html, 'UTF-8'),
        //$pdfGenerator = $this->get('knp_snappy.pdf');
        //return new Response($pdfGenerator->getOutputFromHtml($html, ['encoding' => 'utf-8']),
            200,
            [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' =>
                    'attachment; filename="' .
                    "logs_" .
                    $type .
                    '.pdf"'
            ]
        );
    }

    /**
     * @Route("/search-result/export/csv/{type}", name="logs_search_result_export_csv")
     * @param Request $request
     * @param $type
     * @return StreamedResponse
     */
    public function exportCVSLogsAction(Request $request, $type)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $form = $this->createForm(new LogFilterType());

        $hasFilter = $this->get('request')->query->has('log_filter');
        if ($hasFilter) {

            $logs = $em->getRepository("FSUserBundle:Log")->getLogsByType($type);

            $form->submit($request);

            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $logs);

            $logs = $logs->getQuery()->execute();

        } else {
            $logs = [];
        }

        $logs = array_unique($logs, SORT_REGULAR);

        $response = new StreamedResponse();
        $response->setCallback(function () use ($logs) {

            $handle = fopen('php://output', 'w+');

            // Add a row with the names of the columns for the CSV file
            fputcsv($handle, array('user', 'action', 'type', 'name', 'time'), ',');
            /** @var Log $log */
            foreach ($logs as $log) {
                if ($log->getUser() != null) {
                    $user = $log->getUser()->getUsername();
                } else {
                    $user = "Failed Login";
                }
                fputcsv($handle, array(
                    $user,
                    $log->getAction(),
                    $log->getObjectType(),
                    $log->getObjectName(),
                    $log->getActionDate()->format('g:i:s A m/d/Y')
                ), ',');
            }
            fclose($handle);
        });

        $response->setStatusCode(200);
        $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
        $response->headers->set('Content-Disposition', 'attachment; filename="logs_' . $type . '.csv"');

        return $response;
    }

    /**
     * @Route("/search-result/export/logs/pdf", name="alerts_search_result_export_pdf")
     */
    public function exportPDFAlertsAction(Request $request)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $form = $this->createForm(new AlertFilterType());

        $hasFilter = $this->get('request')->query->has('alert_filter');
        if ($hasFilter) {

            $logs = $em->getRepository("FSCamBundle:Alert")->getAllAlertsForFiltering();

            $form->submit($request);

            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $logs);

            $logs = $logs->getQuery()->execute();

        } else {
            $logs = [];
        }

        $logs = array_unique($logs, SORT_REGULAR);

        $html = $this->renderView('FSUserBundle:Log/Pdf:singleAlert.html.twig', [
            'logs' => $logs,
        ]);

        $pdfGenerator = $this->get('spraed.pdf.generator');
        return new Response($pdfGenerator->generatePDF($html, 'UTF-8'),
        //$pdfGenerator = $this->get('knp_snappy.pdf');
        //return new Response($pdfGenerator->getOutputFromHtml($html, ['encoding' => 'utf-8']),
            200,
            [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' =>
                    'attachment; filename="' .
                    "logs_Alert" .
                    '.pdf"'
            ]
        );

    }

    /**
     * @Route("/search-result/export/logs/csv", name="alerts_search_result_export_csv")
     * @param Request $request
     * @return StreamedResponse
     */
    public function exportCVSAlertsAction(Request $request)
    {
        $em = $this->getDoctrine()
            ->getManager();

        $form = $this->createForm(new AlertFilterType());

        $hasFilter = $this->get('request')->query->has('alert_filter');
        if ($hasFilter) {

            $logs = $em->getRepository("FSCamBundle:Alert")->getAllAlertsForFiltering();

            $form->submit($request);

            $this->get('lexik_form_filter.query_builder_updater')->addFilterConditions($form, $logs);

            $logs = $logs->getQuery()->execute();

        } else {
            $logs = [];
        }

        $logs = array_unique($logs, SORT_REGULAR);

        $response = new StreamedResponse();
        $response->setCallback(function () use ($logs) {

            $handle = fopen('php://output', 'w+');

            // Add a row with the names of the columns for the CSV file
            fputcsv($handle, array('type', 'name', 'message', 'time'), ',');
            /** @var Alert $log */
            foreach ($logs as $log) {

                fputcsv($handle, array(
                    $log->getAlertType(),
                    $log->getName(),
                    $log->getMessage(),
                    $log->getCreatedAt()->format('g:i:s A m/d/Y')
                ), ',');
            }
            fclose($handle);
        });

        $response->setStatusCode(200);
        $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
        $response->headers->set('Content-Disposition', 'attachment; filename="logs_Alert.csv"');

        return $response;
    }
}