<?php

namespace FS\UserBundle\Controller;

use FS\UserBundle\Entity\UserGroupRight;
use FS\UserBundle\Form\Type\IORecordType;
use FS\UserBundle\Model\UserGroupSearchResult;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use FS\UserBundle\Entity\UserGroup;
use FS\UserBundle\Form\Type\UserGroupType;
use Symfony\Component\Form\Form;
use Symfony\Component\Form\FormBuilder;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class UserGroupAdminController
 * @Route("/admin/user-group")
 * @Security(expression="user.hasRight('admin--users-and-groups') || is_granted('ROLE_ADMIN')")
 */
class UserGroupAdminController extends Controller
{

    /**
     * @Template()
     */
    public function listAction()
    {
        $em = $this->getDoctrine()->getManager();

        $userGroups = $em->getRepository('FSUserBundle:UserGroup')->getAllGroups();

        $userGroups = $this->get('fs_user.container_manager')->filterArray($userGroups);

        $woGroup = $em->getRepository('FSUserBundle:User')->findByUserGroup(null, ['username' => 'ASC']);

        $adminGroup = $em->getRepository('FSUserBundle:UserGroup')->findOneBy(['id'=>1]);
        if (!$this->getUser()->getUserGroup()->getContainer()->isVisible($adminGroup->getContainer())) {
            $adminGroup = null;
        }

        return [
            'userGroups' => $userGroups,
            'woGroup' => $woGroup,
            'adminGroup' =>$adminGroup,
        ];
    }

    /**
     * Displays a form to create a new UserGroup entity.
     *
     * @Route("/new", name="admin_user_group_new")
     * @Template()
     */
    public function newAction()
    {
        $userGroup = new UserGroup();
        $userGroup->setSearchResults(UserGroupSearchResult::getUserGroupSearchResultsList());
        $form = $this->createForm('user_group', $userGroup);

        return array(
            'userGroup' => $userGroup,
            'form' => $form->createView(),
        );
    }

    /**
     * Creates a new UserGroup entity.
     *
     * @Route("/create", name="admin_user_group_create")
     * @Method("POST")
     * @Template("FSUserBundle:UserGroupAdmin:new.html.twig")
     */
    public function createAction(Request $request)
    {
        $userGroup = new UserGroup();
        $form = $this->createForm('user_group', $userGroup);

        if ($form->submit($request)->isValid()) {
            $userGroup->addSelfToAccessedGroups();
            $em = $this->getDoctrine()->getManager();
            $em->persist($userGroup);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'New group has been successfully added'
            );

            return $this->redirect($this->generateUrl('admin_user_group_edit', ['id' => $userGroup->getId()]));
        }

        return array(
            'userGroup' => $userGroup,
            'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to edit an existing UserGroup entity.
     *
     * @Route("/edit/{id}", name="admin_user_group_edit")
     * @Template()
     * @param UserGroup $userGroup
     * @return array
     */
    public function editAction(UserGroup $userGroup)
    {

        $editForm = $this->createForm('user_group', $userGroup);
        $editFormView = $editForm->createView();

        if ($this->getUser()->getUserGroup() == $userGroup) {
            // Setting admin right checkbox to disabled to prevent removing admin right
            foreach ($editFormView->children['rights']->children as $right) {
                if (UserGroupRight::GRANT_ADMIN === $right->vars['value']) {
                    $right->vars['attr']['disabled'] = 'disabled';
                    $right->vars['checked'] = 'true';
                }
            }
        }

        return array(
            'userGroup' => $userGroup,
            'form' => $editFormView,
        );
    }

    /**
     * Displays a form to edit an existing UserGroup entity.
     *
     * @Route("/editIO/{id}", name="admin_io_group_edit")
     * @Template()
     * @param UserGroup $userGroup
     * @return array
     */
    public function editIOAction(UserGroup $userGroup)
    {
        $editIOForm = $this->createForm(new IORecordType(), $userGroup);
        $editIOFormView = $editIOForm->createView();

        return array(
            'userGroup' => $userGroup,
            'form' => $editIOFormView,
        );
    }

    /**
     * Edits an existing UserGroup entity.
     *
     * @Route("/update/{id}", name="admin_user_group_update")
     * @Method("POST")
     * @Template("FSUserBundle:UserGroupAdmin:edit.html.twig")
     */
    public function updateAction(UserGroup $userGroup, Request $request)
    {
        $editForm = $this->createForm('user_group', $userGroup);
        if ($editForm->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            if ($this->getUser()->getUserGroup() == $userGroup) {
                $userGroupRight = new UserGroupRight();
                $userGroupRight->setGranted(UserGroupRight::GRANT_ADMIN);
                $userGroup->addRight($userGroupRight);
                $em->persist($userGroup);
            }
            $em->flush();
            return $this->redirect($this->generateUrl('admin_user_group_edit', ['id' => $userGroup->getId()]));
        }

        $editFormView = $editForm->createView();

        if ($this->getUser()->getUserGroup() == $userGroup) {
            // Setting admin right checkbox to disabled to prevent removing admin right
            foreach ($editFormView->children['rights']->children as $right) {
                if (UserGroupRight::GRANT_ADMIN === $right->vars['value']) {
                    $right->vars['attr']['disabled'] = 'disabled';
                    $right->vars['checked'] = 'true';
                }
            }
        }

        return array(
            'userGroup' => $userGroup,
            'form' => $editFormView,
        );

    }

    /**
     * Edits an existing UserGroup entity.
     *
     * @Route("/updateIO/{id}", name="admin_io_group_update")
     * @Method("POST")
     * @Template("FSUserBundle:UserGroupAdmin:editIO.html.twig")
     * @param UserGroup $userGroup
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function updateIOAction(UserGroup $userGroup, Request $request)
    {
        $editForm = $this->createForm(new IORecordType(), $userGroup);
        if ($editForm->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            if ($this->getUser()->getUserGroup() == $userGroup) {
                $userGroupRight = new UserGroupRight();
                $userGroupRight->setGranted(UserGroupRight::GRANT_ADMIN);
                $userGroup->addRight($userGroupRight);
                $em->persist($userGroup);
            }
            $em->flush();
        }

        return $this->redirect($this->generateUrl('admin_io_group_edit', ['id' => $userGroup->getId()]));
    }

    /**
     * Displays a form to delete a UserGroup entity.
     *
     * @Route("/{id}/remove", name="admin_user_group_remove")
     * @Template()
     */
    public function removeAction(UserGroup $userGroup, Request $request)
    {
        $form = $this->createDeleteForm($userGroup->getId());

        return array('userGroup' => $userGroup, 'form' => $form->createView());
    }

    /**
     * Deletes a UserGroup entity.
     *
     * @Route("/{id}/delete", name="admin_user_group_delete")
     * @Method("POST")
     */
    public function deleteAction(UserGroup $userGroup, Request $request)
    {
        $form = $this->createDeleteForm($userGroup->getId());
        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $em->remove($userGroup);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'Group has been successfully deleted'
            );
        }

        return $this->redirect($this->generateUrl('admin_user'));
    }


    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return Form
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }

    /**
     * Displays a form to delete a UserGroup entity.
     *
     * @Route("/{id}/copy", name="admin_user_group_copy")
     * @Template()
     */
    public function copyGroupAction(UserGroup $userGroup, Request $request)
    {
        $form = $this->createCloneForm($userGroup->getId());

        return array('userGroup' => $userGroup, 'form' => $form->createView());
    }

    /**
     * Deletes a UserGroup entity.
     *
     * @Route("/{id}/clone", name="admin_user_group_clone")
     * @Method("POST")
     * @param UserGroup $userGroup
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function cloneAction(UserGroup $userGroup, Request $request)
    {
        $form = $this->createCloneForm($userGroup->getId());
        if ($form->submit($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();
            $newUserGroup = $userGroup->makeDeepCopy();
            $newUserGroup->setName($userGroup->getName(). "(Copy)");
            $em->persist($newUserGroup);
            $em->flush();
        }

        return $this->redirect($this->generateUrl('admin_user'));
    }


    /**
     * Create Clone form
     *
     * @param  integer $id
     * @return Form
     */
    protected function createCloneForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
            ->add('id', 'hidden')
            ->getForm();
    }
}