<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 22.03.2016
 * Time: 13:46
 */

namespace FS\UserBundle\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Model\ContainerizedInterface;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity(repositoryClass="FS\UserBundle\Repository\LdapSynchronizeRepository")
 * @ORM\Table(name="ldap_synchronize")
 */
class LdapSynchronize implements ContainerizedInterface
{
    use FSContainerTrait;

    const LDAP_SYNCHRONIZE__SYNC_INTERVAL_NONE = 'none';
    const LDAP_SYNCHRONIZE__SYNC_INTERVAL_DAILY = 'daily';
    const LDAP_SYNCHRONIZE__SYNC_INTERVAL_WEEKLY = 'weekly';


    const LDAP_SYNCHRONIZE__STATUS_SYNCHRONIZED = 'synced';
    const LDAP_SYNCHRONIZE__STATUS_NOT_SYNCHRONIZED = 'not_synced';
    const LDAP_SYNCHRONIZE__STATUS_RECURRENCE_SYNCHRONIZATION = 'rec_synced';

    /**
     * @var integer
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var integer
     *
     * @Assert\NotBlank()
     *
     * @ORM\Column(type="text")
     */
    private $name;
    /**
     * @var integer
     *
     * @Assert\NotBlank()
     *
     * @ORM\Column(type="text")
     */
    private $bindScript;
    /**
     * @var UserGroup
     *
     * @Assert\NotBlank()
     *
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\UserGroup")
     * @ORM\JoinColumn(name="user_group_id", referencedColumnName="id", onDelete="SET NULL", nullable=true)
     */
    private $userGroup;
    /**
     * @var string
     *
     * @Assert\NotBlank()
     *
     * @ORM\Column(type="string", options={"default":LdapSynchronize::LDAP_SYNCHRONIZE__SYNC_INTERVAL_NONE})
     */
    private $syncInterval;
    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime")
     */
    private $dateCreate;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $dateUpdate;

    /**
     * @var string
     *
     * @ORM\Column(type="string", options={"default":LdapSynchronize::LDAP_SYNCHRONIZE__STATUS_NOT_SYNCHRONIZED})
     */
    private $status;


    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="FS\UserBundle\Entity\User", mappedBy="ldapSyncSchedule", cascade={"all"})
     */
    private $usersInSync;

    /**
     * LdapSynchronize constructor.
     */
    public function __construct()
    {
        $this->dateCreate = new \DateTime();

        $this->syncInterval = self::LDAP_SYNCHRONIZE__SYNC_INTERVAL_NONE;
        $this->status = self::LDAP_SYNCHRONIZE__STATUS_NOT_SYNCHRONIZED;

        $this->usersInSync = new ArrayCollection();
    }

    public static function getSyncIntervalList()
    {
        return [
            self::LDAP_SYNCHRONIZE__SYNC_INTERVAL_NONE => ucfirst(self::LDAP_SYNCHRONIZE__SYNC_INTERVAL_NONE),
            self::LDAP_SYNCHRONIZE__SYNC_INTERVAL_DAILY => ucfirst(self::LDAP_SYNCHRONIZE__SYNC_INTERVAL_DAILY),
            self::LDAP_SYNCHRONIZE__SYNC_INTERVAL_WEEKLY => ucfirst(self::LDAP_SYNCHRONIZE__SYNC_INTERVAL_WEEKLY),
        ];
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getBindScript()
    {
        return $this->bindScript;
    }

    /**
     * @param int $bindScript
     */
    public function setBindScript($bindScript)
    {
        $this->bindScript = $bindScript;
    }

    /**
     * @return int
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param int $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return UserGroup
     */
    public function getUserGroup()
    {
        return $this->userGroup;
    }

    /**
     * @param UserGroup $userGroup
     */
    public function setUserGroup($userGroup)
    {
        $this->userGroup = $userGroup;
    }

    /**
     * @return string
     */
    public function getSyncInterval()
    {
        return $this->syncInterval;
    }

    /**
     * @param string $syncInterval
     */
    public function setSyncInterval($syncInterval)
    {
        $this->syncInterval = $syncInterval;
    }

    /**
     * @return \DateTime
     */
    public function getDateCreate()
    {
        return $this->dateCreate;
    }

    /**
     * @param \DateTime $dateCreate
     */
    public function setDateCreate($dateCreate)
    {
        $this->dateCreate = $dateCreate;
    }

    /**
     * @return \DateTime
     */
    public function getDateUpdate()
    {
        return $this->dateUpdate;
    }

    /**
     * @param \DateTime $dateUpdate
     */
    public function setDateUpdate($dateUpdate)
    {
        $this->dateUpdate = $dateUpdate;
    }

    /**
     * @return ArrayCollection
     */
    public function getUsersInSync()
    {
        return $this->usersInSync;
    }


    /**
     * @param User $user
     * @return $this
     */
    public function addUser(User $user)
    {
        $user->setLdapSyncSchedule($this);

        $this->usersInSync[] = $user;
        return $this;
    }

    /**
     * @param User $user
     */
    public function removeUser(User $user)
    {
        $this->usersInSync->removeElement($user);
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
    }

}