<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 02.04.2015
 * Time: 19:57
 */

namespace FS\UserBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Model\ContainerizedInterface;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Log
 *
 * @ORM\Table(name="log")
 * @ORM\Entity(repositoryClass="FS\UserBundle\Repository\LogRepository")
 */
class Log implements ContainerizedInterface
{

    use FSContainerTrait;

    const OB_TYPE_USER = 'User';
    const OB_TYPE_ROOM = 'Room';
    const OB_TYPE_CAMERA = 'Camera';
    const OB_TYPE_RECORDING = 'Recording';
    const OB_TYPE_VIDEO = 'Video';
    const OB_TYPE_ADMIN = 'Admin';


    public static $actions =
        [
            self::OB_TYPE_USER => [
                'logged' => 'Logged',
                'attempt' => 'Attempted to login'
            ],
            self::OB_TYPE_ROOM => [
                'view' => 'Viewed'
            ],
            self::OB_TYPE_CAMERA => [
                'view' => 'Viewed'
            ],
            self::OB_TYPE_RECORDING => [
                'start' => 'Start',
                'stop' => 'Stop',
                'schedule' => 'Scheduled',
                'scheduleStart' => 'Schedule start',
                'scheduleStop' => 'Schedule stop',
                'scheduleBlock' => 'Schedule block',
            ],
            self::OB_TYPE_VIDEO => [
                'view' => 'Viewed',
                'download' => 'Downloaded',
                'delete' => 'Deleted'
            ],
            self::OB_TYPE_ADMIN => [
                'object' => [
                    'user' => 'User',
                    'userGroup' => 'User Group',
                    'room' => 'Room',
                    'camera' => 'Camera',
                    'wowza' => 'Wowza',
                    'template' => 'Template'
                ],
                'action' => [
                    'add' => 'Add',
                    'remove' => 'Remove',
                    'edit' => 'Edit'
                ]
            ]
        ];


    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;


    /**
     * @var User
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\User")
     * @ORM\JoinColumn(name="user_id", referencedColumnName="id", onDelete="SET NULL")
     */
    private $user;

    /**
     * @var string
     *
     * @ORM\Column(type="string")
     */
    private $objectType;

    /**
     * @var integer
     *
     * @ORM\Column(type="bigint", nullable=true)
     */
    private $objectId;

    /**
     * @var string
     *
     * @ORM\Column(type="string")
     */
    private $action;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime")
     */
    private $actionDate;

    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    private $objectName;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $extraInformation;

    public function __construct()
    {
        $this->actionDate = new \DateTime();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set objectType
     *
     * @param string $objectType
     * @return Log
     */
    public function setObjectType($objectType)
    {
        $this->objectType = $objectType;

        return $this;
    }

    /**
     * Get objectType
     *
     * @return string
     */
    public function getObjectType()
    {
        return $this->objectType;
    }

    /**
     * Set objectId
     *
     * @param integer $objectId
     * @return Log
     */
    public function setObjectId($objectId)
    {
        $this->objectId = $objectId;

        return $this;
    }

    /**
     * Get objectId
     *
     * @return integer
     */
    public function getObjectId()
    {
        return $this->objectId;
    }

    /**
     * Set action
     *
     * @param string $action
     * @return Log
     */
    public function setAction($action)
    {
        $this->action = $action;

        return $this;
    }

    /**
     * Get action
     *
     * @return string
     */
    public function getAction()
    {
        return $this->action;
    }

    /**
     * Set actionDate
     *
     * @param \DateTime $actionDate
     * @return Log
     */
    public function setActionDate($actionDate)
    {
        $this->actionDate = $actionDate;

        return $this;
    }

    /**
     * Get actionDate
     *
     * @return \DateTime
     */
    public function getActionDate()
    {
        return $this->actionDate;
    }

    /**
     * Set extraInformation
     *
     * @param string $extraInformation
     * @return Log
     */
    public function setExtraInformation($extraInformation)
    {
        $this->extraInformation = $extraInformation;

        return $this;
    }

    /**
     * Get extraInformation
     *
     * @return string
     */
    public function getExtraInformation()
    {
        return $this->extraInformation;
    }

    /**
     * Set user
     *
     * @param \FS\UserBundle\Entity\User $user
     * @return Log
     */
    public function setUser(\FS\UserBundle\Entity\User $user = null)
    {
        $this->user = $user;

        return $this;
    }

    /**
     * Get user
     *
     * @return \FS\UserBundle\Entity\User
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * @return mixed
     */
    public function getObjectName()
    {
        return $this->objectName;
    }

    /**
     * @param $objectName
     * @return $this
     */
    public function setObjectName($objectName)
    {
        $this->objectName = $objectName;
        return $this;
    }

    /**
     * @param array $logs
     * @return array
     */
    public static function getEntitiesIds(array $logs)
    {
        /** @var Log $log */
        foreach ($logs as $log) {
            $ids[] = $log->getObjectId();
        }

        return $ids;
    }
}
