<?php

namespace FS\UserBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;

/**
 * @ORM\Entity
 * @ORM\Table(name="user_groups_rights")
 * @UniqueEntity(
 *     fields={"group", "granted"},
 *     errorPath="group"
 * )
 */
class UserGroupRight
{
    // MONITORING
    const GRANT_MONITORING = 'monitoring';
    const GRANT_MONITORING__TALKBACK = 'monitoring--talkback';
    const GRANT_MONITORING__PTZ_CONTROL = 'monitoring--ptz-control';
    const GRANT_MONITORING__EDIT_PRESETS = 'monitoring--edit-presets';
    const GRANT_MONITORING__CAN_PTZ_ALL_AUTHORS = 'monitoring--can-ptz-all-authors';

    // REVIEW
    const GRANT_REVIEW = 'review';
    const GRANT_REVIEW__ADD_AND_EDIT_MARKERS_AND_INFO = 'review--add-and-edit-markers-and-info';
    const GRANT_REVIEW__DELETE_MARKERS = 'review--delete-markers';
    const GRANT_REVIEW__PLAYBACK_TOOLS = 'review--playback-tools';
    const GRANT_REVIEW__DELETE_RECORDINGS = 'review--delete-recordings';
    const GRANT_REVIEW__DOWNLOAD_RECORDINGS = 'review--download-recordings';
    const GRANT_REVIEW__EDIT_INFORMATION = 'review--edit-information';
    const GRANT_REVIEW__EDIT_SHARING = 'review--edit-sharing';
    const GRANT_REVIEW__EDIT_RETENTION = 'review--edit-retention';

    // SCHEDULE
    const GRANT_SCHEDULE = 'schedule';
    const GRANT_SCHEDULE__ADD_SCHEDULE = 'schedule--add-schedule';
    const GRANT_SCHEDULE__EXCEPTIONS = 'schedule--exceptions';
    const GRANT_SCHEDULE__EDIT_SCHEDULES = 'schedule--edit-schedules';
    const GRANT_SCHEDULE__DELETE_REMOVE_SCHEDULES = 'schedule--delete-and-remove-schedules';
    const GRANT_SCHEDULE__SHARING = 'schedule--sharing';
    const GRANT_SCHEDULE__RETENTION = 'schedule--rentention';
    const GRANT_SCHEDULE__CONTROL = 'schedule--control';

    // RECORDING
    const GRANT_RECORDING = 'recording';
    const GRANT_RECORDING__START_RECORDING = 'recording--start-recording';
    const GRANT_RECORDING__STOP_ALL_AUTHORS = 'recording--stop-all-authors';
    const GRANT_RECORDING__SHARING = 'recording--sharing';
    const GRANT_RECORDING__RETENTION = 'recording--retention';
    const GRANT_RECORDING__ADD_MARKERS = 'recording--add-markers';

    // ADMIN
    const GRANT_ADMIN = 'admin';
    const GRANT_ADMIN__GENERAL = 'admin--general';
    const GRANT_ADMIN__TEMPLATES = 'admin--templates';
    const GRANT_ADMIN__ROOMS = 'admin--rooms';
    const GRANT_ADMIN__USERS_AND_GROUPS = 'admin--users-and-groups';
    const GRANT_ADMIN__LOGS = 'admin--logs';
    const GRANT_ADMIN__HELP = 'admin--help';
    const GRANT_ADMIN__WOWZA = 'admin--wowza';
    const GRANT_ADMIN__CONTAINERS = 'admin--containers';
    const GRANT_ADMIN__UPDATE = 'admin--update';

    /********* NON GROUP RULES *************/
    /**
     * UPLOAD
     *
     */
    const GRANT_UPLOAD = 'upload';

    /**
     * ALERTS
     */
    const GRANT_ALERT = 'alert';
    /**
     * DOWNLOAD
     *
     * @deprecated
     */
    const GRANT_DOWNLOAD = 'download';

    /**
     * DELETE
     *
     * @deprecated
     */
    const GRANT_DELETE = 'delete';


    /**
     * @var integer
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var UserGroup
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\UserGroup", inversedBy="rights")
     * @ORM\JoinColumn(name="user_group_id", referencedColumnName="id")
     **/
    private $group;

    /**
     * @var string
     *
     * @ORM\Column(type="string")
     */
    private $granted;

    public static function getGruntsList()
    {
        return [
            // MONITORING
            self::GRANT_MONITORING,
            self::GRANT_MONITORING__TALKBACK,
            self::GRANT_MONITORING__PTZ_CONTROL,
            self::GRANT_MONITORING__EDIT_PRESETS,
            self::GRANT_MONITORING__CAN_PTZ_ALL_AUTHORS,
            // REVIEW
            self::GRANT_REVIEW,
            self::GRANT_REVIEW__ADD_AND_EDIT_MARKERS_AND_INFO,
            self::GRANT_REVIEW__DELETE_MARKERS,
            self::GRANT_REVIEW__PLAYBACK_TOOLS,
            self::GRANT_REVIEW__DELETE_RECORDINGS,
            self::GRANT_REVIEW__DOWNLOAD_RECORDINGS,
            self::GRANT_REVIEW__EDIT_INFORMATION,
            self::GRANT_REVIEW__EDIT_SHARING,
            self::GRANT_REVIEW__EDIT_RETENTION,
            // SCHEDULE
            self::GRANT_SCHEDULE,
            self::GRANT_SCHEDULE__ADD_SCHEDULE,
            self::GRANT_SCHEDULE__EXCEPTIONS,
            self::GRANT_SCHEDULE__EDIT_SCHEDULES,
            self::GRANT_SCHEDULE__DELETE_REMOVE_SCHEDULES,
            self::GRANT_SCHEDULE__SHARING,
            self::GRANT_SCHEDULE__RETENTION,
            self::GRANT_SCHEDULE__CONTROL,
            // RECORDING
            self::GRANT_RECORDING,
            self::GRANT_RECORDING__START_RECORDING,
            self::GRANT_RECORDING__STOP_ALL_AUTHORS,
            self::GRANT_RECORDING__SHARING,
            self::GRANT_RECORDING__RETENTION,
            self::GRANT_RECORDING__ADD_MARKERS,
            //UPLOAD
            self::GRANT_UPLOAD,
            //ALERT
            self::GRANT_ALERT,
            // ADMIN
            self::GRANT_ADMIN,
            self::GRANT_ADMIN__GENERAL,
            self::GRANT_ADMIN__TEMPLATES,
            self::GRANT_ADMIN__ROOMS,
            self::GRANT_ADMIN__USERS_AND_GROUPS,
            self::GRANT_ADMIN__LOGS,
            self::GRANT_ADMIN__HELP,
            self::GRANT_ADMIN__WOWZA,
            self::GRANT_ADMIN__CONTAINERS,
            self::GRANT_ADMIN__UPDATE
        ];
    }

    public function getGrantParent($grant)
    {
        $list = [
            // MONITORING
            self::GRANT_MONITORING => null,
            self::GRANT_MONITORING__TALKBACK => self::GRANT_MONITORING,
            self::GRANT_MONITORING__PTZ_CONTROL => self::GRANT_MONITORING,
            self::GRANT_MONITORING__EDIT_PRESETS => self::GRANT_MONITORING,
            self::GRANT_MONITORING__CAN_PTZ_ALL_AUTHORS => self::GRANT_MONITORING,
            // REVIEW
            self::GRANT_REVIEW => null,
            self::GRANT_REVIEW__ADD_AND_EDIT_MARKERS_AND_INFO => self::GRANT_REVIEW,
            self::GRANT_REVIEW__DELETE_MARKERS => self::GRANT_REVIEW,
            self::GRANT_REVIEW__PLAYBACK_TOOLS => self::GRANT_REVIEW,
            self::GRANT_REVIEW__DELETE_RECORDINGS => self::GRANT_REVIEW,
            self::GRANT_REVIEW__DOWNLOAD_RECORDINGS => self::GRANT_REVIEW,
            self::GRANT_REVIEW__EDIT_INFORMATION => self::GRANT_REVIEW,
            self::GRANT_REVIEW__EDIT_SHARING => self::GRANT_REVIEW,
            self::GRANT_REVIEW__EDIT_RETENTION => self::GRANT_REVIEW,
            // SCHEDULE
            self::GRANT_SCHEDULE => null,
            self::GRANT_SCHEDULE__ADD_SCHEDULE => self::GRANT_SCHEDULE,
            self::GRANT_SCHEDULE__EXCEPTIONS => self::GRANT_SCHEDULE,
            self::GRANT_SCHEDULE__EDIT_SCHEDULES => self::GRANT_SCHEDULE,
            self::GRANT_SCHEDULE__DELETE_REMOVE_SCHEDULES => self::GRANT_SCHEDULE,
            self::GRANT_SCHEDULE__SHARING => self::GRANT_SCHEDULE,
            self::GRANT_SCHEDULE__RETENTION => self::GRANT_SCHEDULE,
            self::GRANT_SCHEDULE__CONTROL => self::GRANT_SCHEDULE,
            // RECORDING
            self::GRANT_RECORDING => null,
            self::GRANT_RECORDING__START_RECORDING => self::GRANT_RECORDING,
            self::GRANT_RECORDING__STOP_ALL_AUTHORS => self::GRANT_RECORDING,
            self::GRANT_RECORDING__SHARING => self::GRANT_RECORDING,
            self::GRANT_RECORDING__RETENTION => self::GRANT_RECORDING,
            //UPLOAD
            self::GRANT_UPLOAD => null,
            //ALERT
            self::GRANT_ALERT => null,
            // ADMIN
            self::GRANT_ADMIN => null,
            self::GRANT_ADMIN__GENERAL => self::GRANT_ADMIN,
            self::GRANT_ADMIN__TEMPLATES => self::GRANT_ADMIN,
            self::GRANT_ADMIN__ROOMS => self::GRANT_ADMIN,
            self::GRANT_ADMIN__USERS_AND_GROUPS => self::GRANT_ADMIN,
            self::GRANT_ADMIN__LOGS => self::GRANT_ADMIN,
            self::GRANT_ADMIN__HELP => self::GRANT_ADMIN,
            self::GRANT_ADMIN__WOWZA => self::GRANT_ADMIN,
            self::GRANT_ADMIN__CONTAINERS => self::GRANT_ADMIN,
            self::GRANT_ADMIN__UPDATE => self::GRANT_ADMIN
        ];

        return array_key_exists($grant, $list) ? $list[$grant] : null;
    }

    public static function getSubRightsList($parent)
    {
        $list = [
            // MONITORING
            self::GRANT_MONITORING => [
                self::GRANT_MONITORING__TALKBACK,
                self::GRANT_MONITORING__PTZ_CONTROL,
                self::GRANT_MONITORING__EDIT_PRESETS,
                self::GRANT_MONITORING__CAN_PTZ_ALL_AUTHORS,
            ],
            // REVIEW
            self::GRANT_REVIEW => [
                self::GRANT_REVIEW__ADD_AND_EDIT_MARKERS_AND_INFO,
                self::GRANT_REVIEW__DELETE_MARKERS,
                self::GRANT_REVIEW__PLAYBACK_TOOLS,
                self::GRANT_REVIEW__DELETE_RECORDINGS,
                self::GRANT_REVIEW__DOWNLOAD_RECORDINGS,
                self::GRANT_REVIEW__EDIT_INFORMATION,
                self::GRANT_REVIEW__EDIT_SHARING,
                self::GRANT_REVIEW__EDIT_RETENTION,
            ],

            // SCHEDULE
            self::GRANT_SCHEDULE => [
                self::GRANT_SCHEDULE__ADD_SCHEDULE,
                self::GRANT_SCHEDULE__EXCEPTIONS,
                self::GRANT_SCHEDULE__EDIT_SCHEDULES,
                self::GRANT_SCHEDULE__DELETE_REMOVE_SCHEDULES,
                self::GRANT_SCHEDULE__SHARING,
                self::GRANT_SCHEDULE__RETENTION,
                self::GRANT_SCHEDULE__CONTROL,
            ],

            // RECORDING
            self::GRANT_RECORDING => [
                self::GRANT_RECORDING__START_RECORDING,
                self::GRANT_RECORDING__STOP_ALL_AUTHORS,
                self::GRANT_RECORDING__SHARING,
                self::GRANT_RECORDING__RETENTION,
            ],

            // Admin
            self::GRANT_ADMIN => [
                self::GRANT_ADMIN__GENERAL,
                self::GRANT_ADMIN__TEMPLATES,
                self::GRANT_ADMIN__ROOMS,
                self::GRANT_ADMIN__USERS_AND_GROUPS,
                self::GRANT_ADMIN__LOGS,
                self::GRANT_ADMIN__HELP,
                self::GRANT_ADMIN__WOWZA,
                self::GRANT_ADMIN__CONTAINERS,
                self::GRANT_ADMIN__UPDATE
            ]
        ];

        return $list[$parent];
    }

    public function isGranted($right)
    {
        return $right == $this->getGranted(); // || $this->getGrantParent($right) == $this->getGranted()

    }

    /**
     * @return UserGroup
     */
    public function getGroup()
    {
        return $this->group;
    }

    /**
     * @param mixed $group
     * @return $this
     */
    public function setGroup(\FS\UserBundle\Entity\UserGroup $group)
    {
        $this->group = $group;

        return $this;
    }

    /**
     * @return string
     */
    public function getGranted()
    {
        return $this->granted;
    }

    /**
     * @param string $grunted
     * @return $this
     */
    public function setGranted($grunted)
    {
        $this->granted = $grunted;

        return $this;
    }


}