<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 06.04.2015
 * Time: 18:05
 */

namespace FS\UserBundle\EventListener;


use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Event\LifecycleEventArgs;
use FS\BasicBundle\Model\Loggable;
use FS\CamBundle\Entity\Template;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Entity\Wowza;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Entity\Log;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class AdminEntityListener implements EventSubscriber
{
    /**
     * @var ContainerInterface
     */
    private $container;
    /**
     * @var EntityManager $em
     */
    private $em;

    private $requestStack;

    /**
     * @param ContainerInterface $container
     * @param RequestStack $requestStack
     */
    public function __construct(ContainerInterface $container, RequestStack $requestStack)
    {
        $this->container = $container;
        $this->requestStack = $requestStack;
    }

    /**
     * @return array
     */
    public function getSubscribedEvents()
    {
        return array(
            'postPersist',
            'postUpdate',
            'preRemove'
        );
    }

    private function writeLog(Loggable $entity, $objectType, $action, $extra, $container = null)
    {
        $user = $this->getUser();
        if (is_null($user) && !is_object($user)) $user = null;

        $extraFields = function(Log $log) use ($extra, $container) {
            $log->setExtraInformation($extra)
                ->setContainer($container);
        };

        $this->container->get('fs_user.util.logger')
            ->logAction($objectType, $entity, $user, $action, $extraFields);
    }

    private function process($cmd, LifecycleEventArgs $args)
    {
        $entity = $args->getEntity();
        $this->em = $args->getEntityManager();

        if ($entity instanceof User) {
            if (
                $this->requestStack->getCurrentRequest() != null &&
                $this->requestStack->getCurrentRequest()->get("_route") != 'fos_user_security_check'
            ) {
                $container = null;
                /** @var User $entity */
                if ($entity->getUserGroup() != null) {
                    $container = $entity->getUserGroup()->getContainer();
                }

                $this->writeLog(
                    $entity,
                    Log::$actions[Log::OB_TYPE_ADMIN]['object']['user'],
                    Log::$actions[Log::OB_TYPE_ADMIN]['action'][$cmd],
                    Log::OB_TYPE_ADMIN,
                    $container
                );
            }

        } else if ($entity instanceof UserGroup) {
            $this->writeLog(
                $entity,
                Log::$actions[Log::OB_TYPE_ADMIN]['object']['userGroup'],
                Log::$actions[Log::OB_TYPE_ADMIN]['action'][$cmd],
                Log::OB_TYPE_ADMIN//,
                //$entity->getContainer()
            );
        } else if ($entity instanceof Room) {
            $this->writeLog(
                $entity,
                Log::$actions[Log::OB_TYPE_ADMIN]['object']['room'],
                Log::$actions[Log::OB_TYPE_ADMIN]['action'][$cmd],
                Log::OB_TYPE_ADMIN,
                $entity->getContainer()
            );
        } else if ($entity instanceof Camera) {
            $this->writeLog(
                $entity,
                Log::$actions[Log::OB_TYPE_ADMIN]['object']['camera'],
                Log::$actions[Log::OB_TYPE_ADMIN]['action'][$cmd],
                Log::OB_TYPE_ADMIN//,
            //$entity->getRoom()->getContainer()
            );
        } else if ($entity instanceof Wowza) {
            $this->writeLog(
                $entity,
                Log::$actions[Log::OB_TYPE_ADMIN]['object']['wowza'],
                Log::$actions[Log::OB_TYPE_ADMIN]['action'][$cmd],
                Log::OB_TYPE_ADMIN
            );

        } else if ($entity instanceof Template) {
            $this->writeLog(
                $entity,
                Log::$actions[Log::OB_TYPE_ADMIN]['object']['template'],
                Log::$actions[Log::OB_TYPE_ADMIN]['action'][$cmd],
                Log::OB_TYPE_ADMIN,
                $entity->getContainer()
            );
        }
    }

    /**
     * @param LifecycleEventArgs $args
     */
    public function postUpdate(LifecycleEventArgs $args)
    {
        $this->process('edit', $args);
    }

    public function postPersist(LifecycleEventArgs $args)
    {
        $this->process('add', $args);
    }

    public function preRemove(LifecycleEventArgs $args)
    {
        $this->process('remove', $args);
    }

    public function getUser()
    {
        if (!$this->container->get('security.context')) {
        }

        if (null === $token = $this->container->get('security.context')->getToken()) {
            return;
        }

        if (!is_object($user = $token->getUser())) {
            return;
        }

        return $user;
    }

}