<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 03.04.2015
 * Time: 21:15
 */

namespace FS\UserBundle\EventListener;

use Doctrine\Common\Persistence\ObjectManager;
use Doctrine\ORM\EntityManager;
use FOS\UserBundle\Model\UserManagerInterface;
use FS\UserBundle\Entity\Log;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Util\Logger;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Security\Core\Event\AuthenticationEvent;
use Symfony\Component\Security\Core\Event\AuthenticationFailureEvent;
use Symfony\Component\Security\Http\Event\InteractiveLoginEvent;
use Symfony\Component\Security\Core\AuthenticationEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class AuthenticationListener implements EventSubscriberInterface
{

    protected $objectManager;
    protected $requestStack;
    protected $userManager;
    protected $logger;

    public function __construct(
        EntityManager $objectManager,
        RequestStack $requestStack,
        UserManagerInterface $userManager,
        Logger $logger
    )
    {
        $this->objectManager = $objectManager;
        $this->requestStack = $requestStack;
        $this->userManager = $userManager;
        $this->logger = $logger;
    }

    /**
     * getSubscribedEvents
     *
     * @return    array
     */
    public static function getSubscribedEvents()
    {
        return array(
            AuthenticationEvents::AUTHENTICATION_FAILURE => 'onAuthenticationFailure',
            AuthenticationEvents::AUTHENTICATION_SUCCESS => 'onAuthenticationSuccess',
        );
    }

    /**
     * onAuthenticationFailure
     *
     * @param    AuthenticationFailureEvent $event
     */
    public function onAuthenticationFailure(AuthenticationFailureEvent $event)
    {

        $ip = $this->requestStack->getCurrentRequest()->getClientIp();

        $username = $event->getAuthenticationToken()->getUsername();
        /** @var User $user */
        $user = $this->userManager->findUserByUsername($username);

        $log = new Log();
        if ($user != null) {
            $log->setObjectId($user->getId());
            if($user->getUserGroup() != null){
                $log->setContainer($user->getUserGroup()->getContainer());
            }
        } else {
            $log->setObjectId(null);
        }
        $log->setUser(null);
        $log->setObjectName($username);
        $log->setObjectType(Log::OB_TYPE_USER);
        $log->setAction(Log::$actions[Log::OB_TYPE_USER]['attempt'] . ' from ip = ' . $ip);
        $log->setExtraInformation('username = ' . $username);

        $this->objectManager->persist($log);
        $this->objectManager->flush();

    }

    /**
     * onAuthenticationSuccess
     *
     * @param AuthenticationEvent|InteractiveLoginEvent $event
     */
    public function onAuthenticationSuccess(AuthenticationEvent $event)
    {
        $ip = $this->requestStack->getCurrentRequest()->getClientIp();
        $user = $event->getAuthenticationToken()->getUser();

        $extraFields = function(Log $log) use ($user) {
            if($user->getUserGroup() != null){
                $log->setContainer($user->getUserGroup()->getContainer());
            }
        };

        $this->logger->logAction(
            Log::OB_TYPE_USER,
            $user,
            $user,
            Log::$actions[Log::OB_TYPE_USER]['logged'].' from ip ='.$ip,
            $extraFields);
    }
}