<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 06.04.2015
 * Time: 16:55
 */

namespace FS\UserBundle\EventListener;


use Doctrine\ORM\EntityManager;
use FOS\UserBundle\Doctrine\UserManager;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Record;
use FS\UserBundle\Entity\Log;
use FS\UserBundle\Util\Logger;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\Security\Core\SecurityContext;

class LogPathListener
{
    private $userManager;
    private $entityManager;
    private $requestStack;
    private $request;
    private $securityContext;
    private $logger;

    /**
     * @param RequestStack $requestStack
     * @param UserManager $userManager
     * @param EntityManager $entityManager
     * @param SecurityContext $securityContext
     */
    public function __construct
    (
        RequestStack $requestStack,
        UserManager $userManager,
        EntityManager $entityManager,
        SecurityContext $securityContext,
        Logger $logger
    )
    {
        $this->requestStack = $requestStack;
        $this->request = $requestStack->getCurrentRequest();
        $this->userManager = $userManager;
        $this->entityManager = $entityManager;
        $this->securityContext = $securityContext;
        $this->logger = $logger;
    }

    public function onKernelRequest(GetResponseEvent $event)
    {
        $routeName = $this->request->get('_route');
        switch ($routeName) {
            ///-----------------------
            // Record logs
            case "record_show":
                $record = $this->entityManager->getRepository('FSCamBundle:Record')
                    ->find((int)$this->request->attributes->all()['id']);
                $user = $this->getUser();
                if($record == null){
                    break;
                }

                $extraFields = function(Log $log) use ($record) {
                    $log->setContainer($record->getContainer());
                };

                $this->logger->logAction(
                    Log::OB_TYPE_VIDEO,
                    $record,
                    $user,
                    Log::$actions[Log::OB_TYPE_VIDEO]['view'],
                    $extraFields);
                break;

            case "record_download":
                if($this->request->attributes->all()['record'] instanceof Record){
                    $record = $this->request->attributes->all()['record'];
                } else {
                    $record = $this->entityManager->getRepository('FSCamBundle:Record')
                        ->find((int)$this->request->attributes->all()['record']);
                }
                $user = $this->getUser();
                if($record == null){
                    break;
                }
                if($this->request->attributes->all()['camera'] instanceof Camera){
                    $camera = $this->request->attributes->all()['camera']->getId();
                } else {
                    $camera = (int)$this->request->attributes->all()['camera'];
                }

                $extraFields = function(Log $log) use ($record, $camera) {
                    $log->setContainer($record->getContainer())
                        ->setExtraInformation("cameraId = " . $camera);
                };

                $this->logger->logAction(
                    Log::OB_TYPE_VIDEO,
                    $record,
                    $user,
                    Log::$actions[Log::OB_TYPE_VIDEO]['download'],
                    $extraFields);
                break;
            case "record_delete":
                $record = $this->entityManager->getRepository('FSCamBundle:Record')
                    ->find((int)$this->request->attributes->all()['id']);
                $user = $this->getUser();
                if($record == null){
                    break;
                }

                $extraFields = function(Log $log) use ($record) {
                    $log->setContainer($record->getContainer())
                        ->setExtraInformation('User delete record');
                };

                $this->logger->logAction(
                    Log::OB_TYPE_VIDEO,
                    $record,
                    $user,
                    Log::$actions[Log::OB_TYPE_VIDEO]['delete'],
                    $extraFields);
                break;
            default:
                return;
        }
    }

    public function getUser()
    {
        if (!$this->securityContext) {
        }

        if (null === $token = $this->securityContext->getToken()) {
            return;
        }

        if (!is_object($user = $token->getUser())) {
            return;
        }

        return $user;
    }
}