<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 21.12.2015
 * Time: 19:53
 */

namespace FS\UserBundle\Form\Type;


use FS\UserBundle\Entity\Container;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class ContainerType extends AbstractType
{
    private $containerManager;

    /**
     * @param ContainerManager $containerManager
     */
    public function __construct(ContainerManager $containerManager)
    {
        $this->containerManager = $containerManager;
    }

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $data = $builder->getForm()->getData();
        $list = $this->containerManager->getContainersList();

        /** @var Container $data */
        if($data){
            $list = $this->removeFromArray($list, $data);
            foreach($data->toArray() as $child){
                $list = $this->removeFromArray($list, $child);
            }
        }

        $builder->add('name', 'text', [
            'label' => "Container name"
        ]);
        $builder->add('parent',
            'entity',
            [
                'class' => 'FS\UserBundle\Entity\Container',
                'multiple' => false,
                'label' => 'Parent Container',
                'choices' => $list,
            ]
        );
    }

    private function removeFromArray($array, $item) {
        $index = array_search($item, $array);
        if ($index !== false) {
            unset($array[$index]);
        }

        return $array;
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\UserBundle\Entity\Container'
            )
        );
    }

    /**
     * Returns the name of this type.
     *
     * @return string The name of this type
     */
    public function getName()
    {
        return "fs_container";
    }
}