<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 22.03.2016
 * Time: 16:00
 */

namespace FS\UserBundle\Form\Type;


use Doctrine\ORM\EntityManager;
use FS\UserBundle\Entity\LdapSynchronize;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class LdapSynchronizeType extends AbstractType
{
    private $containerManager;
    private $em;

    /**
     * @param EntityManager $entityManager
     * @param ContainerManager $containerManager
     */
    public function __construct(EntityManager $entityManager, ContainerManager $containerManager)
    {
        $this->containerManager = $containerManager;
        $this->em = $entityManager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('name', 'text', [
                    'label' => 'Name',
                    'required' => true,
                ]
            )
            ->add('container',
                'entity', [
                    'class' => 'FSUserBundle:Container',
                    'property' => 'name',
                    'choices' => $this->containerManager->getContainersList(),
                    'required' => true,
                    'label' => 'Container',
                    'attr' => [
                        'style' => 'width:100%'
                    ],

                ]
            )
            ->add('bindScript', 'textarea',
                [
                    'attr' => [
                        'rows' => 3,
                    ],
                    'label' => 'Bind Script',
                    'required' => true,
                ]
            )
            ->add('userGroup',
                'entity', [
                    'class' => 'FSUserBundle:UserGroup',
                    'property' => 'name',
                    'empty_value' => '',
                    'choices' => $this->containerManager->filterArray(
                        $this->em->getRepository('FSUserBundle:UserGroup')->findAll()
                    ),
                    'required' => false,
                    'label' => 'Group to Add',
                    'attr' => [
                        'style' => 'width:100%'
                    ],
                ]
            )
            ->add('syncInterval', 'choice', [
                'choices' => LdapSynchronize::getSyncIntervalList(),
                'label' => 'Sync Interval',
                'multiple' => false,
                'attr' => [
                    'style' => 'width:100%'
                ],
            ]);
    }

    /**
     * @param OptionsResolverInterface $resolver
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            [
                'data_class' => 'FS\UserBundle\Entity\LdapSynchronize'
            ]
        );
    }

    public function getName()
    {
        return 'fs_ldap_sync';
    }
}