<?php

namespace FS\UserBundle\Form\Type;

use Lexik\Bundle\FormFilterBundle\Filter\FilterOperands;
use Lexik\Bundle\FormFilterBundle\Filter\Query\QueryInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class LogFilterType extends AbstractType
{
    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add(
                'name',
                'filter_text',
                [
                    'condition_pattern' => FilterOperands::STRING_BOTH,
                    'attr' => ['placeholder' => 'search criteria'],
                    'apply_filter' => function (QueryInterface $filterQuery, $field, $values) {
                        if (!empty($values['value'])) {
                            /** @var \Doctrine\ORM\QueryBuilder $qb */
                            $qb = $filterQuery->getQueryBuilder();

                            $value = $filterQuery->getExpr()->literal('%' . $values['value'] . '%');
                            $qb->andWhere(
                                $filterQuery->getExpr()->orx(
                                    $filterQuery->getExpr()->like('user.username', $value),
                                    $filterQuery->getExpr()->like('user.displayName', $value),
                                    $filterQuery->getExpr()->like('logs.action', $value),
                                    $filterQuery->getExpr()->like('logs.objectType', $value),
                                    $filterQuery->getExpr()->like('logs.objectName', $value)
                                )
                            );
                        }
                    },
                ]
            )
            ->add(
                'startedAt',
                'filter_date_range',
                [
                    'left_date_options' => ['widget' => 'single_text', 'format' => 'MM/dd/yyyy'],
                    'right_date_options' => ['widget' => 'single_text', 'format' => 'MM/dd/yyyy'],
                    'apply_filter' => function (QueryInterface $filterQuery, $field, $values) {
                        if (!empty($values['value'])) {
                            /** @var \Doctrine\ORM\QueryBuilder $qb */
                            $qb = $filterQuery->getQueryBuilder();

                            $from = $values['value']['left_date'][0];
                            $to = $values['value']['right_date'][0];
                            if (!empty($from)) {
                                $qb->andWhere('logs.actionDate > :from')
                                    ->setParameter('from', $from->format('Y-m-d'));
                            }
                            if (!empty($to)) {
                                $qb->andWhere('logs.actionDate < :to')
                                    ->setParameter('to', $to->modify('+ 1 day')->format('Y-m-d'));
                            }

                        }
                    },
                ]
            );
    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'csrf_protection' => false,
                'validation_groups' => array('filtering') // avoid NotBlank() constraint-related message
            )
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'log_filter';
    }
}
