<?php

namespace FS\UserBundle\Form\Type;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityRepository;
use FS\BasicBundle\Utils\ArrayCollectionFilter;
use FS\CamBundle\Entity\Template;
use FS\CamBundle\Template\TemplateTypes;
use FS\UserBundle\Entity\UserGroup;
use FS\UserBundle\Entity\UserGroupRight;
use FS\UserBundle\Model\UserGroupSearchResult;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Doctrine\Common\Persistence\ObjectManager;
use Symfony\Component\Security\Core\SecurityContext;


class UserGroupType extends AbstractType
{
    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var SecurityContext
     */
    private $context;


    private $containerManager;

    /**
     * @param ObjectManager $om
     * @param SecurityContext $context
     * @param ContainerManager $containerManager
     */
    public function __construct(
        ObjectManager $om,
        SecurityContext $context,
        ContainerManager $containerManager
    )
    {
        $this->om = $om;
        $this->context = $context;
        $this->containerManager = $containerManager;
    }

    /**
     * @return \FS\UserBundle\Entity\User
     */
    private function getUser()
    {
        return $this->context->getToken()->getUser();
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {

        $builder->add('name',
            null,
            [
                'label' => 'Name',
                'label_attr' => ['class' => 'main-input'],
            ]
        );

        $adminGroup = false;
        /** @var UserGroup $formUserGroup */
        $formUserGroup = $builder->getData();
        if ($formUserGroup != null && $formUserGroup->getId() == 1) {
            $adminGroup = true;
        }
        $builder->add(
            'container',
            'entity',
            [
                'label' => 'Container',
                'label_attr' => ['class' => 'main-input'],
                'class' => 'FSUserBundle:Container',
                'choices' => $this->containerManager->getContainersList(),
                'disabled' => $adminGroup,
            ]
        );


        $builder->add(
            'markerTemplate',
            'entity',
            [
                'label' => 'Marker',
                'label_attr' => ['class' => 'main-input'],
                'class' => 'FSCamBundle:Template',
                'choices' => $this->containerManager->filterArray(
                    $this->om->getRepository('FSCamBundle:Template')->getMarkerTemplates()
                ),

            ]
        );
        $builder->add(
            'infoTemplate',
            'entity',
            [
                'label' => 'Information',
                'label_attr' => ['class' => 'main-input'],
                'class' => 'FSCamBundle:Template',
                'choices' => $this->containerManager->filterArray(
                    $this->om->getRepository('FSCamBundle:Template')->getInfoTemplates()
                ),

            ]
        );

        /** @var UserGroup $userGroup */
        $userGroup = $builder->getData();
        $rights = [];
        foreach ($userGroup->getRights() as $right) {
            $rights[] = $right->getGranted();
        }

        $builder->add(
            'rights',
            'choice',
            [
                'expanded' => true,
                'multiple' => true,
                'attr' => ['class' => 'with-hr'],
                'label_attr' => ['class' => 'main-input hr-line-all check'],
                'choices' => [
                    // MONITORING
                    UserGroupRight::GRANT_MONITORING => 'Observe',
                    UserGroupRight::GRANT_MONITORING__TALKBACK => '-- Talkback',
                    UserGroupRight::GRANT_MONITORING__PTZ_CONTROL => '-- PTZ control',
                    UserGroupRight::GRANT_MONITORING__EDIT_PRESETS => '-- Edit Presets',
                    UserGroupRight::GRANT_MONITORING__CAN_PTZ_ALL_AUTHORS => '-- PTZ all authors',
                    // REVIEW
                    UserGroupRight::GRANT_REVIEW => 'Review',
                    UserGroupRight::GRANT_REVIEW__ADD_AND_EDIT_MARKERS_AND_INFO => '-- Add / Edit Markers',
                    UserGroupRight::GRANT_REVIEW__DELETE_MARKERS => '-- Remove Markers',
                    UserGroupRight::GRANT_REVIEW__PLAYBACK_TOOLS => '-- Playback Tools',
                    UserGroupRight::GRANT_REVIEW__DELETE_RECORDINGS => '-- Delete Recordings',
                    UserGroupRight::GRANT_REVIEW__DOWNLOAD_RECORDINGS => '-- Download Recordings',
                    UserGroupRight::GRANT_REVIEW__EDIT_INFORMATION => '-- Edit information',
                    UserGroupRight::GRANT_REVIEW__EDIT_SHARING => '-- Edit Sharing',
                    UserGroupRight::GRANT_REVIEW__EDIT_RETENTION => '-- Edit Retention',
                    // SCHEDULE
                    UserGroupRight::GRANT_SCHEDULE => 'Schedule',
                    UserGroupRight::GRANT_SCHEDULE__ADD_SCHEDULE => '-- Add Schedules',
                    UserGroupRight::GRANT_SCHEDULE__EXCEPTIONS => '-- Exceptions',
                    UserGroupRight::GRANT_SCHEDULE__EDIT_SCHEDULES => '-- Edit Schedules',
                    UserGroupRight::GRANT_SCHEDULE__DELETE_REMOVE_SCHEDULES => '-- Delete / Remove Schedules',
                    UserGroupRight::GRANT_SCHEDULE__SHARING => '-- Sharing',
                    UserGroupRight::GRANT_SCHEDULE__RETENTION => '-- Retention',
                    UserGroupRight::GRANT_SCHEDULE__CONTROL => '-- Control',
                    // RECORDING
                    UserGroupRight::GRANT_RECORDING => 'Recording',
                    UserGroupRight::GRANT_RECORDING__START_RECORDING => "-- Start Recording",
                    UserGroupRight::GRANT_RECORDING__STOP_ALL_AUTHORS => '-- Stop all authors',
                    UserGroupRight::GRANT_RECORDING__SHARING => '-- Sharing',
                    UserGroupRight::GRANT_RECORDING__RETENTION => '-- Retention',
                    UserGroupRight::GRANT_RECORDING__ADD_MARKERS => '-- Add markers',
                    // UPLOAD
                    UserGroupRight::GRANT_UPLOAD => 'Upload',
                    UserGroupRight::GRANT_ALERT => 'Alert',
                    // ADMIN
                    UserGroupRight::GRANT_ADMIN => 'Admin',
                    UserGroupRight::GRANT_ADMIN__GENERAL => '-- General',
                    UserGroupRight::GRANT_ADMIN__TEMPLATES => '-- Templates',
                    UserGroupRight::GRANT_ADMIN__ROOMS => '-- Rooms',
                    UserGroupRight::GRANT_ADMIN__USERS_AND_GROUPS => '-- Users & Groups',
                    UserGroupRight::GRANT_ADMIN__LOGS => '-- Logs',
                    UserGroupRight::GRANT_ADMIN__HELP => '-- Help',
                    UserGroupRight::GRANT_ADMIN__CONTAINERS => '-- Containers',
                    UserGroupRight::GRANT_ADMIN__WOWZA => '-- Wowza',
                    UserGroupRight::GRANT_ADMIN__UPDATE => '-- Update'
                ],
                'mapped' => false,
                'data' => $rights,
                'disabled' => $adminGroup
            ]
        );

        $builder->addEventListener(
            FormEvents::SUBMIT,
            function (FormEvent $event) {
                $userGroup = $event->getForm()->getData();
                $rights = $event->getForm()->get('rights')->getData();


                foreach (UserGroupRight::getGruntsList() as $right) {
                    if (in_array($right, $rights)) {
                        // check right existing, if not add
                        if (!$userGroup->hasRight($right)) {
                            $userGroupRight = new UserGroupRight();
                            $userGroupRight->setGranted($right);
                            $userGroup->addRight($userGroupRight);
                        }
                    } else {
                        // check right existing, if exists delete
                        if ($userGroup->hasRight($right)) {
                            $userGroupRights = $userGroup->getRights()->filter(
                                function ($entry) use ($right) {
                                    return $entry->getGranted() == $right;
                                }
                            );
                            foreach ($userGroupRights as $userGroupRight) {
                                $userGroup->removeRight($userGroupRight);
                            }

                        }
                    }
                }
            }
        );
        $builder->add(
            'rooms',
            'genemu_jqueryselect2_entity',
            [
                'class' => 'FS\CamBundle\Entity\Room',
                'property' => 'name',
                'multiple' => true,
                'required' => true,
                'label' => 'Rooms',
                'label_attr' => ['class' => 'main-input hr-line-all add'],
                'attr' => ['placeholder' => 'Add New Room', 'class' => 'col-xs-12 with-hr'],
                'choices' => $this->containerManager->filterArray(
                    $this->om->getRepository('FSCamBundle:Room')->findAll()
                )
            ]
        );

        $builder->add('accessedUsersAndGroups', 'users_and_groups', [
            'label' => 'Video Access',
            'attr' => ['class' => 'with-hr'],
            'label_attr' => ['class' => 'main-input hr-line-all check'],
            // 'disabled' => $adminGroup,
        ]);

        $builder->add(
            'defaultRetentionRule',
            'retention',
            [
                'label_attr' => ['class' => 'hr-line-all none'],
                'may_inherit' => false
            ]
        );
        $builder->add('ioRecordName', 'text', [
                'label' => 'I/O Record Name'
            ]
        );
        $builder->add('recordDuration', null, [
            'label' => 'Max Record Duration (minutes)'
        ]);

        $builder->add('defaultReviewView', 'choice', [
            'choices' => UserGroup::getReviewViewList(),
            'label' => 'Search Results Default',
            'attr' => [
                'style' => 'width:100%',
            ],
        ]);

        $builder->add('searchResults',
            'genemu_jqueryselect2_choice',
            [
                'multiple' => true,
                'required' => true,
                'label' => 'Search Results Display',
                'attr' => [
                    'class' => 'col-xs-12'
                ],
                'choices' => UserGroupSearchResult::getUserGroupSearchResults(),
            ]
        );

        $builder->add(
            'restrictAccess', 'restrict_access', [
                'label' => 'Allow access',
                'label_attr' => ['class' => 'hr-line-all none'],
                'disabled' => $adminGroup,
            ]
        );
        $builder->add(
            'expiration', 'expiration', [
                'label_attr' => ['class' => 'hr-line-delimiter'],
                'label' => ' ',
                'disabled' => $adminGroup,
            ]
        );
        $builder->add(
            'disabledAccess', 'checkbox', [
                'label' => 'Disable Group',
                'label_attr' => ['style' => "display:none"],
                'disabled' => $adminGroup,
            ]
        );
    }

    public function finishView(FormView $view, FormInterface $form, array $options)
    {
//        foreach ($view->children as $item) {
//
//            if (in_array($item->vars['value'], $disabled, true)) {
//                $item->vars['attr']['disabled'] = 'disabled';
//                $item->vars['checked'] = true;
//            }
//        }
    }


    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\UserBundle\Entity\UserGroup',
            )
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'user_group';
    }
}
