<?php

namespace FS\UserBundle\Form\Type;

use Doctrine\Common\Collections\ArrayCollection;
use FS\BasicBundle\Utils\ArrayCollectionFilter;
use FS\UserBundle\Util\ContainerManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Doctrine\Common\Persistence\ObjectManager;
use Symfony\Component\Security\Core\SecurityContext;


class UserType extends AbstractType
{

    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var SecurityContext
     */
    private $context;

    private $containerManager;

    /**
     * @param ObjectManager $om
     * @param SecurityContext $context
     * @param ContainerManager $containerManager
     */
    public function __construct
    (
        ObjectManager $om,
        SecurityContext $context,
        ContainerManager $containerManager
    )
    {
        $this->om = $om;
        $this->context = $context;
        $this->containerManager = $containerManager;
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder->add(
            'username',
            null,
            [
                'label' => 'User name',
                'label_attr' => [
                    'class' => 'main-input'
                ],
                'attr' =>
                    [
                        'autocomplete' => 'off',
                        'maxLength' => 20
                    ],
                'max_length' => 20
            ]
        );
        $builder->add(
            'plainPassword',
            'repeated',
            [
                'type' => 'password',
                'invalid_message' => 'The password fields must match.',

                'required' => true,
                'first_options' => ['label' => 'Password'],
                'second_options' => ['label' => 'Confirm Password'],
                'options' => [
                    'label_attr' => [
                        'class' => 'main-input'
                    ],
                ]
            ]
        );
        $builder->add(
            'userGroup',
            'entity',
            [
                'label' => 'Group',
                'class' => 'FS\UserBundle\Entity\UserGroup',
                'empty_value' => '',
                'label_attr' => [
                    'class' => 'main-input'
                ],
                'choices' => $this->containerManager->filterArray(
                        $this->om->getRepository('FSUserBundle:UserGroup')->getAllGroupsForChoice()
                ),
                'attr' => array('style' => 'width:100%'),
            ]
        );
        $builder->add("pin", null, [
            'label' => "Mobile PIN"
        ]);
        $builder->add('cardNumber');
        $builder->add(
            'sharedRooms',
            'genemu_jqueryselect2_entity',
            [
                'class' => 'FS\CamBundle\Entity\Room',
                'property' => 'name',
                'multiple' => true,
                'required' => false,
                'label' => 'Rooms',
                'attr' => [
                    'placeholder' => 'Add new Room',
                    'class' => 'col-xs-12 hide-input'
                ],
                'choices' => $this->containerManager->filterArray(
                    $this->om->getRepository('FSCamBundle:Room')->findAll()
                )
            ]
        );
        $builder->add('displayName');
        $builder->add('accessedUsersAndGroups', 'users_and_groups', [
            'label' => 'Video Access',
            'attr' => ['class' => 'with-hr'],
            'label_attr' => ['class' => 'hr-line-all none']
        ]);
        $builder->add(
            'restrictAccess', 'restrict_access', [
                'label' => 'Allow access',
                'label_attr' => ['class' => 'hr-line-all none']
            ]
        );
        $builder->add(
            'expiration', 'expiration', [
                'label_attr' => ['class' => 'hr-line-delimiter'],
                'label' => ' ',
            ]
        );
        $builder->add(
            'disabledAccess', null, [
                'label' => 'Disable User',
                'label_attr' => ['style'=>"display:none"],
            ]
        );

    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\UserBundle\Entity\User',
                'validation_groups' => function (FormInterface $form) {
                    $data = $form->getData();
                    if ($data->getId()) {
                        return array('AdminEdit');
                    } else {
                        return array('AdminCreate');
                    }
                },
            )
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'user';
    }
}
