<?php

namespace FS\UserBundle\Repository;

use Doctrine\ORM\EntityRepository;
use FS\UserBundle\Entity\User;

class UserGroupRepository extends EntityRepository
{
    public function findAllExceptOneUserJoinedToUser()
    {
        $query = $this->getEntityManager()
            ->createQuery('SELECT g, u FROM FSUserBundle:UserGroup g LEFT JOIN g.users u ORDER BY g.name ASC, u.username ASC ');
        try {
            $result = $query->getResult();
            return $result;
        } catch (\Doctrine\ORM\NoResultException $e) {
            return null;
        }
    }

    public function findAccessedToUser(User $user)
    {

        $qb = $this
            ->createQueryBuilder('groups');

        $qb
            ->leftJoin('groups.users', 'users')
            ->leftJoin('users.accessedUsers', 'accessedUsers')
            ->leftJoin('users.accessedGroups', 'accessedGroups')
            ->leftJoin('groups.accessedUsers', 'groupAccessedUsers')
            ->leftJoin('groups.accessedGroups', 'groupAccessedGroups')
            ->andWhere(
                $qb->expr()->orX(
                    $qb->expr()->eq('groupAccessedGroups.id', ':userGroupId')
                )
            )
            ->setParameter('userGroupId', $user->getUserGroup()->getId());


        return $qb->getQuery()->getResult();
    }

    public function getAllGroups()
    {
        $qb = $this
            ->createQueryBuilder('groups');
        $qb->select('groups');
        $qb->where('groups.id <> 1');
        $qb->orderBy('groups.name');

        return $qb->getQuery()->getResult();
    }

    public function getAllGroupsForChoice()
    {
        $qb = $this
            ->createQueryBuilder('groups');
        $qb->select('groups');

        return $qb->getQuery()->getResult();
    }

}
