<?php

namespace FS\UserBundle\Repository;

use Doctrine\ORM\EntityRepository;
use FS\CamBundle\Entity\Room;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Entity\User;

class UserRepository extends EntityRepository
{
    public function findAccessedToUser(User $user)
    {

        $qb = $this
            ->createQueryBuilder('user');

        $qb->leftJoin('user.userGroup', 'group')
            ->leftJoin('user.accessedUsers', 'accessedUsers')
            ->leftJoin('user.accessedGroups', 'accessedGroups')
            ->leftJoin('group.accessedUsers', 'groupAccessedUsers')
            ->leftJoin('group.accessedGroups', 'groupAccessedGroups')
            ->andWhere('user.id <> :userId')
            ->andWhere(
                $qb->expr()->orX(
                    $qb->expr()->eq('accessedUsers.id', ':userId'),
                    $qb->expr()->eq('accessedGroups.id', ':userGroupId'),
                    $qb->expr()->eq('groupAccessedUsers.id', ':userId'),
                    $qb->expr()->eq('groupAccessedGroups.id', ':userGroupId')
                )
            )
            ->setParameter('userId', $user->getId())
            ->setParameter('userGroupId', $user->getUserGroup()->getId());


        return $qb->getQuery()->getResult();
    }

    public function findLDAPUsersByPath($domain, $bindScript)
    {
        $qb = $this->createQueryBuilder('u')
            ->select('u')
            ->where('u.ldapDomain = :ldapDomain')
            ->andWhere('u.deletedAt is NULL')
            ->setParameter('ldapDomain', $domain);
        if ($bindScript) {
            $qb->andWhere('u.ldapScript = :ldapScript')
                ->setParameter('ldapScript', $bindScript);
        }


        return $qb->getQuery()
            ->getResult();
    }

    public function findUsersForRoom(Room $room, $container)
    {
        $qb = $this
            ->createQueryBuilder('u');
        $qb
            ->leftJoin('u.userGroup', 'group')
            ->leftJoin('group.rooms', 'room')
            ->leftJoin('group.container', 'container')
            ->where('room = :room')
            ->andWhere('u.deletedAt is NULL');
        $orX = $qb->expr()->orX();
        $id = 0;
        while ($container){
            $orX->add('container = :container_'.$id);
            $qb->setParameter('container_'.$id, $container);
            if ($container->getParent()) {
                $container = $container->getParent();
                $id++;
            } else $container = null;
        }
        $qb
            ->andWhere($orX)
            ->setParameter('room', $room);

        return $qb;
    }

    public function findLDAPUserByName($username)
    {
        $qb = $this->createQueryBuilder('u')
            ->select('u')
            ->where('u.username = :username')
            ->andWhere('u.type = :type')
            ->andWhere('u.deletedAt is NULL')
            ->setParameter('type', User::TYPE_LDAP)
            ->setParameter('username', $username);

        return $qb->getQuery()
            ->getOneOrNullResult();
    }

    public function findLDAPUserByDN($dn)
    {
        $qb = $this->createQueryBuilder('u')
            ->select('u')
            ->where('u.ldapDN = :dn')
            ->andWhere('u.type = :type')
            ->andWhere('u.deletedAt is NULL')
            ->setParameter('type', User::TYPE_LDAP)
            ->setParameter('dn', $dn);

        return $qb->getQuery()
            ->getOneOrNullResult();
    }
}
