$(document).ready(function() {
    var Time = function Time(restItem, id, startTime, endTime)
    {
        this.restItem = restItem;
        this.restId = id;
        var restStartTimeList = restItem.find('.restrict-start-fields-list');
        var restEndTimeList = restItem.find('.restrict-end-fields-list');
        this.sH = 0;
        this.sM = 0;
        this.eH = 0;
        this.eM = 0;

        var restrictFrom = '<div class="restrict-from-'+id+'"></div>';
        var restrictTo = '<div class="restrict-to-'+id+'"></div>';
        restItem.find('.restrict-global').append(restrictFrom);
        restItem.find('.restrict-global').append(restrictTo);
        this.$restrictFrom = restItem.find('.restrict-from-'+id);
        this.$restrictTo = restItem.find('.restrict-to-'+id);

        if (typeof startTime === 'undefined') {
            var newRestrictStart = restStartTimeList.attr('data-prototype').replace(/__name__/g, id);
            var newRestrictEnd = restEndTimeList.attr('data-prototype').replace(/__name__/g, id);
            restStartTimeList.append('<li>'+newRestrictStart+'</li>').find('input');
            restEndTimeList.append('<li>'+newRestrictEnd+'</li>').find('input');
        } else {
            this.sM = (startTime / 60) % 60;
            this.sH = ((startTime / 60) - this.sM) / 60;
            this.eM = (endTime / 60) % 60;
            this.eH = ((endTime / 60) - this.eM) / 60;
        }
        var inputGL;
        restStartTimeList.find('li').each(function() {
            var input = $(this).find('input');
            if (input.attr('id').split('_').pop() == id) inputGL = input;
        });
        this.$restStartTime = inputGL;
        restEndTimeList.find('li').each(function() {
            var input = $(this).find('input');
            if (input.attr('id').split('_').pop() == id) inputGL = input;
        });
        this.$restEndTime = inputGL;

        this.isTimeEqual = function() {
            return this.sH == this.eH && this.sM == this.eM;
        };

        this.formatAMPM = function(hours, minutes) {
            var ampm = hours >= 12 ? 'PM' : 'AM';

            hours = hours % 12;
            hours = hours ? hours : 12;
            if (hours < 10) hours = '0' + hours;
            minutes = minutes < 10 ? '0' + minutes : minutes;

            return hours + ':' + minutes + ' ' + ampm;
        };
        this.showCurrentTime();
    };

    Time.prototype.destroy = function() {
        this.$restStartTime.parent().remove();
        this.$restEndTime.parent().remove();
        this.$restrictFrom.remove();
        this.$restrictTo.remove();
        this.$startHours.closest('.custom-row').remove();
    };

    Time.prototype.populateForm = function() {
        var $restrictIntervalPrototype = $('#restrict-interval-prototype');
        $restrictIntervalPrototype.before(
            '<div class="custom-row"><div class="restrict-interval-'+this.restId+'">' +
            $restrictIntervalPrototype.html() +
            '</div></div>'
        );

        var $interval = $('.restrict-interval-'+this.restId);

        this.$startHours = $interval.find('.restrict-start-time-hours');
        this.$startMinutes = $interval.find('.restrict-start-time-minutes');
        this.$endHours = $interval.find('.restrict-end-time-hours');
        this.$endMinutes = $interval.find('.restrict-end-time-minutes');

        this.setFormTime();
    };

    Time.prototype.getTimestampStart = function() {
        var startHours = +this.$startHours.val();
        var startMinutes = +this.$startMinutes.val();
        return +((startHours*60) + startMinutes) * 60;
    };

    Time.prototype.getTimestampEnd = function() {
        var endHours = +this.$endHours.val();
        var endMinutes = +this.$endMinutes.val();
        return ((endHours*60) + endMinutes) * 60;
    };

    Time.prototype.isNoAccess = function() {
        return (this.sH == null || this.sM == null || this.eH == null || this.eM == null)
            || this.isTimeEqual();
    };

    Time.prototype.showCurrentTime = function() {
        this.$restrictFrom.html(this.formatAMPM(this.sH, this.sM));
        this.$restrictTo.html(this.formatAMPM(this.eH, this.eM));
    };

    Time.prototype.setFormTime = function() {
        this.$startHours.val(this.sH);
        this.$endHours.val(this.eH);
        this.$startMinutes.val(this.sM);
        this.$endMinutes.val(this.eM);
    };

    Time.prototype.saveFormTime = function() {
        var startHours = +this.$startHours.val();
        var startMinutes = +this.$startMinutes.val();
        var endHours = +this.$endHours.val();
        var endMinutes = +this.$endMinutes.val();

        var startTimestamp = +((startHours*60) + startMinutes) * 60;
        var endTimestamp = +((endHours*60) + endMinutes) * 60;

        if (startTimestamp > endTimestamp) return false;
        this.sH = startHours;
        this.sM = startMinutes;
        this.eH = endHours;
        this.eM = endMinutes;

        this.$restStartTime.val(startTimestamp);
        this.$restEndTime.val(endTimestamp);
    };

    var RestrictItem = function RestrictItem(item)
    {
        this.restItem = item;
        this.itemsCount = 0;
        this.isEnabledField = item.find('.restrict-field-is-enabled');
        this.restrictStatus = item.find('.restrict-status');
        this.$restStartTimeList = item.find('.restrict-start-fields-list');
        this.$restEndTimeList = item.find('.restrict-end-fields-list');
        this.$restrictGlobal = item.find('.restrict-global');

        this.restTime = [];
        var global = this;

        if (this.isEnabledField.is(':checked')) {
            this.restItem.find('.restrict-access-edit').addClass('btn-active');
        }

        this.prepareRestTime = function() {
            var restTimeStart = [], restTimeEnd = [];
            this.$restStartTimeList.find('li').each(function() {
                restTimeStart.push($(this).find('input').val());
            });
            this.$restEndTimeList.find('li').each(function(item) {
                restTimeEnd.push($(this).find('input').val());
            });
            restTimeStart.forEach(function(elem, i) {
                global.restTime.push(new Time(global.restItem, global.itemsCount, elem, restTimeEnd[i]));
                global.itemsCount++;
            });
        };
        if (this.isEnabledField.is(':checked')) this.prepareRestTime();

        this.showRestrictStatus = function() {
            this.restrictStatus.show();
            if (this.isEnabledField.is(':checked')) {
                if (this.restTime.length === 0) {
                    this.restrictStatus.html('<span>Full Access</span>');
                } else {
                    this.restrictStatus.html("");
                }
            } else {
                this.restrictStatus.html('<span class="restrict-access-no">No Access</span>');
            }
        };

        this.showCurrentTime = function() {
            if (this.restTime.length == 0) {
                this.$restrictGlobal.hide();
            } else {
                this.$restrictGlobal.show();
                this.restTime.forEach(function(time) {
                    time.showCurrentTime();
                });
            }
        };
        this.showCurrentTime();
        this.showRestrictStatus();

        /* This function checks if time intervals are selected properly */
        this.isValid = function(timeArray) {
            var previousEnd = -1, isValid = true;
            timeArray.sort(function(a, b) {
                if (a.getTimestampStart() < b.getTimestampStart()) return -1;
                else if (a.getTimestampStart() > b.getTimestampStart()) return 1;
                else return 0;
            });
            timeArray.forEach(function(time) {
                if (time.getTimestampEnd() <= time.getTimestampStart()) {
                    isValid = 'End time must be bigger than start time';
                    return;
                } else if (time.getTimestampStart() <= previousEnd) {
                    isValid = 'Time intervals should not overlay each other';
                    return;
                }
                previousEnd = time.getTimestampEnd();
            });
            return isValid;
        };

        $(global.restItem).on('click', '.restrict-field-is-enabled', function() {
            var $self = $(this);
            global.showRestrictStatus();
            if($self.is(":checked")){
                global.restItem.find('.restrict-access-edit').addClass('btn-active').show();
                global.$restrictGlobal.show();
            } else {
                global.restItem.find('.restrict-access-edit').removeClass('btn-active').hide();
                global.$restrictGlobal.hide();
            }
        }).on('click', '.restrict-access-edit', function() {
            var $self = $(this);
            var $error = $('.restrict-date-error');
            $error.hide();
            if(!global.isEnabledField.is(":checked")){
                return false;
            }

            var accessId = $self.data('id');
            $('#restrict-modal-id').val(accessId);

            var $modal = $('#ajax-modal');
            var params = {backdrop: 'static'};
            params.height = 200;
            params.width = 450;

            $modal.modal(params);

            var temporaryTime = [];
            function setFormTime() {
                $('div[class^="restrict-interval"]').remove();
                if (global.restTime.length === 0) {
                    var time = new Time(global.restItem, 0);
                    global.itemsCount++;
                    temporaryTime.push(time);
                    time.populateForm();
                }
                global.restTime.forEach(function(elem) {
                    elem.populateForm();
                });
            }
            setFormTime();

            $('#restrict-modal-save').click(function () {
                var mergedTime = global.restTime.concat(temporaryTime);
                var isValid = global.isValid(mergedTime);
                if (isValid !== true) {
                    $error.html(isValid);
                    $error.show();
                    return false;
                }
                global.restTime = mergedTime;
                global.restTime.forEach(function(time) {
                    time.saveFormTime();
                });
                global.showCurrentTime();
                global.showRestrictStatus();
            });
            $('#restrict-modal-clear').click(function () {
                global.restTime = global.restTime.concat(temporaryTime);
                global.restTime.forEach(function(elem) {
                    elem.destroy();
                });
                global.itemsCount = 0;
                global.restTime = [];
                global.showCurrentTime();
                global.showRestrictStatus();
            });
            $('#restrict-add-interval').click(function() {
                var time = new Time(global.restItem, global.itemsCount);
                global.itemsCount++;
                temporaryTime.push(time);
                time.populateForm();
            });
            $('#ajax-modal .close').click(function() {
                temporaryTime.forEach(function(time) {
                    time.destroy();
                    global.itemsCount--;
                });
            });
            $('#ajax-modal').on('click', '.fa-times', function() {
                var id = $(this).parent().attr('class').split('-').pop();
                var buffer = global.restTime, deleteId = -1;
                buffer.forEach(function(time, idItem) {
                    if (time.restId == id) {
                        deleteId = idItem;
                        time.destroy();
                    }
                });
                if (deleteId >= 0) global.restTime.splice(deleteId, 1);
                buffer = temporaryTime; deleteId = -1;
                buffer.forEach(function(time, idItem) {
                    if (time.restId == id) {
                        deleteId = idItem;
                        time.destroy();
                    }
                });
                if (deleteId >= 0) temporaryTime.splice(deleteId, 1);
            });
        });
    };

    var RestrictItems = [];

    $('.restrict-access-item').each(function(index, value){
        RestrictItems.push(new RestrictItem($(this)));
    });
});