<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 16.01.15
 * Time: 12:08
 */

namespace FS\UserBundle\Security\Core\Authentication;


use Doctrine\ORM\EntityManager;
use FS\UserBundle\Entity\User;
use Symfony\Component\Security\Core\Authentication\Provider\DaoAuthenticationProvider;
use Symfony\Component\Security\Core\Authentication\Token\UsernamePasswordToken;
use Symfony\Component\Security\Core\Encoder\EncoderFactoryInterface;
use Symfony\Component\Security\Core\Exception\AuthenticationException;
use Symfony\Component\Security\Core\Exception\BadCredentialsException;
use Symfony\Component\Security\Core\User\UserCheckerInterface;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\UserProviderInterface;

/**
 * Class FSAuthenticationProvider
 * @package FS\UserBundle\Security\Core\Authentication
 */
class FSAuthenticationProvider extends DaoAuthenticationProvider
{
    private $om;

    public function __construct
    (
        UserProviderInterface $userProvider,
        UserCheckerInterface $userChecker,
        $providerKey,
        EncoderFactoryInterface $encoderFactory,
        $hideUserNotFoundExceptions = true,
        EntityManager $om
    )
    {
        parent::__construct($userProvider, $userChecker, $providerKey, $encoderFactory, $hideUserNotFoundExceptions);

        $this->om = $om;
    }

    /**
     * {@inheritdoc}
     */
    protected function checkAuthentication(UserInterface $user, UsernamePasswordToken $token)
    {
        //$currentUser = $token->getUser();
        /** @var User $user */
        $type = $user->getType();

        if(!$user->hasAccessNow())
        {
            throw new AuthenticationException('Access Restricted');
        }

        if($user->isAccessExpired())
        {
            throw new AuthenticationException('Access Expired');
        }
        if($user->isDisabledAccess())
        {
            throw new AuthenticationException('Access Disabled');
        }

        if ($type == 'ldap') {
            $ldapServer = $this->om->getRepository('FSUserBundle:LdapServer')->getLdapServer();
            if(!$ldapServer){
                throw new AuthenticationException('LDAP disabled');
            }
            $conn = @ldap_connect($ldapServer->getServer(), '389');

            @ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3);

            $ldapBind = @ldap_bind($conn, $user->getLdapDN(), $token->getCredentials());

            @ldap_close($conn);

            if ($ldapBind) {
                return;
            } else {
                throw new BadCredentialsException('The presented password is invalid.');
            }

        } else {
            parent::checkAuthentication($user, $token);
        }
    }
} 