<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 22.03.2016
 * Time: 13:32
 */

namespace FS\UserBundle\Util;

use Doctrine\ORM\EntityManager;
use FOS\UserBundle\Doctrine\UserManager;
use FS\UserBundle\Entity\LdapServer;
use FS\UserBundle\Entity\LdapSynchronize;
use FS\UserBundle\Entity\User;

/**
 * Class LdapSynchronizer
 * @package FS\UserBundle\Util
 */
class LdapSynchronizer
{
    /** @var EntityManager entityManager */
    private $entityManager;

    /** @var UserManager */
    private $userManager;

    /** @var  LdapServer $ldapServer */
    private $ldapServer;

    /**
     * LdapSynchronizer constructor.
     * @param EntityManager $entityManager
     * @param UserManager $userManager
     */
    public function __construct(EntityManager $entityManager, UserManager $userManager)
    {
        $this->entityManager = $entityManager;
        $this->userManager = $userManager;

        $this->ldapServer = $this->entityManager
            ->getRepository('FSUserBundle:LdapServer')
            ->getLdapServer();
    }

    public function synchronize(LdapSynchronize $ldapSynchronize)
    {
        $ldapSynchronizeResult = [];

        $ldapSynchronizeResult['type'] = 'ldap_sync_error';
        $ldapSynchronizeResult['body'] = 'Unable to connect LDAP server';
        $ldapSynchronizeResult['status'] = false;

        $oldUsers = $ldapSynchronize->getUsersInSync()->toArray();
        $newUsers = [];
        $userGroup = $ldapSynchronize->getUserGroup();

        $conn = @ldap_connect($this->ldapServer->getServer(), '389');
        if ($conn) {
            @ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3);
            $ldapBind = @ldap_bind($conn, $this->ldapServer->getUsername(), $this->ldapServer->getPassword());
            if ($ldapBind) {
                $search = @ldap_search(
                    $conn,
                    $ldapSynchronize->getBindScript() . ',' . $this->ldapServer->getDn(),
                    'cn=*',
                    [
                        'member',
                        'memberUid',
                        'memberOf',
                        'uniquemember'
                    ]);
                if ($search) {
                    $entry = @ldap_first_entry($conn, $search);
                    $attrs = @ldap_get_attributes($conn, $entry);

                    $entries = [];
                    $mainFiled = 'member';

                    if (array_key_exists('member', $attrs)) {
                        $mainFiled = 'member';
                    } elseif (array_key_exists('memberUid', $attrs)) {
                        $mainFiled = 'memberUid';
                    } elseif (array_key_exists('memberOf', $attrs)) {
                        $mainFiled = 'memberOf';
                    } elseif (array_key_exists('uniquemember', $attrs)) {
                        $mainFiled = 'uniquemember';
                    }

                    $count = $attrs[$mainFiled]['count'];
                    $members = $attrs[$mainFiled];

                    for ($i = 0; $i < $count; ++$i) {
                        $fields = ["ou", "sn", "cn", "givenname", "mail", "dn", "uid", "samaccountname"];
                        if ($mainFiled == 'memberUid') {
                            $bindScript = "(&(uid=" . $members[$i] . ")(|(objectClass=person)(objectClass=inetOrgperson)(objectClass=account)))";
                            $search = @ldap_search($conn, $this->ldapServer->getDn(), $bindScript, $fields);
                        } else {
                            $bindScript = $members[$i];
                            $search = @ldap_search($conn, $bindScript, 'cn=*', $fields);
                        }
                        $user = @ldap_get_entries($conn, $search);
                        $entries[] = $user[0];
                    }
                    if(!array_key_exists('count',$entries)){
                        $entries['count'] = count($entries);
                    }
                    $count = 0;

                    for ($i = 0; $i < $entries['count']; ++$i) {
                        if (array_key_exists('uid', $entries[$i])) {
                            $username = $entries[$i]['uid'][0];
                        } else if (array_key_exists('samaccountname', $entries[$i])) {
                            $username = $entries[$i]['samaccountname'][0];
                        } else {
                            $username = $entries[$i]['cn'][0];
                        }

                        $key = $this->arraySearchUsers($username, $oldUsers);

                        $userWithDn = $this->entityManager->getRepository('FSUserBundle:User')->findLDAPUserByDN($entries[$i]['dn']);
                        if($userWithDn){
                            unset($oldUsers[$key]);
                            continue;
                        }

                        if ($key !== false) {
                            unset($oldUsers[$key]);
                        } else {

                            $count++;
                            $userToAdd = new User();
                            $userToAdd->addSelfToAccessedUsers();
                            $userToAdd->setEnabled(true);
                            $userToAdd->setUsername($username);
                            $userToAdd->setType(User::TYPE_LDAP);
                            $userToAdd->setPlainPassword('LdapUser' . $username);
                            if ($userGroup) {
                                $userToAdd->setUserGroup($userGroup);
                            }
                            $userToAdd->setLdapScript($ldapSynchronize->getBindScript());

                            $userToAdd->setLdapDomain($this->ldapServer->getDn());
                            $userToAdd->setLdapDN($entries[$i]['dn']);

                            $newUsers[] = $userToAdd;
                        }
                    }
                    $usersFromDB = $this->entityManager->getRepository('FSUserBundle:User')->findAll();
                    foreach ($newUsers as $nUser)
                    {
                        if (!($this->arraySearchUsers($nUser->getUsername(), $usersFromDB))) {
                            $ldapSynchronize->addUser($nUser);
                            $this->userManager->updateUser($nUser);
                        } else {
                            $count--;
                        }
                    }
                    foreach ($oldUsers as $oldUser)
                    {
                        $ldapSynchronize->removeUser($oldUser);

                        $em = $this->entityManager;

                        $oldUser->setUsernameCanonical(hash('md5', $oldUser->getUsername().$oldUser->getId()));
                        $oldUser->setEmailCanonical(hash('md5', $oldUser->getEmail().$oldUser->getId()));
                        $oldUser->setDeletedAt(new \DateTime());

                        $em->persist($oldUser);
                    }
                    $this->entityManager->persist($ldapSynchronize);
                    $this->entityManager->flush();

                    if ($count == 1) $ldapSynchronizeResult['body'] = $count . ' user added';
                    else $ldapSynchronizeResult['body'] = $count . ' users added';
                } else {
                    $ldapSynchronizeResult['body'] = 'No users';
                }
                $ldapSynchronizeResult['status'] = true;
                $ldapSynchronizeResult['type'] = 'ldap_sync_note';
            }
        }
        return $ldapSynchronizeResult;
    }

    /**
     * @param $username
     * @param $usersArray
     * @return bool|int|string
     */
    private function arraySearchUsers($username, $usersArray)
    {
        foreach ($usersArray as $key => $value) {
            /** @var User $value */
            if ($value->getUsername() === $username && !$value->isDeleted()) {
                return $key;
            }
        }
        return false;
    }
}