#!/bin/bash
### Function to parse YML files
### http://stackoverflow.com/questions/5014632/how-can-i-parse-a-yaml-file-from-a-linux-shell-script
### https://swarm.workshop.perforce.com/files/guest/perforce_software/sdp/dev/Server/Unix/p4/common/lib/libYAML.sh?v=1
parse_yaml () {
    local prefix=$2
    local s
    local w
    local fs
    s='[[:space:]]*'
    w='[a-zA-Z0-9_]*'
    fs="$(echo @|tr @ '\034')"
    sed -ne "s|^\($s\)\($w\)$s:$s\"\(.*\)\"$s\$|\1$fs\2$fs\3|p" \
        -e "s|^\($s\)\($w\)$s[:-]$s\(.*\)$s\$|\1$fs\2$fs\3|p" "$1" |
    awk -F"$fs" '{
    indent = length($1)/7;
    vname[indent] = $2;
    for (i in vname) {if (i > indent) {delete vname[i]}}
        if (length($3) > 0) {
            vn=""; for (i=0; i<indent; i++) {vn=(vn)(vname[i])("_")}
            printf("%s%s%s=(\"%s\")\n", "'"$prefix"'",vn, $2, $3);
        }
    }' | sed 's/_=/+=/g'
}
### Function to compare versions
### http://stackoverflow.com/questions/4023830/bash-how-compare-two-strings-in-version-format
vercomp () {
    if [[ $1 == $2 ]]; then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2) # careful here
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++)); do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++)); do
        if [[ -z ${ver2[i]} ]]; then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]})); then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]})); then
            return 2
        fi
    done
    return 0
}
### Function to log
log () {
    echo "[$(date +'%Y-%m-%d %H:%M:%S')] $1" >> "${0%/*}"/app/logs/update.log
    echo $1
}
### Function to exit if any exception rises
fallback () {
    if [[ $? -ne 0 ]]; then
        if [ -z "$1" ]; then log "Something gone wrong!"
        else log $1; fi

        cd "${0%/*}"
        tar -xzf src-backup.tar.gz

        ### Remove temporary directory
        rm -rf tmp
        rm src-backup.tar.gz
        ### Set rights for created files
        find src -type f -exec chmod 664 {} \;
        find src -type d -exec chmod 775 {} \;
        find src -type d -exec chmod g+s {} \;

        exit 1
    fi
}
### Go to script location and create files backup
log "Starting website update."
cd "${0%/*}"
log "Preparing backup"
tar -zcf src-backup.tar.gz app src web --exclude=web/video
eval $(parse_yaml app/config/parameters.yml "orig_")
fallback "Could not parse parameters file"
### Extract patch archive into tmp directory
cd tmp
fallback "Tmp directory is not available"
mkdir patch
log "Unpacking patch file"
tar -xzf patch.tar.gz -C patch
fallback "Failed trying to unzip patch file"
cd patch
eval $(parse_yaml update.yml "upd_")
fallback "Could not parse update file"
### Copy content of update to the destination
vercomp "$upd_site_version" "$orig_parameters_site_version"
### Check versions
if [[ $? -eq 1 ]]; then
    count=$(find -type f|wc -l)
    echo "num_of_files=$count"
    find -type f | while read file; do
        ### Check if file should be updated
        isUpdate=true
        for i in "${upd_exclude[@]}"; do
            if [[ $file == $i* ]]; then
                isUpdate=false
            fi
        done
        ### And then update if it is
        if [ "$isUpdate" = true ]; then
            cp --parents $file "${0%/*}"
            fallback "Failed trying to copy $file"
            log "Successfully copied $file"
        fi
    done
    ### Call extra.sh if any
    if [ -f extra.sh ]; then
        chmod a+x extra.sh
        ./extra.sh
    fi
    ### Update website version
    cd "${0%/*}"
    sed -i "s/\(site_version:\).*/\1 $upd_site_version/" app/config/parameters.yml
    log "Successfully updated to version $upd_site_version"
else
    cd "${0%/*}"
    log "Update version $upd_site_version must be greater than the website version $orig_parameters_site_version"
fi
### Remove temporary directory
rm -rf tmp
rm src-backup.tar.gz
### Set rights for created files
find src -type f -exec chmod 664 {} \;
find src -type d -exec chmod 775 {} \;
find src -type d -exec chmod g+s {} \;
### Call assets.sh
log "Updating assets"
#chmod +x assets.sh
sh assets.sh >> "${0%/*}"/app/logs/update.log
echo "update_finished=true"
log "Success!"