<?php

namespace Lexik\Bundle\FormFilterBundle\Filter\Doctrine;

use Lexik\Bundle\FormFilterBundle\Filter\Query\QueryInterface;
use Lexik\Bundle\FormFilterBundle\Filter\Doctrine\Expression\DBALExpressionBuilder;

use Doctrine\DBAL\Query\QueryBuilder;

/**
 * @author Jeremy Barthe <j.barthe@lexik.fr>
 */
class DBALQuery implements QueryInterface
{
    /**
     * @var QueryBuilder $queryBuilder
     */
    private $queryBuilder;

    /**
     * @var DBALExpressionBuilder $expr
     */
    private $expressionBuilder;

    /**
     * Constructor.
     *
     * @param QueryBuilder $queryBuilder
     */
    public function __construct(QueryBuilder $queryBuilder)
    {
        $this->queryBuilder      = $queryBuilder;
        $this->expressionBuilder = new DBALExpressionBuilder($this->queryBuilder->expr());
    }

    /**
     * {@inheritDoc}
     */
    public function getEventPartName()
    {
        return 'dbal';
    }

    /**
     * {@inheritDoc}
     */
    public function getQueryBuilder()
    {
        return $this->queryBuilder;
    }

    /**
     * Get QueryBuilder expr.
     *
     * @return Doctrine\DBAL\Query\Expression\ExpressionBuilder
     */
    public function getExpr()
    {
        return $this->queryBuilder->expr();
    }

    /**
     * Get root alias.
     *
     * @return string
     */
    public function getAlias()
    {
        $from = $this->queryBuilder->getQueryPart('from');

        return $from[0]['alias'];
    }

    /**
     * Get expr class.
     *
     * @return \Lexik\Bundle\FormFilterBundle\Filter\Doctrine\Expression\ExpressionBuilder
     */
    public function getExpressionBuilder()
    {
        return $this->expressionBuilder;
    }
}
